/*
Address.java - interface to the Address table
Copyright (C) 2012 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/
package ess;

// import java.sql.Connection;
import java.sql.*;
public class Address extends AdisoftDbase {
	// Need to add the methodology of opening up a new message file for each rpt.
	//
	    // private ServerSystemTable systemTable;

	    public final int ADDRESS = 0;
	    public final int PERS_NUM = ADDRESS + 1;
	    public final int SCAN_MODE = PERS_NUM + 1;
	    public final int CLERK = SCAN_MODE + 1;
	    private String[] tags = {"ADDRESS","PERS_NUM","SCAN_MODE","CLERK"};
	    private String[] values = {"","","NONE",""};
	    private String[] scanModes = {"Upload","Add","Create","Inactive","Blocked","Confirm"};
	    private String sqlConfirmSearch = "SELECT * FROM ADDRESS WHERE PERS_NUM = '$persnum$' AND SCAN_MODE = 'Confirm'";
	    ServerSystemTable systemTable;
	    
	   // SCAN_MODE values UPLOAD, ADD, CREATE, INACTIVE, BLOCKED, PRENOTE
	    /* 
	     * UPLOAD - adds the scan to the scan upload screen (default)
	     * ADD - will add receipt to a report in a certain status otherwise UPLOAD
	     * CREATE - will create a report if needed, then do the ADD
	     * INACTIVE - reporter/auditor removed device from their active list
	     * BLOCKED - system blocked this account for some reason
	     * PRENOTE - going thru initial handshake after the "REGISTER pers_num" message received
	     */
	    public Address()
	    {
	//caller needs to call setDB().        
//	        SystemTable = new ServerSystemTable();
//	        smtp_address = SystemTable.getSystemString("SMTP_ADDRESS","192.168.0.21");
	    }
	    
	    public Address(String DBUrl, String DBUser, String DBPassword)
	    {
	        setDB(DBUrl, DBUser, DBPassword);
	        setSystemTable();
	    }
	    
	    public Address(Connection DBConnection)
	    {
	        setConnection(DBConnection);
	        setSystemTable();
	    }

	    public void setSystemTable()
	    {
	        systemTable = new ServerSystemTable();
	        systemTable.setConnection(super.getConnection());
	    }

	    public int getKeyPosition()
	    {
	    	return ADDRESS;
	    }
	    
	    public String getKeyName()
	    {
	    	return tags[getKeyPosition()];
	    }

	    private String getPersnumName()
	    {
	    	return tags[PERS_NUM];
	    }
	    
	    
	    public void setAddress(String x)
	    {
	        values[ADDRESS] = x;
	    }
	    public void setPers_num(String x)
	    {
	        values[PERS_NUM] = x;
	    }
	    public void setScanMode(String x)
	    {
	        values[SCAN_MODE] = x;
	    }
	    public void setClerk(String x)
	    {
	        values[CLERK] = x;
	    }

	    public String getAddress()
	    {
	        return(values[ADDRESS]);
	    }
	    public String getPers_num()
	    {
	    	return(values[PERS_NUM]);
	    }
	    public String getClerk()
	    {
	    	return(values[CLERK]);
	    }

	    public String getScanMode()
	    {
	    	return(values[SCAN_MODE]);
	    }
	    public String[] getTags()
	    {
	        return(tags);
	    }
	    
	    public String[] getValues() 
	    {
	        return(values);
	    }
	    
	    public String[] getAvailableScanModes()
	    {
	    	return scanModes;
	    }
	    
	    public boolean isValidAddress(String emailAddress) {
	      return this.setResultSet("ADDRESS", getKeyName(), emailAddress);  //check for multiple records
	    }
	    
	    public String getOwner(String emailAddress)
	    {
	       String retVal = null;
	       if (isValidAddress(emailAddress))
	       {
	    	   try {
	    		   retVal = super.myResult.getString(getPersnumName());
	    	   } catch (SQLException e){
	    		   AuditTrail.println("[500] Address.getOwner - cannot access the Pers_num column");
	    		   e.printStackTrace();
	    	   }
	       }
	       return retVal;
	    }
	    
	    public void updateColumn(String reference, String tag, String newValue)
	    {
	        String[] tags = {tag};
	        String[] values = {newValue};
	        setSimple(ADDRESS,reference, tags, values);
	    }
	    
	    public void setSimple(String[] elementTags, String[] elementValues) {
	    //The key here is the first member of the ElementTags array    
	       setSimple("ADDRESS", elementTags[0], elementValues[0], elementTags, elementValues);
	    }
	  
	    public void setSimple(int keyTag, String KeyValue, String[] elementTags, String[] elementValues)
	    {
	        setSimple("ADDRESS", tags[keyTag], KeyValue, elementTags, elementValues);
	    }
	    public void setSimple(int keyTag, String KeyValue)
	    {
	        setSimple("ADDRESS", tags[keyTag], KeyValue, tags, values);
	    }
	    public int setNewAddress(String persnum, String emailAddress)
	    {
    		//final int ADDRESS_UPDATE_OK = 0;
    		//final int ADDRESS_EXISTS = 1;
    		final int ADDRESS_INCOMPLETE = 2;
    		final int PERSNUM_INVALID = 3;
    		final int ADDRESS_UPDATE_ERROR = -1;

    	    PersonnelTable pt = new PersonnelTable();
    		pt.setConnection(getConnection());
    		pt.setSQLTerminator(getSQLTerminator());
    		pt.setSQLStrings();
    		
	    	int retVal = ADDRESS_UPDATE_ERROR;
	    	if (persnum == null) 
	    	{
	    		retVal = ADDRESS_INCOMPLETE;
	    	} else
	    	{
	    		if (pt.setPersInfoAll(persnum))
	    		{
	    			try {
	    				this.setAddress(emailAddress);
	    				this.setPers_num(persnum);
	    				this.setScanMode("Confirm");
	    				this.setClerk(persnum);
	    				// setInsert will always return either a 0 or a 1.  A one means the address already exists
	    				retVal = this.setInsert("ADDRESS", tags[this.getKeyPosition()], values[this.getKeyPosition()], tags, values);
	    			} catch (Exception e) {
	    				AuditTrail.println("[500] Address.setNewAddress serious error - see logs: " + e.toString());
	    				e.printStackTrace();
	    			} 
	    		} else 
	    		{
	    			retVal = PERSNUM_INVALID;
	    		}
	    	}
	    	return retVal;
	    }
	    
	    public boolean setAddresses2Confirm(String persnum)
	    {
	    	// Need to take from the system.xml file
	    	String sqlConfirm = sqlConfirmSearch.replace("$persnum$", persnum);
	    	return super.setResultSet(sqlConfirm);
	    }
	    
	    public String getOneAddress2Confirm(String persnum)
	    {
	    	String retVal = "NONENONE";
	    	if (setAddresses2Confirm(persnum)) {
	    		retVal = getAddressFromTable();
	    	}
	    	return retVal;
	    }
	    
	    public String getAddressFromTable()
	    {
	    	String retVal; 
    		try {
    			retVal = myResult.getString("ADDRESS");
    		} catch (SQLException e) {
    			retVal = "ERROR";
    			AuditTrail.println("[500] Address.getOneAddress2Confirm - error in getting information - see logs");
    			e.printStackTrace();
    		}
            return retVal;
	    }
	    
		public String getNextAddressFromTable()
		{
		    String retVal = "NONENONE";
		    try {
			  if (myResult.next())
			  {
				  retVal = myResult.getString("ADDRESS");
			  }
		    } catch (SQLException e) {
		    	retVal = "ERROR";
		    	AuditTrail.println("[500] Address.getNextAddress - error in getting information - see logs");
		    	e.printStackTrace();
		    }
		    return retVal;
		}


	}



