/*
AdisoftDOM.java - Generic Adisoft DOM access   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.DocType;
import org.jdom.Attribute;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import org.jdom.output.Format;
import org.jdom.output.EscapeStrategy;

public class AdisoftDOM {
  
  protected static AuditTrail Logfile;  //static???
  protected Document CommonDOM;
  private boolean OmitDeclaration = true; //most usage doesn't want this
  private String DbaseURL = "";
  private String DbaseUser = "";
  private String DbasePassword = "";
  private Connection dbConnection;
  private AdisoftDbase db;
  private boolean domProcessed = false;
  protected String sqlTerminator = ";";
  private EscapeStrategy EscStr = new EscapeNone(); 
  
  public AdisoftDOM () {
  }
  
  public AdisoftDOM (String FileName) {
    File InputFile = new File(FileName);
    setDOM(InputFile);
  }
  
  public void setDOM (Document InputDOM) {
    CommonDOM = InputDOM;
    domProcessed = true;
  }
  
  public void setDB(String Url) {
    DbaseURL = Url;
  }
  
  public void setDB(String Url, String User, String Password) {
    DbaseURL = Url;
    DbaseUser = User;
    DbasePassword = Password;
  }
  
  public void setConnection(Connection dbConn) {
    dbConnection = dbConn;
  }
  
  public Connection getConnection() {
    return dbConnection;
  }
  
  public boolean getDOMProcessed () {
    return domProcessed;
  }
  
  public void setEscapeStrategy(EscapeStrategy X) {
    EscStr = X;
  }
    
  public void setSQLTerminator(String X) {
    sqlTerminator = X;
  }
  public String getSQLTerminator() {
    return sqlTerminator;
  }
  
  public void setDOM (File InputFile) {  //Populates CommonDOM from an XML file
    // useage: File myfile = new File("InputFile"); obj.setDOM(myfile);
    domProcessed = false;
    SAXBuilder builder = new SAXBuilder();
    CommonDOM = new Document(new Element("container"));  // was "empty"
    try {
      CommonDOM = builder.build(InputFile);
      //Logfile.println("[000] AdisoftDOM.SetDOM loaded " + InputFile.getName());
      domProcessed = true;
    } catch (org.jdom.JDOMException e) {
      AuditTrail.println("[400] AdisoftDOM.setDOM (1) XML file DOM problem: " + InputFile.getName());
      AuditTrail.println("[400] " + e.toString());
    } catch (java.io.IOException e) {
      AuditTrail.println("[400] AdisoftDOM.setDOM (1) XML file IO problem: " + InputFile.getName());
      AuditTrail.println("[400] " + e.toString());
    }
  }  
    
  public void setDOM (String InputString) {  //Populates CommonDOM from a string
    // useage: Uses a string for an input stream.
    domProcessed = false;
    StringReader SR = new StringReader(InputString);
    SAXBuilder builder = new SAXBuilder();
    CommonDOM = new Document(new Element("container"));  // was "empty"
    try {
      CommonDOM = builder.build(SR);
      domProcessed = true;
    } catch (org.jdom.JDOMException e) {
      AuditTrail.println("[400] AdisoftDOM.setDOM (2) XML string problem" + InputString);
      AuditTrail.println("[400] " + e.toString());
    } catch (java.io.IOException e) {
      AuditTrail.println("[400] AdisoftDOM.setDOM (2) XML other problem: " + InputString);
      AuditTrail.println("[400] " + e.toString());
    }
  } 
  
  public void setDOMFromTable (String TableName) {
    domProcessed = false;
    CommonDOM = new Document(new Element("table"));
    if (DbaseURL.equals("")) {
       db = new AdisoftDbase();
       db.setConnection(dbConnection);
       db.setSQLTerminator(sqlTerminator);
    } else { 
       db = new AdisoftDbase(DbaseURL,DbaseUser,DbasePassword);
       db.setSQLTerminator(sqlTerminator);
    }
    if (db.setTable(TableName)){
      ResultSet rs = db.getResultSet();
      int j = 1;
      // Integer k = new Integer(j);
      try {
         ResultSetMetaData md = rs.getMetaData();
         Element TableTopElement = CommonDOM.getRootElement();
         TableTopElement = TableTopElement.setAttribute(new Attribute("name",TableName));
         do {
           Element Item = new Element("rowset");  //consider using an attribute - changing name to rowset
           Item = Item.setAttribute(new Attribute("sequence",Integer.toString(j++)));
           for (int i = 1; i <= md.getColumnCount(); i++) {
             Element Field = new Element(getTrim(md.getColumnName(i)).toLowerCase());
             Field = Field.setText(getTrim(db.myResult.getString(i)));
             Item = (Element) Item.addContent(Field); 
           }
           TableTopElement = (Element) TableTopElement.addContent(Item);
         } while (rs.next());
         rs.close();
         domProcessed = true;
      } catch (SQLException e) {
         e.printStackTrace();
         AuditTrail.println("[500] AdisoftDOM.setDOMFromTable MetaData retrieval problem for " + TableName);  
      }
    } else {
      AuditTrail.println("[400] AdisoftDOM.setDOMFromTable cannot retrieve table " + TableName);  
    }
    db.close();
  } 
  public String getTrim(String x) {
    if (x == null) return "";
    return x.trim();
  }

  public String getDOMTableValueWhere(String SubTable, String EleAgent, String WhereValue, String EleReturn) {
    /*
    Assuming SubTable = 'table2', EleAgent = 'Agent', WhereValue = 'KalaKala', EleReturn = 'Return'
    wlll "produce" a SQL as such
    SELECT Return FROM THISTABLE.table2 WHERE Agent = 'KalaKala';
    and will return '123' from the DOM below:
    
    <root>
      <table1>
        <rowset>
          <Agent>KalaKala</Agent>
          <Return>XYZ</Agent>
        </rowset>
        <rowset>
          <Agent>HotuMotu</Agent>
          <Return>ABC</Agent>
        </rowset>
      </table1>
      <table2>
        <rowset>
          <Agent>KalaKala</Agent>
          <Return>123</Agent>
        </rowset>
        <rowset>
          <Agent>HotuMotu</Agent>
          <Return>456</Agent>
        </rowset>
      </table2>
    </root>
    */

    Element rowset;
    Element SubElement;
    
    if (SubTable.equals("")) {
      SubElement = CommonDOM.getRootElement();
      //Logfile.println("[000] AdisoftDOM.getDOMTableValueWhere one table");
    } else {
      SubElement = CommonDOM.getRootElement().getChild(SubTable.trim());
      //Logfile.println("[500] AdisoftDOM.getDOMTableValueWhere subtable: " + SubTable);
    }
    
    String retVal = "";
     
    if (SubElement == null) 
    {
      AuditTrail.println("[400] AdisoftDOM.getDOMTableValueWhere table not found: " + SubTable);
    } else { 
        
        
      java.util.List rowList = SubElement.getChildren();
    
      int i = 0;
      if (rowList.size() > 0) {
        do {
          rowset = (Element) rowList.get(i);  // get a row from the database
          if (rowset.getChild(EleAgent) != null) { //fleet .toLowerCase()these 4 lines
            if (rowset.getChild(EleAgent).getText().equals(WhereValue.trim())) {
              if (rowset.getChild(EleReturn) != null) {  
                retVal = rowset.getChild(EleReturn).getText();
                i = rowList.size();
              } else {
                AuditTrail.println("[400] AdisoftDOM.getDOMTableValueWhere: Could not find return value for " + EleReturn);
              }  
            }    // not equals is not an error condition.
          }  
          i = i + 1;
        } while (i < rowList.size());
        if (i == rowList.size()){
          AuditTrail.println("[400] AdisoftDOM.getDOMTableValueWhere: (didn't find row) " + SubTable +"." + EleAgent + " = '" + WhereValue + "', return: " + EleReturn );
        } 
      }
    }
    return retVal; 
  }

  
  public String getDOMTableValueWhere(String EleAgent, String WhereValue, String EleReturn) {
    return getDOMTableValueWhere("", EleAgent, WhereValue, EleReturn);
  }
 
  public String getDOMTableValueFor(String SubTable, String WhereColumn, String defaultValue) {
     String retVal = getDOMTableValueFor(SubTable, WhereColumn);
     if (retVal == "") retVal = defaultValue;
     return retVal;
  }
 
  public String getDOMTableValueFor(String SubTable, String WhereColumn) {
    //assuming SubTable = 'Agent', WhereColumn = 'SalesAgent'
    //wlll "produce" a SQL as such
    //SELECT SalesAgent FROM THISTABLE.Agent;
    //and will return 'Tom Jones' from the DOM below:
    //
    //<root>
    //  <Agent>
    //     <TopAgent>KalaKala</Agent>
    //     <SalesAgent>Tom Jones</SalesAgent>
    //  </Agent>
    //  <Leader>
    //     <Makai>HotuMotu</Makai>
    //     <Luna>ABC</Luna>
    //  </Leader>
    //</root>
    //
    String retVal = ""; 
   
    Element rowset = (Element) CommonDOM.getRootElement().getChild(SubTable);  // get a row from the database
    if (rowset != null && rowset.getChild(WhereColumn) != null) {  
       retVal = rowset.getChild(WhereColumn).getTextNormalize();
    } else {
       AuditTrail.println("[500] AdisoftDOM.getDOMTableValueFor: Could not find "+ SubTable + "." + WhereColumn);
    }  
    return retVal; 
  }
  
  public String getDOMTableValueFor(String WhereColumn) {
    //Just returns a simple value with no subtables
    String retVal = ""; 
   
    Element rowset = (Element) CommonDOM.getRootElement();  // get a row from the database
    if (rowset != null && rowset.getChild(WhereColumn) != null) {  
       retVal = rowset.getChild(WhereColumn).getTextNormalize();
    } else {
       AuditTrail.println("[500] AdisoftDOM.getDOMTableValueFor: Could not find " + WhereColumn);
    }  
    return retVal; 
  }
  
  public Element getElement(String SubTable)
  {
    //Logfile.println("[000] AdisoftDOM.getElement getting: " + SubTable);
    Element rowset = (Element) CommonDOM.getRootElement().getChild(SubTable);
    return rowset;
  }
  
  public String getSubElementText(String subTable)
  {
    return getSubElementText(CommonDOM.getRootElement(), subTable);
  }
  
  public static String getSubElementText(Element x, String subTable)
  {
    String retText = null;
    Element rowset = (Element) x .getChild(subTable);
    if (rowset != null) retText = rowset.getText();
    if (retText == null) retText = "";
    return retText;
  }
  
  public String[] getDOMTableArrayFor(String SubTable, String WhereColumn) {
    String[] retVal = new String[0];
    Element rowset = (Element) CommonDOM.getRootElement().getChild(SubTable);
    Element item;
    if (rowset != null && rowset.getChild(WhereColumn) != null) {  
       java.util.List FeedList = rowset.getChildren(WhereColumn);
       int j = FeedList.size();
       retVal = new String[j];
       for (int i = 0; i < j; i++) {
          item = (Element) FeedList.get(i);
          retVal[i] = item.getTextNormalize();
       }
    } else {
       AuditTrail.println("[500] AdisoftDOM.getDOMTableArrayFor (1): Could not find "+ SubTable + "." + WhereColumn + " multiple return");
    }  
    return retVal;
  }
  
  public void setDOMTableValueFor(String SubTable, String WhereColumn, String SetString) {
	    Element rowset = (Element) CommonDOM.getRootElement().getChild(SubTable);
	    Element item;
	    if (rowset != null && rowset.getChild(WhereColumn) != null) {  
	       java.util.List FeedList = rowset.getChildren(WhereColumn);
	       int j = FeedList.size();
	       for (int i = 0; i < j; i++) {
	          item = (Element) FeedList.get(i);
	          item.setText(SetString);
	       }
	    } else {
	       AuditTrail.println("[500] AdisoftDOM.setDOMTableValueFor (1): Could not find "+ SubTable + "." + WhereColumn + " multiple return");
	    }  
  }
  
  public String[] getDOMTableArrayFor(String WhereColumn, String ifColumn, String ifEquals) {
    return getDOMTableArrayFor("", WhereColumn, ifColumn, ifEquals);
  }
  
  public String[] getDOMTableArrayFor(String SubTable, String WhereColumn, String ifColumn, String ifEquals) {
//Caution - this routine has not been tested.
    String[] origVal = new String[0];
    String[] retVal = new String[0];
    Element rowset; 
    if (SubTable.equals("")) {
       rowset = (Element) CommonDOM.getRootElement();
    } else {
       rowset = (Element) CommonDOM.getRootElement().getChild(SubTable);
    }
    Element item;
    Element check;
    String chk;
    int counter = 0;
    if (rowset != null && rowset.getChild(WhereColumn) != null) {  
       java.util.List FeedList = rowset.getChildren(WhereColumn);
       java.util.List checkList = rowset.getChildren(ifColumn);
       int j = FeedList.size();
       origVal = new String[j];
       for (int i = 0; i < j; i++) {
          
          check = (Element) checkList.get(i);
          chk = check.getTextNormalize();
          if (chk != null && chk.equalsIgnoreCase(ifEquals))
          {
             item = (Element) FeedList.get(i);
             origVal[i] = item.getTextNormalize();
             counter++;
          }
       }
       if (counter != 0) {
           retVal = new String[counter];
           counter = 0;
           for (int i = 0; i < j; i++) {
             if (origVal[i] != null) {
                retVal[counter++] = origVal[i];
             }
           }
       }
    } else {
       AuditTrail.println("[500] AdisoftDOM.getDOMTableArrayFor (2): Could not find "+ SubTable + "." + WhereColumn);
    }  
    return retVal;
  }
  
  public String[] getDOMTableArrayWhere(String WhereColumn, String ifColumn, String ifEquals) {
    return getDOMTableArrayWhere("", WhereColumn, ifColumn, ifEquals);
  }
  
  public String[] getDOMTableArrayWhere(String SubTable, String WhereColumn, String ifColumn, String ifEquals) {
    String[] retVal = new String[0];
    Element rowset; 
    Element parentset;
    
    if (SubTable.equals("")) {
       parentset = (Element) CommonDOM.getRootElement();
    } else {
       parentset = (Element) CommonDOM.getRootElement().getChild(SubTable);
    }
    Element item;
    Element check;
    String chk;
    int counter = 0;
    java.util.List itemsList = parentset.getChildren();
    int j = itemsList.size();
    String[] origVal = new String[j];
    for (int k = 0; k < j; k++) {
      rowset = (Element) itemsList.get(k);  //later on figure a way to have more than one be item 
      item = (Element) rowset.getChild(WhereColumn.toLowerCase());
      if (rowset != null && item != null) {  
        check = (Element) rowset.getChild(ifColumn.toLowerCase());
        chk = check.getTextNormalize();
        if (chk != null && chk.equalsIgnoreCase(ifEquals))
        {
           origVal[k] = item.getTextNormalize();
           counter++;
        }
      }
    }
    if (counter != 0) {
      retVal = new String[counter];
      counter = 0;
        for (int i = 0; i < j; i++) {
          if (origVal[i] != null) {
            retVal[counter++] = origVal[i];
        }
      }
    } else {
      AuditTrail.println("[400] AdisoftDOM.getDOMTableArrayWhere: Could not find any "+ SubTable + "." + WhereColumn);
    }
    return retVal;
  }
  
  
  
  public void setDocType() {
    CommonDOM.setDocType(new DocType("xml"));
  }
  
  public void setOmitDeclaration(boolean OmitHeader) {
    OmitDeclaration = OmitHeader;
  }
  
  public Document getDOM() {
    return CommonDOM;
  }  
  
  public Element getRootElement() {
    return CommonDOM.getRootElement();
  }  
  
  public Document getReportDOM() {
    //Need to refine this to actually find a report element incase it is buried.
    Element TopEle = CommonDOM.getRootElement();
    TopEle = TopEle.getChild("report");
    Document ReturnDOM = new Document((Element) TopEle.clone());
    return ReturnDOM;
  }
  
  public String getThirdLevelInfo(String first, String second, String third) {
	    Element TopEle = CommonDOM.getRootElement();
	    String ReturnString = null;
	    try {
	    	ReturnString = TopEle.getChild(first).getChild(second).getChild(third).getText();
	    } catch (java.lang.NullPointerException e) {
	        AuditTrail.println("[400] AdisoftDOM.getThirdLevelInfo not found: " + first + "."+ second + "."+ third + ".");	
	    }
	    if (ReturnString == null) ReturnString = "";
	    return ReturnString;
  }
  public String setThirdLevelInfo(String first, String second, String third, String textString) {
	    Element TopEle = CommonDOM.getRootElement();
	    String ReturnString = null;
	    try {
	    	TopEle.getChild(first).getChild(second).getChild(third).setText(textString);
	    } catch (java.lang.NullPointerException e) {
	        AuditTrail.println("[500] AdisoftDOM.setThirdLevelInfo not found: " + first + "."+ second + "."+ third + ".");	
	    }
	    if (ReturnString == null) ReturnString = "";
	    return ReturnString;
  }

  public String toString(Element BasicElement) {
    Element FirstElement = (Element) BasicElement.clone();
    Element SecondElement = new Element("document");
    SecondElement = (Element) SecondElement.addContent(FirstElement);
    Document ThisDOM = new Document(SecondElement);
    return toString(ThisDOM);
  }  
    
  public String toString() {    //This puts the DOM to an XML String
    return toString(CommonDOM);
  }
  
  public String toString(boolean NL) {    //This puts the DOM to an XML String
    return toString(CommonDOM, NL);
  }
  
  public String toString(Document ThisDOM) {
    return toString(ThisDOM, true);
  }

  public String toString(Document ThisDOM, boolean NL) {    //This puts the DOM to an XML String
    Format F = Format.getRawFormat();
    XMLOutputter ShowXML = new XMLOutputter();
    //F.setNewlines(NL);
    F.setOmitDeclaration(OmitDeclaration);
    F.setLineSeparator("\r\n");
    F.setEscapeStrategy((EscapeStrategy) EscStr);
    ShowXML.setFormat(F);
    return (String) ShowXML.outputString(ThisDOM);
  }

  public void setNormal() {
    //Logfile.println("[000] Set Normal"); 
    setNormal1();
    setNormal2(); 
  }
  
  public void setNormal1() {
  //This routine takes any element that has a javascript array text and breaks the
  //array pairs into xml name,value pairs as content of the original xml element
    //Logfile.println("[000] Set Normal 1"); 
    array2Elements CheckThis = new array2Elements();
    performAgainstDOM(CheckThis);
  }
  
  public void setNormal2() {
    //This routine creates a normalized version of the current DOM by reading, modifyin the
    //current DOM and placing the new content into a newly created DOM and then finally
    //replacing the current DOM with the new COM.
    //Logfile.println("[000] Set Normal 2"); 
    
    //getting all the content to a list (assuming that this does not have any text content)
    java.util.List elementList = CommonDOM.getRootElement().getChildren();  
    
    //naming the high level - need to make this flexible                                                        
    Element report = new Element("report");   
    
    //This loop traverses all the content in the high level element   
    for (int j = 0 ;j < elementList.size(); j++) {
    
      //Get an element from this objects content list
      Element EleObj = (Element) elementList.get(j);
      
      //Get the name of this element and Form the "new" blank element that will 
      //receive the content of the original object
      String EleName = EleObj.getName();
      Element RLevel = new Element(EleName);
      
      //Calls the Normalize routing which is a recursive routine that go through all
      //the elements and contructs 
      normalize(RLevel, EleObj);
      
      //Adds the New element to the top level elements contnent list
      report = (Element) report.addContent(RLevel);
    }
    
    //replacing the current CommonDOM
    CommonDOM = new Document(report);
    
    //This section will normalize the naming of items
    Check4Under CheckThis = new Check4Under();
    performAgainstDOM(CheckThis);
    
  } //end of CreateNormal()  
  
  // Need to enhance this to:
  // 1. Carry attributes along (important)


  public void setAllowEmptyText(boolean allowEmpty) {
    allowEmptyText = allowEmpty;
  }
  public boolean getAllowEmptyText() {
    return allowEmptyText;
  }
  private boolean allowEmptyText = false;  //used below to allow empty text in elements.

  private void normalize (Element RLevel, Element ElementContents) {
    //This routine recursively traces thru element-contents are transfers each branch/text over
    //to a need element.
    java.util.List EleList = ElementContents.getChildren();
    //Dumb check for type - need to recode for more general later on
    java.util.List AttrList = ElementContents.getAttributes();
    for (int h = 0; h < AttrList.size(); h++) {
      Attribute Single_Attr = (Attribute) AttrList.get(h);
      Single_Attr = Single_Attr.detach();
      RLevel = RLevel.setAttribute(Single_Attr);
    }    
    //RLevel = RLevel.setAttributes(AttrList);
    //First check for existenece and then move the text content
    if (ElementContents.getTextNormalize().length() != 0 || allowEmptyText) {
      RLevel = RLevel.setText(ElementContents.getTextNormalize());        
    }    
    for (int k = 0; k < EleList.size(); k++ ) {
      //Get an element from the "parents" list of elements.
      Element EleObj = (Element) EleList.get(k);
      //Establish flags for text and element content, and get a name
      boolean hasKids = EleObj.getContentSize() > 0;
      boolean hasText = (EleObj.getTextNormalize().length() != 0 || allowEmptyText);
      String EleName = EleObj.getName();
      //Check and see if this a underscore name indicating an embedded item;
      StringTokenizer st = new StringTokenizer(EleName,"_");
      if (st.countTokens() > 2 ){
        //Get the "parent" name and create an element for it (e.g., expense_1);
        EleName = st.nextToken() + "_" + st.nextToken();
        //Get any children of these so that we can trace down the tree 
        Element newElement = singleElement(EleName, (java.util.List) RLevel.getChildren());
        //Get the "child" name and create an element for it
        String Ele2Name = st.nextToken();
        while (st.hasMoreTokens()) Ele2Name += "_" + st.nextToken(); //JH 2005-12-19
        Element new2Element = new Element(Ele2Name);
        //Pass in the object that we are processing
        normalize(new2Element, EleObj);
        //Add the trees to highest level
        newElement = (Element) newElement.addContent(new2Element);
        RLevel = (Element) RLevel.addContent(newElement);
      } else {
      //There is something to do if anything shows up. Else just skip.
        if (hasKids || hasText) {
          //If we are to do anything then we need a new element
          Element newElement = new Element(EleName);
          //Check to see it the element has any text and if it does, then add it with its text
          if (hasText) {
            //Get the text from the current element and place it into the new "blank" element
            newElement = newElement.setText(EleObj.getTextNormalize());
            //If there aren't any children elements, then "post" this to the parent element
          }
          //If Children, then recursively process then and "post" to parent element
          if (hasKids) {
            //Get a list of the Children
            //java.util.List Ele2List = EleObj.getChildren();
            //Perform recursion for this
            normalize(newElement, EleObj);
          }
          //Add all content to the parent element
          RLevel = (Element) RLevel.addContent(newElement);       
        } // end -> hasKids || hasText
      }   // end -> is this a dot item
    }     // end of for loop 
  }       // end of Mormalize routine
  
  public Element singleElement(String EleName, List EleList) {
    int i = 0;
    Element newElement = new Element(EleName);
    Element workEle;
    while (i < EleList.size()) {
      workEle = (Element) EleList.get(i);  
      if ( workEle.getName().equals(EleName)){
        newElement = (Element) EleList.get(i);
        newElement = (Element) newElement.detach();
        break;
      }
      i++;  
    }    
    return newElement;
  }
  
  public void performAgainstDOM(Object Execute) {
    Element CurrentElement = CommonDOM.getRootElement();
    AdisoftTreeAPI4DOM Perform = (AdisoftTreeAPI4DOM) Execute;
    walkTree(CurrentElement, Perform);    
  }
  
  private void walkTree(Element CurrentElement, AdisoftTreeAPI4DOM Perform) {
    Perform.doMethod(CurrentElement);
    if (CurrentElement != null && (CurrentElement.getContentSize() > 0)) {
      java.util.List elementList = CurrentElement.getChildren();
      for (int i = 0; i < elementList.size(); i++) {
        walkTree( (Element) elementList.get(i), Perform); 
      }
    }
  }
  
  public class Check4Under extends ess.AdisoftTreeAPI4DOM{
    //This is an example on how to write something to use with the performAgainstDOM function
    //The class is then implemented like the following:
    //    Check4Under CheckThis = new Check4Under();
    //    performAgainstDOM(CheckThis);
    //This example was an alternative in the "sequence" attribute method - see code for more usage

    public Check4Under () {
    }    
    public void doMethod(Element CurrentElement) {
      String EleName = CurrentElement.getName();
      //Check and see if this a dot name indicating an embedded item;
      StringTokenizer st = new StringTokenizer(EleName,"_");
      if (st.countTokens() == 2){
        String firstHalf = st.nextToken();
        String secondHalf = st.nextToken();
        try {
          if (Integer.parseInt(secondHalf) > -1){
            CurrentElement.setName(firstHalf);
            CurrentElement.setAttribute("sequence",secondHalf);
          }
        } catch (NumberFormatException e) {
          //Logfile.println("[000] AdisoftDOM Check4Under - non-sequenced: " + CurrentElement.getName());  
        }
      }  
    }
  }
  
  public class array2Elements extends ess.AdisoftTreeAPI4DOM{
   //This class is called against each element and will adjust the content of that
   //element if a [['x','y'],['x','y'],...['x','y']] javascript array structure is found into content with
   //x and y as the name and text value of a DOM element.
   //Currently this is only programmed for the simple case as experienced by the 
   //expense reporting software.  This limits the number of pop-up window levels whichthat that
   //application can support/while saving data, to the parent plus one pop-up.
   
    public array2Elements () {
    }  
    
    public void doMethod(Element curElement)
    {
       String origText = curElement.getTextTrim();  
       StringTokenizer st = new StringTokenizer(origText,"[,']",true);
       ArrayList stack = new ArrayList();
       int skipRBCount = 0;
       String lastToken;
       String token = "";
       String value = "";
       boolean newElement = true;
       boolean expectingClosingQuote = false;
       String[] delimiters = {"[", "]", ",", "'"};
       final int LEFT_BRACKET = 0;
       final int RIGHT_BRACKET = LEFT_BRACKET + 1;;
       final int COMMA = RIGHT_BRACKET + 1;
       final int QUOTE = COMMA + 1;
    
       Element child;
    
       if (origText.length() > 1 && origText.substring(0,2).equals("[[")){
         curElement = curElement.setText(""); 
         while (st.hasMoreTokens())
         {
           lastToken = token;
           token = st.nextToken();
           switch (Utilities.findString(token, delimiters))
           {
            case LEFT_BRACKET:
                if (newElement == false)
                {
                    newElement = true;
                    skipRBCount++;
                }
                break;
            case COMMA:
                if (expectingClosingQuote == true)
                    value += token;
                break;
            case QUOTE:
                if (expectingClosingQuote == true)
                    expectingClosingQuote = false;
                else
                    expectingClosingQuote = true;
                break;
            case RIGHT_BRACKET:
                if ((lastToken.equals("]")) && (skipRBCount > 0))
                    skipRBCount--;
                else
                {
                    if (newElement == false)
                    {
                        newElement = true;
                        curElement = setValue(curElement, value);
                        value = "";
                    }
                    child = curElement;
                    curElement = popElement(stack);
                    if ((curElement != null)&&(child != null))
                        curElement.addContent(child);
                }
                break;
            default:
                if (newElement == true)
                {
                    pushElement(curElement, stack);
                    curElement = setName(curElement, token);
                    newElement = false;
                }    
                else
                {
                    if (expectingClosingQuote == true)
                        value += token;
                }
                break;
           }
         }
       }
    }
    private void pushElement(Element curElement, ArrayList stack)
    {
      stack.add(curElement);
    }
    private Element popElement(ArrayList stack)
    {
      if (stack.size() > 0)
        return ((Element) stack.remove(stack.size()-1));
      else return (null);
    }
    private Element setName(Element curElement, String token)
    {   
      if ((token != null) && (token.length() > 0))
        return(new Element(token));
      else return null;
    }

    private Element setValue(Element curElement, String token)
    {
      if ((token != null) && (token.length() > 0))
        return((Element) curElement.addContent(token));
      else return null;
    }
    
  }  // end of the embedded class
     
  public void setPromoteSubElements(String parentName) {
    
    promoteSubElements CheckThis = new promoteSubElements(parentName);
    performAgainstDOM(CheckThis);
    
  }   
     
  public class promoteSubElements extends ess.AdisoftTreeAPI4DOM{
     private String parentName;
     
     public promoteSubElements(String X) {
        parentName = X;
     }

//The following function requires JDOM 1.0 level (version 1 lib uses .99)
     public void doMethod(Element CurrentElement) {
        Element subElement;
        if (CurrentElement.getName().equals(parentName)) {
           Element myParent = (Element) CurrentElement.getParent();
           java.util.List myChildren = CurrentElement.getChildren();
           if (myChildren.size() > 0) {
             //int i = 0;
             //int j = myChildren.size();
             do {
                subElement = (Element) myChildren.get(0);
                //i = i + 1;
                subElement = (Element) subElement.detach();
                myParent = (Element) myParent.addContent(subElement);
             } while (myChildren.size() > 0);
           }
           CurrentElement = (Element) CurrentElement.detach();
        }
     }
  }
}  //end of main class
