/*
AdisoftDbConn.java - Database connection manager  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;


//import java.net.URL;
import java.sql.*;
//import java.util.*;

//
//  This object opens up the database pointed to by emailrobotAdisoftDbase in the Windows ODBC
//  and has a special routine to get information out of the SYSTEM table which acts as
//  a parameter file for the emailrobot project.
//

public class AdisoftDbConn {

    private Connection dbCon;         // was public static until 5/2/2002
                                     // be careful not to overwrite this - need to protect it somehow
                                     // may want to create a reference to dbCon thru dbCon()
                                     // will have to change ServerSystemTable class
    static AuditTrail Logfile;
    public Statement myState;
    public ResultSet myResult;
    private String url = "";     //This string holds the url, which previously defaulted to "jdbc:odbc:adisoftdbase"
    private String dbUser = "";
    private String dbPassword = "";
    private static String dbDriver = ""; 
    private boolean localConnection = false;

    public AdisoftDbConn() {
        Logfile = new AuditTrail();
    }

    public AdisoftDbConn(String newUrl) {
        Logfile = new AuditTrail();
        url = newUrl;
        AdisoftDbaseStart();
    }
    
    public AdisoftDbConn(String newUrl, String newUser, String newPassword){
        Logfile = new AuditTrail();
        url = newUrl;
        dbUser = newUser;
        dbPassword = newPassword;
        AdisoftDbaseStart();
    }

    public AdisoftDbConn(AuditTrail CurrentLog) {
        Logfile = CurrentLog;
    }

    public AdisoftDbConn(String newUrl, AuditTrail CurrentLog) {
        Logfile = CurrentLog;
        url = newUrl;
        AdisoftDbaseStart();
    }
    
    public void setDBDriver(String X) {
        dbDriver = X;
    }

    public void setDBUser(String X) {
        dbUser = X;
    }
    
    public void setDBPassword(String X) {
        dbPassword = X;
    }
    
    public void setDB(String newUrl) {
        setURL(newUrl);
    }
    public void setDB(String newUrl, String newUser, String newPassword) {
        setURL(newUrl,newUser,newPassword);
    }
    public void setURL(String newUrl) {
        url = newUrl;
        AdisoftDbaseStart();
    }  

    public void setURL(String newURL, String newUser, String newPassword) {
        setDBUser(newUser);
        setDBPassword(newPassword);
        setURL(newURL);
    }

    public Connection getConnection() {
        return dbCon;
    }

    public void setConnection(Connection x) {
        dbCon = x;
    }

    private void AdisoftDbaseStart() {
        AdisoftSoftStart();
    }

    private void AdisoftSoftStart() {

        // If the dbCon object hasn't been instantiated yet
        
        try {
           if (dbCon == null || dbCon.isClosed()) {
        
                Class.forName(properJDBCString(url));
                AuditTrail.println("[000] AdisoftDbConn.SoftStart Connection: " + url + ", user " + dbUser);
                try {
                    dbCon = DriverManager.getConnection(url,dbUser,dbPassword);
                    dbCon.setAutoCommit(true);
                    localConnection = true;
                } catch (java.lang.Exception ex){
                    AuditTrail.println("[500] AdisoftDbConn.SoftStart Failed to open: " + url + ", user " + dbUser);
                    AuditTrail.println("[500] AdisoftDbConn.SoftStart Drivers: " + DriverManager.getDrivers());
                    ex.printStackTrace();
                }
           } else {
              AuditTrail.println("[000] AdisoftDbConn.SoftStart Previous DB driver used for " + url) ;
           }    
        } catch (SQLException e) {
           AuditTrail.println("[500] AdisoftDbConn.SoftStart SQL exception: "+ e.toString());
        } catch (ClassNotFoundException e) {
           AuditTrail.println("[500] AdisoftDbConn.SoftStart DB driver class not found: "+properJDBCString(url));
        }
    }

    private void AdisoftHardStart() {
        try {
            Class.forName(properJDBCString(url));
            AuditTrail.println("[000] AdisoftDbConn.HardStart Connection: " + url + ", user " + dbUser);
            try {
                dbCon = DriverManager.getConnection(url,dbUser,dbPassword);
                dbCon.setAutoCommit(true);
                localConnection = true;
            } catch (java.lang.Exception ex){
                AuditTrail.println("[500] AdisoftDbConn.hardStart Failed to open: " + url + ", user " + dbUser);
                AuditTrail.println("[500] AdisoftDbConn.HardStart Drivers: " + DriverManager.getDrivers());
                ex.printStackTrace();
            }
        }   catch (ClassNotFoundException e) {
            AuditTrail.println("[500] AdisoftDbConn.HardStart DB driver class not found: "+properJDBCString(url));
        }
    }

    public void close() {
        try {
            if (localConnection) dbCon.close();  
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbConn.close DB close failure");
        }
    }
    private String properJDBCString(String xURL) {
        String db_driver = dbDriver;
        if (db_driver == null || db_driver.equals(""))
        {
           if (xURL.toUpperCase().indexOf("REMOTEDB") > -1) {
              db_driver = "com.nogginware.remotedb.Driver";   // Nogginware ODBC-ODBC bridge
           } else if (xURL.toUpperCase().indexOf("CODEBASE") > -1) {
              db_driver = "codebase.jdbc.CodeBaseDriver";   // Codebase JDBC
           } else if (xURL.toUpperCase().indexOf("ODBC") > -1) {
              db_driver = "sun.jdbc.odbc.JdbcOdbcDriver";   // Default ODBC 
           } else if (xURL.toUpperCase().indexOf("ORACLE") > -1) {
              db_driver = "oracle.jdbc.driver.OracleDriver";   // Default for ORACLE 
           } else if (xURL.toUpperCase().indexOf("SQLSERVER") > -1) {
              db_driver = "com.microsoft.jdbc.sqlserver.SQLServerDriver";   // Default for MS SQL Server 
           } else {
              db_driver = "com.mysql.jdbc.Driver";          // MySQL Native - the real default
           }
           AuditTrail.println("[000] AdisoftDbConn.properJDBCString default: " + db_driver);
        } else {
           AuditTrail.println("[000] AdisoftDbConn.properJDBCString set: " + db_driver);
        }
        return db_driver;
    }
}
