/*
AdisoftDbase.java - Generic database utilities  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

import java.sql.*;
import java.util.*;
import org.jdom.Element;
import org.jdom.Attribute;

//
//  This object opens up the database pointed to by emailrobotAdisoftDbase in the Windows ODBC
//  and has a special routine to get information out of the SYSTEM table which acts as
//  a parameter file for the emailrobot project.
//

public class AdisoftDbase {

    public Connection dbCon;         // was public static until 5/2/2002
                                     // be careful not to overwrite this - need to protect it somehow
                                     // may want to create a reference to dbCon thru dbCon()
                                     // will have to change ServerSystemTable class
    static AuditTrail Logfile;
    public Statement myState;
    public ResultSet myResult;
    static Double DConvert;
    protected static String dbDriver = "";
    private String url = "";     //This string holds the url, which previously defaulted to "jdbc:odbc:adisoftdbase"
    private String dbUser = "";
    private String dbPassword = "";
    private boolean localConnection = false;
    private String sqlDateType = "YYYY-MM-DD";
    private String sqlDateStr = "'$date$'";
    private String sqlNo = "No";
    private String sqlYes = "Yes";
    private String sqlAcceptNulls = "No";
    private String sqlBlankDate = "  /  /  ";
    private boolean useNullCheck = true;          // jh 2007-10-12
    protected String sqlLockTable = "";           // e.g. "LOCK TABLES $table$ WRITE";
    protected String sqlUnlockTable = "";         // e.g. "COMMIT" or "UNLOCK TABLES";
    protected String sqlIncrement = "";           // e.g. "UPDATE $table$ SET $column$ TO $column$ + 1;";
    private CustomDate date = new CustomDate();  
    private String encryptFlag = "No";
    protected String sqlTerminator = ";";
    private boolean sqlCommitRequired = false;      //Some drivers don't perform an autocommit
    private String sqlReservedWordList = ";";
    private boolean truncateStrings = true;
    private boolean checkKey = true;
    
    protected static AdisoftDOM schemaDOM = new AdisoftDOM(); //schema table
    protected static boolean schemaSet = false;   //Can I get rid of this by using null?
    protected Element columnNames;
    protected String physicalTableName;           //Future usage
    private String inputDateFormat = "MM/dd/yyyy";    //jh 2010-01-28 - not used yet, readied for getDateFromUserStr() method
        
    public AdisoftDbase() {
        Logfile = new AuditTrail();
    }

    public AdisoftDbase(String newUrl) {
        Logfile = new AuditTrail();
        url = newUrl;
        AdisoftDbaseStart();
    }
    
    public AdisoftDbase(String newUrl, String newUser, String newPassword){
        Logfile = new AuditTrail();
        url = newUrl;
        dbUser = newUser;
        dbPassword = newPassword;
        AdisoftDbaseStart();
    }

    public AdisoftDbase(String newUrl, String newUser, String newPassword, String DBDriver){
        Logfile = new AuditTrail();
        url = newUrl;
        dbUser = newUser;
        dbPassword = newPassword;
        setDBDriver(DBDriver);
        AdisoftDbaseStart();
    }
public AdisoftDbase(AuditTrail CurrentLog) {
        Logfile = CurrentLog;
    }

    public AdisoftDbase(String newUrl, AuditTrail CurrentLog) {
        Logfile = CurrentLog;
        url = newUrl;
        AdisoftDbaseStart();
    }
    
    public void setDBDriver(String X) {
        dbDriver = X;
    }
    
    public void setInputDateFormat(String x) {  
        inputDateFormat = x;
    }
    public String getInputDateFormat() {  
        return inputDateFormat;
    }

    public void setSQLDateType(String x) {
        sqlDateType = x;
    }
    public void setSQLDateStr(String x) {
        sqlDateStr = x;
    }
    public void setSQLYes(String x) {
        sqlYes = x;
    }
    public void setSQLNo(String x) {
        sqlNo = x;
    }
    public void setAcceptNulls(String x) {
        sqlAcceptNulls = x;
    }
    public void setUseNullCheck(String x) {
        if (x.toUpperCase().equals("YES") )
        {
           useNullCheck = true;
        } else {
           useNullCheck = false;
        }
    }
    
    public void setSQLBlankDate(String x) {
        sqlBlankDate = x;
    }
    public void setDBUser(String X) {
        dbUser = X;
    }
    public void setEncryption(String X) {
        encryptFlag = X;
    }
    public void setDBPassword(String X) {
        dbPassword = X;
    }
    public void setLockTable(String X) {
        sqlLockTable = X;
    }
    public void setUnlockTable(String X) {
        sqlUnlockTable = X;
    }
    public void setSQLIncrement(String X) {
        sqlIncrement = X;
    }
    
    public void setSQLTerminator(String X) {
        sqlTerminator = X;
    }
    
    public void setTruncationOff() {
        truncateStrings = false;
    }
    
    public void setTruncationOn() {
        truncateStrings = true;
    }
    
    public void setSQLReservedWordList(String X) {
        sqlReservedWordList = X;
    }
    
    public void setCheckKey(boolean x) {
        checkKey = x;
    }
    
    public void resetSchemaTable(String schemaFileName)
    {
        schemaSet = true;
        setSchemaTable(schemaFileName);
    }
    
    public void setSchemaTable(String schemaFileName)
    {
        if (!schemaSet) {
           java.io.File schemaFile = new java.io.File(schemaFileName);
           schemaDOM.setDOM(schemaFile);
           schemaSet = true;
        }
    }
    public void setSchemaDOM(AdisoftDOM schema)
    {
        schemaDOM = schema;
    }
    
    
    public void setAutoCommit(boolean x) {
        try {
          dbCon.setAutoCommit(x );
        } catch (SQLException e) {
          AuditTrail.println("[400] AdisoftDbase.setAutoCommit failure");
          AuditTrail.println("[400] " + e.toString());
          sqlCommitRequired = true;
        }
    }
    
    public void setCommit() {
        try {
          dbCon.commit();
        } catch (SQLException e) {
          AuditTrail.println("[500] AdisoftDbase.setCommit failure");
          AuditTrail.println("[500] " + e.toString());
          e.printStackTrace();
        }
    }
    
    public void setDB(String newUrl) {
        setURL(newUrl);
    }
    public void setDB(String newUrl, String newUser, String newPassword) {
        setURL(newUrl,newUser,newPassword);
    }
    public void setURL(String newUrl) {
        url = newUrl;
        AdisoftDbaseStart();
    }  

    public void setURL(String newURL, String newUser, String newPassword) {
        setDBUser(newUser);
        setDBPassword(newPassword);
        setURL(newURL);
    }

    public void setConnection(Connection x) {
        dbCon = x;
    }
    
    public Connection getConnection() {
        return dbCon;
    }

    public String getSQLTerminator() {
        return sqlTerminator;
    }

    private void AdisoftDbaseStart() {
        AdisoftSoftStart();
    }

    private void AdisoftSoftStart() {

        // If the dbCon object hasn't been instantiated yet

        
         try {
            if (dbCon == null || dbCon.isClosed()) {
                Class.forName(properJDBCString(url));
                AuditTrail.println("[000] AdisoftDbase.SoftStart new connection: " + url + ", user " + dbUser);
                try {
                    dbCon = DriverManager.getConnection(url,dbUser,dbPassword);
                    dbCon.setAutoCommit(true);
                    localConnection = true;
                } catch (java.lang.Exception ex){
                    AuditTrail.println("[500] AdisoftDbase.SoftStart Failed to open: " + url + ", user " + dbUser);
                    AuditTrail.println("[500] AdisoftDbase.SoftStart Drivers: " + DriverManager.getDrivers());
                    AuditTrail.println("[500] AdisoftDbase " + ex.toString());
                    ex.printStackTrace();
                }
            } else {
                AuditTrail.println("[000] AdisoftDbase.SoftStart using existing: " + url) ;
            }  
         } catch (SQLException e) {
            AuditTrail.println("[500] AdisoftDbase.SoftStart SQL exception: "+ e.toString());
         } catch (ClassNotFoundException e) {
            AuditTrail.println("[500] AdisoftDbase.SoftStart DB driver class not found: "+properJDBCString(url));
         }
            
    }

    private void AdisoftHardStart() {
        try {
            Class.forName(properJDBCString(url));
            AuditTrail.println("[000] AdisoftDbase.HardStart Connection: " + url + ", user " + dbUser);
            try {
                dbCon = DriverManager.getConnection(url,dbUser,dbPassword);
                dbCon.setAutoCommit(true);
                localConnection = true;
            } catch (java.lang.Exception ex){
                AuditTrail.println("[500] AdisoftDbase.hardStart Failed to open: " + url + ", user: " + dbUser);
                AuditTrail.println("[500] AdisoftDbase.HardStart Drivers: " + DriverManager.getDrivers());
                ex.printStackTrace();
            }
        }   catch (ClassNotFoundException e) {
            AuditTrail.println("[500] AdisoftDbase.HardStart DB driver class not found: "+properJDBCString(url));
        }
    }
    public boolean setPersistance(String sqlInsertMethod,String schemaName,
                                  String[] tagList,String[] valueList)
    {
        String[] encryptList = new String[0];
        return setPersistance(sqlInsertMethod, schemaName, tagList, valueList, encryptList, true);
    }   
    
    public boolean setPersistance(String sqlInsertMethod,String schemaName,
                                  String[] tagList,String[] valueList, String[] encryptList)
    {
        return setPersistance(sqlInsertMethod, schemaName, tagList, valueList, encryptList, true);
    }   
    
    public boolean setPersistance(String sqlInsertMethod,String schemaName,
                                  String[] tagList,String[] valueList, 
                                  String[] encryptList, boolean checkIndex)
    {
        Element schemaElement = schemaDOM.getElement(schemaName);
        Element table = schemaElement.getChild("columns");
        Element primaryKey = schemaElement.getChild("key");
        ArrayList parameters = Utilities.make1SListFrom2(tagList,valueList); 
        boolean status = setPersistance(sqlInsertMethod,schemaName,primaryKey,table,parameters,encryptList);
        return status;
    }
    
    public boolean setPersistance(String persistanceMethod,
                                  String cTable,
                                  Element primaryKey,                                  
                                  Element table,
                                  ArrayList parameters,
                                  String[] encryptList)
    {
        
// Parameters is a ArrayList.  Each element is a String[] array with the 0
// item being the name, and the 1 item being the value

       ArrayList tags = new ArrayList();
       ArrayList physicals = new ArrayList();
       ArrayList logicals = new ArrayList(); 
       ArrayList numerics = new ArrayList(); 
       ArrayList dates = new ArrayList(); 
       ArrayList encrypts = new ArrayList(); //will change this later
       ArrayList values = new ArrayList(); 
       ArrayList sizes = new ArrayList();
       Element tag;
       Attribute tagInfo;
       Attribute nameInfo;
       Attribute protectionInfo;
       String tagName;
       String otherName;
       String type;
       Attribute sizeInfo;
       final String[] types = {"string","logical","numeric","date"};
       final String[] protection = {"scramble"};
       final int STRING = 0;
       final int LOGICAL = 1;
       final int NUMERIC = 2;
       final int DATE = 3;
       
       ArrayList keys = new ArrayList();
       ArrayList keyvalues = new ArrayList();
       String keyName;
       Element keyMember;
       String value;
       
       // establish primary key list
       java.util.List primaryKeyList = primaryKey.getChildren();
        
       for (int i = 0; i < primaryKeyList.size(); i++)
       {
            keyMember = (Element) primaryKeyList.get(i);
            keyName = keyMember.getName();
            keys.add(keyName);      
       }
       
       java.util.List columns = table.getChildren();
        
       for (int i = 0; i < columns.size(); i++)
       {
            tag = (Element) columns.get(i);
            tagName = tag.getName();  
            nameInfo = tag.getAttribute("physical");  
            //physical attribute in the schema.xml holds the physical column name.
            if (nameInfo == null || nameInfo.getValue().trim().equals(""))
            {
               otherName = tagName; 
            } else {
               otherName = nameInfo.getValue().trim();
            }
            //This section just passes along the encrypted field information
            for (int z = 0; z < encryptList.length; z++)
            {
                encrypts.add(encryptList[z]);
            }
            //Later, we'll go to this code and drive all the encryption from here
            //Will do this when XML2ESS is fixed for files.
            //protectionInfo = tag.getAttribute("protection");  
            //if (protectionInfo != null 
            //    && protectionInfo.getValue().toUpperCase().trim().equals("SCRAMBLE"))
            //{
            //   encrypts.add(otherName);
            //}
            
            tags.add(tagName);
            physicals.add(otherName);
            values.add("");
            sizes.add("");
            tagInfo = tag.getAttribute("type");
            type = tagInfo.getValue();
            
            switch(Utilities.findString( type, types))
            {
               case STRING:
                  // Want to put a check here in case the length is missing...
                  sizeInfo = tag.getAttribute("length");
                  sizes.set(i,sizeInfo.getValue());
                  break;
               case LOGICAL:
                  logicals.add(otherName);
               case NUMERIC:
                  numerics.add(otherName);
               case DATE:
                  dates.add(otherName);
            }
// add encrypt addition here...?? Could be a good space for encryption ??
       }
       
       int j;
       String[] x;
       String sizeString;
       int stringLen;
       for (int i = 0; i < parameters.size(); i++)
       {
            x = (String[]) parameters.get(i);
            j = tags.indexOf(x[0]);
            if (j > -1)
            {
               sizeString = (String) sizes.get(j); 
               if (sizeString.equals("") || !truncateStrings)  //need to be aware of encryption tail/head/etc
               {
                  values.set(j,x[1]);
               } else {
                  stringLen = Integer.parseInt(sizeString);
                  if (x[1].length() > stringLen)
                  {
                    values.set(j,x[1].substring(0, stringLen));
                  } else {
                    values.set(j,x[1]);
                  }
               }
            }
       }
       
       for (int i = 0; i < keys.size(); i++)
       {
            tagName = (String) keys.get(i);
            j = tags.indexOf(tagName);
            if (j > -1) 
            { 
                value = (String) values.get(j);
                keyvalues.add(value);
            }
       }
       
       return setPersistance(persistanceMethod, cTable, keys, physicals, logicals, numerics, dates, encrypts, values, keyvalues); 
    }
    
    // Besides setting a record for the first time, this routine will also check to see 
    // if the key is unique
    public boolean setPersistance(String persistanceMethod,
                                  String cTable,
                                  ArrayList keyList, 
                                  ArrayList tagList, 
                                  ArrayList logicalFields, 
                                  ArrayList numericFields, 
                                  ArrayList dateFields, 
                                  ArrayList encryptFields, 
                                  ArrayList valueList,
                                  ArrayList keyvaluelist)
    {
       boolean retVal = false; 
       String[] tags = set2ArrayFromAL(tagList); 
       String[] logicals = set2ArrayFromAL(logicalFields);
       String[] numerics = set2ArrayFromAL(numericFields);
       String[] dates = set2ArrayFromAL(dateFields);
       String[] encrypts = set2ArrayFromAL(encryptFields);
       String[] values = set2ArrayFromAL(valueList);   
       
       boolean keySearch = false;  // if no keys are specified, this will post
       if (checkKey && keyList.size() > 0)
       {
           String[] keys = set2ArrayFromAL(keyList); 
           String[] keyvalues = set2ArrayFromAL(keyvaluelist); 
           String keyString = getUpdateString(keys, logicals, numerics, dates, encrypts, keyvalues, " AND ");
           String resultStr = "SELECT * FROM " + cTable + " WHERE " + keyString + sqlTerminator;
           keySearch = setResultSet(resultStr);    
       }
       
       if (!keySearch)
       { 
          retVal = setPersistance(persistanceMethod, cTable, tags, logicals, numerics, dates, encrypts, values);   
       }
       return retVal;
    }
    
    private String[] set2ArrayFromAL(ArrayList x)
    {  
      String[] retArray = new String[x.size()];
      for (int i = 0; i < x.size(); i++)
      {
        retArray[i] = (String) x.get(i);
      }
      return retArray;
    }
    public boolean setPersistance(String persistanceMethod,
                                  String cTable, 
                                  String[] tagList, 
                                  String[] logicalFields, 
                                  String[] numericFields, 
                                  String[] dateFields, 
                                  String[] encryptFields, 
                                  String[] valueList)
    {
        String SQLStatement;
        boolean status = true;
        //Method is either SimpleSQL or PreparedStatement
        if (persistanceMethod.toUpperCase().equals("PREPAREDSTATEMENT"))
        {
           status = setPreparedInsert(cTable, tagList, logicalFields, numericFields, dateFields, encryptFields, valueList);
            
        } else {
           SQLStatement = getInsertSQL(cTable, tagList, logicalFields, numericFields, dateFields, encryptFields, valueList);
           if (doSQLExecute(SQLStatement) == 1) {
             AuditTrail.println("[000] AdisoftDbase.setPersistance [" + cTable + "] " + persistanceMethod + " succeeded"); 
             //closeMyState();
           } else {
             AuditTrail.println("[500] AdisoftDbase.setPersistance failed: " + SQLStatement);
             status = false;
           }
        }
        return (status);
    }
    //==================================
    public String[] getPhysicalNames(String cTable,
                                  String[] tagList)
    {
    // This is probably changing the name of the tags way back in the report object so it 
    // it should allow tags not to be found (as the won't be once they are changed - but the
    // receipt, expense shouldn't be changed twice anyway at this point.
        Element schemaElement = schemaDOM.getElement(cTable);
        Element table = schemaElement.getChild("columns");
        java.util.List columns = table.getChildren();
        Element tag;
        Attribute nameInfo;
        String tagName;
        String[] retArray = new String[tagList.length];

        for (int i = 0; i < retArray.length; i++)
        {
            retArray[i] = tagList[i];
        }
        
        for (int i = 0; i < retArray.length; i++)
        {
//AuditTrail.println("[---] AdisoftDbase.getPhysicalNames tagList: " + retArray[i]);            
            for (int j = 0; j < columns.size(); j++)
            {
               tag = (Element) columns.get(j);
               tagName = tag.getName();
               
               if (retArray[i].equalsIgnoreCase(tagName))
               {
//AuditTrail.println("[---] AdisoftDbase.getPhysicalNames tagName found: " + tagName);                
                  nameInfo = tag.getAttribute("physical");  
                  //physical attribute in the schema.xml holds the physical column name.
                  if (nameInfo != null && !nameInfo.getValue().trim().equals(""))
                  {
//AuditTrail.println("[---] AdisoftDbase.getPhysicalNames replacement: " + nameInfo.getValue().trim());                    
                      retArray[i] = nameInfo.getValue().trim();
                  }
               }
            }
        }
        return retArray;
    }
    
    
    
    public boolean setUpdate(String persistanceMethod,
                                  String cTable,
                                  Element primaryKey,
                                  Element table,
                                  ArrayList parameters)
    {
        
// Parameters is a ArrayList.  Each element is a String[] array with the 0
// item being the name, and the 1 item being the value

// Must have key values that have not changed - this means that the process calling
// this cannot change the key values for the row

// future updates 1) let key values change 2) Only post items that have changed

       ArrayList tags = new ArrayList(); 
       ArrayList physicals = new ArrayList();
       ArrayList keys = new ArrayList();
       ArrayList logicals = new ArrayList(); 
       ArrayList numerics = new ArrayList(); 
       ArrayList dates = new ArrayList(); 
       ArrayList encrypts = new ArrayList(); 
       ArrayList values = new ArrayList(); 
       ArrayList keyvalues = new ArrayList();
       ArrayList itemsUpdated = new ArrayList();
       ArrayList sizes = new ArrayList();
       
       Element keyMember;
       Element tag;
       Attribute tagInfo;
       Attribute nameInfo;
       Attribute sizeInfo;
       
       String keyName;
       String tagName;
       String otherName;
       String value;
       String type;
       final String[] types = {"string","logical","numeric","date"};
       final int STRING = 0;
       final int LOGICAL = 1;
       final int NUMERIC = 2;
       final int DATE = 3;
       
       // set the physical table name
       Attribute physicalTableAttribute = table.getAttribute("physical");  
       //physical attribute at the table tag holds the physical table name.
       if (physicalTableAttribute == null || physicalTableAttribute.getValue().trim().equals(""))
       {
          physicalTableName = cTable; 
       } else {
          physicalTableName = physicalTableAttribute.getValue().trim();
       }
       
       // establish primary key list
       java.util.List primaryKeyList = primaryKey.getChildren();
        
       for (int i = 0; i < primaryKeyList.size(); i++)
       {
            keyMember = (Element) primaryKeyList.get(i);
            keyName = keyMember.getName();
            keys.add(keyName);      
       }
       // establish the various table control lists
       java.util.List columns = table.getChildren();
        
       for (int i = 0; i < columns.size(); i++)
       {
            tag = (Element) columns.get(i);
            tagName = tag.getName();  
            nameInfo = tag.getAttribute("physical");  
            //physical attribute in the schema.xml holds the physical column name.
            if (nameInfo == null || nameInfo.getValue().trim().equals(""))
            {
               otherName = tagName; 
            } else {
               otherName = nameInfo.getValue().trim();
            }
            tags.add(tagName);
            physicals.add(otherName);
            values.add("");
            sizes.add("");
            tagInfo = tag.getAttribute("type");
            type = tagInfo.getValue();
            
            switch(Utilities.findString( type, types))
            {
               case STRING:
                  break;
               case LOGICAL:
                  logicals.add(otherName);
               case NUMERIC:
                  numerics.add(otherName);
               case DATE:
                  dates.add(otherName);
            }
// add encrypt addition here...
       }
       
// set items from screen and then create list that indicates that they have changed  
//// want to check to see if a value has changed later on
       
       int j;
       String[] x;
       String sizeString;
       int stringLen;
       for (int i = 0; i < parameters.size(); i++)
       {
            x = (String[]) parameters.get(i);
            j = tags.indexOf(x[0]);  
            if (j > -1)
            {
               itemsUpdated.add((String) physicals.get(j));  //get the converted value here                
               sizeString = (String) sizes.get(j); 
               if (sizeString.equals(""))
               {
                  values.set(j,x[1]);
               } else {
                  stringLen = Integer.parseInt(sizeString);
                  if (x[1].length() > stringLen)
                  {
                    values.set(j,x[1].substring(0, stringLen));
                  } else {
                    values.set(j,x[1]);
                  }
               }
            }
       }
       
       for (int i = 0; i < physicals.size();)   //was tags
       {
            tagName = (String) physicals.get(i);
            j = itemsUpdated.indexOf(tagName);
            if (j == -1) 
            { 
                physicals.remove(i);
                values.remove(i);
            } else {
                i = i + 1;
            }
       }
       
//        
       for (int i = 0; i < keys.size(); i++)
       {
            tagName = (String) keys.get(i);
            j = physicals.indexOf(tagName);
            if (j > -1) 
            { 
                value = (String) values.get(j);
                keyvalues.add(value);
            }
       }
       // changed tags to physicals..
       // Is this where we want to change the table name also?? Jim Holton 
       return setUpdate(persistanceMethod, physicalTableName, keys, physicals, logicals, numerics, dates, encrypts, values, keyvalues); 
    }
 
    
    public boolean setUpdate(String persistanceMethod,
                                  String cTable,
                                  ArrayList keyList,
                                  ArrayList tagList, 
                                  ArrayList logicalFields, 
                                  ArrayList numericFields, 
                                  ArrayList dateFields, 
                                  ArrayList encryptFields, 
                                  ArrayList valueList,
                                  ArrayList keyvaluelist)
    {
       String[] keys = set2ArrayFromAL(keyList); 
       String[] tags = set2ArrayFromAL(tagList); 
       String[] logicals = set2ArrayFromAL(logicalFields);
       String[] numerics = set2ArrayFromAL(numericFields);
       String[] dates = set2ArrayFromAL(dateFields);
       String[] encrypts = set2ArrayFromAL(encryptFields);
       String[] values = set2ArrayFromAL(valueList);
       String[] keyvalues = set2ArrayFromAL(keyvaluelist);
       
       return setUpdate(persistanceMethod, cTable, keys, tags, logicals, numerics, dates, encrypts, values, keyvalues);   
    }
    
    
    public boolean setUpdate(String persistanceMethod,
                                  String cTable, 
                                  String[] keyList,
                                  String[] tagList, 
                                  String[] logicalFields, 
                                  String[] numericFields, 
                                  String[] dateFields, 
                                  String[] encryptFields, 
                                  String[] valueList,
                                  String[] keyValues)
    {
        String SQLStatement;
        boolean status = true;
        
        //Method is either SimpleSQL or PreparedStatement
        if (persistanceMethod.toUpperCase().equals("PREPAREDSTATEMENT"))
        {
           AuditTrail.println("[500] AdisoftDbase.setUpdate failed: prepared statement updates not supported.");
           status = false;  
          // status = setPreparedInsert(cTable, keyList, tagList, logicalFields, numericFields, dateFields, encryptFields, valueList,keyValues);
            
        } else {
           SQLStatement = getUpdateSQL(cTable, keyList, tagList, logicalFields, numericFields, dateFields, encryptFields, valueList, keyValues);
           if (doSQLExecute(SQLStatement) == 1) {
             AuditTrail.println("[000] AdisoftDbase.setUpdate [" + cTable + "] " + persistanceMethod + " succeeded"); 
             //closeMyState();
           } else {
             AuditTrail.println("[500] AdisoftDbase.setUpdate failed: " + SQLStatement);
             status = false;
           }
        }
        return (status);
    }
    
    public boolean setDelete(String persistanceMethod,
                                  String cTable, 
                                  String[] keyList,
                                  String[] tagList, 
                                  String[] logicalFields, 
                                  String[] numericFields, 
                                  String[] dateFields, 
                                  String[] encryptFields, 
                                  String[] valueList,
                                  String[] keyValues)
    {
        String SQLStatement;
        boolean status = true;
        
        //Method is either SimpleSQL or PreparedStatement
        if (persistanceMethod.toUpperCase().equals("PREPAREDSTATEMENT"))
        {
           AuditTrail.println("[500] AdisoftDbase.setDelete failed: prepared statement updates not supported.");
           status = false;  
          // status = setPreparedInsert(cTable, keyList, tagList, logicalFields, numericFields, dateFields, encryptFields, valueList,keyValues);
            
        } else {
           SQLStatement = getDeleteSQL(cTable, keyList, tagList, logicalFields, numericFields, dateFields, encryptFields, valueList, keyValues);
           if (doSQLExecute(SQLStatement) == 1) {
             AuditTrail.println("[000] AdisoftDbase.setDelete [" + cTable + "] " + persistanceMethod + " succeeded"); 
             //closeMyState();
           } else {
             AuditTrail.println("[500] AdisoftDbase.setDelete failed: " + SQLStatement);
             status = false;
           }
        }
        return (status);
    }
    
    
    
    public int doSQLExecute(String SQLStatement) {

        // This can be used to execute UPDATE, INSERT or DELETE.  It can also be
        // used for other statements that don't return any result set.

        int resultCount;   //number of rows updated

        try {
            if (myState == null) myState = dbCon.createStatement();
            resultCount = myState.executeUpdate(SQLStatement);
            AuditTrail.println("[000] AdisoftDbase.doSQLExecute: " + SQLStatement + ", resultCount: " + Integer.toString(resultCount));
            if (sqlCommitRequired) setCommit();
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbase.doSQLExecute Failed to access database: " + SQLStatement);
            AuditTrail.println("[500] " + ex.toString());
            resultCount = -1;
        }
/*        finally
        {
            // This is probably unnecessary as UPDATE, INSERT and DELETE should
            // be closing the statement immediately themselves in most databases.
            if (myState != null) closeMyState();
        }   */
        return resultCount;
    }

    public ResultSet getResultSet() {
        return myResult;
    }
   

    public boolean setResultSet(String SQLStatement)
    {
        //This method is used for obtaining a result set from a SELECT statement

        try {

            if (myState == null) myState = dbCon.createStatement();
            AuditTrail.println("[000] AdisoftDbase.setResultSet SQLStatement: " + SQLStatement);
            myResult = myState.executeQuery(SQLStatement);

            if (myResult.next()) {
                return true;
            } else {
                AuditTrail.println("[000] AdisoftDbase.setResultSet nothing found: " + SQLStatement);
//Should we close the result set? jh 6/7/2005
                return false;
            }


        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbase.setResultSet Failed [1]: " + SQLStatement);
            AuditTrail.println("[500] " + ex.toString());
            return false;
        }
    }
    
    public ResultSet jdbcResultSet(String SQLStatement, Statement jdbcState) {

        //This method is used for obtaining a result set from a SELECT statement
        ResultSet jdbcResult; 
        try {

            if (jdbcState == null) jdbcState = dbCon.createStatement();
            AuditTrail.println("[000] AdisoftDbase.jdbcResultSet SQLStatement: " + SQLStatement);
            jdbcResult = jdbcState.executeQuery(SQLStatement);

            if (jdbcResult.next()) {
                return jdbcResult;
            } else {
                AuditTrail.println("[000] AdisoftDbase.jdbcResultSet nothing found: " + SQLStatement);
//Should we close the result set? jh 6/7/2005
                return null;
            }


        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbase.jdbcResultSet Failed [1]: " + SQLStatement);
            AuditTrail.println("[500] " + ex.toString());
            return null;
        }
    }
//Caller needs to call CloseMyState() after done with the record.
    public boolean setResultSet(String Table, String Name, String Value) {

        // This method gets a result set from a simple (i.e., one column equal to) SELECTION
        // criteria.
        String sqlString = "SELECT * FROM " + Table.toUpperCase() + " WHERE " + Name.toUpperCase() + " = '" + Value + "'" + sqlTerminator; 
        //String sqlString = "SELECT * FROM " + Table.toUpperCase() + " WHERE RTRIM(" + Name.toUpperCase() + ") = '" + Value + "'" + sqlTerminator; 

        try {

            if (myState == null) myState = dbCon.createStatement();
            AuditTrail.println("[000] AdisoftDbase.setResultSet query: " + sqlString );
            myResult = myState.executeQuery(sqlString);

            if (myResult.next()) {
                return true;
            } else {
                AuditTrail.println("[000] AdisoftDbase.setResultSet did not Find: " + Name + "," + Value + " in " + Table);
//Should we close the result set? jh 6/7/2005
                return false;
            }


        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbase.setResultSet Failed to access database [2]: " + Table + " for " + Name + " = " + Value);
            AuditTrail.println("[500] " + sqlString);
            AuditTrail.println("[500] " + ex.toString());
            return false;
        }
    }
    public boolean setResultSet(String Table, String Name, String Value, String orderString) {

        // This method gets a result set from a simple (i.e., one column equal to) SELECTION
        // criteria.
       
        String sqlString = "SELECT * FROM " + Table.toUpperCase() + " WHERE " + Name.toUpperCase() + " = '" + Value + "' ORDER BY " + orderString + sqlTerminator;

        try {

            if (myState == null) myState = dbCon.createStatement();
            AuditTrail.println("[000] AdisoftDbase.setResultSet query: " + sqlString);
            myResult = myState.executeQuery( sqlString );
            //myResult = myState.executeQuery("SELECT * FROM " + Table.toUpperCase() + " WHERE RTRIM(" + Name.toUpperCase() + ") = '" + Value + "' ORDER BY " + orderString + sqlTerminator );

            if (myResult.next()) {
                return true;
            } else {
                AuditTrail.println("[000] AdisoftDbase.setResultSet did not Find: " + Name + "," + Value + " in " + Table);
//Should we close the result set? jh 6/7/2005
                return false;
            }


        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbase.setResultSet Failed to access database [3]: " + Table + " for " + Name + " = " + Value);
            AuditTrail.println("[500] " + sqlString);
            AuditTrail.println("[500] " + ex.toString());
            return false;
        }
    }
    public void setSimple(String Table, String[] ElementTags, String[] ElementValues) {
    //The key here is the first member of the ElementTags array    
       setSimple(Table, ElementTags[0], ElementValues[0], ElementTags, ElementValues);
    }
  
    public void setSimple(String Table, String KeyTag, String KeyValue, String[] ElementTags, String[] ElementValues) {
    //     
        String SQLStatement = "SQL statement not yet constructed";
        
        try {
            if (setResultSet(Table, KeyTag,KeyValue)) {
               AuditTrail.println("[000] AdisoftDbase.setSimple Existing report");
               String TagString = constructUpdList(ElementTags,ElementValues);
               SQLStatement = "UPDATE " + Table + " SET " + TagString +" WHERE " + KeyTag + " = '" + KeyValue + "'" + sqlTerminator;
//               SQLStatement = "UPDATE " + Table + " SET " + TagString +" WHERE RTRIM(" + KeyTag + ") = '" + KeyValue + "'" + sqlTerminator;
            } else {
               AuditTrail.println("[000] AdisoftDbase.setSimple New report");
               String TagString = constructInsList(ElementTags,",");
               String ValueString = constructInsList(ElementValues,"','");
               SQLStatement = "INSERT INTO " + Table + " (" + TagString +") VALUES ('" + ValueString + "')" + sqlTerminator;
            }
            
            if (doSQLExecute(SQLStatement) == 1) {
                AuditTrail.println("[000] AdisoftDbase.setSimple Update succeeded: " + SQLStatement);
                //dbCon.commit();
            } else {
                AuditTrail.println("[500] AdisoftDbase.setSimple Update failed: " + SQLStatement);
            }
        } catch (java.lang.Exception ex){
            ex.printStackTrace();    //want to get this to the log file
            AuditTrail.println("[500] AdisoftDbase.setSimple Update failed for " + KeyTag + ", value: " + KeyValue);
            AuditTrail.println("[500] AdisoftDbase.setSimple SQL: " + SQLStatement);
            AuditTrail.println("[500] " + ex.toString());
        }
    }
   public boolean writeSimple(String Table, String KeyTag, String KeyValue, String[] ElementTags, String[] ElementValues) {
    //     
        String SQLStatement = "SQL statement not yet constructed";
        boolean retVal = false;
        
        try {
            if (setResultSet(Table, KeyTag,KeyValue)) {
               AuditTrail.println("[000] AdisoftDbase.setSimple Existing report");
               String TagString = constructUpdList(ElementTags,ElementValues);
               SQLStatement = "UPDATE " + Table + " SET " + TagString +" WHERE " + KeyTag + " = '" + KeyValue + "'" + sqlTerminator;
            } else {
               AuditTrail.println("[000] AdisoftDbase.setSimple New report");
               String TagString = constructInsList(ElementTags,",");
               String ValueString = constructInsList(ElementValues,"','");
               SQLStatement = "INSERT INTO " + Table + " (" + TagString +") VALUES ('" + ValueString + "')" + sqlTerminator;
            }
            
            if (doSQLExecute(SQLStatement) == 1) {
                AuditTrail.println("[000] AdisoftDbase.writeSimple Update succeeded: " + SQLStatement);
                //dbCon.commit();
                retVal = true;
            } else {
                AuditTrail.println("[500] AdisoftDbase.writeSimple Update failed: " + SQLStatement);
                retVal = false;
            }
        } catch (java.lang.Exception ex){
            ex.printStackTrace();    //want to get this to the log file
            AuditTrail.println("[500] AdisoftDbase.setSimple Update failed for " + KeyTag + ", value: " + KeyValue);
            AuditTrail.println("[500] AdisoftDbase.setSimple SQL: " + SQLStatement);
            AuditTrail.println("[500] " + ex.toString());
            retVal = false;
        }
        return retVal;
    }

   public int setInsert(String Table, String KeyTag, String KeyValue, String[] ElementTags, String[] ElementValues) {
	    //     
	        String SQLStatement = "SQL statement not yet constructed";
	        int retVal = 0;
	        
	        try {
	            if (setResultSet(Table, KeyTag,KeyValue)) {
	               AuditTrail.println("[400] AdisoftDbase.setInsert existing entry: " + KeyValue);
	               retVal = 1;
	            } else {
	               AuditTrail.println("[000] AdisoftDbase.setInsert new report: " + KeyValue);
	               String TagString = constructInsList(ElementTags,",");
	               String ValueString = constructInsList(ElementValues,"','");
	               SQLStatement = "INSERT INTO " + Table + " (" + TagString +") VALUES ('" + ValueString + "')" + sqlTerminator;
		            if (doSQLExecute(SQLStatement) == 1) {
		                AuditTrail.println("[000] AdisoftDbase.setInsert insert succeeded: " + SQLStatement);
		                //dbCon.commit();
		                retVal = 0;
		            } else {
		                AuditTrail.println("[500] AdisoftDbase.setInsert insert failed: " + SQLStatement);
		                retVal = -1;
		            }

	            }
	        } catch (java.lang.Exception ex){
	            ex.printStackTrace();    //want to get this to the log file
	            AuditTrail.println("[500] AdisoftDbase.setInsert insert failed for " + KeyTag + ", value: " + KeyValue);
	            AuditTrail.println("[500] AdisoftDbase.setInsert SQL: " + SQLStatement);
	            AuditTrail.println("[500] " + ex.toString());
	            retVal = -1;
	        }
	        return retVal;
	    }

   private String constructUpdList(String[] Tags, String[] Values){
     String ValueString = "";
     String C = "";
     for (int j = 0; j < Tags.length; j++) {
       ValueString += C + Tags[j] + " = '" + Values[j] + "'";
       C = ",";
     }   
     return ValueString;
   }

   private String constructInsList(String[] Values, String Delimiter){
     String ValueString = "";
     String C = "";
     for (int j = 0; j < Values.length; j++) {
       ValueString += C + Values[j];
       C = Delimiter;
     }   
     return ValueString;
   }
   public boolean setTable(String Table) {
      // was SELECT ALL below  
      if (setResultSet("SELECT ALL * FROM " + Table.toUpperCase() + sqlTerminator)) {
         AuditTrail.println("[000] AdisoftDbase.setTable returned: " + Table);
         return true;
      } else {
         AuditTrail.println("[500] AdisoftDbase.setTable did not return: " + Table);
         return false;
      }

   }

   public void writeLog(String LogEntry) {

            AuditTrail.println(LogEntry);   //allows any routine access to the AdisoftDbase log.....
   }
    
    public void closeMyState()
    {
        if (myState != null)
        {
            try{
                myState.close();
                myState = null;
                //AuditTrail.println("[000] AdisoftDbase.closeMyState");
            } catch (java.lang.Exception ex){
                ex.printStackTrace();
                AuditTrail.println("[500] AdisoftDbase.closeMyState failure");
                AuditTrail.println("[500] " + ex.toString());
            }
        } else {
            AuditTrail.println("[400] AdisoftDbase.closeMyState null state detected");
        }    
    }

    public void close() {
        try {
            if (myState != null) closeMyState();
            if (localConnection) {
                if (dbCon != null) {
                  //AuditTrail.println("[000] AdisoftDbase.close");
                  dbCon.close();
                  dbCon = null;
                } else {
                  AuditTrail.println("[000] AdisoftDbase.close null detected");
                }
            }
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbase.close failure");
            AuditTrail.println("[500] " + ex.toString());
        }
    }
    
    public void closeResultSet() {
        try 
        {
           myResult.close();
        } catch (java.sql.SQLException ex){
           AuditTrail.println("[500] AdisoftDbase.closeResultSet failure");
           AuditTrail.println("[500] " + ex.toString());
        }
    }
    
    private String properJDBCString(String xURL) {
        String db_driver = dbDriver;
        if (db_driver == null || db_driver.equals(""))
        {
          if (xURL.toUpperCase().indexOf("REMOTEDB") > -1) {
            db_driver = "com.nogginware.remotedb.Driver";   // Nogginware ODBC-ODBC bridge
          } else if (xURL.toUpperCase().indexOf("CODEBASE") > -1) {
            db_driver = "codebase.jdbc.CodeBaseDriver";   // Codebase JDBC
          } else if (xURL.toUpperCase().indexOf("ODBC") > -1) {
            db_driver = "sun.jdbc.odbc.JdbcOdbcDriver";   // Default ODBC 
          } else if (xURL.toUpperCase().indexOf("ORACLE") > -1) {
            db_driver = "oracle.jdbc.driver.OracleDriver";   // Default for ORACLE 
          } else if (xURL.toUpperCase().indexOf("SQLSERVER") > -1) {
            db_driver = "com.microsoft.sqlserver.jdbc.SQLServerDriver";   // Default for MS SQL Server 
          } else {
            db_driver = "com.mysql.jdbc.Driver";          // MySQL Native - the real default
          }
           AuditTrail.println("[000] AdisoftDbase.properJDBCString default: " + db_driver);
        } else {
           AuditTrail.println("[000] AdisoftDbase.properJDBCString set: " + db_driver);
        }
        return db_driver;
    }

  //deprecated - use adisoft.Utilities.getNReplace()
  public String SQLReplace(String x, String oldVal, String newVal) {
    // x = rough sql statement e.g., "select $name$ from USERTABLE;" 
    // oldVal = token e.g., '$name$'
    // newVal = value e.g., 'employee_name'
    if (newVal != null) {
      int y = x.indexOf(oldVal);
      while (y > -1) {
         x = x.substring(0,y) + newVal + x.substring(y + oldVal.length());
         y = x.indexOf(oldVal);
      }   
    }
    return x;
  }

/***** SQL functions ****/
//Create the 'INSERT INTO' from the tags and values array.
//All values are treated as strings, i.e., with delimiters around the values, except
//for the logical data fields, such as REIMB, FORCED, etc. These data fields are treated
//as logical fields and no delimiter is placed around the data value.

   public boolean setPreparedInsert(String table, String[] tags, String[] logicalFields, String[] numericFields, String[] dateFields, String[] encryptFields, String[] values)
   {
        boolean status = true;
        String tagString = "";
        String valueString = "";
        String delimiter = "";
        String sDate;
        String retStr = "Statement not defined";
        try {    
            for (int i = 0; i < values.length; i++)
            {
                if (values[i].length()> 0 || sqlAcceptNulls.toUpperCase().equals("NO"))
                {
                    tagString +=delimiter + reservedCheck(tags[i]);
                    valueString += delimiter + "?";
                    delimiter = ", ";
                }
            }
            
            retStr = "INSERT INTO " + table +" (" + tagString + ") VALUES (" +
                             valueString + ")";
                             
            PreparedStatement ps = dbCon.prepareStatement(retStr);                             

            int j = 1;
            for (int i = 0; i < values.length; i++)
            {
                if (values[i].length()> 0 || sqlAcceptNulls.toUpperCase().equals("NO"))
                {
                    // remove after testing
                    // AuditTrail.println("[---] Field: " + tags[i] + " ==> " + values[i]); 
                    if (Utilities.findString(tags[i], logicalFields) != -1){
                        String logicValue = values[i];
                        if (logicValue.length() == 0) logicValue = "No";
                        if (logicValue.equals("1")) logicValue = "Yes";
                        if (logicValue.equals("0")) logicValue = "No";
                        if (logicValue.toUpperCase().equals("YES")) {
                            ps.setBoolean(j++, true);
                        } else  if (logicValue.toUpperCase().equals("NO")){
                            ps.setBoolean(j++, false);
                        } else {
                            ps.setBoolean(j++, false);  //maybe throw an error instead
                        }
                    } else if (Utilities.findString(tags[i], numericFields) != -1){
                        if (values[i].length() == 0) values[i] = "0";
                        ps.setDouble(j++, Double.valueOf(values[i]).doubleValue());
                    } else if (encryptFlag.equalsIgnoreCase("YES") && Utilities.findString(tags[i], encryptFields) != -1){
                        // AuditTrail.println("[---] Hit encryption handle");
                        byte[] encryptedString = values[1].getBytes();
                        ps.setBytes(j++,encryptedString);
                        // ps.setString(j++,values[i]);
                    } else if (Utilities.findString(tags[i], dateFields) != -1){
                        if (values[i].length() == 0) {
                           java.sql.Date sqlDate = new java.sql.Date((long) 0.00);                            
                           ps.setDate(j++, sqlDate);
                        }else {
                           java.util.Date dDate = CustomDate.getDateFromStr(values[i]);
                           java.sql.Date sqlDate = new java.sql.Date(dDate.getTime());
                           ps.setDate(j++, sqlDate);
                        }
                    } else {  
                        ps.setString(j++,values[i]);
                    }
                }
            } 
            ps.executeUpdate();
            AuditTrail.println("[000] AdisoftDbase.setPreparedInsert " + table + ": " + values[1] + " succeeded");            
        } catch (java.sql.SQLException ex) {
            status = false;
            ex.printStackTrace();    //want to get this to the log file
            AuditTrail.println("[500] AdisoftDbase.setPreparedInsert " + table + ": " + values[1] + " failed: " + retStr);            
            AuditTrail.println("[500] " + ex.toString());
        }

        return status;
   }
  
   public String getInsertSQL(String table, String[] tags, String[] logicalFields, String[] numericFields, String[] dateFields, String[] encryptFields, String[] values)
   {
        String tagString = "";
        String valueString = "";
        String delimiter = "";
        String sDate;
        
        int j = 0;
        for (int i = 0; i < values.length; i++)
        {
            if (values[i].length()> 0 || sqlAcceptNulls.toUpperCase().equals("NO"))
            {
             // comment next line out when not debugging   
             //AuditTrail.println("[---] AdisoftDbase.getInsertSQL: " + tags[i] + " = " + values[i]);       
                tagString +=delimiter + reservedCheck(tags[i]);
                if (Utilities.findString(tags[i], logicalFields) != -1){
                    String logicValue = values[i];
                    if (logicValue.length() == 0) logicValue = "No";
                    if (logicValue.toUpperCase().equals("YES")) {
                      valueString += delimiter + sqlYes;
                    } else  if (logicValue.toUpperCase().equals("NO")){
                      valueString += delimiter + sqlNo;
                    } else {
                      valueString += delimiter + logicValue;
                    }
                } else if (Utilities.findString(tags[i], numericFields) != -1){
                    if (values[i].length() == 0) values[i] = "0";
                    valueString += delimiter + values[i];
                } else if (encryptFlag.equalsIgnoreCase("YES") && Utilities.findString(tags[i], encryptFields) != -1){
                    valueString += delimiter + values[i];
                } else if (Utilities.findString(tags[i], dateFields) != -1){
                    if (values[i].length() == 0) {
                       sDate = sqlBlankDate;
                    } else {
                      AuditTrail.println("[000] AdisoftDbase getInsertSQL date: " + values[i] + " (" + inputDateFormat + ")");
                      java.util.Date dDate =  date.getDateFromStr(values[i], inputDateFormat);
                      if (sqlDateType.equalsIgnoreCase("YYYY-MM-DD")) { 
                        sDate = CustomDate.getSQLDate(dDate);
                      } else if (sqlDateType.equalsIgnoreCase("DD-MMM-YYYY")) {   
                        sDate = CustomDate.getOracleDate(dDate);
                      } else if (sqlDateType.equalsIgnoreCase("YYYYMMDD")) {   
                        sDate = CustomDate.getExportDate(dDate);
                      } else {    // -- MM/DD/YYYY
                        sDate = CustomDate.getSimpleDate(dDate);
                      }
                      sDate = SQLReplace(sqlDateStr,"$date$",sDate);
                    }
                    valueString += delimiter + sDate ;
                } else {  
                    valueString += delimiter + "'" + repStr(values[i]) + "'";
                }
                delimiter = ", ";
            }
        }
        String retStr = "INSERT INTO " + table +" (" + tagString + ") VALUES (" +
                             valueString + ")";
        return retStr;
   }
   
   public String getUpdateSQL(String table, String[] keys, String[] tags, String[] logicalFields, String[] numericFields, String[] dateFields, String[] encryptFields, String[] values, String[] keyValues)
   {
     String valueString = getUpdateString(tags, logicalFields, numericFields, dateFields, encryptFields, values, ", ");
     String keyString = getUpdateString(keys, logicalFields, numericFields, dateFields, encryptFields, keyValues, " AND ", true);
     String retStr = "UPDATE " + table +" SET " + valueString + " WHERE " + keyString;
     return retStr;
   }
   
   public String getDeleteSQL(String table, String[] keys, String[] tags, String[] logicalFields, String[] numericFields, String[] dateFields, String[] encryptFields, String[] values, String[] keyValues)
   {
     String keyString = getUpdateString(keys, logicalFields, numericFields, dateFields, encryptFields, keyValues, " AND ");
     String retStr = "DELETE FROM " + table + " WHERE " + keyString;
     return retStr;
   }
   
   
   public String getUpdateString(String[] tags, String[] logicalFields, String[] numericFields, String[] dateFields, String[] encryptFields, String[] values, String delimitWith)
   {
        return getUpdateString(tags, logicalFields, numericFields, dateFields, encryptFields, values, delimitWith, false);
   }
   
   public String getUpdateString(String[] tags, String[] logicalFields, String[] numericFields, String[] dateFields, String[] encryptFields, String[] values, String delimitWith, boolean createIndex)
   {
        String tagString = "";
        String valueString = "";
        String keyString = " ";
        String delimiter = "";
        String endWith = "";
        String equalSign = " = ";
        String sDate;
        String workStr;
        
        if (createIndex)   //jh 2007-10-12
        { 
            delimiter = "(";   
            delimitWith = ") " + delimitWith + " (";
            endWith = ")";
        }
        
        // int j = 0;
        for (int i = 0; i < values.length; i++)
        {
            AuditTrail.println("[000] AdisoftDbase.getUpdateString item: " + tags[i] + ", " +  values[i]);
            if (values[i].length()> 0 || sqlAcceptNulls.toUpperCase().equals("NO"))
            {
            // AuditTrail.println("[000] AdisoftDbase.getInsertSQL: " + tags[i] + " = " + values[i]);       
                valueString +=delimiter + reservedCheck(tags[i]);
                if (Utilities.findString(tags[i], logicalFields) != -1){
                    String logicValue = values[i];
                    if (logicValue.length() == 0) logicValue = "No";
                    if (logicValue.toUpperCase().equals("YES")) {
                      valueString += equalSign + sqlYes;
                    } else  if (logicValue.toUpperCase().equals("NO")){
                      valueString += equalSign + sqlNo;
                    } else {
                      valueString += equalSign + logicValue;
                    }
                } else if (Utilities.findString(tags[i], numericFields) != -1){
                    if (values[i].length() == 0) values[i] = "0";
                    valueString += equalSign + values[i];
                } else if (encryptFlag.equalsIgnoreCase("YES") && Utilities.findString(tags[i], encryptFields) != -1){
                    valueString += equalSign + values[i];
                } else if (Utilities.findString(tags[i], dateFields) != -1){
                    if (values[i].length() == 0) {
                       sDate = sqlBlankDate;
                    } else {
                      AuditTrail.println("[000] AdisoftDbase getUpdateString date: " + values[i] + " (" + inputDateFormat + ")");
                      java.util.Date dDate =  date.getDateFromStr(values[i],inputDateFormat);
                      if (sqlDateType.equalsIgnoreCase("YYYY-MM-DD")) { 
                        sDate = CustomDate.getSQLDate(dDate);
                      } else if (sqlDateType.equalsIgnoreCase("DD-MMM-YYYY")) {    // -- MM/DD/YYYY
                        sDate = CustomDate.getOracleDate(dDate);
                      } else if (sqlDateType.equalsIgnoreCase("YYYYMMDD")) {   
                        sDate = CustomDate.getExportDate(dDate);
                      } else {    // -- MM/DD/YYYY
                        sDate = CustomDate.getSimpleDate(dDate);
                      }
                      sDate = SQLReplace(sqlDateStr,"$date$",sDate);
                    }
                    valueString += equalSign + sDate;
                } else {  
                    workStr = repStr(values[i]);
                    if (workStr.equals("") && createIndex && useNullCheck ) {
                       valueString += " IS NULL OR " + reservedCheck(tags[i]) + equalSign + "'" + workStr + "'";
                    } else {
                       valueString += equalSign + "'" + workStr + "'";
                    }
                }
                delimiter = delimitWith;
            }
        }
        if (!valueString.equals("")) valueString += endWith;
        return valueString;
   }
   
   public String reservedCheck(String tag)
   {
    if (sqlReservedWordList.indexOf(";" + tag + ";") > -1)
       tag = "\"" + tag + "\"";
    return tag;
   }
   
   public String str(String x, int xsize) {
   //limits the size of a field - used for SQL columns  
      if (x.length() > xsize) x = x.substring(0, xsize);
      return x;
   }

   public static String repStr(String x) {
      String oldChar = "\'";
      String newChar = "\'\'";
      x = repStr(x, oldChar, newChar);
/*
Need to figure out which characters need to be escaped.
*/
      return x; 
   }
   
   public static String repStr(String x, String oldChar, String newChar) 
   {
      int startLocate = 0;
      int oldLength = oldChar.length();
      int newLength = newChar.length();
      
      int locate = x.indexOf(oldChar, startLocate);
      while (locate != -1) {
          x = x.substring(0,locate) + newChar + x.substring(locate + oldLength);
          startLocate = locate + newLength;
          locate = x.indexOf(oldChar, startLocate);
      }
      return x;
   }
   
   public String getTrim(String x) {
      if (x == null) return "";
      return x.trim();
   }
   
   public void setColumns(String x) {
      Element root = schemaDOM.getRootElement();
      Element columns = (Element) root.getChild(x);
      columnNames = (Element) columns.getChild("columns");
   }
   
   public String getColumnName(String x) {
      String retVal = x;
      Element item = (Element) columnNames.getChild(x);
      try {
        Attribute physical = item.getAttribute("physical");
        if (!(physical == null)) retVal = physical.getValue(); 
      } catch (java.lang.NullPointerException ex) {
        AuditTrail.println("[400] AdisoftDbase.getColumnName: no physical: " + x);
      } catch (java.lang.Exception ex){
        AuditTrail.println("[500] AdisoftDbase.getColumnName: Error finding " + x);
        AuditTrail.println("[500] Error: " + ex.toString());
        ex.printStackTrace();
      }
      return retVal;
   }
   
   public String SQLReplaceWArray(String x, ArrayList values) {
     return SQLReplaceWArray(x,values,"","");
   }
   
   public String SQLReplaceWArray(String x, ArrayList values, String head, String tail){
    
     String[] Params; 
    
     for (int i = 0; i < values.size(); i++)
     {
       Params = (String[]) values.get(i); 
       x = SQLReplace(x, head + Params[0] + tail, Params[1]);
     }
     return x;
   }
   public String getSQLArrayValue(ArrayList values, String x){
    
     String[] Params; 
     String ret_val = null; 
    
     for (int i = 0; i < values.size(); i++)
     {
       Params = (String[]) values.get(i); 
       if (Params[0].equalsIgnoreCase(x)) 
       {
          ret_val = Params[1];
          i = values.size();
       }
     }
     return ret_val;
   }
   
   public java.util.ArrayList getArrayList(String[] x, String[] y)
   {
	   
	   java.util.ArrayList values = new java.util.ArrayList(); 
	   String ParamName;
	   String ParamValue;
	   String[] Params;

	   for (int i = 0; i < x.length; i++){
		      Params = new String[2];
		      ParamName = x[i];
		      ParamValue = y[i]; 
		      Params[0] = ParamName;
		      if (ParamValue == null) ParamValue = "";
		      Params[1] = ParamValue;     
		      values.add(Params);
		   }
	   return values;
   }
   
}
   
