/*
Approver.java - determines who can approve a report 
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.io.*;
//import java.util.*;
import java.sql.*;
//import java.net.URL;

public class Approver
{

    static AuditTrail Logfile;
    AdisoftDbase db;
    private Connection dbConnection;
    PersonnelTable approver = null;
    int approvalStatus = 0;
    final int OK = 0;
    final int CANNOT_APPROVE_THIS_REPORT = 1;
    final int MANAGER_RECORD_NOT_FOUND = 2;
    final int CANNOT_APPROVE_OWN_REPORT = 3;
    final int REPORTER_NOT_IN_DATABASE = 4;
    private String SQL1;
    private String SQL2;
    private String SQL3;
    private String SQL4;
    private String SQL_Depart;
    private DepartmentTable depart;
    protected String sqlTerminator = ";";
    private AdisoftDOM schemaDOM = new AdisoftDOM(); //schema table
    
    
 //need to use the connection stuff for this
    public Approver(PersonnelTable approver, Connection x, String sqlTerminator) {
        this.approver = approver;
        dbConnection = x;
        setConnection(x);
        depart = new DepartmentTable();
        depart.setConnection(x);
        setSQLStrings();
    }
    
    public Approver() {
        //used for a Javabean call
    }
    
    public void setApprover(PersonnelTable approver) {
        this.approver = approver;
        //setConnection(x);
        //depart = new DepartmentTable();
        //depart.setConnection(x);
    }
    
    public void setConnection(Connection x) {
        dbConnection = x;
    }
    
    public void setSQLTerminator(String X) {
        sqlTerminator = X;
    }

    
    public void setUpFiles() {
        depart = new DepartmentTable();
        depart.setConnection(dbConnection);
        depart.setSQLTerminator(sqlTerminator);
        setSQLStrings();
        depart.setSQLStrings(SQL_Depart);
    }
        
    public void setSQLStrings() {
        //rewrite this for to do the SysTable SQL from this object
        ServerSystemTable SysTable = new ServerSystemTable();
        AdisoftDOM SystemDOM = new AdisoftDOM();
        SysTable.setConnection(dbConnection);
        SysTable.setSQLTerminator(sqlTerminator);
        String system_file = SysTable.getSystemString("XMLSYSTEM","C:\\WORK\\"); 
        java.io.File SystemFile = new java.io.File(system_file);
        SystemDOM.setDOM(SystemFile); 
        AuditTrail.println("[000] Approver.setSQLStrings from " + system_file);
        SQL1 = SystemDOM.getDOMTableValueFor("personneltable","emailinfoallsql");
        SQL2 = SystemDOM.getDOMTableValueFor("personneltable","persinfosql");
        SQL3 = SystemDOM.getDOMTableValueFor("personneltable","persinfoallsql");
        SQL4 = SystemDOM.getDOMTableValueFor("personneltable","persnuminfosql"); 
        SQL_Depart = SystemDOM.getDOMTableValueFor("departtable","departinfosql");
        java.io.File schemaFile = new java.io.File(SysTable.getSystemString("XMLSCHEMA","C:\\WORK\\"));
        schemaDOM.setDOM(schemaFile);
        
    }     
        
    public int getApprovalStatus()
    {
        return(approvalStatus);
    }
    
    public boolean canApprove(PersonnelTable reporter, String methodOfApproval,
                            int levelOfApproval) {
        return (canApprove(reporter, "","", methodOfApproval, levelOfApproval));
    }
                    
    public boolean canApprove(PersonnelTable reporter, String chargedDepart,
    		String chargedCompany, String methodOfApproval,int levelOfApproval) {
      //Below was 'DEPARTMENT MANAGER'              
      String[] methods = {"Manager", "DepartmentManager", "Admin", "ManagerUnlessXcharge", "ManagerOrAdmin", "Auditor", "CEO", "NoApproval", "AdminOrAuditor"};
      final int MANAGER = 0;
      final int DEPART_MANAGER = 1;
      final int ADMIN = 2;
      final int MANAGERUNLESSXCHARGE = 3;
      final int MANAGERORADMIN = 4;
      final int AUDITOR = 5;
      final int CEO = 6;
      final int NOAPPROVAL = 7;
      final int ADMINORAUDITOR = 8;
      
      approvalStatus = CANNOT_APPROVE_OWN_REPORT;
      if ((approver.equals(reporter) == false) && approver.isAlternateApprover())
      {
        
         approvalStatus = OK;
         
      } else
      {
         if (approver.equals(reporter) == false)
         {
           switch(Utilities.findString(methodOfApproval, methods))
           {
            case MANAGER:
                approvalStatus = canApproveAsManager(reporter, levelOfApproval);
                break;
            case DEPART_MANAGER:
                approvalStatus = canApproveAsDepartManager(reporter, levelOfApproval);
                break;
            case ADMIN:
                approvalStatus = canApproveAsAdmin(reporter);
                break;
            case MANAGERUNLESSXCHARGE:
                approvalStatus = canApproveAsManagerUnlessXcharge(reporter, chargedDepart, chargedCompany, levelOfApproval);
                break;
            case MANAGERORADMIN:
                if (canApproveAsManager(reporter, levelOfApproval) == OK)
                {
                    approvalStatus = OK;
                } else {
                    approvalStatus = canApproveAsDesignee(reporter);
                }
                break;
            case AUDITOR:
                approvalStatus = canApproveAsAuditor(reporter);
                break;
            case CEO:
                approvalStatus = canApproveAsCEO(reporter);
                break;    
            case NOAPPROVAL:
                approvalStatus = CANNOT_APPROVE_THIS_REPORT;
                break;
            case ADMINORAUDITOR:
                if (canApproveAsAdmin(reporter) == OK)
                {
                    approvalStatus = OK;
                } else {
                    approvalStatus = canApproveAsAuditor(reporter);
                }
                break;
                
           }
         }
      }
      return(approvalStatus == OK ? true : false);
    }
    
    //Can approve if I am the reporter's manager (up to nth level of approvale)
    private int canApproveAsManager(PersonnelTable reporter, int levelOfApproval)
    {
        approvalStatus = CANNOT_APPROVE_THIS_REPORT;
        String persNum = reporter.getPersNum();
        if ((persNum != null) && (persNum.length() > 0))
        {
            //can change to check the string against mine.  Although an obvious one,
            //won't really speed up the performance for a lot of the stuff as the
            //negative case out weighs the positive one.
            //PersonnelTable manager = new PersonnelTable(persNum, dbConnection, SQL1, SQL2, SQL3, SQL4);
            PersonnelTable manager = new PersonnelTable();
            manager.setConnection(dbConnection);
            manager.setSQLStrings(SQL1, SQL2, SQL3, SQL4);
            manager.setSchemaDOM(schemaDOM); //JH 2005-04-19
            manager.setColumns("USER"); //JH 2005-04-19
            manager.setPersNumInfo(persNum);
// Logfile.println("[---] Approver.M routine 1 : " + reporter.getPersNum() + ", " + Integer.toString(levelOfApproval));

            while((manager != null) && (approvalStatus != 0) && ((levelOfApproval--)> 0)){
// Logfile.println("[---] Approver.M routine 2 : " + manager.getPersNum() + ", " + Integer.toString(levelOfApproval));
                if (!manager.getManager().equals("") && manager.setPersNumInfo(manager.getManager()) == true)
                {
// Logfile.println("[---] Approver.M routine 3 : " + manager.getPersNum() + ", " + Integer.toString(levelOfApproval));
                    if (manager.isApprover())  //JH 9-19-2003
                    {
                        if (approver.equals(manager) == true)
                          approvalStatus = 0;
// Logfile.println("[---] Approver.M approval run"); 
                    }else{
                        levelOfApproval++;
                    }
                }
                else
                {
                    if (manager != null)
                    {
                        manager.close();
                        manager = null;
                    }
                }
            }
                
            if (manager != null) {    //jh 12-2-2002
                manager.close();
                manager = null;
            }
        }
        else
            approvalStatus = REPORTER_NOT_IN_DATABASE;
            
        return(approvalStatus);
    }
    
    //can approve if I am the department manager (up to nth level)
    private int canApproveAsDepartManager(PersonnelTable reporter, int levelOfApproval)
    {
        //DepartmentTable depart = new DepartmentTable(reporter.getDepartment(),DBUrl, DBUser, DBPassword);
        boolean retDepartVal = depart.setDepartInfo(reporter.getDepartment(), reporter.getCompany()); //jh 1-7-2004
        
        PersonnelTable manager = new PersonnelTable();
        manager.setConnection(dbConnection);
        manager.setSQLStrings(SQL1, SQL2, SQL3, SQL4);
        manager.setSchemaDOM(schemaDOM); //JH 2005-04-19
        manager.setColumns("USER"); //JH 2005-04-19
        
        approvalStatus = CANNOT_APPROVE_THIS_REPORT;
        while ((manager != null) && (retDepartVal) && 
                (approvalStatus != 0) && ((levelOfApproval--)> 0))
        {
            if (depart.getManager().equals("") || manager.setPersNumInfo(depart.getManager()) == true)
            {
                if (!manager.getPersNum().equals("") && approver.equals(manager) == true)
                {
                    approvalStatus = 0;
                } else {
                    if (!manager.isApprover())
                    {
                        levelOfApproval++;  //JH 9-19-2003
                    }
                }
                if (depart.getSuperDepartment().equals("") || depart.setDepartInfo(depart.getSuperDepartment(), depart.getCompany()) == false)
                {
                    retDepartVal = false;
                }  
            }
            else
            {
                manager.close();
                manager = null;
            }
        }
        
        if (manager != null) {    //jh 12-2-2002
            manager.close();
            manager = null;
        }
        
        return(approvalStatus);
    }
    
    //Can approve if I am the department admin
    private int canApproveAsAdmin(PersonnelTable reporter)
    {
        
        //DepartmentTable depart = new DepartmentTable(reporter.getDepartment(), DBUrl, DBUser, DBPassword);
        boolean retDepartVal = depart.setDepartInfo(reporter.getDepartment());
        
        PersonnelTable admin = new PersonnelTable();
        admin.setConnection(dbConnection);
        admin.setSQLStrings(SQL1, SQL2, SQL3, SQL4);
        admin.setSchemaDOM(schemaDOM); //JH 2005-04-19
        admin.setColumns("USER"); //JH 2005-04-19
        approvalStatus = CANNOT_APPROVE_THIS_REPORT;

        if ((retDepartVal) && (admin != null))
            if (admin.setPersNumInfo(depart.getAdmin()) == true)
                if (approver.equals(admin) == true)  
                    approvalStatus = 0;
 
        if (admin != null)
            admin.close();
            
        //if (depart != null)
            //depart.close();
        
        return(approvalStatus);
    }
    
    private int canApproveAsDesignee(PersonnelTable reporter)
    {
        
        //DepartmentTable depart = new DepartmentTable(reporter.getDepartment(), DBUrl, DBUser, DBPassword);
        boolean retDepartVal = depart.setDepartInfo(reporter.getDepartment(), reporter.getCompany());
        
        PersonnelTable admin = new PersonnelTable();
        admin.setConnection(dbConnection);
        admin.setSQLStrings(SQL1, SQL2, SQL3, SQL4);
        admin.setSchemaDOM(schemaDOM); //JH 2005-04-19
        admin.setColumns("USER"); //JH 2005-04-19
        
        approvalStatus = CANNOT_APPROVE_THIS_REPORT;

        if ((retDepartVal) && (admin != null) && !depart.getAdmin().equals(""))
            if (admin.setPersNumInfo(depart.getAdmin()) == true)
                if (approver.equals(admin) == true)  
                  if(!admin.getManager().equals(reporter.getPersNum()))
                    approvalStatus = 0;
 
        if (admin != null)
            admin.close();
            
        //if (depart != null)
            //depart.close();
        
        return(approvalStatus);
    }
    
    
    
    
    //Can approve if I am the manager (up to nth level of approvale) of the charged department
//Need to add a super department chain here like in the department manager.
    private int canApproveAsManagerUnlessXcharge(PersonnelTable reporter, String chargedDepart, String chargedCompany, int levelOfApproval)
    {
        String reporterDepart = reporter.getDepartment();
        approvalStatus = CANNOT_APPROVE_THIS_REPORT;
        
        if ((chargedDepart == null) || (chargedDepart.length() == 0) || (chargedDepart.equals(reporterDepart))) {
            approvalStatus = canApproveAsManager(reporter, levelOfApproval);
        } else
        {
            boolean retDepartVal = depart.setDepartInfo(chargedDepart, chargedCompany);
                    
            PersonnelTable manager = new PersonnelTable();
            manager.setConnection(dbConnection);
            manager.setSQLStrings(SQL1, SQL2, SQL3, SQL4);
            manager.setSchemaDOM(schemaDOM); //JH 2005-04-19
            manager.setColumns("USER"); //JH 2005-04-19
            manager.setPersNumInfo(depart.getManager());
             
            while((approvalStatus == CANNOT_APPROVE_THIS_REPORT) && ((levelOfApproval--)> 0))
            {
                if (approver.equals(manager) == true)
                {
                    approvalStatus = OK;
                }
                else
                {    
                    if (!manager.getManager().equals("") && !manager.isApprover()) 
                    {
                        levelOfApproval++;  //JH 9-19-2003
                    }
                    
                    if (manager.getManager().equals("") || manager.setPersNumInfo(manager.getManager()) == false)
                    {
                        approvalStatus = MANAGER_RECORD_NOT_FOUND;
                    }
                }
            }
            if (manager != null) {    //jh 12-2-2002
                manager.close();
                manager = null;
            }
        }    
        Logfile.println("[000] Approver.MXC routine returns: " + Integer.toString(approvalStatus));
        return(approvalStatus);
    }
    
    private int canApproveAsAuditor(PersonnelTable reporter)
    {
        int approvalStatus = CANNOT_APPROVE_THIS_REPORT;
        if (approver.isAuditor()) approvalStatus = 0;
        return approvalStatus;
    }
    private int canApproveAsCEO(PersonnelTable reporter)
    {
        int approvalStatus = CANNOT_APPROVE_THIS_REPORT;
        if (approver.isCEO()) approvalStatus = 0;
        return approvalStatus;
    }
}