/*
ApproverEmailAddress.java - determines who can approve a report and returns their email address  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.io.*;
//import java.util.*;
import java.sql.*;
//import java.sql.*;
//import org.jdom.Document; //See if I can find this
//import org.jdom.Element;
//import org.jdom.DocType;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;

public class ApproverEmailAddress implements RouteTo
{
    static AuditTrail Logfile;
    protected Connection DBConn;
    protected AdisoftDbase db;//
    protected ServerSystemTable systemTable;
    protected PersonnelTable submitter;
    protected PersonnelTable receiver;      //This is the routTo Object
    protected String persNum = "";
    protected String duplicateSignatureOK = "";
    protected String chargedDepart = "";
    protected String chargedCompany = "";
    protected String limitRequired = "";
    protected double reportAmount = (double) 0.00;
    protected String adminSigned = "";
    protected String signed1 = "";
    protected String signed2 = "";
    protected boolean atLeastOnce = false;
    protected String sqlTerminator = ";";
    protected double currencyConversionRate = (double) 1.00;
    
    public void init(Connection DBConnection, String SQLTerminator) 
    {
        DBConn = DBConnection;
        db = new AdisoftDbase();
        db.setConnection(DBConn);
        sqlTerminator = SQLTerminator;
        setDatabases();
    }

    public void init(String DBUrl, String DBUser, String DBPassword, String SQLTerminator)
    {
        db = new AdisoftDbase(DBUrl, DBUser, DBPassword);
        DBConn = db.getConnection();
        sqlTerminator = SQLTerminator;        
        setDatabases(); //jh 3/3/03
    }
    
    private void setDatabases()
    {
        systemTable = new ServerSystemTable();
        systemTable.setConnection(DBConn);
        systemTable.setSQLTerminator(sqlTerminator);
        submitter = new PersonnelTable();
        submitter.setConnection(DBConn);
        submitter.setSQLTerminator(sqlTerminator);        
        submitter.setSQLStrings();
    }
    
    public void setCurrencyConversion(double multiplier)
    {
    	currencyConversionRate = multiplier;
    }
    
    public String getEmailAddress(String persNum)
    {
    //    this.persNum = this.persNum;
        return ("");
    }
/**
Returns the email address for the admin of the department of the report submitter, i.e., persNum.

Look at changing this. This is really a set routine.

*/
    public String getEmailAddress(String persNum, String duplicateSignatureOK,
    		        String chargedDepart, String chargedCompany,
    		        String limitRequired, String reportAmount, String adminSigned, 
    		        String signed1, String signed2, boolean atLeastOnce)
    {
        String email = "";
        
        this.persNum = persNum;
        this.duplicateSignatureOK = duplicateSignatureOK;
        this.chargedDepart = chargedDepart;
        this.chargedCompany = chargedCompany;
        this.limitRequired = limitRequired;
        if (reportAmount.equals("")) reportAmount = "0.00";
        this.reportAmount = Double.parseDouble(reportAmount);
        this.adminSigned = adminSigned;
        this.signed1 = signed1;
        this.signed2 = signed2;
        this.atLeastOnce = atLeastOnce;

        return email;
    }

    public void close()
    {
        if (systemTable != null)
        {
            systemTable.close();
            systemTable = null;
        }
        if (submitter != null)
        {
            submitter.close();
            submitter = null;
        }
        if (db != null)
        {
            db.close();
            db = null;
        }
    }
    protected boolean validateDuplicateSignatures(String signer)
    {
        boolean result = true;
        
        if (duplicateSignatureOK.equalsIgnoreCase("NO"))
           if ((adminSigned.length() > 0 && signer.equalsIgnoreCase(adminSigned)) ||
                 (signed1.length() > 0 && signer.equalsIgnoreCase(signed1)) || 
                 (signed2.length() > 0 && signer.equalsIgnoreCase(signed2)))
            result = false;
            
        return(result);
    }
    protected boolean isOnceAndOnlyOnce(String signer)
    {
        boolean result = false;
        
        if (atLeastOnce) {
           if ((adminSigned.length() > 0 && signer.equalsIgnoreCase(adminSigned)) ||
                 (signed1.length() > 0 && signer.equalsIgnoreCase(signed1)) || 
                 (signed2.length() > 0 && signer.equalsIgnoreCase(signed2))) {
                result = false;
           } else {
            	result = true;
           }
    
        }
        return(result);
    }
    
    protected String getManagerEmail()
    {
        String email = "";
        
        PersonnelTable manager = submitter;
        String managerPersNum = manager.getManager();
        while (email.length() == 0 && manager.setPersNumInfo(managerPersNum))
        {
            if (validateDuplicateSignatures(managerPersNum))
                if (manager.isApprover() 
                && (limitRequired.equalsIgnoreCase("NO") 
               		|| isOnceAndOnlyOnce(manager.getPersNum())	
                    || manager.validLimit(reportAmount,currencyConversionRate)))
                    email = manager.getEmailAddress();
            managerPersNum = manager.getManager();
        }
        if (email.length() > 0) receiver = manager;        
        return email;        
    }
    
    protected String getDepartManagerEmail()
    {
        String email = "";
        
        DepartmentTable depart = new DepartmentTable();
        depart.setConnection(DBConn);
        depart.setSQLTerminator(sqlTerminator); //jh 2006-03-01
        depart.setSQLStrings();
        if (depart.setDepartInfo(submitter.getDepartment().trim(), submitter.getCompany()))
        {
            PersonnelTable manager = submitter;
            String managerPersNum = depart.getManager();
            while (email.length() == 0 && manager.setPersNumInfo(managerPersNum))
            {
                if (validateDuplicateSignatures(managerPersNum)&& (manager.equals(submitter) == false))
                    if (manager.isApprover() && (limitRequired.equalsIgnoreCase("NO") || manager.validLimit(reportAmount,currencyConversionRate)))
                        email = manager.getEmailAddress();
                managerPersNum = manager.getManager();
            } 
            if (email.length() > 0) receiver = manager;
        }
            
        if (depart != null)
            depart.close();
            
        return(email);
    }
    
    protected String getXChargeDepartManagerEmail()
    {
        String email = "";
        DepartmentTable depart = new DepartmentTable();
        depart.setConnection(DBConn);
        depart.setSQLTerminator(sqlTerminator); //jh 2006-03-01
        depart.setSQLStrings();
        if (depart.setDepartInfo(chargedDepart, chargedCompany))
        {
            PersonnelTable manager = new PersonnelTable();
            manager.setConnection(DBConn);
            manager.setSQLTerminator(sqlTerminator); //jh 2006-03-10
            manager.setSQLStrings(); //jh 2006-03-10
        
            String managerPersNum = depart.getManager();
            while (email.length() == 0 && manager.setPersNumInfo(managerPersNum))
            {
                if (validateDuplicateSignatures(managerPersNum)) {
                   if (manager.equals(submitter) == false) {
                      if (manager.isApprover() && (limitRequired.equalsIgnoreCase("NO") || manager.validLimit(reportAmount,currencyConversionRate))) {
                         email = manager.getEmailAddress();
                      } else {
                        AuditTrail.println("[000] Approver Email XCharge - limit too low: " + managerPersNum);
                      }
                   } else {
                    AuditTrail.println("[000] Approver Email XCharge - manager same as submitter: " + managerPersNum);
                   }
                } else {
                    AuditTrail.println("[000] Approver Email XCharge - duplicate signer: " + managerPersNum);
                }
                managerPersNum = manager.getManager();
            }
            if (email.length() > 0) receiver = manager;
        }
            
        if (depart != null)
            depart.close();
        return(email);
    }
    public String getRouteToPersNum() 
    {
        String persnum = "";
        if (receiver != null) persnum = receiver.persnum;
        return persnum;
    }
}