/*
AuditTrail.java - Writes messages to an audit log 
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


// making the routines "static" has caused some of the other objects code to be obsolete
// where they pass audit trails to be passed along or instansiated.  Since this really doesn't
// cause any problems I have left most of them in and will remove them at my convenience.

package ess;

import java.io.*;
import java.util.*;

public class AuditTrail {

// The AuditTrail class provides an audit trail to any object, method, or process
// When the object is instantiated it opens up the auditlog and println-ing to the
// AuditTrail object causes an entry to be made into the log

    static CustomDate dateRoutine = new CustomDate();
    static BufferedWriter FileHandle;           //Redefines FileHandle in OutputTextFile
    static int NumberOfInstances = 0;
    static OutputTextFile AuditLogFile = new OutputTextFile();
    static ArrayList ActiveUsers = new ArrayList();
    static ArrayList ReportList = new ArrayList();
    static boolean detailLog = false;

    public AuditTrail() {          //used by apps
        //openAuditTrail("/var/ess/expense.log");
        //Later work in a set and move everything down one.
        File x = new File("/var/ess");
        if (x.isDirectory())
        {
           openAuditTrail("/var/ess/expense.log");
        } else
        {
           x = new File("~/var/ess"); 
           if (x.isDirectory())
           {
              openAuditTrail("~/var/ess/expense.log");
           } else
           {
              x = new File("c:/ess"); 
              if (x.isDirectory())
              {
                 openAuditTrail("c:/ess/expense.log");
              } else 
              {
                 x = new File("~"); 
                 if (x.isDirectory())
                 {
                    //could be c:\winnt\system32 for windows
                    openAuditTrail("~/expense.log");
                 } else 
                 {
                    openAuditTrail("expense.log");
                 }
              }
           }
        }
    }

    public boolean setDetailLog(boolean detailFlag) {
    	detailLog = detailFlag;
    	return detailLog;
    }
    
    public boolean setDetailLog() {
    	return setDetailLog(!detailLog);
    }
    
    public boolean getDetailLog() {
    	return detailLog;
    }
    
    public static synchronized void openAuditTrail(String NameOfTheLog) {
        if (NumberOfInstances == 0) {
            try {
                System.out.println("Attempting to create log: " + NameOfTheLog);
                AuditLogFile.createFileHandle(NameOfTheLog);
                NumberOfInstances++;
                println("[262] ESS audit log has started: " + NameOfTheLog);
                System.out.println("ESS audit log has started");
            } catch (IOException e) {
                System.out.println("ESS audit log has failed (1)");
            }
        } else {
            NumberOfInstances++;
        }
    }

    public static synchronized void println(String logEntry) {
        try {
//Might consider doing a flag and an either-or depending on whether the
//log file has opened
        	if (detailLog || isImportant(logEntry)) {
               String x = CustomDate.entryDate() + " , " + logEntry;
               AuditLogFile.write(x + "\r\n");
               //System.out.println(x);  // 2007-09-04
        	}
        } catch (IOException e) {
            System.out.println("[500] Adisoft audit log has failed (2)");
        }
    }

    private static boolean isImportant(String logEntry){
    	boolean retVal = true;
    	if (logEntry.indexOf("[000]") > -1) retVal = false;
    	return retVal;
    }
    
    public static synchronized void setInitQueues() {
        ActiveUsers.clear();
        ReportList.clear();
    }
    public static synchronized void setAddUser(String email, 
                  String session, String IP, String name)
    {
        String[] user = new String[4];
        if (locateElement(ActiveUsers, email, session) < 0) {
            user[0] = email;
            user[1] = session;
            user[2] = IP;
            user[3] = name;
            ActiveUsers.add(user);
        }
    }
    public static synchronized void setRemoveUser(String email, String session)
    {
        int x = locateElement(ActiveUsers, email, session);
        if (x > -1) {
            ActiveUsers.remove(x);
        }
    }
    
    public static synchronized void setRemoveUsers(String session) {
        String[] k;
        for (int i = 0; i < ActiveUsers.size(); i++) {
            k = (String[]) ActiveUsers.get(i);
            if (session.equalsIgnoreCase(k[1])){
                ActiveUsers.remove(i);
            }
        }
    }
    
    public static synchronized ArrayList getUserList()
    {
        return ActiveUsers;
    }
    
    public static synchronized void setAddReport(String reference, String session, String name)
    {
        String[] report = new String[3];
        if (locateElement(ReportList, reference, session) < 0) {
            report[0] = reference;
            report[1] = session;
            report[2] = name;
            ReportList.add(report);
        }
    }

    public static synchronized void setRemoveReport(String reference, String session)
    {
        int x = locateElement(ReportList, reference, session);
        if (x > -1) {
            ReportList.remove(x);
        }
        
    }
    
    public static synchronized void setRemoveReports(String session) {
        String[] k;
        for (int i = 0; i < ReportList.size(); i++) {
            k = (String[]) ReportList.get(i);
            if (session.equalsIgnoreCase(k[1])){
                ReportList.remove(i);
            }
        }
    }
    
    public static synchronized boolean isReportInUse(String reference, String mySession)
    {
        boolean retVal = false;
        String[] k;
        for (int i = 0; i < ReportList.size(); i++) {
            k = (String[]) ReportList.get(i);
            if (reference.equalsIgnoreCase(k[0]) && !mySession.equalsIgnoreCase(k[1]))
            {
                retVal = true;
                i = ReportList.size();
            }
        }
        return retVal;
    }
    
    public static synchronized boolean isReportInMyUse(String reference, String mySession)
    {
        boolean retVal = false;
        String[] k;
        for (int i = 0; i < ReportList.size(); i++) {
            k = (String[]) ReportList.get(i);
            if (reference.equalsIgnoreCase(k[0]) && mySession.equalsIgnoreCase(k[1]))
            {
                retVal = true;
                i = ReportList.size();
            }
        }
        return retVal;
    }
    
    public static synchronized ArrayList getReportList()
    {
        return ReportList;
    }
    
    public static synchronized int locateElement(ArrayList x, String y, String z) {
        int retVal = -1;
        String[] k;
        for (int i = 0; i < x.size(); i++) {
            k = (String[]) x.get(i);
            if (y.equalsIgnoreCase(k[0]) && z.equalsIgnoreCase(k[1])){
                retVal = i;
                i = x.size();
            }
        }
        return retVal;
    }
    public boolean isDummy(String x)
    {
        println("[---] Dummy => " + x);
        return true;
    }
}
