/*
CustomDate.java - date handling routines  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

import java.util.*;
import java.text.*;

public class CustomDate {

// This class holds methods that return dates in specialized formats

    public Date date = new Date();
    public Date time = new Date();
    public static Date localDate;
    private String userFormat = "MM/dd/yyyy";
    
    static Locale teanga = new Locale("en");

    public static SimpleDateFormat simpleDate = new SimpleDateFormat("MM/dd/yyyy");
    public static SimpleDateFormat simpleTime = new SimpleDateFormat("HH:mm:ss");
    public static SimpleDateFormat xBaseDate = new SimpleDateFormat("yyyy-MM-dd");
    static SimpleDateFormat exportDate = new SimpleDateFormat("yyyyMMdd");
    static SimpleDateFormat simplerDate = new SimpleDateFormat("MM/dd/yy");
    static SimpleDateFormat simpleMonth = new SimpleDateFormat("MM/yy");
    static SimpleDateFormat onlyMonth = new SimpleDateFormat("MM");
    static SimpleDateFormat sapDate = new SimpleDateFormat("MMddyyyy");
    static SimpleDateFormat greatPlainsDate = new SimpleDateFormat("MMddyy");
    static SimpleDateFormat julianDate = new SimpleDateFormat("yyyyDDD");    
    static SimpleDateFormat monthDay = new SimpleDateFormat("MM/dd");
    static SimpleDateFormat monthDayEuro = new SimpleDateFormat("dd/MM");
    static SimpleDateFormat nameDay = new SimpleDateFormat("dd MM yyyy");
    static SimpleDateFormat longDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    static SimpleDateFormat GMTDate = new SimpleDateFormat("dd MMM yyyy HH:mm:ss zzz", teanga);
    static SimpleDateFormat oracleDate = new SimpleDateFormat("dd-MMM-yyyy", teanga);  
    static SimpleDateFormat oracleSQLDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.s");
    static SimpleDateFormat nachaDate = new SimpleDateFormat("yyMMdd");
    static SimpleDateFormat nachaTime = new SimpleDateFormat("HHmm");
    static SimpleDateFormat instanceDate = new SimpleDateFormat("yyyyMMddHHmmsss");

    //private String timeZoneString = "GMT-8:00";
    
    
    //public void setTimeZone(String X)
    //{
    //	timeZoneString = X;
    //}
    
    public CustomDate() {
    }
    
// Basic date set and get    
    public void setDate(Date x)
    {
        date = x;
    }
    
    public void setUserDateFormat(String x)
    {
    	userFormat = x;
    }
    
    public String getUserDateFormat()
    {
    	return userFormat;
    }

    public Date getDate()
    {
        return date;
    }

    public String getGMTString() {
        return date.toGMTString();
    }
    
// sets date from a string
    public static synchronized Date getDateFromString(String sDate) {
        //synchronized(pos) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        return simplerDate.parse(sDate,pos);
        //}
    }
    public static synchronized Date getDateFromStr(String sDate) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        return simpleDate.parse(sDate,pos);
    }
    public Date getDateFromStr(String sDate, String sDateFormat) {
    	SimpleDateFormat userDate = new SimpleDateFormat(sDateFormat);
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        return userDate.parse(sDate,pos);
    }
    
    public String getStrFromDate(Date dDate, String sDateFormat) {
    	SimpleDateFormat userDate = new SimpleDateFormat(sDateFormat);
        return userDate.format(dDate);
    }
    
    public String getUserDateStr(String sDate, String sDateFormat) {
    	String retVal;
    	try 
    	{
    		retVal = getStrFromDate(getDateFromStr(sDate),sDateFormat);
    	} catch (java.lang.NullPointerException e)
    	{
    		retVal = "";
    	}
    	return retVal;
    }
    public String getUserDateStr(String sDate) {
    	return getUserDateStr(sDate,userFormat); 
    }
    public static synchronized Date getOracleSQLDate(String sDate) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        return oracleSQLDate.parse(sDate,pos);
    }
    public static synchronized Date getDateFromXBase(String sDate) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        return xBaseDate.parse(sDate,pos);
    }
    public static synchronized Date getDateFromGMTDate(String sDate) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        return GMTDate.parse(sDate,pos);
    }
    public static synchronized Date getDateFromExportDate(String sDate) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        return exportDate.parse(sDate,pos);
    }
    public static synchronized Date getDateFromExcelDate(String sDate) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        if (sDate.length() == 7) sDate = "0" + sDate;
        return sapDate.parse(sDate,pos);
    }
    public static synchronized Date getLongDateFromXBase(String sDate, String sTime) {
        ParsePosition pos = new ParsePosition(0);
        pos.setIndex(0);
        sDate += " " + sTime;
        return longDate.parse(sDate,pos);
    }
    public static synchronized Date getDateFromSQL(String sDate) {
        return getDateFromXBase(sDate);
    }

// gets the various date formats that are needed
    public static String HTTPDate() {
        Date DefaultDate = new Date();
        return HTTPDate(DefaultDate);
    }

    public static String getInstanceDate() {
        return genericDateString(instanceDate,new Date());
    }
        
    public static String HTTPDate (Date date) {
        // The HTTPDate is used to return date in the mime information that accompanies
        // the HTTP handshake.
        String[] weekdays = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
        // might have to add logic to sync the Day to the GMT date in the event
        // of a day rollover
        return weekdays[date.getDay()] + ", " + date.toGMTString();
    }
    

    public static synchronized String genericDateString(SimpleDateFormat dateFormat, Date x) {
        String newDateString = dateFormat.format(x);
        if (newDateString == null || newDateString.equals("12/30/1899")) 
                        newDateString = "";  //only if above condition
        return newDateString;
    }

    public String getUserDateString(Date date)
    {
    	return getStrFromDate(date, userFormat);
    }
    
    
    public static String logDate() {
        Date DefaultDate = new Date();
        return logDate(DefaultDate);
    }

    public static String logDate(Date date) {
        // This is used to write a standardized string date to the log file
        // It may also be used for various other reporting
        //CommonFunctions parseMe = new CommonFunctions();
        String DateString = date.toString();
        String[] ReturnArray = CommonFunctions.parsDelimStr(DateString," ");
        return ReturnArray[1] + ReturnArray[2];   //Returning Month and day
    }
    
    public static String getStandardDateString(String x)
    {
        //This method can be expended later to convert based upon the
        //date format that is being used.
        return getSimpleDate(getDateFromStr(x));
    }
    public String getStandardDateString(String x, String sDateFormat)
    {
        return getSimpleDate(getDateFromStr(x, sDateFormat));
    }

    public static String getExportDate(Date date) {
        return genericDateString(exportDate,date);
    }
    public static String getSQLDate(Date date) {
        return genericDateString(xBaseDate,date);
    }
    public static String getOracleDate(Date date) {
        return genericDateString(oracleDate,date);
    }
    public static String getGMTDate(Date date) {
        return genericDateString(GMTDate,date);
    }
    public static String getNameDate(Date date) {
        return genericDateString(nameDay,date);
    }
    public static String getUSDate(Date date) {
        return genericDateString(simplerDate,date);
    }
    public static String getSimpleDate(Date date) {
        return genericDateString(simpleDate, date);
    }
    public static String getSimpleTime(Date time) {
        return genericDateString(simpleTime,time);
    }
    
    // This two functions are really just for testing.
    public static String getLocalDate() {
    	//TimeZone TZ = TimeZone.getTimeZone("GMT-8:00");
    	//TimeZone TZ = TimeZone.getTimeZone(timeZoneString); 
    	TimeZone TZ = TimeZone.getDefault(); 
    	Calendar cal = Calendar.getInstance(TZ);
    	localDate = cal.getTime();
    	return genericDateString(simpleDate, localDate);
    }
    public String getLocalTime() {
    	//TimeZone TZ = TimeZone.getTimeZone("GMT-8:00"); 
    	//TimeZone TZ = TimeZone.getTimeZone(timeZoneString); 
    	TimeZone TZ = TimeZone.getDefault(); 
    	Calendar cal = Calendar.getInstance(TZ);
    	localDate = cal.getTime();
    	return genericDateString(simpleTime,localDate);
    }
    
    public static String getMonthDay(Date date) {
        return genericDateString(monthDay,date);
    }
    
    public String getUserMonthDay(Date date) {
    	   return getUserMonthDay(date, userFormat);
    }
    
    public String getUserMonthDay(Date date, String sUserDateFormat) {
        String retVal;
    	if (sUserDateFormat.indexOf("MM") < sUserDateFormat.indexOf("dd"))
    	{
    		retVal = genericDateString(monthDay,date);
    	} else
    	{
    		retVal = genericDateString(monthDayEuro,date);
    	}
    	return retVal;
    }

    
    public static String getMonth(Date date) {
        return genericDateString(onlyMonth,date);
    }
    public static String getSAPDate(Date date) {
        return genericDateString(sapDate,date);
    }
    public static String getGreatPlainsDate(Date date) {
        return genericDateString(greatPlainsDate,date);
    }
    public static String getJulianDate(Date date) {
        return genericDateString(julianDate,date);
    }
    public static String getNachaDate(Date date) {
        return genericDateString(nachaDate,date);
    }
    public static String getNachaTime(Date time) {
        return genericDateString(nachaTime,time);
    }
    public static String entryDate() {
        Date DefaultDate = new Date();
        return DefaultDate.toString();
    }
    public static String getPreviousMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, -1);
        return simpleMonth.format(calendar.getTime());
    }
    public static String getPreviousWeekend(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        while (calendar.get(Calendar.DAY_OF_WEEK) != Calendar.SATURDAY) {
            calendar.add(Calendar.DATE, -1);
        }
        return simpleDate.format(calendar.getTime());
    }
// Various utilites to create new dates
    public static Date addDays(Date today, int days)
    {
        Calendar day = new GregorianCalendar();
        day.setTime(today);
        day.add(Calendar.DATE, days);
        return (day.getTime());
    }
    public static Date tomorrow(Date today) {
        return(addDays(today, 1));
    }
    public static Date getPrevSunday(Date today) {
        Calendar day = new GregorianCalendar();
        day.setTime(today);
        int dayOfWeek = day.get(Calendar.DAY_OF_WEEK);
        return(addDays(today, -(dayOfWeek-1)));
    }   
    public static int getDayOfWeek(Date today) {
        Calendar day = new GregorianCalendar();
        day.setTime(today);
        int dayOfWeek = day.get(Calendar.DAY_OF_WEEK);
        return(dayOfWeek);
    }   
    public static Date adjustEpoch(Date date) {
//There is a problem with the '0000-00-00' or a null?        
        GregorianCalendar epoch = new GregorianCalendar(1990, 1, 1);
        if (date.before(epoch.getTime())) {
            epoch.setTime(date);
            epoch.add(Calendar.YEAR, 100);
            date = epoch.getTime();
        }
        return date;
    }
    
    public static String setInternalFromSQL(String x)
    {
        x = getSimpleDate(getDateFromSQL(x));
        return x;
    }
}