/*
DB2Audit.java - Loads the XML2ESS class from the database  
Copyright (C) 2005 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;
//import java.io.*;
//import java.util.*;
import java.text.*;
import java.sql.*;

import java.lang.reflect.Field;
import org.jdom.Element;
import org.jdom.Document;

public class DB2Audit extends ReportDisplayAudit {
   private String reportVoucher = "";
   private boolean reportFound = false;
   static CustomDate CD = new CustomDate();
   
   private DecimalFormat money;
   private DecimalFormat rate;
   private DecimalFormatSymbols numericRepresentation = new DecimalFormatSymbols();
   private char aPeriod = '.';
   private char aComma = ',';

   private AdisoftDbase schemaDB = new AdisoftDbase();
   
   public DB2Audit () {
   }
   
   public void setDateFormat(String x)
   {
	   AuditTrail.println("[000] DB2Audit.setDateFormat: " + x);
	   super.setDateFormat(x);
	   db.setInputDateFormat(x);
   }
   
   public void setReportVoucher(String X) {
     reportVoucher = X;
   }
   public void set(String X) {
     setReportVoucher(X);
     set();
   } 
   public void set() {
     try {
        if (db.setResultSet("REPORT","VOUCHER",reportVoucher,"VOUCHER")) {
           ST.setConnection(dbCon);
           ST.setSQLTerminator(sqlTerminator);    
           schemaDB = new AdisoftDbase();
           schemaDB.setConnection(dbCon);
           schemaDB.setSQLTerminator(sqlTerminator);
           schemaDB.setCheckKey(false);  //jh 2005-06-03
           String schemaFileName = ST.getSystemString("XMLSCHEMA","C:\\WORK\\schema.xml");
           schemaDB.setSchemaTable(schemaFileName);
           schemaDB.setColumns("REPORT"); 

           numericRepresentation.setDecimalSeparator(aPeriod);
           numericRepresentation.setGroupingSeparator(aComma);
           money = new DecimalFormat("0.00",numericRepresentation);
           rate = new DecimalFormat("0.####",numericRepresentation);

           populateReport();
           if (Rep.CURRENCY.equals("")) 
           {   
        	   super.setCompany(Rep.COMPANY);
        	   Rep.CURRENCY = HomeCurrency;
           }
           
           //String SQLx = "SELECT * FROM RECEIPT WHERE VOUCHER = '" + reportVoucher + "' AND DTOS(REC_DATE) = DTOS(REC_DATE)" ;
           //if (db.setResultSet(SQLx)) {
           if (super.db.setResultSet("RECEIPT","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSReceipt P = new ESSReceipt();
               // P.ExpenseLink = genReference++;
               // P.set(E, P.ExpenseLink);
               schemaDB.setColumns("RECEIPT");               
               populateReceipt(P);
               ESSReceipts.addElement(P);  
             } while (db.myResult.next());
           }
           //SQLx = "SELECT * FROM EXPENSE WHERE VOUCHER = '" + reportVoucher + "' AND RECEIPT = RECEIPT AND LINEITEM = LINEITEM" ;
           //if (db.setResultSet(SQLx)) {
           if (db.setResultSet("EXPENSE","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSExpense P = new ESSExpense();
               schemaDB.setColumns("EXPENSE");               
               populateExpense(P);
               ESSExpenses.addElement(P);  
             } while (db.myResult.next());
           }
           if (db.setResultSet("PURPOSE","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSPurpose P = new ESSPurpose();
               schemaDB.setColumns("PURPOSE");               
               populatePurpose(P);
               ESSPurposes.addElement(P);  
             } while (db.myResult.next());
           }
           if (db.setResultSet("EXTENDED","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSExtended P = new ESSExtended();
               schemaDB.setColumns("EXTENDED");               
               populateExtended(P);
               ESSExtendeds.addElement(P);  
             } while (db.myResult.next());
           }
        } 

        setUnScramble();  //reconcile the usage of getPossiblyScrambledValue() below - something is not working
        normalizeESSReport();
        setESSGL();
        reportFound = true;

     } catch (SQLException e) {
        AuditTrail.println("[500] DB2ESS.set: Report not loaded SQL Exception");
        AuditTrail.println("[500]" + e.toString());
        e.printStackTrace();
        reportFound = false;
     }
     db.close();
   }
   
   public boolean getReportFound() {
     return reportFound;
   }
   
   private void populateReport() throws SQLException{
      Rep.RP_STAT = getValue("RP_STAT");
      Rep.PERS_NUM = getValue("PERS_NUM");
      Rep.CRE_DATE = getXDateValue("CRE_DATE");
      Rep.CRE_TIME = getValue("CRE_TIME");
      Rep.VOUCHER = getValue("VOUCHER");
      Rep.PVOUCHER = getValue("PVOUCHER");
      Rep.SUB_DATE = getXDateValue("SUB_DATE");
      Rep.SUB_TIME = getValue("SUB_TIME");
      Rep.CUR_DATE = getXDateValue("CUR_DATE");
      Rep.CUR_TIME = getValue("CUR_TIME");
      Rep.ACT_DATE = getXDateValue("ACT_DATE");
      Rep.ACT_TIME = getValue("ACT_TIME");
      Rep.PAYMENT = getBooleanValue("PAYMENT");
      Rep.NAME = getPossiblyScrambledValue("NAME");
      Rep.LOCATION = getValue("LOCATION");
      Rep.PHONE = getValue("PHONE");
      Rep.DEPART = getValue("DEPART");
      Rep.COMPANY =  getValue("COMPANY");    // JH 20080530 getReportCompany(Rep.DEPART)
      Rep.RE_AMT = getNumberValue("RE_AMT");
      Rep.AD_AMT = getNumberValue("AD_AMT");
      Rep.RC_AMT = getNumberValue("RC_AMT");
      Rep.SIGN1 = getValue("SIGN1");
      Rep.SIGN2 = getValue("SIGN2");
      Rep.SIGN3 = getValue("SIGN3");
      //Rep.FILENAME = getValue(25);
      //Rep.OCCODE = getValue(28);
      Rep.FORCED = getBooleanValue("FORCED");
      Rep.VEHICLE = getValue("VEHICLE");
      Rep.MONTH = getValue("MONTH");
      Rep.SERVDATE = getXDateValue("SERVDATE");
      Rep.START = getValue("START");
      Rep.FINISH = getValue("FINISH");
      Rep.BUSINESS = getValue("BUSINESS");
      Rep.CLERK = getValue("CLERK");
      Rep.EXTRA1 = getValue("EXTRA1");
      Rep.MANAGER = getValue("MANAGER");
      Rep.COMMENT = getValue("COMMENT");
      Rep.EXTRA3 = getValue("EXTRA3");
      Rep.PHY_REC = getValue("PHY_REC");
      Rep.RECEIVED = getXDateValue("RECEIVED");
      //Rep.ORD_NAME = getValue(55);      
      Rep.SIGN4 = getValue("SIGN4");
      Rep.ADMIN1 = getValue("ADMIN1");
      Rep.ADMIN2 = getValue("ADMIN2");
      Rep.WEEKEND = getXDateValue("WEEKEND");
      Rep.CURRENCY = getValue("CURRENCY");
      Rep.HISTORY = getValue("HISTORY");
      Rep.UP_DATE = getXDateValue("UP_DATE");
      Rep.UP_STAT = getValue("UP_STAT");
      Rep.SG1_DATE = getXDateValue("SG1_DATE");
      Rep.SG2_DATE = getXDateValue("SG2_DATE");
      Rep.AUD_DATE = getXDateValue("AUD_DATE");
      Rep.AD1_DATE = getXDateValue("AD1_DATE");
      Rep.AD2_DATE = getXDateValue("AD2_DATE");
      setCompany(Rep.COMPANY);
   }

   private String getXDateValue (String N) throws SQLException {
      String Default = "";
      String Xdate = getValue(N, Default);
      
      if (Xdate.equals("1899-12-30")) Xdate = "";  //VFP 5.0 Blank Date
      if (Xdate.equals("0002-11-30")) Xdate = "";  //MySQL Blank Date (Considered a bug in this version)
      if (Xdate.equals("0000-00-00")) Xdate = "";  //MySQL Blank Date 
      
      if (!Xdate.equals("")) {
         Xdate = CustomDate.getSimpleDate(CustomDate.getDateFromXBase(Xdate));
      }
      return Xdate;
   }

   private String getIntegerValue (String N) throws SQLException {
      String Default = "3";
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         int Y = java.lang.Float.valueOf(X).intValue();
         X = java.lang.Integer.toString(Y);
      }
      return X;
   }
   
   private String getNumberValue (String N) throws SQLException {
      String Default = "0.00";
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         try {
            X = money.format(money.parse(X));
         } catch (ParseException e) {
            X = Default;
            AuditTrail.println("[500] DB2ESS.getNumberValue - Data parsing error");
         }
      }
      return X;
   }

   private String getRateValue (String N) throws SQLException {
      String Default = "";
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         try {
            X = rate.format(rate.parse(X));
         } catch (ParseException e) {
            X = Default;
            AuditTrail.println("[500] DB2ESS.getRateValue - Data parsing error");
       }
      }
      return X;
   }
   private String getBooleanValue (String N) throws SQLException {
      String Default = "";
      String X = getValue(N, Default);
      if (X.equals("1")) {
         X = "Yes";
      } else {
         X = "No";
      }
      return X;
   }

   private String getValue (String N) throws SQLException {
      //cannot retrieve numbers, dates, or booleans this way
      return getValue(N, "");
   } 
   
   private String getValue (String N, String Default) throws SQLException {
      //gets a value from a single column
      String retVal;
      //AuditTrail.println("[---] N = " + N);
      String X = schemaDB.getColumnName(N);
      //AuditTrail.println("[---] X = " + X);
      String Z = db.myResult.getString(X);
      //AuditTrail.println("[---] Z = " + Z);
      String E = getTrim(Z);
      if (E != null) {
        retVal = E;  
      } else {    
        retVal = Default;
      }
      return retVal;
   }
   
   private String getPossiblyScrambledValue(String N) throws SQLException {
      String retVal = "";
      if (super.encrypt.equalsIgnoreCase("YES"))
      {
        retVal = getScrambledValue(N, retVal);
      } else 
      {
        retVal = getValue(N, retVal);
      }
      return retVal;
   }


   private String getScrambledValue (String N, String Default) throws SQLException {
      //gets a value from a single column
      //String E = getTrim(db.myResult.getString(N));
      String retVal;
      byte[] bArray = db.myResult.getBytes(N);
      if (bArray.length == 0) { 
        retVal = Default;
      } else {
        String E = new String(bArray);
        if (E != null) {
          retVal = E;  
        } else {    
          retVal = Default;
        }
      }
      return retVal;
   }
   
   private void populateReceipt(ESSReceipt P) throws SQLException {
      P.RCPTTYPE = getValue("RCPTTYPE");
      P.CHARGE = getValue("CHARGE");
      P.VOUCHER = getValue("VOUCHER");
      P.RECEIPT = getValue("RECEIPT");
      P.REFER = getValue("REFER");
      P.REC_AMT = getNumberValue("REC_AMT");
      P.REC_DATE = getXDateValue("REC_DATE");
      P.AMOUNT  = getNumberValue("AMOUNT");
      P.REIMB = getBooleanValue("REIMB");
      P.FORCED = getBooleanValue("FORCED");
      P.CRE_DATE = getXDateValue("CRE_DATE");
      P.CHECK = getBooleanValue("CHECK");
      P.MERCHANT = getValue("MERCHANT");
      P.UNITS = getValue("UNITS");
      P.XSOURCE = getValue("XSOURCE");
      P.XRATE = getRateValue("XRATE");
      P.XDATE = getXDateValue("XDATE");
      P.BEG_DATE = getXDateValue("BEG_DATE");
      P.LOCATION = getValue("LOCATION");
      P.OFFSET = getValue("OFFSET");
      P.OK_STAT = getValue("OK_STAT");
      P.OK_REF = getValue("OK_REF");
   }
   private void populateExpense(ESSExpense P) throws SQLException{
      P.EXPENSE = getValue("EXPENSE");
      P.AMOUNT = getNumberValue("AMOUNT");
      P.VOUCHER = getValue("VOUCHER");
      P.RECEIPT = getValue("RECEIPT");
      P.PURPOSE = getValue("PURPOSE");
      P.LINEITEM = getValue("LINEITEM");
      P.BILLTYPE = getIntegerValue("BILLTYPE");
      P.COMMENT = getPossiblyScrambledValue("COMMENT");
      P.DEPART = getValue("DEPART");
      P.ChargedDepart = P.DEPART;  //JH 1-29-2007
      P.CRE_DATE = getXDateValue("CRE_DATE");
      P.EXP_DATE = getXDateValue("EXP_DATE");
      P.REC_AMT = getNumberValue("REC_AMT");
      P.XRATE = getRateValue("XRATE");
      P.XDATE = getXDateValue("XDATE");
      P.UNITS = getValue("UNITS");
      P.XSOURCE = getValue("XSOURCE");
      P.PERSPLIT = getValue("PERSPLIT");
      P.COMPANY = getValue("COMPANY");
      //P.category = getValue(xx);
   }
   private void populatePurpose(ESSPurpose P) throws SQLException{
      P.PURPOSE = getValue("PURPOSE");
      P.VOUCHER = getValue("VOUCHER");
      P.CLIENT = getPossiblyScrambledValue("CLIENT");
      P.LOCATION = getPossiblyScrambledValue("LOCATION");
      P.COMMENT = getPossiblyScrambledValue("COMMENT");
      P.PROJECT = getPossiblyScrambledValue("PROJECT");
      P.STEPNO = getValue("STEPNO");
      P.BEG_DATE = getXDateValue("BEG_DATE");
      P.END_DATE = getXDateValue("END_DATE");
      P.DAYS = getValue("DAYS");
      P.NIGHTS = getValue("NIGHTS");
      P.CLIENTNO = getPossiblyScrambledValue("CLIENTNO");
      P.EXTENDED = getBooleanValue("EXTENDED");
      P.TIMESLIP = getBooleanValue("TIMESLIP");
   }
   private void populateExtended(ESSExtended P) throws SQLException{
      P.VOUCHER = getValue("VOUCHER");  //What about purpose??
      P.LINEITEM = getValue("LINEITEM");
      P.NAME = getPossiblyScrambledValue("NAME");
      P.REMARKS = getPossiblyScrambledValue("REMARKS");
      P.CLIENT = getPossiblyScrambledValue("CLIENT");
      P.LOCATION = getPossiblyScrambledValue("LOCATION");
      P.COMMENT = getPossiblyScrambledValue("COMMENT");
      P.EXTRAITEM = getValue("EXTRAITEM");
   }
   public String getTrim(String x) {
      if (x == null) return "";
      return x.trim();
   }
   
//========================================================================

  public void set(AdisoftDOM ReportDOM) {
  //compatible to the Adisoft DOM extension  
    set(ReportDOM.getDOM());
  }  
  public void set(Document ReportDOM) {
    Element report = ReportDOM.getRootElement();
    set(report, 0);
  }
  
  public void set(Element report, int Link){
    setReport(report, Link);
    normalizeWeight();
    normalizeSplitClients();
    normalizeESSReport();
    setESSGL();
  }


  public void setReport(Element report, int Link, boolean useNewVoucher) {
  //overrrides the method in the XML2ESS super class  
  //calls the routines to load up the objects representing the various sections of an ESS report  
    java.util.List sections = report.getChildren();
    for (int i = 0; i < sections.size(); i++) {   //these are the sections
      Element section = (Element) sections.get(i);
      java.util.List sectionList = section.getChildren();
      if (sectionList.size() > 0) {
        String secType = section.getName();
        if (secType.equals("header")) {
          if (!setESSReport(section, useNewVoucher)){
            AuditTrail.println("[400] XML2ESS.setReport: Report process aborted");
            i = sections.size();
          }    
        } else if (secType.equals("purpose")) { 
          setESSPurpose(section); 
        } else if (secType.equals("split")) { 
          //setSplitClient(section, Link); 
        } else if (secType.equals("receipt")) { 
          setESSReceipt(section);  
        } else if (secType.equals("expense")) { 
          setESSExpense(section, Link); 
        } else if (secType.equals("attendeelist")) { 
          setESSAttendeeList(section, Link);
        } else if (secType.equals("attendee")) { 
          setESSAttendee(section, Link);    
        }
      }
    }
  }  

  public void setReport(Element report, int Link) {
    setReport(report, Link, false);
  }  
  
  private boolean setESSReport(Element E, boolean useNewVoucher) {
    //used to create a report object.
    setElements(E, Rep, Rep.getDateFields(), Rep.getNumericFields());
    Rep.set();
    setAccountingInfo();
    //Rep.RE_AMT = getReimburse();
    return true;
  } 
 
  private void setESSPurpose(Element E) {
    //used to create a Purpose object.
    String Reference = E.getChildText("PURPOSE");
    ESSPurpose P;
    if (isESSPurpose(Reference)) {
       P = getESSPurpose(Reference);
    } else {
       P = new ESSPurpose();
       ESSPurposes.addElement(P);
    }
    //P.set();  //reversed JH 2006-04-26
    setElements(E,P,P.getDateFields(), P.getNumericFields());
    P.set();  //look at a better modularization and the order    
    P.PurposeLink = Integer.parseInt(P.PURPOSE.substring(P.PURPOSE.length()-7));
    setReport(E, P.PurposeLink);
  } 
   
  private void setESSReceipt(Element E) {
    //used to create a receipt object.
    String Reference = E.getChildText("RECEIPT");
    ESSReceipt P;
    // In current usage, the else clause will be used as report edits as of 6-26-2006 
    // have an object persistance of page.
    if (isESSReceipt(Reference)) {
       AuditTrail.println("[000] DB2Audit.setESSReceipt Existing Receipt:" + Reference);       
       P = getESSReceipt(Reference);        
    } else {
       AuditTrail.println("[000] DB2Audit.setESSReceipt New Receipt:" + Reference);
       P = new ESSReceipt();
       P.setUpdateListWithReceipt();
       ESSReceipts.addElement(P);
    }
    setElements(E,P, P.getDateFields(), P.getNumericFields());
    P.set();  //look at a better modularization    
    P.normalizeType();  //Jim Holton 11-10-2004
    setReport(E, P.ExpenseLink); 
  } 

  private String LastLineItem;  //Use by setAttendee JH 10-31-2007   
  private void setESSExpense(Element E, int ReceiptLink) {
    //used to create an expense object.
    String Reference1 = E.getChildText("RECEIPT");
    String Reference2 = E.getChildText("LINEITEM");
    ESSExpense P;
    if (isESSExpense(Reference1, Reference2)) {
       AuditTrail.println("[000] DB2Audit.setESSExpense Existing Expense:" + Reference1 + "-" + Reference2);        
       //ESSReceipt R = getESSReceipt(ReceiptLink);
       //P = getESSExpenseLineItem(R.RECEIPT, Reference2);
       P = getESSExpenseLineItem(Reference1, Reference2);   //JH 2006-04-27
    } else {
       if (Reference2 == null) {
         AuditTrail.println("[000] DB2Audit.setESSExpense New Expense[1]:" + Reference1 + "-" +Reference2);
       } else {
         AuditTrail.println("[000] DB2Audit.setESSExpense New Expense[2]:" + Reference1 + "-" +Reference2);
       }
       P = new ESSExpense();
       
       ESSExpenses.addElement(P);
    }
    
    setElements(E,P, P.getDateFields(), P.getNumericFields());
    if (P.LINEITEM == null || P.LINEITEM.equals("")) {   //JH 2007-10-31
       String workingNumber;
       do {
          workingNumber = getESSReferenceString(genReference++);
       } while (isLineItem(workingNumber));     
       P.LINEITEM = workingNumber;
    }
    LastLineItem = P.LINEITEM;
    P.set(ReceiptLink);  //look at a better modularization 
    
    setReport(E, P.LineLink);
  }   
  
  protected void setESSAttendeeList (Element E, int ListLink) {
    //used to create an AttendeeList.
    String Reference = E.getChildText("LINEITEM");
    ESSAttendeeList P;
    if (Reference == null) {
       P = new ESSAttendeeList();
       ESSAttendeeLists.addElement(P);
    } else {
       P = getESSAttendeeList(Reference);
    }
    setElements(E,P, P.getDateFields(), P.getNumericFields());
    P.normalize(ListLink);
    setReport(E, ListLink);
  }  
   
  private void setESSAttendee (Element E, int Link) {
    //used to create an Attendee.
    String Reference = E.getChildText("EXTRAITEM");
//AuditTrail.println("[---] DB2Audit.setESSAttendee Reference: " + Reference);     
    ESSExtended P;
    if (Reference == null) {
       P = new ESSExtended();
       ESSExtendeds.addElement(P);
//AuditTrail.println("[---] DB2Audit.setESSAttendee new ESSExtended");       
    } else {
    	//AuditTrail.println("[---] DB2Audit.setESSAttendee lastLineItem: " + LastLineItem);    
       P = getESSAttendee(LastLineItem,Reference);
//AuditTrail.println("[---] DB2Audit.setESSAttendee using a previous");       
    }
    setElements(E,P, P.getDateFields(), P.getNumericFields());
//AuditTrail.println("[---] DB2Audit.setESSAttendee NAME: " + P.NAME);    
//AuditTrail.println("[---] DB2Audit.setESSAttendee EXTRAITEM: " + P.EXTRAITEM);    
    if (P.LINEITEM == null || P.LINEITEM.equals("")) {   //JH 2007-10-31
       if (LastLineItem != null) {
          P.LINEITEM = LastLineItem;  //Link is an integer
       }
    }
//AuditTrail.println("[---] DB2Audit.setESSAttendee LINEITEM: " + P.LINEITEM);  
//AuditTrail.println("[---] DB2Audit.setESSAttendee persistance: " + P.persistance);
    P.normalize(Link);
    //setReport(E, Link);
  }    
   
  private void setElements(Element CurrentElement, Object obj, String[] dateFields, String[] numberFields) {
    String name;
    String value;
    if (CurrentElement != null && (CurrentElement.getContentSize() > 0)) {
      java.util.List elementList = CurrentElement.getChildren();
      for (int i = 0; i < elementList.size(); i++) {
        Element e = (Element) elementList.get(i);
        name = e.getName();
        value = e.getText().trim();
        value = numberAdjustment(name, value, numberFields);  //JH 2011-4-20 and the next line too.
        value = dateAdjustment(name, value, dateFields);
        setField(obj, name, value);
      }
    }
  }
  
  
  
  private String dateAdjustment(String name, String value, String[] dateFields)
  {
	  for (int i = 0; i < dateFields.length; i++)
	  {
		  if (name.equals(dateFields[i]))
		  {
			  if (value != null && value.length() > 1)
			  {
				  try {
					  AuditTrail.println("[000] DB2Audit.dateAdjustment: " + name + ", " + value);
					  value = date.getStandardDateString(value, super.getDateFormat());
				  } catch(java.lang.NullPointerException ex) {
					  AuditTrail.println("[500] DB2Audit.dateAdjustment NullPointer: " + name + ", " + value + ",error: " + ex.toString()); 
				  } catch(java.lang.Exception ex) {
					  AuditTrail.println("[500] DB2Audit.dateAdjustment Exception: " + name + ", " + value + ",error: " + ex.toString()); 
				  }
			  }
			  i = dateFields.length;
		  }
	  }
	  return value;
  }
  
  private String numberAdjustment(String name, String value, String[] numberFields)
  {
	  for (int i = 0; i < numberFields.length; i++)
	  {
		  if (name.equals(numberFields[i]))
		  {
		      if (value != null && value.length() > 1)
		      {
		    	  AuditTrail.println("[000] DB2Audit.numberAdjustment: " + name + ", " + value);
		    	  value = Utilities.replaceStr(value,separatorStr,"");
		    	  if (!decimalStr.equals(".")) value = Utilities.replaceStr(value,decimalStr,".");
		    	  i = numberFields.length;
		      }
		  }
	  }
	  return value;
  }
  
  
  private void setField(Object obj, String name, String value) {
    Field f;
    Class c = obj.getClass();   //Performance?? :-> can we incorporate above?? 
    try {
        f = c.getField(name);
        f.set(obj, value);
    } catch (NoSuchFieldException e) {
        AuditTrail.println("[000] DB2Audit.setField NoSuchFieldException "  + name + " = " + value);
    } catch (IllegalAccessException e) {
        AuditTrail.println("[500] DB2Audit.setField IllegalAccessException "  + name + " = " + value);
    }
  }
  
  private String getStringField(Object obj, String name) {
    Field f;
    String x = "";
    Class c = obj.getClass();
    try {
        f = c.getField(name);
        x = (String) f.get(obj);
    } catch (NoSuchFieldException e) {
        AuditTrail.println("[000] DB2Audit.getStringField NoSuchFieldException " + name);
    } catch (IllegalAccessException e) {
        AuditTrail.println("[500] DB2Audit.getStringField IllegalAccessException " + name);
    }
    return x;
  }
  
  private String[] getStringFieldArray(Object obj, String[] name)
  {
    String[] X = new String[name.length];
    for (int i = 0; i < name.length; i++)
    {
        X[i] = getStringField(obj, name[i]);
    }
    return X;
  }
  public boolean setHeaderUpdate()
  {
	    String stack_date_format = super.getDateFormat();  //JH 2011-04-21
	    String internal_date_format = "MM/dd/yyyy";
	    super.setDateFormat(internal_date_format);
	    db.setInputDateFormat(internal_date_format);

	    ReportElementAPI X = (ReportElementAPI) Rep;
	    boolean retVal = setUpdate(X, "REPORT");
	    
	    super.setDateFormat(stack_date_format);
	    db.setInputDateFormat(stack_date_format);
	 
	    return retVal;
  }
  
  public void setHeaderWithStatus() {
    Rep.setUpdateListWithStatus();
  }
  
  public boolean setUpdate()
  {
    String stack_date_format = super.getDateFormat();  //JH 2011-04-21
    String internal_date_format = "MM/dd/yyyy";
    super.setDateFormat(internal_date_format);
    db.setInputDateFormat(internal_date_format);

    ReportElementAPI X = (ReportElementAPI) Rep;
    boolean updateStatus = setUpdate(X, "REPORT");

    if (updateStatus == true)
    {
        int i = 0;
        while ((i < ESSPurposes.size()) && (updateStatus == true))
        {
            X = (ReportElementAPI) ESSPurposes.elementAt(i);
            updateStatus = setUpdate(X, "PURPOSE");
            i++;
        }
        
        i = 0;
        while ((i < ESSReceipts.size()) && (updateStatus == true))
        {
            X = (ReportElementAPI) ESSReceipts.elementAt(i);
            updateStatus = setUpdate(X, "RECEIPT");
            i++;
        }
        i = 0;
        ESSExpense Y;
        while ((i < ESSExpenses.size()) && (updateStatus == true))
        {
            //check for a delete persistance and if that is the case, do that not this
            Y = (ESSExpense) ESSExpenses.elementAt(i);
            if (Y.getPersistance().equalsIgnoreCase("DELETE"))
            {
               updateStatus = deleteExpense(Y.RECEIPT, Y.LINEITEM);
            } else 
            {
               updateStatus = setUpdate((ReportElementAPI) Y, "EXPENSE");
            }
            i++;
        }
        
       
        i = 0;
        ESSExtended Z;
        while ((i < ESSExtendeds.size()) && (updateStatus == true))
        {
            //need to resolve the deleting of an individual attendee also.
            Z = (ESSExtended) ESSExtendeds.elementAt(i);
            if (Z.getPersistance().equalsIgnoreCase("DELETE"))
            {
               updateStatus = deleteExtended(Z.LINEITEM, Z.EXTRAITEM);
            } else 
            {
               updateStatus = setUpdate((ReportElementAPI) Z, "EXTENDED");
            }
            i++;
        } 
       
    }
    
    super.setDateFormat(stack_date_format);
    db.setInputDateFormat(stack_date_format);
    
    return updateStatus;
  }
  
  
  public boolean deletePurpose(String PurposeNo) {
    ESSPurpose x = getESSPurpose(PurposeNo);
    boolean retVal = ( !x.PURPOSE.equals("") ) && ESSPurposes.remove( (Object) x );
    if (retVal)
    {
        x.persistance = "delete";
        retVal = setUpdate( (ReportElementAPI) x, "PURPOSE");
    }
    return retVal;
  }

  public boolean deleteReceipt(String ReceiptNo) {
    ESSReceipt x = getESSReceipt(ReceiptNo);
    boolean retVal = ( !x.RECEIPT.equals("") ) && ESSReceipts.remove( (Object) x );
    if (retVal)
    {
        String[] expenses = getExpenseRefs4Receipt(ReceiptNo);
        for (int i = 0; i < expenses.length; i++) 
        {
           retVal = retVal && deleteExpense(ReceiptNo, expenses[i]);
        }
        if (retVal)
        {
           x.persistance = "delete";
           retVal = setUpdate( (ReportElementAPI) x, "RECEIPT");
        }
    }
    return retVal;
  }

  public boolean deleteExpense(String ReceiptNo, String lineitem) {
    //  Right now, the lineitem is unique
    ESSExpense x = getESSExpenseLineItem(ReceiptNo, lineitem);
    boolean retVal = ( !x.LINEITEM.equals("") ) && ESSExpenses.remove( (Object) x );
    if (retVal)
    {
        String[] extendeds = getExtendedRefs4Expense(lineitem);
        for (int i = 0; i < extendeds.length; i++) 
        {
           retVal = retVal && deleteExtended(lineitem, extendeds[i]);
        }
        if (retVal)
        {
           x.persistance = "delete";
           retVal = setUpdate( (ReportElementAPI) x, "EXPENSE");
        }
    }
    return retVal;
  }

  public boolean deleteExtended(String lineitem, String extraitem) {
    ESSExtended x = getESSAttendee(lineitem, extraitem);
    boolean retVal = ( !x.LINEITEM.equals("") ) && ESSExtendeds.remove( (Object) x );
    if (retVal)
    {
        x.persistance = "delete";
        retVal = setUpdate( (ReportElementAPI) x, "EXTENDED");
    }
    return retVal;
  }

  public boolean setUpdate(ReportElementAPI X, String Table)
  {
    boolean updateStatus = true;
    String persistanceType = getStringField(X,"persistance");
    if (!persistanceType.equals("")) 
    {
      AuditTrail.println("[000] DB2Audit.setUpdate (" + Table + ") + persistance: " + persistanceType);    
      if (persistanceType.equalsIgnoreCase("update"))
      {
         String[] keyList = db.getPhysicalNames(Table,X.getKeyList());
         String[] tagList = db.getPhysicalNames(Table,X.getUpdateList());
         String[] logicalFields = db.getPhysicalNames(Table,X.getLogicalFields());
         String[] numericFields = db.getPhysicalNames(Table,X.getNumericFields());
         String[] dateFields = db.getPhysicalNames(Table,X.getDateFields());
         String[] encryptFields = db.getPhysicalNames(Table,X.getEncryptFields());
         String[] valueList = this.getStringFieldArray(X, X.getUpdateList());
         String[] keyValues = this.getStringFieldArray(X, X.getKeyList());
         updateStatus = db.setUpdate("SIMPLESQL", Table, keyList, tagList, logicalFields, 
                         numericFields, dateFields, encryptFields, valueList, keyValues);
         if (updateStatus) {
               AuditTrail.println("[000] DB2Audit.setUpdate persistance status OK"); 
               X.updateNormalize();
               X.setPersistanceFlag("");
         } else {
               AuditTrail.println("[500] DB2Audit.setUpdate persistance status FAILED");        
         }

      } else if (persistanceType.equalsIgnoreCase("insert"))
      {
         updateStatus = X.setPersistance();  // want to move to db eventually
         if (updateStatus) X.updateNormalize();
         if (updateStatus) X.setPersistanceFlag("");                         
         
      } else if (persistanceType.equalsIgnoreCase("delete"))
      {
         String[] keyList = db.getPhysicalNames(Table,X.getKeyList());
         String[] tagList = db.getPhysicalNames(Table,X.getUpdateList());
         String[] logicalFields = db.getPhysicalNames(Table,X.getLogicalFields());
         String[] numericFields = db.getPhysicalNames(Table,X.getNumericFields());
         String[] dateFields = db.getPhysicalNames(Table,X.getDateFields());
         String[] encryptFields = db.getPhysicalNames(Table,X.getEncryptFields());
         String[] valueList = this.getStringFieldArray(X,tagList);
         String[] keyValues = this.getStringFieldArray(X, keyList);
         updateStatus = db.setDelete("SIMPLESQL", Table, keyList, tagList, logicalFields, 
                         numericFields, dateFields, encryptFields, valueList, keyValues);
         //Probably need to remove the object here

      }
    }                   
  
    return updateStatus;
  }

}