/*
DB2ESS.java - Loads the XML2ESS class from the database  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;
//import java.io.*;
//import java.util.*;
import java.text.*;
import java.sql.*;

public class DB2ESS extends ReportDisplayESS {
   private String reportVoucher = "";
   private boolean reportFound = false;
   static CustomDate CD = new CustomDate();
   //private NumberFormat money = NumberFormat.getCurrencyInstance();
   private DecimalFormat money;
   private DecimalFormat rate;
   private DecimalFormatSymbols numericRepresentation = new DecimalFormatSymbols();
   private char aPeriod = '.';
   private char aComma = ',';
   private AdisoftDbase schemaDB = new AdisoftDbase();
   
   public DB2ESS () {
   }
   
   public void setReportVoucher(String X) {
     reportVoucher = X;
   }
   public void set(String X) {
     setReportVoucher(X);
     set();
   } 
   public void set() {
     try {
        if (db.setResultSet("REPORT","VOUCHER",reportVoucher,"VOUCHER")) {
        	numericRepresentation.setDecimalSeparator(aPeriod);
        	numericRepresentation.setGroupingSeparator(aComma);
        	money = new DecimalFormat("0.00",numericRepresentation);
        	rate = new DecimalFormat("0.####",numericRepresentation);
           ST.setConnection(dbCon);
           ST.setSQLTerminator(sqlTerminator);    
           schemaDB = new AdisoftDbase();
           schemaDB.setConnection(dbCon);
           schemaDB.setSQLTerminator(sqlTerminator);
           schemaDB.setCheckKey(false);  //jh 2005-06-03
           String schemaFileName = ST.getSystemString("XMLSCHEMA","C:\\WORK\\schema.xml");
           schemaDB.setSchemaTable(schemaFileName);
           schemaDB.setColumns("REPORT"); 
           populateReport();
           if (Rep.CURRENCY.equals("")) 
           {   
        	   super.setCompany(Rep.COMPANY);
        	   Rep.CURRENCY = HomeCurrency;
           }
           //String SQLx = "SELECT * FROM RECEIPT WHERE VOUCHER = '" + reportVoucher + "' AND DTOS(REC_DATE) = DTOS(REC_DATE)" ;
           //if (db.setResultSet(SQLx)) {
           if (super.db.setResultSet("RECEIPT","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSReceipt P = new ESSReceipt();
               // P.ExpenseLink = genReference++;
               // P.set(E, P.ExpenseLink);
               schemaDB.setColumns("RECEIPT");               
               populateReceipt(P);
               ESSReceipts.addElement(P);  
             } while (db.myResult.next());
           }
           //SQLx = "SELECT * FROM EXPENSE WHERE VOUCHER = '" + reportVoucher + "' AND RECEIPT = RECEIPT AND LINEITEM = LINEITEM" ;
           //if (db.setResultSet(SQLx)) {
           if (db.setResultSet("EXPENSE","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSExpense P = new ESSExpense();
               schemaDB.setColumns("EXPENSE");               
               populateExpense(P);
               ESSExpenses.addElement(P);  
             } while (db.myResult.next());
           }
           if (db.setResultSet("PURPOSE","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSPurpose P = new ESSPurpose();
               schemaDB.setColumns("PURPOSE");               
               populatePurpose(P);
               ESSPurposes.addElement(P);  
             } while (db.myResult.next());
           }
           if (db.setResultSet("EXTENDED","VOUCHER",reportVoucher,"VOUCHER")) {
             do  {
               ESSExtended P = new ESSExtended();
               schemaDB.setColumns("EXTENDED");               
               populateExtended(P);
               ESSExtendeds.addElement(P);  
             } while (db.myResult.next());
           }
        } 

        setUnScramble();  //reconcile the usage of getPossiblyScrambledValue() below - something is not working
        normalizeESSReport();
        setESSGL();
        reportFound = true;

     } catch (SQLException e) {
        AuditTrail.println("[500] DB2ESS.set: Report not loaded SQL Exception");
        AuditTrail.println("[500]" + e.toString());
        e.printStackTrace();
        reportFound = false;
     }
     db.close();
   }
   
   public boolean getReportFound() {
     return reportFound;
   }
   
   private void populateReport() throws SQLException{
      Rep.RP_STAT = getValue("RP_STAT");
      Rep.PERS_NUM = getValue("PERS_NUM");
      Rep.CRE_DATE = getXDateValue("CRE_DATE");
      Rep.CRE_TIME = getValue("CRE_TIME");
      Rep.VOUCHER = getValue("VOUCHER");
      Rep.PVOUCHER = getValue("PVOUCHER");
      Rep.SUB_DATE = getXDateValue("SUB_DATE");
      Rep.SUB_TIME = getValue("SUB_TIME");
      Rep.CUR_DATE = getXDateValue("CUR_DATE");
      Rep.CUR_TIME = getValue("CUR_TIME");
      Rep.ACT_DATE = getXDateValue("ACT_DATE");
      Rep.ACT_TIME = getValue("ACT_TIME");
      Rep.PAYMENT = getBooleanValue("PAYMENT");
      Rep.NAME = getPossiblyScrambledValue("NAME");
      Rep.LOCATION = getValue("LOCATION");
      Rep.PHONE = getValue("PHONE");
      Rep.DEPART = getValue("DEPART");
      Rep.COMPANY = getValue("COMPANY");    // JH 20080530 getReportCompany(Rep.DEPART)
      Rep.RE_AMT = getNumberValue("RE_AMT");
      Rep.AD_AMT = getNumberValue("AD_AMT");
      Rep.RC_AMT = getNumberValue("RC_AMT");
      Rep.SIGN1 = getValue("SIGN1");
      Rep.SIGN2 = getValue("SIGN2");
      Rep.SIGN3 = getValue("SIGN3");
      //Rep.FILENAME = getValue(25);
      //Rep.OCCODE = getValue(28);
      Rep.FORCED = getBooleanValue("FORCED");
      Rep.VEHICLE = getValue("VEHICLE");
      Rep.MONTH = getValue("MONTH");
      Rep.SERVDATE = getXDateValue("SERVDATE");
      Rep.START = getValue("START");
      Rep.FINISH = getValue("FINISH");
      Rep.BUSINESS = getValue("BUSINESS");
      Rep.CLERK = getValue("CLERK");
      Rep.EXTRA1 = getValue("EXTRA1");
      Rep.MANAGER = getValue("MANAGER");
      Rep.COMMENT = getValue("COMMENT");
      Rep.EXTRA3 = getValue("EXTRA3");
      Rep.PHY_REC = getValue("PHY_REC");
      Rep.RECEIVED = getXDateValue("RECEIVED");
      //Rep.ORD_NAME = getValue(55);      
      Rep.SIGN4 = getValue("SIGN4");
      Rep.ADMIN1 = getValue("ADMIN1");
      Rep.ADMIN2 = getValue("ADMIN2");
      Rep.WEEKEND = getXDateValue("WEEKEND");
      Rep.CURRENCY = getValue("CURRENCY");
      Rep.HISTORY = getValue("HISTORY");
      Rep.UP_DATE = getXDateValue("UP_DATE");
      Rep.UP_STAT = getValue("UP_STAT");
      Rep.SG1_DATE = getXDateValue("SG1_DATE");
      Rep.SG2_DATE = getXDateValue("SG2_DATE");
      Rep.AUD_DATE = getXDateValue("AUD_DATE");
      Rep.AD1_DATE = getXDateValue("AD1_DATE");
      Rep.AD2_DATE = getXDateValue("AD2_DATE");
      setCompany(Rep.COMPANY);      
   }

   private String getXDateValue (String N) throws SQLException {
      String Default = "";
      String Xdate = getValue(N, Default);
      
      if (Xdate.equals("1899-12-30")) Xdate = "";  //VFP 5.0 Blank Date
      if (Xdate.equals("0002-11-30")) Xdate = "";  //MySQL Blank Date (Considered a bug in this version)
      if (Xdate.equals("0000-00-00")) Xdate = "";  //MySQL Blank Date 
      
      if (!Xdate.equals("")) {
         Xdate = CustomDate.getSimpleDate(CustomDate.getDateFromXBase(Xdate));
      }
      return Xdate;
   }

   private String getIntegerValue (String N) throws SQLException {
      String Default = "3";
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         int Y = java.lang.Float.valueOf(X).intValue();
         X = java.lang.Integer.toString(Y);
      }
      return X;
   }
   
   private String getNumberValue (String N) throws SQLException {
      String Default = "0.00";
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         try {
            X = money.format(money.parse(X));
            if (X.indexOf(",") > X.indexOf("."))
            {
            	AuditTrail.println("[500] DB2ESS.getNumberValue - Data parsing error: " + X);
            }
         } catch (ParseException e) {
            X = Default;
            AuditTrail.println("[500] DB2ESS.getNumberValue - Data parsing error");
         }
      }
      return X;
   }

   private String getRateValue (String N) throws SQLException {
      String Default = "";
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         try {
            X = rate.format(rate.parse(X));
         } catch (ParseException e) {
            X = Default;
            AuditTrail.println("[500] DB2ESS.getRateValue - Data parsing error");
       }
      }
      return X;
   }
   private String getBooleanValue (String N) throws SQLException {
      String Default = "";
      String X = getValue(N, Default);
      if (X.equals("1")) {
         X = "Yes";
      } else {
         X = "No";
      }
      return X;
   }

   private String getValue (String N) throws SQLException {
      //cannot retrieve numbers, dates, or booleans this way
      return getValue(N, "");
   } 
   
   private String getValue (String N, String Default) throws SQLException {
      String retVal;
      String X = schemaDB.getColumnName(N);
      String Z = db.myResult.getString(X);
      String E = getTrim(Z);
      if (E != null) {
        retVal = E;  
      } else {    
        retVal = Default;
      }
      return retVal;
   }
   
   private String getPossiblyScrambledValue(String N) throws SQLException {
      String retVal = "";
      if (super.encrypt.equalsIgnoreCase("YES"))
      {
        retVal = getScrambledValue(N, retVal);
      } else 
      {
        retVal = getValue(N, retVal);
      }
      return retVal;
   }


   private String getScrambledValue (String N, String Default) throws SQLException {
      //gets a value from a single column
      //String E = getTrim(db.myResult.getString(N));
      String retVal;
      byte[] bArray = db.myResult.getBytes(N);
      if (bArray.length == 0) { 
        retVal = Default;
      } else {
        String E = new String(bArray);
        if (E != null) {
          retVal = E;  
        } else {    
          retVal = Default;
        }
      }
      return retVal;
   }
   
   
   
   private void populateReceipt(ESSReceipt P) throws SQLException {
      P.RCPTTYPE = getValue("RCPTTYPE");
      P.CHARGE = getValue("CHARGE");
      P.VOUCHER = getValue("VOUCHER");
      P.RECEIPT = getValue("RECEIPT");
      P.REFER = getValue("REFER");
      P.REC_AMT = getNumberValue("REC_AMT");
      P.REC_DATE = getXDateValue("REC_DATE");
      P.AMOUNT  = getNumberValue("AMOUNT");
      P.REIMB = getBooleanValue("REIMB");
      P.FORCED = getBooleanValue("FORCED");
      P.CRE_DATE = getXDateValue("CRE_DATE");
      P.CHECK = getBooleanValue("CHECK");
      P.MERCHANT = getValue("MERCHANT");
      P.UNITS = getValue("UNITS");
      P.XSOURCE = getValue("XSOURCE");
      P.XRATE = getRateValue("XRATE");
      P.XDATE = getXDateValue("XDATE");
      P.BEG_DATE = getXDateValue("BEG_DATE");
      P.LOCATION = getValue("LOCATION");
      P.OFFSET = getValue("OFFSET");
      P.OK_STAT = getValue("OK_STAT");
      P.OK_REF = getValue("OK_REF");
   }
   private void populateExpense(ESSExpense P) throws SQLException{
      P.EXPENSE = getValue("EXPENSE");
      P.AMOUNT = getNumberValue("AMOUNT");
      P.VOUCHER = getValue("VOUCHER");
      P.RECEIPT = getValue("RECEIPT");
      P.PURPOSE = getValue("PURPOSE");
      P.LINEITEM = getValue("LINEITEM");
      P.BILLTYPE = getIntegerValue("BILLTYPE");
      P.COMMENT = getPossiblyScrambledValue("COMMENT");
      P.DEPART = getValue("DEPART");
      P.ChargedDepart = P.DEPART;  //JH 3-2-2007
      P.CRE_DATE = getXDateValue("CRE_DATE");
      P.EXP_DATE = getXDateValue("EXP_DATE");
      P.REC_AMT = getNumberValue("REC_AMT");
      P.XRATE = getRateValue("XRATE");
      P.XDATE = getXDateValue("XDATE");
      P.UNITS = getValue("UNITS");
      P.XSOURCE = getValue("XSOURCE");
      P.PERSPLIT = getValue("PERSPLIT");
      P.COMPANY = getValue("COMPANY");
      //P.category = getValue(xx);
   }
   private void populatePurpose(ESSPurpose P) throws SQLException{
      P.PURPOSE = getValue("PURPOSE");
      P.VOUCHER = getValue("VOUCHER");
      P.CLIENT = getPossiblyScrambledValue("CLIENT");
      P.LOCATION = getPossiblyScrambledValue("LOCATION");
      P.COMMENT = getPossiblyScrambledValue("COMMENT");
      P.PROJECT = getPossiblyScrambledValue("PROJECT");
      P.STEPNO = getValue("STEPNO");
      P.BEG_DATE = getXDateValue("BEG_DATE");
      P.END_DATE = getXDateValue("END_DATE");
      P.DAYS = getValue("DAYS");
      P.NIGHTS = getValue("NIGHTS");
      P.CLIENTNO = getPossiblyScrambledValue("CLIENTNO");
      P.EXTENDED = getBooleanValue("EXTENDED");
      P.TIMESLIP = getBooleanValue("TIMESLIP");
   }
   private void populateExtended(ESSExtended P) throws SQLException{
      P.VOUCHER = getValue("VOUCHER");  //What about purpose??
      P.LINEITEM = getValue("LINEITEM");
      P.NAME = getPossiblyScrambledValue("NAME");
      P.REMARKS = getPossiblyScrambledValue("REMARKS");
      P.CLIENT = getPossiblyScrambledValue("CLIENT");
      P.LOCATION = getPossiblyScrambledValue("LOCATION");
      P.COMMENT = getPossiblyScrambledValue("COMMENT");
      P.EXTRAITEM = getValue("EXTRAITEM");
   }
   public String getTrim(String x) {
      if (x == null) return "";
      return x.trim();
   }

}