/*
DB2Guideline.java - loads the guide class from the database  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/
package ess;
//import java.io.*;
//import java.util.*;
import java.text.*;
import java.sql.*;

public class DB2Guideline extends ExpenseReport {
   private String reportVoucher = "";
   private boolean reportFound = false;
   static CustomDate CD = new CustomDate();
   //private NumberFormat money = NumberFormat.getCurrencyInstance();
   //private DecimalFormat money = new DecimalFormat("0.00");
   //private DecimalFormat rate = new DecimalFormat("0.####");
   private DecimalFormat money;
   private DecimalFormat rate;
   private DecimalFormatSymbols numericRepresentation = new DecimalFormatSymbols();
   private char aPeriod = '.';
   private char aComma = ',';
   private AdisoftDbase db;
   private AdisoftDOM acctTable;
   static AuditTrail Logfile;
   
   public DB2Guideline (String voucher, String Url, String DbaseUser,String DbasePassword,
                        String SQLTerm, ServerSystemTable ST) {
     DbaseURL = Url;
     DbaseUser = DbaseUser;
     DbasePassword = DbasePassword;
     sqlTerminator = SQLTerm;

     db = new AdisoftDbase(Url,DbaseUser,DbasePassword);
     db.setSQLTerminator(sqlTerminator);
     db.setSchemaTable(ST.getSystemString("XMLSCHEMA","C:\\WORK\\"));
     
     acctTable = new AdisoftDOM();
     acctTable.setDB(Url,DbaseUser,DbasePassword);
     acctTable.setSQLTerminator(sqlTerminator);     
     setDefaults(db.getConnection(),sqlTerminator);
     set(voucher);
   }
   public DB2Guideline (String voucher, Connection dbConn, String SQLTerm,
                        ServerSystemTable ST) {
    AuditTrail.println("[000] DB2Guideline started");
     dbConnection = dbConn;
     sqlTerminator = SQLTerm;
     
     db = new AdisoftDbase();
     db.setConnection(dbConn);
     db.setSQLTerminator(sqlTerminator);
     db.setSchemaTable(ST.getSystemString("XMLSCHEMA","C:\\WORK\\"));
     
     acctTable = new AdisoftDOM();
     acctTable.setConnection(dbConn);
     acctTable.setSQLTerminator(sqlTerminator);
     setDefaults(dbConn,sqlTerminator);
     set(voucher);
   }

   public void setReportVoucher(String X) {
     reportVoucher = X;
   }
   public void set(String X) {
     setReportVoucher(X);
     set();
   } 
   public void set() {
	 	
     numericRepresentation.setDecimalSeparator(aPeriod);
	 numericRepresentation.setGroupingSeparator(aComma);
	 money = new DecimalFormat("0.00",numericRepresentation);
	 rate = new DecimalFormat("0.####",numericRepresentation);
	   
     try {
        db.setColumns("REPORT");
        if (db.setResultSet("REPORT","VOUCHER",reportVoucher)) {
           populateReport();
           db.closeMyState();
           
           db.setColumns("EXPENSE");
           if (db.setResultSet("EXPENSE","VOUCHER",reportVoucher)) {
             int sequence = 0;
             acctTable.setDOMFromTable("ACCOUNT");
             do  {
                sequence++;
               populateExpense(Integer.toString(sequence));
             } while (db.myResult.next());
             db.closeMyState();
           }
           db.setColumns("RECEIPT");
           if (db.setResultSet("RECEIPT","VOUCHER",reportVoucher)) {
                setSortBy(BY_RECEIPTS);
                rptExpenses.sort();
             do  {
                populateReceipt();
             } while (db.myResult.next());
             
             restoreSortBy();
             db.closeMyState();
           }
           db.setColumns("PURPOSE");
           if (db.setResultSet("PURPOSE","VOUCHER",reportVoucher)) {
             do  {
               populatePurpose();
             } while (db.myResult.next());
             db.closeMyState();
           }
          rptExpenses.sort();
        } 
     } catch (SQLException e) {

        AuditTrail.println("[500] DB2Guideline.set: Report not loaded SQL Exception");
        AuditTrail.println("[500]" + e.toString());        
        e.printStackTrace();
     }
     db.close();
     reportFound = true;
   }
   
   public boolean getReportFound() {
     return reportFound;
   }
   
   
   // Need to change the way that this is all done - use the schema file...
   private void populateReport() throws SQLException{
      //int PERS_NUM = 2;
      //int CRE_DATE = 3;
      //int NAME = 14;
      //int LOCATION = 15;
      //int PHONE = 16;
      //int DEPART = 17;
      //int SUB_DATE = 7;
      
      String persNum = getValue("PERS_NUM");
      String name = getValue("NAME");
      String phone = getValue("PHONE");
      String loc = getValue("LOCATION");
      String depart = getValue("DEPART");
      String currency = getValue("CURRENCY");
      java.util.Date createDate = (java.util.Date) getXDateValue("CRE_DATE");
      String xdate = getValue("SUB_DATE", "");
      java.util.Date submitDate;
      if (xdate.equals("")) {
          submitDate = createDate;  // was new java.util.Date(xdate);
      } else {
          submitDate = CustomDate.getDateFromXBase(xdate);
      }
      header = new Header(persNum, name, phone, loc, depart, createDate, " ", " ",submitDate, currency);
   }

   private java.util.Date getXDateValue (String N) throws SQLException {
      String Default = "";
      String Xdate = getValue(N, Default);
      
      return CustomDate.getDateFromXBase(Xdate);
   }

   private int getIntegerValue (String N) throws SQLException {
      String Default = "X";
      String X = getValue(N, Default);
      int Y = 0;
      
      if (!X.equals(Default)) {
         Y = java.lang.Float.valueOf(X).intValue();
      }
      return Y;
   }
   
   private double getNumberValue (String N) throws SQLException {
      String Default = "0.00";
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         try {
            X = money.format(money.parse(X));
         } catch (ParseException e) {
            X = Default;
            AuditTrail.println("[500] DB2ESS.getNumberValue - Data parsing error");
         }
      }
      return (Double.parseDouble(X));
   }

   private double getRateValue (String N) throws SQLException {
      String Default = "0.00";   //was an empty string -jh 4/15/2005
      String X = getValue(N, Default);
      if (!X.equals(Default)) {
         try {
            X = rate.format(rate.parse(X));
         } catch (ParseException e) {
            X = Default;
            AuditTrail.println("[500] DB2ESS.getRateValue - Data parsing error");
       }
      }
      return (Double.parseDouble(X));
   }
   private String getBooleanValue (String N) throws SQLException {
      String Default = "";
      String X = getValue(N, Default);
      if (X.equals("1")) {
         X = "Yes";
      } else {
         X = "No";
      }
      return X;
   }

   private String getValue (String N) throws SQLException {
      return getValue(N, "");
   } 
   
   private String getValue (String N, String Default) throws SQLException {
      //gets a value from a single column
      String E = db.myResult.getString(db.getColumnName(N));
      if (E != null) {
       return E;  
      } else {    
       return Default;
      }
   }
   
   private void populateReceipt() throws SQLException {
      //int CHARGE = 2;
      //int RECEIPT = 4;
      //int REC_DATE = 6;
      //int XRATE = 8;
      //int UNITS = 9;
      //int AMOUNT  = 12;
      //int REIMB = 15;
      
      String receipt = getValue("RECEIPT");
      double amount = getNumberValue("AMOUNT");
      java.util.Date recDate = getXDateValue("REC_DATE");
      java.util.Date begDate = getXDateValue("BEG_DATE");
      String charge = getValue("CHARGE");
      String reimb = getBooleanValue("REIMB");
      String units = getValue("UNITS");
      float xrate = (float) getRateValue("XRATE");  //Changed from get number value...
      
      //Missing comments and xref
      receipts.addElement(new Receipt(receipt, amount, units, xrate, recDate, getReceiptExpenses(receipt), "", charge,
                           reimb, begDate, ""));
   }
   private void populateExpense(String sequence) throws SQLException{
      //int EXPENSE = 1;
      //int AMOUNT = 2;
      //int RECEIPT = 4; 
      //int PURPOSE = 5;
      //int COMMENT = 15;
      //int EXP_DATE = 17;
      //int PERSPLIT = 24;
      java.util.Date expDate = getXDateValue("EXP_DATE");
      String expense = getValue("EXPENSE");
//Need expense category      
      String category = expense;
      double amount = getNumberValue("AMOUNT");
      String receipt = getValue("RECEIPT");
      int perSplit = getIntegerValue("PERSPLIT");
      String comment = getValue("COMMENT");
      String purpose = getValue("PURPOSE");
      
//Need reimberseStatus??

      rptExpenses.addElement(new Expense(receipt, sequence, expDate, expense, 
                    getExpenseCategory(expense), amount, perSplit, 
                    comment, purpose, " "));
   }
   
   private String getExpenseCategory(String expenseType)
   {
        return (acctTable.getDOMTableValueWhere("expense", getTrim(expenseType), "category"));
   }
    public String getTrim(String x) {
      if (x == null) return "";
      return x.trim();
   }
  
   private void populatePurpose() throws SQLException{
      //int PURPOSE = 1;
      //int CLIENT = 3;
      //int LOCATION = 4;
      //int COMMENT = 5;
      //int PROJECT = 6;
      //int STEPNO = 13;
      //int BEG_DATE = 7;
      //int END_DATE = 8;
      //int DAYS = 9;
      //int NIGHTS = 10;
      //int CLIENTNO = 11;
      
      String project = getValue("PROJECT");
      String stepNo = getValue("STEPNO");
      String clientNo = getValue("CLIENTNO");
      String client = getValue("CLIENT");
      String loc = getValue("LOCATION");
      String comment = getValue("COMMENT");
      java.util.Date begDate = getXDateValue("BEG_DATE");
      java.util.Date endDate = getXDateValue("END_DATE");
      int days = getIntegerValue("DAYS");
      int nights = getIntegerValue("NIGHTS");
      String purpose = getValue("PURPOSE");
      purposes.addElement(new Purpose(project, stepNo,
                clientNo, client, loc,
                comment, begDate, endDate,
                days, nights, purpose));
   }
}