/*
DBXML2ESS.java - Sets XML2ESS for a DOM that has database column name as elements (Audit) 
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.io.*;
//import java.util.*;
//import java.text.*;
//import java.sql.*;
import java.lang.reflect.Field;
import org.jdom.Element;
import org.jdom.Document;

public class DBXML2ESS extends ReportDisplayESS {
   private int ReceiptLink = 0;
   String secType = "";
   
   public DBXML2ESS () {
   }
   
  public void set(AdisoftDOM ReportDOM) {
  //compatible to the Adisoft DOM extension  
    set(ReportDOM.getDOM());
  }  
  public void set(Document ReportDOM) {
    Element report = ReportDOM.getRootElement();
    set(report, 0);
  }
  
  public void set(Element report, int Link){
    setReport(report, Link);
    normalizeWeight();
    normalizeSplitClients();
    normalizeESSReport();
    setESSGL();
  }
   
  public void setReport(Element report, int Link, boolean useNewVoucher) {
  //overrrides the method in the XML2ESS super class  
  //calls the routines to load up the objects representing the various sections of an ESS report  
    java.util.List sections = report.getChildren();
    for (int i = 0; i < sections.size(); i++) {   //these are the sections
      Element section = (Element) sections.get(i);
      java.util.List sectionList = section.getChildren();
      if (sectionList.size() > 0) {
        String secType = section.getName();
        if (secType.equals("header")) {
          if (!setESSReport(section, useNewVoucher)){
            AuditTrail.println("[400] XML2ESS.setReport: Report process aborted");
            i = sections.size();
          }    
        } else if (secType.equals("purpose")) { 
          setESSPurpose(section); 
        } else if (secType.equals("split")) { 
          //setSplitClient(section, Link); 
        } else if (secType.equals("receipt")) { 
          setESSReceipt(section);  
        } else if (secType.equals("expense")) { 
          setESSExpense(section, Link); 
        } else if (secType.equals("attendeelist")) { 
          setESSAttendeeList(section, Link);
        } else if (secType.equals("attendee")) { 
          setESSAttendee(section, Link);    
        } 
      }
    }
    normalizeESSReport();
    setESSGL();
    // db.close();
    // reportFound = true;
  }  

  public void setReport(Element report, int Link) {
    setReport(report, Link, false);
  }  
  
  private boolean setESSReport(Element E, boolean useNewVoucher) {
    //used to create a report object.
    
    setElements(E,Rep);
    Rep.set();
    setAccountingInfo();
    //Rep.RE_AMT = getReimburse();
    return true;
  } 
 
  private void setESSPurpose(Element E) {
    //used to create a Purpose object.
    ESSPurpose P = new ESSPurpose();
    P.set();  //look at a better modularization and the order
    setElements(E,P);
    P.PurposeLink = Integer.parseInt(P.PURPOSE.substring(P.PURPOSE.length()-7));
    ESSPurposes.addElement(P);
    setReport(E, P.PurposeLink);
  } 
   
  private void setESSReceipt(Element E) {
    //used to create a receipt object.
    ESSReceipt P = new ESSReceipt();
    //P.ExpenseLink = genReference++;
    P.set();  //look at a better modularization 
    setElements(E,P);
    P.normalizeType();  //Jim Holton 11-10-2004
    ESSReceipts.addElement(P);
    setReport(E, P.ExpenseLink); 
  } 
  
  private void setESSExpense(Element E, int ReceiptLink) {
    //used to create an expense object.
    ESSExpense P = new ESSExpense();
    P.set(ReceiptLink);  //look at a better modularization 
    setElements(E,P);
    ESSExpenses.addElement(P);
    setReport(E, P.LineLink);
  }   
  
  protected void setESSAttendeeList (Element E, int ListLink) {
    //used to create an AttendeeList.
    ESSAttendeeList P = new ESSAttendeeList();
    //P.set(E, ListLink);  //JH 11/30/2005 - removed 12/1/05
    setElements(E,P);
    P.normalize(ListLink);
    ESSAttendeeLists.addElement(P);
    setReport(E, ListLink);
  }  
   
  private void setESSAttendee (Element E, int Link) {
    //used to create an Attendee.
    ESSExtended P = new ESSExtended();
    //P.set(E, Link);  //JH 11/30/2005 
    //LINEITEM = getESSReferenceString(Link); added as comment JH 11/30/2005
    setElements(E,P);
    P.normalize(Link);
    ESSExtendeds.addElement(P);
        
    //setReport(E, Link);
  }    
   
  private void setElements(Element CurrentElement, Object obj) {
    String name;
    String value;
    if (CurrentElement != null && (CurrentElement.getContentSize() > 0)) {
      java.util.List elementList = CurrentElement.getChildren();
      for (int i = 0; i < elementList.size(); i++) {
        Element e = (Element) elementList.get(i);
        name = e.getName();
        value = e.getText().trim();
        setField(obj, name, value);
      }
    }
  }
  
  private void setField(Object obj, String name, String value) {
    Field f;
    Class c = obj.getClass();
    try {
        f = c.getField(name);
        f.set(obj, value);
    } catch (NoSuchFieldException e) {
        AuditTrail.println("[400] DBXMLESS.setField NoSuchFieldException "  + name + " = " + value);
    } catch (IllegalAccessException e) {
        AuditTrail.println("[500] DBXMLESS.setField IllegalAccessException "  + name + " = " + value);
    }
  }
  
  private String getStringField(Object obj, String name) {
    Field f;
    String x = "";
    Class c = obj.getClass();
    try {
        f = c.getField(name);
        x = (String) f.get(obj);
    } catch (NoSuchFieldException e) {
        AuditTrail.println("[400] DBXMLESS.getStringField NoSuchFieldException " + name);
    } catch (IllegalAccessException e) {
        AuditTrail.println("[500] DBXMLESS.getStringField IllegalAccessException " + name);
    }
    return x;
  }
  
  private String[] getStringFieldArray(Object obj, String[] name)
  {
    String[] X = new String[name.length];
    for (int i = 0; i < name.length; i++)
    {
        X[i] = getStringField(obj, name[i]);
    }
    return X;
  }
  
  
  public boolean setUpdate()
  {
    ReportElementAPI X = (ReportElementAPI) Rep;
    boolean updateStatus = setUpdate(X, "REPORT");
    
    if (updateStatus == true)
    {
        int i = 0;
        while ((i < ESSPurposes.size()) && (updateStatus == true))
        {
            X = (ReportElementAPI) ESSPurposes.elementAt(i);
            updateStatus = setUpdate(X, "PURPOSE");
            i++;
        }
        
        i = 0;
        while ((i < ESSReceipts.size()) && (updateStatus == true))
        {
            X = (ReportElementAPI) ESSReceipts.elementAt(i);
            updateStatus = setUpdate(X, "RECEIPT");
            i++;
        }
        
        i = 0;
        while ((i < ESSExpenses.size()) && (updateStatus == true))
        {
            X = (ReportElementAPI) ESSExpenses.elementAt(i);
            updateStatus = setUpdate(X, "EXPENSE");
            i++;
        }
        
       
        i = 0;
        while ((i < ESSExtendeds.size()) && (updateStatus == true))
        {
            X = (ReportElementAPI) ESSExtendeds.elementAt(i);
            updateStatus = setUpdate(X, "EXTENDED");
            i++;
        } 
       
    }
    return updateStatus;
  }
  
  private boolean setUpdate(ReportElementAPI X, String Table)
  {
    boolean updateStatus = true;
    String persistanceType = getStringField(X,"persistance");
    
    if (!persistanceType.equals("")) 
    {
      if (persistanceType.equalsIgnoreCase("update"))
      {
         String[] keyList = db.getPhysicalNames(Table,X.getKeyList());
         String[] tagList = db.getPhysicalNames(Table,X.getUpdateList());
         String[] logicalFields = db.getPhysicalNames(Table,X.getLogicalFields());
         String[] numericFields = db.getPhysicalNames(Table,X.getNumericFields());
         String[] dateFields = db.getPhysicalNames(Table,X.getDateFields());
         String[] encryptFields = db.getPhysicalNames(Table,X.getEncryptFields());
         String[] valueList = this.getStringFieldArray(X,tagList);
         String[] keyValues = this.getStringFieldArray(X, keyList);
         updateStatus = db.setUpdate("SIMPLESQL", Table, keyList, tagList, logicalFields, 
                         numericFields, dateFields, encryptFields, valueList, keyValues);

      } else if (persistanceType.equalsIgnoreCase("insert"))
      {
         updateStatus = X.setPersistance();  // want to move to db eventually
         
      } else if (persistanceType.equalsIgnoreCase("delete"))
      {
         String[] keyList = db.getPhysicalNames(Table,X.getKeyList());
         String[] tagList = db.getPhysicalNames(Table,X.getUpdateList());
         String[] logicalFields = db.getPhysicalNames(Table,X.getLogicalFields());
         String[] numericFields = db.getPhysicalNames(Table,X.getNumericFields());
         String[] dateFields = db.getPhysicalNames(Table,X.getDateFields());
         String[] encryptFields = db.getPhysicalNames(Table,X.getEncryptFields());
         String[] valueList = this.getStringFieldArray(X,tagList);
         String[] keyValues = this.getStringFieldArray(X, keyList);
         updateStatus = db.setDelete("SIMPLESQL", Table, keyList, tagList, logicalFields, 
                         numericFields, dateFields, encryptFields, valueList, keyValues);

      }
    }                   
  
    return updateStatus;
  }
  
}