/*
DepartmentTable.java - Access to department table
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.net.URL;
import java.sql.*;

//
//  This object extends the Dbase class to provide enhancements for reading and writing
//  to the SYSTEM table which contains setup and control parameters
//

public class DepartmentTable {

    private Statement myState = null;                          // every instance gets a pair of these
    private ResultSet myResult;

    public String depart = "";
    public String descrip = "";
    public String gl_term = "";
    public String occode = "";
    public String company = "";
    public String loc_code = "";
    public String superDepart = "";
    public String manager = "";
    public String admin = "";
    public String guide = "";
    
    public String activeCompany;
    public String currency = ""; 
    public String mileageUnits = "";
    public String auditor = "";
    public String companyName = "";
    
    private String SQL_Find = "SELECT * FROM DEPART WHERE UCASE(DEPART) = '$depart$' ORDER BY DEPART;";
 
    static AuditTrail Logfile;
    private AdisoftDbase DbaseObject = new AdisoftDbase();     // all instances share these
    // private AdisoftHash AH = new AdisoftHash();
    // private DepartmentTable department;
    private Connection dbConn;
    protected String sqlTerminator = ";";
    
    public DepartmentTable() {
      Logfile = new AuditTrail();
    }
/*
    public DepartmentTable(AuditTrail CurrentLog) {
      Logfile = CurrentLog;
    }
    public DepartmentTable(String depart, String Url, String DBUser, String DBPassword, String DBSQLTerminator){
        setDB(Url, DBUser, DBPassword, DBSQLTerminator);
        setDepartInfo(depart);
    }
    
    public DepartmentTable(String depart, Connection x){
        setConnection(x);
        setDepartInfo(depart);
    }
*/    
    public void setDB(String Url, String DBUser, String DBPassword, String DBSQLTerminator){
        AuditTrail.println("[000] DepartmentTable.setDB for: " + Url);
        DbaseObject.setDB(Url, DBUser, DBPassword);
        DbaseObject.setSQLTerminator(DBSQLTerminator);
    }
    
    public void setConnection(Connection x){
        AuditTrail.println("[000] DepartmentTable.setConnection");
        dbConn = x;
        DbaseObject.setConnection(x);
    }
    public void setSQLTerminator(String X) {
        sqlTerminator = X;
    }
    public void setSQLStrings(String x)
    {
        SQL_Find = x;
    }
    
    public void setSQLStrings() {
        //rewrite this for to do the SysTable SQL from this object
        ServerSystemTable SysTable = new ServerSystemTable();
        AdisoftDOM SystemDOM = new AdisoftDOM();
        SysTable.setConnection(dbConn);
        SysTable.setSQLTerminator(sqlTerminator);
        String system_file = SysTable.getSystemString("XMLSYSTEM","C:\\WORK\\"); 
        java.io.File SystemFile = new java.io.File(system_file);
        SystemDOM.setDOM(SystemFile); 
        AuditTrail.println("[000] DepartmentTable.setSQLStrings from " + system_file);
        SQL_Find = SystemDOM.getDOMTableValueFor("departtable","departinfosql");
    }    
        
    public void closeMyState()
    {
        if (myState != null)
        {
            try
            {
                myState.close();
                myState = null;
            }
            catch(Exception e)
            {
                AuditTrail.println("[500] DepartmentTable.closeMyState() close failure");
            }
        }
        
    }
    public void close() {
        try {
           //Logfile.println("[000] DepartmentTable.close");
           closeMyState();
           //DbaseObject.dbCon.close();
           DbaseObject.close(); 
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] DepartmentTable.close DB close failure");
        }
    }
 
    public ResultSet getResult() {
      return myResult;
    }
    

    public boolean setDepartInfo(String searchDepart) {  //department name
        AuditTrail.println("[400] DepartmentTable.setDepartInfo with no company");
    	return setDepartInfo(searchDepart, "");
    }
    
    public boolean setDepartInfo(String searchDepart, String searchCompany) {  //department name

        boolean exitStatus = true;
        String SQLCommand = "not set";
        
        if (depart.toUpperCase().equals(searchDepart.toUpperCase())) {
            //Logfile.println("[000] DepartmentTable.setDepartInfo: Session reuse " + searchDepart);
            return true;
        } else {
          try {
// May want to check for a blank department
            myState = DbaseObject.dbCon.createStatement();
            SQLCommand = DbaseObject.SQLReplace(SQL_Find,"$depart$",searchDepart.toUpperCase()); 
            SQLCommand = DbaseObject.SQLReplace(SQLCommand,"$company$",searchCompany.toUpperCase()); 
            myResult = myState.executeQuery(SQLCommand);
            if (myResult != null)
            { 
                if (myResult.next()) {   //change below later to column locations to generalize
                    depart = getTrim(myResult.getString("DEPART"));
                    descrip = getTrim(myResult.getString("DESCRIP"));
                    gl_term = getTrim(myResult.getString("GL_TERM"));
                    company = getTrim(myResult.getString("COMPANY"));
                    loc_code = getTrim(myResult.getString("LOC_CODE"));
                    superDepart = getTrim(myResult.getString("SUPER"));
                    manager = getTrim(myResult.getString("MANAGER"));
                    admin = getTrim(myResult.getString("CLERK"));
                    guide = getTrim(myResult.getString("GUIDE"));
                    setCompanyInheritance(company);
                    //Logfile.println("[000] DepartmentTable.setDepartInfo: Found department " + searchDepart);
                } else {
                	AuditTrail.println("[400] DepartmentTable.setDepartInfo: " + SQLCommand);
                    AuditTrail.println("[400] DepartmentTable.setDepartInfo: Invalid depart: " + searchDepart);
                    exitStatus = false;
                }
                
                closeMyState();  
                return (exitStatus);
            }
            else
            {
                    AuditTrail.println("[400] DepartmentTable.setDepartInfo: myResult = null");
                    return false;
            }

          } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] DepartmentTable.setDepartInfo: Error finding " + searchDepart);
            AuditTrail.println("[500] DepartmentTable.setDepartInfo: " + SQLCommand);
            return false;
          }
        }
    }
    
    /*  
    public boolean setDepartInfoAll(String SearchKey) {
        boolean exitStatus = true;
        // can replace this with AdisoftDbase usage if I want - JH
        try {

            myState = DbaseObject.dbCon.createStatement();
            myResult = myState.executeQuery("SELECT * FROM DEPART WHERE DEPART = '" + SearchKey + "'" );
            
            if (myResult.next()) {
                AuditTrail.println("[200] DepartmentTable.setDepartInfoAll: Loaded results for " + SearchKey);
            } else {
                AuditTrail.println("[400] DepartmentTable.setDepartInfoAll: Invalid key " + SearchKey);
                exitStatus = false;
            }
            closeMyState();
            return(exitStatus);

        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] DepartmentTable.setDepartInfoAll: Error finding " + SearchKey);
            return false;
        }
    }
*/
    public void resetProfile() {
        depart = "";
        descrip = "";
        gl_term = "";
        occode = "";
        company = "";
        loc_code = "";
        superDepart = "";
        manager = "";
        admin = "";
        guide = "";
        currency = "";
        mileageUnits = "";
    }    
    
    public boolean equals(DepartmentTable other)
    {
        return(depart.equals(other.depart));
    }
    
    public String getCompany()
    {
        return(company);
    }
    
    public String getDepartmentName()
    {
        return(descrip);
    }

    public String getCompanyName()
    {
    	return(companyName);
    }
    public String getManager()
    {
        return(manager);
    }
    
    public String getAdmin()
    {
        return(admin);
    }
    
    public String getSuperDepartment()
    {
        return(superDepart);
    }
    
    public String getTrim(String x) {
      if (x == null) return "";
      return x.trim();
    }

    public String getCurrency()
    {
        return(currency);
    }
    public String getMileageUnits()
    {
        return(mileageUnits);
    }

    public String getAuditor()
    {
    	return auditor;
    }
    
    private void setCompanyInheritance(String activeCompany) 
    {
       String CompanySelect = "COMPANY = '" + activeCompany + "'"; //Later can do some tricky things with OR
       DbaseObject.setResultSet("SELECT CURRENCY, MILEAGE, AUDITOR, DESCRIP FROM COMPANY WHERE " + CompanySelect);

       try {
         currency = DbaseObject.myResult.getString(1); 
         mileageUnits = DbaseObject.myResult.getString(2);
         auditor = DbaseObject.myResult.getString(3);
         companyName = DbaseObject.myResult.getString(4);
       } catch (Exception e) {
           e.printStackTrace();
           AuditTrail.println("[500] Report.setCompanyInheritance IO error: " + e.getMessage());
           currency = "";   //ssssss Need to change this JH 2008-08-15
           mileageUnits = "";
           auditor = "";
           companyName = "";
       }
    }
}