/*
EODSave.java - Runs the EOD jobs to produce various output files
Copyright (C) 2005 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


/*Note: think about using the Status table for stati onther than report - for
example if this process has finished OK.  May can use the system.xml instead
*/

package ess;

import java.util.*;
import java.sql.*;

public class EODSave implements Runnable {
    // This class is used to instantiate feeds in the EOD process
  
  static AuditTrail Logfile;
  private CustomDate Dt = new CustomDate();
  private PersonnelTable PersFile;
  private AdisoftDOM SystemDOM = new AdisoftDOM();
  private ServerSystemTable SysTable = new ServerSystemTable();
  private PersonnelTable Reporter = new PersonnelTable();
  private AdisoftDOM StatusDOM = new AdisoftDOM();
  private AdisoftDOM DepartDOM = new AdisoftDOM();
  private AdisoftDbase Reg = new AdisoftDbase();
  
  private StringTokenizer rp;
  private StringTokenizer st;
  
  private AdisoftDbConn DbConn;
  private String sqlTerminator = ";";
  private String emailAddress = "";
  
  
  static Boolean ProcessRunning = new Boolean(false);

  public boolean getAlreadyRunning()
  {
    synchronized(ProcessRunning) {
        boolean retVal = true;
        if (!ProcessRunning.booleanValue())
        {
            ProcessRunning = Boolean.valueOf("true");
            retVal = false;
        }
        return retVal;
    }
  }
  
  public void setProcessNotRunning()
  {
    ProcessRunning = Boolean.valueOf("false");
  }

  public void setConnection(AdisoftDbConn x)
  {
    DbConn = x;
    SysTable.setConnection(DbConn.getConnection());   
    DepartDOM.setConnection(DbConn.getConnection());
    Reporter.setConnection(DbConn.getConnection());
    Reg.setConnection(DbConn.getConnection()); 
  }

  public void setEmailAddress(String x)
  {
    emailAddress = x;
  }

  public void setVoucherList(StringTokenizer x)
  {
    rp = x;
  }
  public void setStatusList(StringTokenizer x)
  {
    st = x;
  }

  public void setSQLTerminator(String x)
  {
    sqlTerminator = x;
    SysTable.setSQLTerminator(sqlTerminator);
    DepartDOM.setSQLTerminator(sqlTerminator);
    Reporter.setSQLTerminator(sqlTerminator); 
    Reg.setSQLTerminator(sqlTerminator);
  }

  public void run() {
    
    AuditTrail.println("[000] EODSave.java started");

    String report2Approve;
    String report2AppStat;
    String persnum;
    String persname;
    boolean xFlag;
    String newNewStatusCode;

    int voucherNumber = 0;
    
    SystemDOM.setSQLTerminator(sqlTerminator);  //Do we need this?/also next paragraph? JH 2006-03-13
    
    String system_file = SysTable.getSystemString("XMLSYSTEM","C:\\WORK\\"); 
    java.io.File SystemFile = new java.io.File(system_file);
    SystemDOM.setDOM(SystemFile);

    String status_file = SysTable.getSystemString("XMLSTATUS","C:\\WORK\\"); 
    java.io.File StatusFile = new java.io.File(status_file);
    StatusDOM.setDOM(StatusFile); 
    
    DepartDOM.setDOMFromTable("DEPART"); 
    Reporter.setSQLStrings();
        
    String newAsOfDate = Dt.xBaseDate.format(Dt.date);
    String SQLDate = SystemDOM.getDOMTableValueFor("sql","datesql");
    String SQLType = SystemDOM.getDOMTableValueFor("sql","dateformat");
    String executeCommand = SystemDOM.getDOMTableValueFor("endofday","executecommand");
    if (SQLType.equals("MM/DD/YYYY")) { 
      SQLDate = Reg.SQLReplace(SQLDate,"$date$",Dt.simpleDate.format(Dt.date));
    } else if (SQLType.equalsIgnoreCase("DD-MMM-YYYY")){    // oracle
      SQLDate = Reg.SQLReplace(SQLDate,"$date$",Dt.getOracleDate(Dt.date));
    } else { //generate YYYY-MM-DD
      SQLDate = Reg.SQLReplace(SQLDate,"$date$",newAsOfDate);
    } 

    String encryptKeyStr = SystemDOM.getDOMTableValueFor("encrypt","key","15");
    int encryptKey = Integer.parseInt(encryptKeyStr);

   
    boolean actionFlag;
    ess.FeedAPI F;
   
    String[] feedNames = SystemDOM.getDOMTableArrayFor("endofday","feed");
    java.util.Vector feeds = new java.util.Vector();
    int NumberOfFeeds = feedNames.length;
   
    if (NumberOfFeeds > 0){
      actionFlag = true;
      for (int i = 0; i < NumberOfFeeds; i++) {
        try {
           F = (ess.FeedAPI) (Class.forName(feedNames[i])).newInstance();
           F.init(SystemDOM,DbConn.getConnection()); 
           feeds.add(F);
        } catch (java.lang.ClassNotFoundException e) {
            AuditTrail.println("[500] EODSave.java Class Not Found: " + feedNames[i]);
            AuditTrail.println("[500] EODSave.java: " + e.toString());
            e.printStackTrace();
            setEmailMessage("Expense Report EOD Error","EOD Process error - Class not found: " + feedNames[i]);
        } catch (java.lang.NoClassDefFoundError e) {
            AuditTrail.println("[500] EODSave.java No class def: " + feedNames[i]);
            AuditTrail.println("[500] EODSave.java: " + e.toString());
            e.printStackTrace();
            setEmailMessage("Expense Report EOD Error","EOD Process error - Class not found: " + feedNames[i]);
        } catch (java.lang.InstantiationException e) {
            AuditTrail.println("[500] EODSave.java Instantiation Error: " + feedNames[i]);
            AuditTrail.println("[500] EODSave.java: " + e.toString());
            e.printStackTrace();
            setEmailMessage("Expense Report EOD Error","EOD Process error - Instantiation Error: " + feedNames[i]);
        } catch (java.lang.IllegalAccessException e) {
            AuditTrail.println("[500] EODSave.java Illegal Access: " + feedNames[i]);
            AuditTrail.println("[500] EODSave.java: " + e.toString());
            e.printStackTrace();
            setEmailMessage("Expense Report EOD Error","EOD Process error - Illegal Access: " + feedNames[i]);
        }
      }
    } else {
      actionFlag = false;
      AuditTrail.println("[500] EODSave.java - no feed objects found");
      setEmailMessage("Expense Report EOD No files created","No feed objects were found. No feeds produced.");
    }
   // Might want to consider treating the delimiters as tokens??
    while (rp.hasMoreTokens() && actionFlag) {
      report2Approve = rp.nextToken().trim() ;
      AuditTrail.println("[303] EODSave.java starting: " + report2Approve);
      report2AppStat = st.nextToken().trim() ;
      voucherNumber = voucherNumber + 1;
     
      AuditTrail.println("[304] EODSave.java status: " + report2AppStat);
      ess.DB2ESS Rep2 = new ess.DB2ESS();
      Rep2.setConnection(DbConn.getConnection(),sqlTerminator); 
      Rep2.setDenormalizeClient(true);

      Rep2.set(report2Approve);
      if (Rep2.getReportFound()) {
        persnum = Rep2.getPersNum();  //personnel number
        persname = Rep2.getReportName(); //report name

        xFlag = Reporter.setPersNumInfo(persnum); 
        
        if (xFlag ) {   
          for (int i = 0; i < NumberOfFeeds; i++) {
             F = (ess.FeedAPI) feeds.get(i);
             F.post(Reporter, (ess.Report) Rep2);
             // Need some kind of status checking after this
          }  

        } else {  
               setEmailMessage("Expense Report EOD Error", "Feed Generation: Cannot generate info for " + report2Approve + " - reporter not found");
               AuditTrail.println("[500] EODSave.java - Cannot generate info for " + report2Approve + " - reporter not found"); 
              
        }  //Done with the find reporter logic 
      } else {  
          setEmailMessage("Expense Report EOD Error", "Feed Generation: Cannot generate info for " + report2Approve + " - cannot load the report");
          AuditTrail.println("[500] EODSave.java - Cannot read report " + report2Approve + " - possible bad data"); 
      } //If the SQL select is OK
    } //While the tokens are still there

    for (int i = 0; i < NumberOfFeeds; i++) {
      F = (ess.FeedAPI) feeds.get(i);
      F.close();
      // Need some kind of status checking after this
      
    }
    
    ProcessRunning = Boolean.valueOf("false");
    setEmailMessage("Expense Report EOD Finished","Feed creation process has finished.");
    try {
	    // Execute a command with an argument that contains a space
	    Process child = Runtime.getRuntime().exec(executeCommand);
    	AuditTrail.println("[000] EODSave.java execute has been launched: " +  executeCommand );
	    java.io.BufferedReader input = new java.io.BufferedReader(new java.io.InputStreamReader(child.getInputStream()));
        // Need to close this inputstream ^^^^^^^?????
        String line=null;

        while((line=input.readLine()) != null) {
        	AuditTrail.println("[210] EODSave.java execute command return: " + line);
        }
        int exitVal = child.waitFor();
        AuditTrail.println("[211] EODSave.java execute command exit code: " + exitVal);
  	} catch (java.io.IOException e) {
		AuditTrail.println("[500] EODSave.java execute command io failure: " +  executeCommand );
		AuditTrail.println("[500] EODSave.java execute command io failure: " +  e.toString());
        e.printStackTrace();
  	} catch (java.lang.InterruptedException e) {
		AuditTrail.println("[500] EODSave.java execute command interrupted: " +  executeCommand );
		AuditTrail.println("[500] EODSave.java execute command interrupted: " +  e.toString());
        e.printStackTrace();
    } catch(Exception e) {
		AuditTrail.println("[500] EODSave.java execute command failed: " +  executeCommand );
		AuditTrail.println("[500] EODSave.java execute command failed: " +  e.toString());
        e.printStackTrace();
	}


    AuditTrail.println("[282] EODSave.java finished");
  }
  
  public void setEmailMessage( String sujet, String emailMessage)
  
  {
    String from = SysTable.getSystemString("PAL_EMAIL_ADDRESS","expense@adisoft-inc.com");
    String smtpServer = SysTable.getSystemString("SMTP_ADDRESS","207.0.0.1");
    String smtpPort = SysTable.getSystemString("SMTP_PORT","25");
    String smtpDomain = SysTable.getSystemString("SMTP_DOMAIN","adisoft-inc.com");
    String smtpDelimiter = SysTable.getSystemString("SMTP_DELIMITER",".");
        
    SMTPSingle smtp = new SMTPSingle();
    smtp.setSubject(sujet);
    smtp.setIP_URL(smtpServer);
    smtp.setPort(smtpPort);
    smtp.setDomain(smtpDomain);
    smtp.setDelimiter(smtpDelimiter);
    
    if (!smtp.setSMTPMessage (emailMessage, emailAddress, from)) {
    	AuditTrail.println("[400] EODSave.java - Email notification not sent to: " + emailAddress);
    }
  }
}