/*
ESSRules.java - Handles Rules.xml file  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;
//import java.io.*;
import java.util.*;
import java.sql.*;
import org.jdom.Document; //See if I can find this
import org.jdom.Element;

public class ESSRules
{
//ESS Expense Report ERRORS
  final int GOOD_REPORT = 0;
  final int REPORT_ALREADY_PROCESSED = 1;
  final int REPORT_DOES_NOT_EXIST = 2;
  final int BAD_REPORT_CONTENTS = 3;
    
  final int PERS_NUM_NOT_FOUND = 4;
  final int INVALID_CHARGE_TYPE = 5;
  final int INVALID_EXPENSE_TYPE = 6;
  final int FAILED_GUIDELINE = 7;
  final int PERSISTANCE_FAILED = 8;
  
  String[] conditions= {"GOOD_REPORT", "REPORT_ALREADY_PROCESSED", "REPORT_DOES_NOT_EXIST", "BAD_REPORT_CONTENTS",
        "PERS_NUM_NOT_FOUND", "INVALID_CHARGE_TYPE", "INVALID_EXPENSE_TYPE", "FAILED_GUIDELINE",
        "PERSISTANCE_FAILED"};
        
    static AuditTrail Logfile;
    String DBUrl = "";
    String DBUser = "";
    String DBPassword = "";
   private ServerSystemTable systemTable;
//   Vector notifiers = new Vector();
    private SortVector notifiers = new SortVector(new compareNotifiers());
  
    public ESSRules(String DBUrl, String DBUser, String DBPassword, String DBSQLTerminator)
    {
        this.DBUrl = DBUrl;
        this.DBUser = DBUser;
        this.DBPassword = DBPassword;
        systemTable = new ServerSystemTable(DBUrl, DBUser, DBPassword);
        systemTable.setSQLTerminator(DBSQLTerminator);
    }
    
    public ESSRules(Connection dbConn, String DBSQLTerminator)
    {
        systemTable = new ServerSystemTable();
        systemTable.setConnection(dbConn);
        systemTable.setSQLTerminator(DBSQLTerminator);
    }

    public void close() {
        systemTable.close();
    }

    public String getCondString(int code)
    {
        return(code < conditions.length ? conditions[code] : "CONDITION_NOT_FOUND");
    }
    
  public Vector setRoutingObjects(String routingName, Vector conditions)
  {
    
        String rulesFile = systemTable.getSystemString("XMLRULES","c:/visualcafe/projects/adisoft/rules.xml");
        AdisoftDOM rulesTable = new AdisoftDOM(rulesFile);
        Document rulesDOM = rulesTable.getDOM();
        Element root = rulesDOM.getRootElement();
        Element routingEl;
        Iterator notifyItr;

        notifiers.removeAllElements();
 
        if ((root.getName()).equals("rules") )
        {
            Iterator routingItr = root.getChildren().iterator();
//find the routing rules for the expense report                
            while (routingItr.hasNext())
            {
                routingEl = (Element) routingItr.next();
                if ((routingEl.getChildText("name")).equals (routingName) == true)
                {
                    notifyItr = routingEl.getChildren("notification").iterator();
                            
                    while (notifyItr.hasNext())
                    {
                        Element notifyEl = (Element) notifyItr.next();
                        if (Utilities.findString(notifyEl.getChildText("condition"), conditions) != -1)                       
                            notifiers.addElement(new Notifier(notifyEl));
                    }
                }

            }    
            
            notifiers.sort();
        }
        
    return((Vector) notifiers);    
  }
  
    public String getCondition(Object routing)
    {
        return(((Notifier) routing).condition);
    }
    
    public String getRoutingObject(Object routing)
    {
        return(((Notifier) routing).routingObject);
    }
  
    public String getMessage(Object routing)
    {
        return(((Notifier) routing).message);
    }
    
    public String getPropertyMessage(Object routing)
    {
        return(((Notifier) routing).propertyMessage);
    }

    
  class Notifier
  {
    public String condition = "";
    public String routingObject = "";
    public String message = "";
    public String propertyMessage = "";
        
    public Notifier(Element notifier)
    {
        condition = notifier.getChildText("condition");
        routingObject = notifier.getChildText("routingObject");
        message = notifier.getChildText("message");
        propertyMessage = notifier.getChildText("propertyMessage");
    }

    public int compareTo(Notifier other)
    {
        return(this.routingObject.compareTo(other.routingObject));
    }
    
    public String toString()
    {
        return(condition + ", " + routingObject + ", " + message + "\n");
    }
    
  }
  
    class compareNotifiers implements Compare{
        public boolean lessThan(Object left, Object right) {
                return (((Notifier) left).compareTo((Notifier) right) < 0);
        }
        public boolean lessThanOrEqual(Object left, Object right) {
                return (((Notifier) left).compareTo((Notifier) right) <= 0);
        }
    }
        
  
}
