/*
EditTables.java - tools for generating arrays used in ESSProfile, etc.   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.text.DecimalFormatSymbols;

import org.jdom.Element;
import org.jdom.Document;

public class EditTables extends AdisoftDbase {
  
  static AuditTrail AuditLog;    
  private ess.Language myLang = new ess.Language();
  
  Utilities utilities = new Utilities();

  private String EditList;  //used to return results of Required, Default, Single and Double
  
  public EditTables () {
  }
    
  public void setLanguage(String x) {
	  myLang.resetLanguage(x);
  }
  
  public String getEditList() {
    return EditList;
  }
  
  public String getSingle(String SQLStatement) {
    setSingle(SQLStatement);
    return EditList;
  }
  public String getSingle(String SQLStatement, boolean addBlank) {
    setSingle(SQLStatement, addBlank);
    return EditList;
  }
  
  public String getSingleNoQuotes(String SQLStatement) {
    setSingle(SQLStatement,"");
    return EditList;
  }
 
  public String getDefault(String SQLStatement) {
    setDefault(SQLStatement);
    return EditList;
  }
  public String getRequired(String SQLStatement) {
    setRequired(SQLStatement);
    return EditList;
  }
  public String getNameValue(String SQLStatement) {
    setNameValue(SQLStatement);
    return EditList;
  }
  public String getNameValue(String SQLStatement, boolean addBlank) {
    setNameValue(SQLStatement, addBlank);
    return EditList;
  }
  
  public String getNameAndTranslation (String SQLStatement) {
	  setNameAndTranslation(SQLStatement, true);
	  return EditList;
  }
  private String fixEditString(String x) {
    if (x != null) {
      return Utilities.rightTrim(x.replace('\"','\''));
    } else {
      return "";
    }
  }
  
  public void setDefault (String SQLStatement) {
    String Q = "\"";
    try {
      if (setResultSet(SQLStatement)) {
        EditList = Q + fixEditString(myResult.getString(1)) + Q;
      } else {
        EditList = "\"\"";     
      }
    } catch (java.sql.SQLException e) {
      e.printStackTrace();
      AuditTrail.println("[500] EditTables.setDefault - Bad (getResultSet): " + SQLStatement);
      EditList = "";
    }
  } 
  
  public String getUnique(String SQLStatement)
  {
	  setUnique(SQLStatement);
	  return EditList;
  }
  
  public void setUnique (String SQLStatement) {  //future, combine portions of the next two
	    try {
	      if (setResultSet(SQLStatement)) {
	          EditList = fixEditString(myResult.getString(1));
	      } else {
	          EditList = "";     
	      }
	    } catch (java.sql.SQLException e) {
	      e.printStackTrace();
	      AuditTrail.println("[500] EditTables.setUnique - Bad (getResultSet): " + SQLStatement);
	      EditList = "";
	    }
  }  
  
  public void setRequired (String SQLStatement) {
    try {
      if (setResultSet(SQLStatement)) {

        String D = ";";
        String Q = "\"";
        EditList = Q + D; 
        do {
          EditList += fixEditString(myResult.getString(1)) + D;
        } while (myResult.next());
        EditList += Q;
      } else {
        EditList = "\"\"";     
      }
    } catch (java.sql.SQLException e) {
      e.printStackTrace();
      AuditTrail.println("[500] EditTables.setSingle - Bad (getResultSet): " + SQLStatement);
      EditList = "";
    }
  } 
  
  public void setSingle (String SQLStatement, boolean addBlank) {
    if (addBlank) {
      setSingle (SQLStatement);
    } else {
      setSingleNoBlank (SQLStatement);
    }
  }  
  
  public void setSingle (String SQLStatement)
  {
    String Q = "\"";
    setSingle (SQLStatement, Q);
  }
  
  public void setSingle (String SQLStatement, String Q) {  //future, combine portions of the next two
    String D = ",";
    try {
      if (setResultSet(SQLStatement)) {
        EditList = "[" + Q + Q; 
        do {
          EditList += D + Q + fixEditString(myResult.getString(1)) + Q;
        } while (myResult.next());
        EditList += "]";
      } else {
        EditList = "[" + Q + Q + "]";     
      }
    } catch (java.sql.SQLException e) {
      e.printStackTrace();
      AuditTrail.println("[500] EditTables.setSingle - Bad (getResultSet): " + SQLStatement);
      EditList = "[" + Q + Q + "]";
    }
  }  
  public void setSingleNoBlank (String SQLStatement) {
  //deprecated - used setSingle alone.
    try {
      if (setResultSet(SQLStatement)) {
        EditList = "["; 
        String D = "";
        String Q = "\"";
        do {
          EditList += D + Q + fixEditString(myResult.getString(1)) + Q;
          D = ",";
        } while (myResult.next());
        EditList += "]";
      } else {
        EditList = "[]";           // was -> "\"\""; 
      }
    } catch (java.sql.SQLException e) {
      e.printStackTrace();
      AuditTrail.println("[500] EditTables.setSingleNoBreak - Bad (getResultSet): " + SQLStatement);
      EditList = "[]";
    }
  }
  public void setNameValue (String SQLStatement) {
    setNameValue (SQLStatement, true);
  }  
  
  public void setNameValue (String SQLStatement, boolean addBlank) {
    try {
      if (setResultSet(SQLStatement)) {
        String O = "[";
        String C = "]";
        String D;
        String D1 = ",";
        String Q = "\"";
        if (addBlank) {
          D = ",";  
          EditList = O + O + Q + Q + D + Q + Q + C;
        } else {
          D = "";
          EditList = O;
        }
        do {
          EditList += D + O + Q + fixEditString(myResult.getString(1)) + Q + D1 + Q + fixEditString(myResult.getString(2)) + Q + C;
          D = ",";
        } while (myResult.next());
        EditList += C;
      } else {
        EditList = "[]";     
      }
    } catch (java.sql.SQLException e) {
      e.printStackTrace();
      AuditTrail.println("[500] EditTables.setSingle - Bad (getResultSet): " + SQLStatement);
      EditList = "[]";
    }
  } 
  
 
  
  public void setNameAndTranslation (String SQLStatement, boolean addBlank) {
	    try {
	      if (setResultSet(SQLStatement)) {
	        String O = "[";
	        String C = "]";
	        String D;
	        String D1 = ",";
	        String Q = "\"";
	        String column1;
	        if (addBlank) {
	          D = ",";  
	          EditList = O + O + Q + Q + D + Q + Q + C;
	        } else {
	          D = "";
	          EditList = O;
	        }
	        do {
	        	column1 = myResult.getString(1);
	          EditList += D + O + Q + fixEditString(column1) + Q + D1 + Q + fixEditString(myLang.getDataString(column1)) + Q + C;
	          D = ",";
	        } while (myResult.next());
	        EditList += C;
	      } else {
	        EditList = "[]";     
	      }
	    } catch (java.sql.SQLException e) {
	      e.printStackTrace();
	      AuditTrail.println("[500] EditTables.setSingle - Bad (getResultSet): " + SQLStatement);
	      EditList = "[]";
	    }
  } 
  
  public String getList(String SQLStatement) {
    setList(SQLStatement);
    return EditList;
  }
  public String getList(String SQLStatement, boolean addBlank) {
    setList(SQLStatement, addBlank);
    return EditList;
  }
  
  public void setList (String SQLStatement) {
    setList(SQLStatement, true);
  }
   
  public void setList (String SQLStatement, boolean addBlank) {
    try {
      if (setResultSet(SQLStatement)) {
        java.sql.ResultSetMetaData Meta = myResult.getMetaData();
        int number_columns = Meta.getColumnCount();
        String O = "[";
        String C = "]";
        String D;
        String D1 = ",";
        String Q = "\"";
        if (addBlank) {
          D = ",";  
          EditList = O + O + Q + Q + D + Q + Q + C;
        } else {
          D = "";
          EditList = O;
        }
        do {
          EditList += D + O + Q + fixEditString(myResult.getString(1)) + Q; 
          for (int n = 2; n <= number_columns; n++)   
              EditList += D1 + Q + fixEditString(myResult.getString(n)) + Q; 
          EditList += C;
          D = ",";
        } while (myResult.next());
        EditList += C;
      } else {
        EditList = "[]";     
      }
    } catch (java.sql.SQLException e) {
      e.printStackTrace();
      AuditTrail.println("[500] EditTables.setSingle - Bad (getResultSet): " + SQLStatement);
      AuditTrail.println("[500) " + e.toString());
      EditList = "[]";
    }
  } 
  
  public String getElementArray(AdisoftDOM Sys) {
     Document CommonDOM;
     CommonDOM = Sys.getDOM();
     Element parentElement = CommonDOM.getRootElement();
     AuditTrail.println("[000] EditTables.getElementArray [1]");
     return getElementArray(parentElement);
  }
  
  public String getElementArray(AdisoftDOM Sys, String parentElementName) {
     Document CommonDOM;
     CommonDOM = Sys.getDOM();
     Element parentElement = CommonDOM.getRootElement().getChild(parentElementName.trim());
     AuditTrail.println("[000] EditTables.getElementArray [2]: " + parentElementName);
     return getElementArray(parentElement);
  }
  
  public String getElementArray(Element parentElement) {
     java.util.List elementChildren = parentElement.getChildren();   
     String DBase = "[";
     DBase = DBase + setChildren(elementChildren) ;
     DBase = DBase + "]";
     return DBase;
  }
  
  private String setChildren(java.util.List Children) {
    java.util.List DBaseElements;
    String ReturnString = "";
    String Comma = "";
    for (int i = 0; i < Children.size(); i++) {
      Element CE = (Element) Children.get(i);
//      if (CE.getTextNormalize().length() != 0 || (CE.getContentSize() > 0)) {
      if (CE.getContentSize() > 0) {
        ReturnString += Comma + "[\"" + ess.CommonFunctions.restoreSpace(CE.getName()) + "\""; 
        if (CE.getTextNormalize().length() != 0) {
          ReturnString += ",\"" + CE.getText() + "\"" ;  //was getTextNormalize() before
        }
//        if (CE.getContentSize() > 0) {
//          java.util.List DBaseElements = CE.getChildren();
//          ReturnString += ",[" + setChildren(DBaseElements) + "]";
//        }  
        DBaseElements = CE.getChildren();
        if (DBaseElements.size() > 0) {
          ReturnString += ",[" + setChildren(DBaseElements) + "]";
        }  
        ReturnString += "]";
        Comma = ",";
      }
    }
    return ReturnString;
  }
  
  public String getElementList(AdisoftDOM Sys, String parentElementName) {
     Document CommonDOM;
     CommonDOM = Sys.getDOM();
     Element parentElement = CommonDOM.getRootElement().getChild(parentElementName.trim());
     AuditTrail.println("[000] EditTables.getElementList [2]: " + parentElementName);
     return getElementList(parentElement);
  }
  
  public String getElementList(Element parentElement) {
     java.util.List elementChildren = parentElement.getChildren();   
     String DBase = "[";
     String D = "";
     String Q = "\"";
     Element E;
     for (int i = 0; i < elementChildren.size(); i++)
     {
        E = (Element) elementChildren.get(i);
        DBase = DBase + D + Q + E.getText() + Q;
        D = ",";
     }
     DBase = DBase + "]";
     return DBase;
  }
  
   
  
  
  //these versions of getElementArray has an array filter of elements (array) and only
  //return those elements that have a match in the filter.
  
  public String getDOMResultSet(AdisoftDOM Sys, String[] specified) {
     Document CommonDOM;
     CommonDOM = Sys.getDOM();
     Element parentElement = CommonDOM.getRootElement();
     AuditTrail.println("[000] EditTables.getDOMResultSet [1]");
     return getDOMResultSet(parentElement, specified);
  }
  
  public String getDOMResultSet(AdisoftDOM Sys, String parentElementName, String[] specified) {
     Document CommonDOM;
     CommonDOM = Sys.getDOM();
     Element parentElement = CommonDOM.getRootElement().getChild(parentElementName.trim());
     AuditTrail.println("[000] EditTables.getDOMResultSet [2]: " + parentElementName);
     return getDOMResultSet(parentElement, specified);
  }
  
  public String getDOMResultSet(Element parentElement, String[] specified) {
     java.util.List elementChildren = parentElement.getChildren();   
     Element rowElement;
     java.util.List columnElements;
     String comma = "";
     java.util.List rowElements = parentElement.getChildren();
     String DBase = "[";
     for (int i = 0; i < rowElements.size(); i++)
     {
        DBase += comma + "[";
        rowElement = (Element) rowElements.get(i);
        columnElements = rowElement.getChildren();
        DBase += setDOMResultSet(columnElements, specified);
        DBase += "]";
        comma = ",";
     }
     DBase = DBase + "]";
     return DBase;
  }
  
  private String setDOMResultSet(java.util.List Children, String[] specified) {
    java.util.List DBaseElements;
    String ReturnString = "";
    String Comma = "";
    String itemName;
    
    for (int i = 0; i < Children.size(); i++) {
      Element CE = (Element) Children.get(i);
      itemName = CE.getName();
//      if ((CE.getTextNormalize().length() != 0 || (CE.getContentSize() > 0))) {
      if (CE.getContentSize() > 0) {
        if ((Utilities.findString(itemName, specified) > -1) && (CE.getTextNormalize().length() != 0)) {
          ReturnString += Comma + "\"" + CE.getText() + "\"" ;  //was getTextNormalize() before
          Comma = ",";
        }
//        if (CE.getContentSize() > 0) {
//          java.util.List DBaseElements = CE.getChildren();
//          ReturnString += setDOMResultSet(DBaseElements,specified);
//        }  
        DBaseElements = CE.getChildren();
        if (DBaseElements.size() > 0) {
          ReturnString += setDOMResultSet(DBaseElements,specified);
        }  
        //ReturnString += "]";
      }
    }
    return ReturnString;
  }
  
  public String getCheckListFromArray(String[] specified) {
     String comma = "";
     String DBase = ";";
     for (int i = 0; i < specified.length; i++)
     {
        DBase += comma;
        DBase += specified[i];
        comma = ";";
     }
     DBase = DBase + ";";
     return DBase;
  }
  
  public String getDOMSingle(AdisoftDOM Sys, String parentElementName, String[] specified) {
     Document CommonDOM;
     CommonDOM = Sys.getDOM();
     Element parentElement = CommonDOM.getRootElement().getChild(parentElementName.trim());
     AuditTrail.println("[000] EditTables.getDOMSingle: " + parentElementName);
     return getDOMSingle(parentElement, specified);
  }
  
  public String getDOMSingle(Element parentElement, String[] specified) {
     java.util.List elementChildren = parentElement.getChildren();   
     Element rowElement;
     java.util.List columnElements;
     String comma = "";
     java.util.List rowElements = parentElement.getChildren();
     String DBase = "[";
     for (int i = 0; i < rowElements.size(); i++)
     {
        rowElement = (Element) rowElements.get(i);
        columnElements = rowElement.getChildren();
        DBase += comma + setDOMResultSet(columnElements, specified);
        comma = ",";
     }
     DBase = DBase + "]";
     return DBase;
  }
  
  public String getDOMChildNames(AdisoftDOM Sys) {
     Document CommonDOM;
     CommonDOM = Sys.getDOM();
     Element parentElement = CommonDOM.getRootElement();
     AuditTrail.println("[000] EditTables.getChildNames root element");
     return getDOMChildNames(parentElement);
  }
  
  public String getDOMChildNames(Element parentElement) {
     java.util.List elementChildren = parentElement.getChildren();   
     Element rowElement;
     String elementName;
     String comma = "";
     java.util.List rowElements = parentElement.getChildren();
     String DBase = "[";
     for (int i = 0; i < rowElements.size(); i++)
     {
        rowElement = (Element) rowElements.get(i);
        elementName = rowElement.getName();
        DBase += comma + "'" + elementName + "'";
        comma = ",";
     }
     DBase = DBase + "]";
     return DBase;
  }
  
  public String getMileage(String company) {
     AdisoftDOM CommonDOM = new AdisoftDOM();
     CommonDOM.setConnection(super.getConnection());
     CommonDOM.setSQLTerminator(super.getSQLTerminator());
     CommonDOM.setDOMFromTable("MILEAGE");
     Element parentElement = CommonDOM.getRootElement();
     Element rowElement;
     Element x;
     String eleCompany;
     java.util.List rowElements = parentElement.getChildren();
     for (int i = 0; i < rowElements.size(); i++)
     {
        rowElement = (Element) rowElements.get(i);
        x = rowElement.getChild("company");
        eleCompany = x.getTextNormalize();
        if (company.equals("") || company.equalsIgnoreCase(eleCompany)) {
           x = rowElement.getChild("effective");
           if (x != null) {
              x.setText(CustomDate.getSimpleDate(CustomDate.getDateFromXBase(x.getTextNormalize())));             
           }
        } else {
            rowElement.detach();
            i = i - 1;
        }
     }
// Xdate = CustomDate.getSimpleDate(CustomDate.getDateFromXBase(Xdate));     
     AuditTrail.println("[000] EditTables.getMileage [1]");
     //String[] x = {"effective", "rate"};
     //return getDOMResultSet(parentElement, x);
     return getElementArray(parentElement);
  }
  public String getMileage(String company, String dateFormatStr, String decimal) {
	     CustomDate workDate = new CustomDate(); 

	     AdisoftDOM CommonDOM = new AdisoftDOM();
	     CommonDOM.setConnection(super.getConnection());
	     CommonDOM.setSQLTerminator(super.getSQLTerminator());
	     CommonDOM.setDOMFromTable("MILEAGE");
	     Element parentElement = CommonDOM.getRootElement();
	     Element rowElement;
	     Element x;  //working effective date
	     Element y;  //working rate
	     String eleCompany;
	     java.util.List rowElements = parentElement.getChildren();
	     for (int i = 0; i < rowElements.size(); i++)
	     {
	        rowElement = (Element) rowElements.get(i);
	        x = rowElement.getChild("company");
	        eleCompany = x.getTextNormalize();
	        if (company.equals("") || company.equalsIgnoreCase(eleCompany)) {
	           x = rowElement.getChild("effective");
	           if (x != null) {
	              x.setText(workDate.getStrFromDate(workDate.getDateFromXBase(x.getTextNormalize()), dateFormatStr));             
	           }
	           y = rowElement.getChild("rate");
	           if (y != null) {
	              y.setText(getUserAmount(y.getTextNormalize(),decimal));             
	           }
	        } else {
	            rowElement.detach();
	            i = i - 1;
	        }
	     }
	     AuditTrail.println("[000] EditTables.getMileage [2]");
	     return getElementArray(parentElement);
	  }
      protected String getUserAmount(String x, String decimalStr) {
		 String retVal = Utilities.replaceStr(x,".",decimalStr); 
         AuditTrail.println("[000] EditTables.getUserAmount: " + x + ", with decimal: '" + decimalStr + "', for: " + retVal);
         return retVal;
      }  

}