/*
Email.java - represents an email message in memory
Copyright (C) 2004,2012 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.io.*;

public class Email{

// This class represents an email message with attachments
//
	private String emailString;                              //Holds the raw email message
	private String headerContentType = "multipart/mixed";    //Used to detect message with attachments
	private String boundaryString = null;                    //Message - attachment boundary string
	static AuditTrail Log = new AuditTrail();
	private String lineDelimiter = "\r\n";                   //new line sequence 0D0A
	private String[] messageParts = null;                    //Holds the message and attachment in separate cells
	private String beginBoundaryQuote = "boundary=";       //Used the extract the boundary string (originally had a quote)
	private String endBoundaryQuote = null;                   //Was originally set to "\""
	private String beginContentQuote = "Content-Type:";      //Used to extract the content type of each part of the message
	private String endContentQuote = ";";     
	// private String contentType = null;                       //Content type of the message part (change to an array???           
	private int partPointer = -1;                             //points to the content being addressed
	private String beginTransferEncoding = "Content-Transfer-Encoding:";
    private String endTransferEncoding = "\r\n";
    private String beginReturnPath = "Return-Path: <";       //
    private String endReturnPath = ">";
    final String CR = "\r";
    final String LF = "\n";
    String InFileName;
    int msgSectionRegistration = 0;                           //Holds section number where the registration information is
	
	public Email () 
	{
	}
	
	public Email (String x) 
	{
		set(x);
	}
	public void setHeaderContentType (String x)
	{
		headerContentType = x;
	}
	public void setLineDelimiter (String x)
	{
		lineDelimiter = x;
	}
	
	public void setBeginBoundaryQuote (String x)
	{
		beginBoundaryQuote = x;
	}

	public void setEndBoundaryQuote (String x)
	{
		endBoundaryQuote = x;
	}
	
	public void setBeginTransferEncoding (String x)
	{
		beginTransferEncoding = x;
	}
	
	public void setEndTransferEncoding (String x)
	{
		endTransferEncoding = x;
	}
	
	public String getTransferEncoding()
	{
		return getTransferEncoding(partPointer);
	}
	
	public String getTransferEncoding(int i) {
		AuditTrail.println("[000] Email getTransferEncoding: " + Integer.toString(i));
		return extractSubstring(messageParts[i], beginTransferEncoding, endTransferEncoding);
	}

	public String getReturnPath() {
		AuditTrail.println("[000] Email getReturnPath");
		return extractSubstring(messageParts[0], beginReturnPath, endReturnPath);
	}

	
	public void setBeginContentQuote (String x)
	{
		beginContentQuote = x;
	}

	public void setEndContentQuote (String x)
	{
		endContentQuote = x;
	}

	public int getNumberofParts()
	{
		return messageParts.length;
	}
	
	private String getBeginMessage()
	{
		return lineDelimiter + lineDelimiter;
	}

	public boolean getNext()
	{
		boolean retVal = false;
		partPointer++;                
		if (partPointer < messageParts.length) retVal = true;
		return retVal;
	}
	public void setNext()
	{
		partPointer = -1;
	}
	
	public String getContent()
	{
		return getContent(partPointer);
	}
	public String getContent(int part)
	{
        String extractedMessage = extractAbsolute(messageParts[part],getBeginMessage());
		if (extractedMessage.length() > 26) {
		    AuditTrail.println("[000] Email coded image: " + extractedMessage.substring(0,25));
		} else {
			AuditTrail.println("[000] Email coded image: " + extractedMessage);
		}
		return extractedMessage;
	}
	public String getText()
	{
		return getText(partPointer);
	}
	public String getText(int part)
	{
        String extractedMessage = extractSubstring(messageParts[part],getBeginMessage());
		if (extractedMessage.length() > 26) {
		    AuditTrail.println("[000] Email coded image: " + extractedMessage.substring(0,25));
		} else {
			AuditTrail.println("[000] Email coded image: " + extractedMessage);
		}
		return extractedMessage;
	}

	public String getContentType()
	{
		return getContentType(partPointer);
	}
	public String getContentType(int part)
	{
		String cType = null;
		String itemNumber = Integer.toString(part);
		AuditTrail.println("[000] Email getContentType: " + itemNumber);
		if (messageParts[part].indexOf(beginContentQuote) > -1)
		{
			cType = extractAbsolute(messageParts[part],beginContentQuote,endContentQuote);
			AuditTrail.println("[000] Email Content-Type[" + itemNumber + "]: "  + cType);
		} else {
			cType = "TEXT";
			AuditTrail.println("[000] Email Content-Type defaulted to: "  + cType);
		}
		return cType;
	}
		
	public void set(String x){
		emailString = x;    //why ??
		boolean multipartFound = false;
		// String extractedMessage = "";
		boundaryString = null;
		AuditTrail.println("[000] Email setEmail ==== called ====");
		// consider checking for the \n\r thing here with a replaceAll call
		setNext();
		String[] emailLineArray = emailString.split(lineDelimiter);
		for (int i = 0; ((i < emailLineArray.length) && (i < 100)); i++) {
			if (i < 31 ) AuditTrail.println("[000] Email setEmail: " + emailLineArray[i]);
			if ((emailLineArray[i].toUpperCase().indexOf(headerContentType.toUpperCase()) > -1) && !multipartFound)
			{
				// boundaryString = getBoundary(emailLineArray[i]);
				boundaryString = getBoundary(emailString);
			    if (boundaryString != null) multipartFound = true;
			}
		}
		AuditTrail.println("[000] Email multipart found: " + Boolean.toString(multipartFound));
		AuditTrail.println("[000] Email boundary: " + boundaryString);
		if (multipartFound)
		{
			messageParts = emailString.split(boundaryString);
		} else
		{
			messageParts = new String[1];
			messageParts[0] = emailString;
		}
		AuditTrail.println("[000] Email setEmail number of message parts found: " + Integer.toString(messageParts.length));
		for (int i = 0; i < messageParts.length; i++)
		{
			String itemNumber = Integer.toString(i);
			// contentType = getContentType(i);
			int xlen = messageParts[i].length();
			int mlen = xlen;
			if (xlen > 500) xlen = 250;
			AuditTrail.println("[000] Email messagePart [" + itemNumber + "]: " + messageParts[i].substring(0, xlen));
			if ((xlen == 250) && (mlen != xlen)) {
				AuditTrail.println("[000] ..... " + messageParts[i].substring(mlen - (xlen/2), mlen));
			}
			// extractedMessage = extractAbsolute(messageParts[i],getBeginMessage(),boundaryString);
			// extractedMessage = getContent(i);
		}
	}
   
	private String getBoundary(String messageLine) {  //Per RCF 1431 (MIME) standard
		
		String retVal = "";
		if (endBoundaryQuote == null)
		{
			retVal = "--" + extractAbsolute(messageLine, beginBoundaryQuote,"\r");  // get everything to the CR in the next CRLF
			retVal = retVal.replaceAll("\"","");                                    // remove quotes if they exist 
			retVal = retVal.replaceAll("\n","");                                    // incase we have a LF somewhere
			
		} else {
			retVal = "--" + extractAbsolute(messageLine, beginBoundaryQuote,endBoundaryQuote);
		}
		AuditTrail.println("[000] Email getBoundary [" + retVal + "]");
		return retVal;
	}
	
	private String extractAbsolute(String messageLine, String startingQuote)
	{
		return extractAbsolute(messageLine, startingQuote, "");
	}

	/*
	 * This eliminates any spaces from being involved in locating the substring being extracted.
	 */
	private String extractAbsolute(String messageLine, String startingQuote, String endingQuote)
	{
		return extractSubstring(messageLine.replaceAll(" ", ""), startingQuote, endingQuote);
	}

	private String extractSubstring(String messageLine, String startingQuote)
	{
		return extractSubstring(messageLine, startingQuote, "");
	}

	private String extractSubstring(String messageLine, String startingQuote, String endingQuote)
	{
		String retVal = "";
		try {
			messageLine = messageLine.substring(messageLine.toUpperCase().indexOf(startingQuote.toUpperCase())+ startingQuote.length());
			if (!endingQuote.equals("")) messageLine = messageLine.substring(0, messageLine.toUpperCase().indexOf(endingQuote.toUpperCase()));
			if (messageLine.length() > 0) retVal = messageLine.trim();
		} catch (java.lang.StringIndexOutOfBoundsException e) {
			AuditTrail.println("[400] Email extractSubstring error");
		}
		return retVal;
	}
	

	public void saveImage(String FileName, String FileType, ServerSystemTable SystemTable, byte[] imageData, String persnum)
	{
		OutputImageFile toDisk = openOutputImageFile(FileName, FileType, SystemTable, persnum);	
		try {
			toDisk.write(imageData);
		} catch (IOException e) {
			AuditTrail.println("[500] Email.saveImage - IOException error - see server log for more information");
			e.printStackTrace();
		}
        try {
            toDisk.close();
        } catch (IOException e) {
            AuditTrail.println("[500] Email.saveImage - Failed to close image file");
        }
	}

    public OutputImageFile openOutputImageFile(String FileName, String FileType, ServerSystemTable SystemTable, String persnum) {

     	OutputImageFile retVal = new OutputImageFile();
    	String parentFolder = SystemTable.getSystemString("receiptmanagement","parentfolder","/var/ess/receipts/") + persnum;
    	java.io.File persFolder = new java.io.File(parentFolder);
    	if (!persFolder.isDirectory()) 
    	{
    		persFolder.mkdirs();
    		AuditTrail.println("[000] Email.OutputImageFile - directory created: " + parentFolder + persnum);
    	}
        InFileName =  parentFolder + "/" + FileName + "." + FileType;
        try {
             retVal.createFileHandle(InFileName);
             AuditTrail.println("[000] Email Image file: " + InFileName);
        } catch (IOException e) {
             AuditTrail.println("[500] Email.OutputImageFile Image file create has failed (1) - " + e.toString());
        }
        return retVal;
     }
    
    public boolean isRegistration()
    {
    	boolean retVal = false;
    	msgSectionRegistration = -1;
    	do  {
    		msgSectionRegistration = msgSectionRegistration + 1;
    		String textMsg = getText(msgSectionRegistration).toUpperCase();
    		AuditTrail.println("[000] Email.isRegistration text part " + Integer.toString(msgSectionRegistration) + ": " + textMsg);
    		if (textMsg.indexOf("REGISTER") > -1) {
    			retVal = true;
    		}
    	} while(!retVal && msgSectionRegistration < messageParts.length - 1);
    	return retVal;
    }
    public String getRegistrationPersnum()
    {
    	String retVal = null;
    	String workString = getText(msgSectionRegistration).replace(CR, " ");
    	workString = workString.replace(LF, " ");
    	String [] testArray = workString.toUpperCase().split(" ");    //will this handle a null????
    	for (int i = 0; i < testArray.length; i++)
    	{
    		AuditTrail.println("[000] Email.getRegistrationPersnum outer: " + testArray[i]);
    		if (testArray[i].equalsIgnoreCase("REGISTER"))
    		{
    			AuditTrail.println("[000] Email.getRegistrationPersnum found register: " + testArray[i]);
    			i = i + 1;
    			while ( i < testArray.length)
    			{
    				AuditTrail.println("[000] Email.getRegistrationPersnum inner loop: " + testArray[i]);
    				if (testArray[i] != null && !testArray[i].equals("") && !testArray[i].equals(" ") )
    				{
    					retVal = testArray[i];
    					i = testArray.length;
    				}
    				i = i + 1;
    			}
    		}
    	}
    	AuditTrail.println("[000] Email.getRegistrationPersnum: " + retVal);
    	return retVal;
    }
}



    
