/*
FeedAPI.java - instantiates feed process in the EOD 
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.io.*;
import java.util.*; 
//import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Attribute;
import java.sql.*;


public abstract class FeedAPI {

// This class is used instantiating feeds in the EOD process
    int status = 0;
    protected static AuditTrail Logfile;
    private OutputTextFile outFile = new OutputTextFile();
    protected AdisoftDOM Sys = new AdisoftDOM();
  
    protected String quotes = "\"";
    protected boolean quoteDate = true;
    protected CustomDate currentDate = new CustomDate();
    protected int recordLayoutStart = 0;   // If the record layout starts at '0' or '1'
    protected String[] holidays;           // Contains a list of holidays for checking.
    protected Connection connection;  //used if dbase access is required.

    protected AdisoftDOM Description = new AdisoftDOM();  //holds export file descriptions
                                                          //see calling classes for examples

// Items below moved in from fleet for report generailization        
    protected java.io.File SystemFile;
    
    protected java.util.List columnList; //Columns that get read in  
    
    protected java.util.List dateList; // These three are used by subclasses to identify
    protected java.util.List numberList; // elements for changing to the standard internal
    protected java.util.List logicList; // string formatting 2006-06-26
    
    protected Element inputLine; //Element used to retreive whatever column you want.
    protected ess.AdisoftDOM DescribDOM;
    protected String maskInput;
    protected String maskRow;
    protected String[] names;
    
    protected String fileType = "delimited";
    protected String delimitStr = ",";
    protected String dateFormatStr = null;

    public abstract void init(ess.AdisoftDOM sys, Connection x);
    public abstract void post(PersonnelTable p, Report r);  
    public abstract void close(); 

    public void setInit(ess.AdisoftDOM sys, Connection x)
    {      
      Sys = sys;   
      connection = x;
      String systemFileName = sys.getDOMTableValueFor("configuration","xmlexport","C:\\WORK\\"); 
      SystemFile = new java.io.File(systemFileName);
      names = setupExportDescription2Array();
    }
    
    public Element getInputDescElement() {
    	return inputLine;
    }
    
    public void setFileType(String x)
    {
        fileType = x;
    }
    
    public void setMaskInput(String inputMask)
    {
          maskInput = inputMask;
    }

    public String getMaskInput() 
    {
    	return maskInput;
    }
    
    
    public void setMaskRow(String row)
    {
          maskRow = row;
    }
    
    public String getMaskRow()
    {
    	return maskRow;
    }
    
    public int getStatus() 
    {
        return status;
    }
    
    public void setRecordLayoutStart(int x)
    {
        recordLayoutStart = x;
    }
    
    public CustomDate getCurrentDate()
    {
        return currentDate;
    }
    
    public void setDelimitStr(String x)
    {
    	if (x != null) delimitStr = x;
    }
    
    public String getDelimitStr() {
    	return delimitStr;
    }
    
    public void setQuotes(String x)
    {
        if (x != null) quotes = x;
    }
    
    public String getQuotes(){
    	return quotes;
    }
    
    public void setDateFormat(String x)
    {
        dateFormatStr = x;
    }
    
    public String getDateFormat(){
    	return dateFormatStr;
    }

    public void setQuoteDate(String x)
    {
    	if (x != null) {
    		if (x.equalsIgnoreCase("no")) setQuoteDate(false);   //JH 2011-04-12
    	} else
    	{
    		if (x.equalsIgnoreCase("yes")) setQuoteDate(true);
    	}
    }
    
    public boolean getQuoteDate(){
    	return quoteDate;
    }
    
    public void setQuoteDate(boolean x)
    {
        quoteDate = x;
    }

    
    public int openFile(String fileName)
    {
        return openFile(fileName, true);
    }
    
    public int openFile(String fileName, boolean useEODElement)
    {
        //want to put in logic for 
        int retVal = 0;
        if (useEODElement) fileName = Sys.getDOMTableValueFor("endofday","folder","") + fileName;
        outFile.setFile(fileName, false);
        if (outFile.getWriteStatus())
        { 
            AuditTrail.println("[000] FeedAPI.openFile - Opened: " + fileName);
        } else {
            AuditTrail.println("[500] FeedAPI.openFile - Error: " + fileName);
            retVal = -1;
        }
        return retVal;
    }
    public int println(String string)
    {
        return println(string, true);
    }
    public int println(String string, boolean addReturn)
    {
        int retVal = 0;
        try {
            if (addReturn)
            {
                outFile.writeln(string);
            } else {
                outFile.write(string);
            }
        } catch (IOException e) {
            AuditTrail.println("[500] FeedAPI.println - IO Error");
            retVal = -1;
        } catch (Exception e) {
            AuditTrail.println("[500] FeedAPI.println - General Error");
            AuditTrail.println("[500] " + e.toString());
            retVal = -2;
        }
        return retVal;
    }
    public void setClose()
    {
        try {
            outFile.setClose();
            AuditTrail.println("[000] FeedAPI.setClose - OK");
        } catch (IOException e) {
            AuditTrail.println("[500] FeedAPI.setClose - IO Error");
        } catch (Exception e) {
            AuditTrail.println("[500] FeedAPI.setClose - General Error");
        }
    }
    
     public void setField(char[] record, Element field, CustomDate datevalue) 
    {
        final String[] dateTypes = {"yyyymmdd","yymmdd","hhmm","mmddyyyy","mm","mm/dd/yy","dd_mmm_yyyy"};
        final int YYYYMMDD = 0;
        final int YYMMDD = 1;
        final int HHMM = 2;
        final int MMDDYYYY = 3;
        final int MM = 4;
        final int MM_DD_YY = 5;
        final int DD_MMM_YYYY = 6;
        
        String value = "";
        Attribute attrib = field.getAttribute("format");
        String contents = attrib.getValue().toLowerCase();  //JH 2006-06-23
        switch(Utilities.findString( contents, dateTypes))
        {
            case YYYYMMDD:
              value = CustomDate.getExportDate(datevalue.date);
              break;
            case YYMMDD:
              value = CustomDate.getNachaDate(datevalue.date);
              break;
            case HHMM:
              value = CustomDate.getNachaTime(datevalue.time);
              break;
            case MMDDYYYY:
              value = CustomDate.getSAPDate(datevalue.date);
              break;
            case MM:
              value = CustomDate.getMonth(datevalue.date);
              break;
            case MM_DD_YY:
              value = CustomDate.getUSDate(datevalue.date);
              break;
            case DD_MMM_YYYY:
              value = CustomDate.getOracleDate(datevalue.date);
              break;
              
        }  
        setField(record, field, value);        
    }

    public void setField(char[] record, Element field) 
    {
        setField(record, field, field.getText());
    }
    
    public void setField(char[] record, Element field, float lvalue) 
    {
        String value = Float.toString(lvalue);
        setField(record, field, value);        
    }
    
    public void setField(char[] record, Element field, long lvalue) 
    {
        String value = Long.toString(lvalue);
        setField(record, field, value);        
    }
    
    public void setField(char[] record, Element field, double lvalue) 
    {
        double x = (double) 0.00;
        x = lvalue;
        String value = Double.toString(x);
        setField(record, field, value);        
    }
    
    
    public void setField(char[] record, Element field, int ivalue) 
    {
        String value = Integer.toString(ivalue);
        setField(record, field, value);        
    }
    
    public void setField(char[] record, Element field, String value) 
    {
        final String[] dateTypes = {"packeddollar","numeric","packed"};
        final int PACKEDDOLLAR = 0;
        final int NUMERIC = 1;
        final int PACKED = 2;
        Attribute attrib = field.getAttribute("format");
        String contents = attrib.getValue();
        switch(Utilities.findString( contents, dateTypes))
        {
            case PACKEDDOLLAR:
              //Logfile.println("[---] in: " + value);
              double workVar = Double.valueOf(value).doubleValue();
              workVar = workVar * (double) 100;
              long x = java.lang.Math.round(workVar);
              value = Long.toString(x);
              value = value.trim();
              //Logfile.println("[---] out: " + value);              
              break;
        }  
        
        try {
            int x = field.getAttribute("position").getIntValue();
            x = x - recordLayoutStart;
            int y = field.getAttribute("length").getIntValue();
            
            switch(Utilities.findString( contents, dateTypes))
            {
                case PACKEDDOLLAR:
                case PACKED:
                  value = leftPack(value, y, '0'); 
                  break;
                case NUMERIC:
                  value = leftPack(value, y, ' '); 
                  break;
            }  
            setField(record, value, x, y);

        } catch (org.jdom.DataConversionException e) {
            this.println("Check position/length or offset attribute!!!");
            AuditTrail.println("[500] FeedAPI.setField - check position/length or offset attribute");
        }
    }
    
    public void setField(char[] record, String value, int startPosition, int length) 
    {
        //Logfile.println("[---] FeedAPI.setField: " + value);
        char[] asChars = value.toCharArray();
        int lengthOfValue = value.length();
        if (lengthOfValue < length) length = lengthOfValue;
        for (int i = 0; i < length; i++)
        {
            record[startPosition + i] = asChars[i];
        }
    }
    
    public String leftPack(String x, int length, char packChar)
    {
        x = x.trim();
        String retVal = "";
        char[] asChars = x.toCharArray();
        char[] newfield = new char[length];
        Arrays.fill(newfield, packChar);
        int xlen = length - x.length();
        if (xlen < 0)
        {
            retVal = "ERR!";
        } else {
            int j = 0;
            for (int i = xlen ; i < length ; i++)
            {
                newfield[i] = asChars[j++]; 
            }
            retVal = String.valueOf(newfield);
        }
        return retVal;
    }
    
    public long addString2LongforCurrency(long total, String value)
    {
        return Utilities.addString2LongforCurrency(total, value);
    }
    
    public long addString2Long(long total, String value)
    {
        return Utilities.addString2Long(total, value);
    }
  
    public static String makeLong2Currency(long value)
    {
        return Utilities.makeLong2Currency(value);
    }
    
    public CustomDate bump4holiday(CustomDate xdate)
    {   
        String date2check;
        java.util.Date date = xdate.getDate();
        
        Calendar day = new GregorianCalendar();
        day.setTime(date);
        int dayOfWeek = day.get(Calendar.DAY_OF_WEEK);
        
        if (dayOfWeek == Calendar.SATURDAY) {
        	date = CustomDate.addDays(date, 2);
        } else if (dayOfWeek == Calendar.SUNDAY){
        	date = CustomDate.addDays(date, 1);
        }
        
        date2check = CustomDate.getSQLDate(date);
        while (Utilities.findString( date2check, holidays) > -1)
        {
            date = CustomDate.addDays(date, 1);
            xdate.setDate(date);
            date2check = CustomDate.getSQLDate(date);
        }
        
        xdate.setDate(date);
        return xdate;
    }
        public String getDenormalItem(String[] items, int itemWanted)
    {
        String retVal = items[itemWanted];
        return retVal;
    }
    
    protected String[] setupExportDescription2Array()
    {          
        Element xLine;
        DescribDOM = new AdisoftDOM();
        DescribDOM.setDOM(SystemFile);

        xLine = DescribDOM.getElement(maskInput);  //This should be the name;
        
        fileType = xLine.getAttributeValue("type");  //This should be "delimited" or "SDF"
        
        if (fileType.equalsIgnoreCase("DELIMITED")) 
        {	delimitStr = xLine.getAttributeValue("delimiter");  //This should be "delimited" or "SDF"
            if (delimitStr == null) delimitStr = ",";
        }
        inputLine = xLine.getChild(maskRow);
        columnList = inputLine.getChildren();
        String[] names = new String[columnList.size()];
        Element z;
        for (int i = 0; i < columnList.size(); i++)
        {
            z = (Element) columnList.get(i);
            names[i] = z.getName();
        }
        return names;
    }
    
    protected String[] setupAttribute2Array(String Attrib, String[] WantedAttribValues)
    {
        return setupAttribute2Array(Attrib, WantedAttribValues, false);
    }
    
    protected String[] setupAttribute2Array(String Attrib)
    {
        String[] blankArray = {};
        return setupAttribute2Array(Attrib, blankArray, true);
    }
    
    protected String[] setupAttribute2Array()
    {
        return setupAttribute2Array("format");
    }
    
    protected String[] setupAttribute2Array(String Attrib, String[] WantedAttribValues, boolean allFlag)
    {          
        Element xLine;
        Attribute xAttrib;
        String xAttribName;
        String xAttribValue;
        DescribDOM = new AdisoftDOM();
        DescribDOM.setDOM(SystemFile);
        xLine = DescribDOM.getElement(maskInput);
        inputLine = xLine.getChild(maskRow);
        columnList = inputLine.getChildren();
        ArrayList namesList = new ArrayList();
        Element z;
        for (int i = 0; i < columnList.size(); i++)
        {
            z = (Element) columnList.get(i);
            xAttrib = z.getAttribute(Attrib);
            xAttribName = xAttrib.getName();
            xAttribValue = xAttrib.getValue();
            
            if (allFlag)
            {
                namesList.add(xAttribValue);
            }else
            {
            
               for (int k = 0; k < WantedAttribValues.length; k++)
               {
                   if (WantedAttribValues[i].equalsIgnoreCase(xAttribValue))
                   {
                     namesList.add(xAttribName);
                   }
               }
            }
        }
        String[] names = new String[namesList.size()];
        for (int L = 0; L < namesList.size(); L++)
        {
            names[L] = (String) namesList.get(L);
        }
        return names;
    }
    
    protected int getRowIndex(String name)
    {
        return getRowIndex(names, name);
    }
    
    protected int getRowIndex(String[] names, String name)
    {
        int retVal = -1;
        String x;
        for (int i = 0; i < names.length; i++) 
        {
            if (names[i].equalsIgnoreCase(name))
            {
                retVal = i;
                i = names.length;
            }
        }
        return retVal;
    }
    
    //Used to ensure string for export file 
    public static String removeReturn(String x) {
       x = Utilities.replaceStr(x, ",", ";");
       x = Utilities.replaceStr(x, "\n", ":");
       x = Utilities.replaceStr(x, "\r", ":");
       return x; 
    }

    public String quoteString(String x)
    {
        return "\"" + x + "\"";
    }
    
    protected String FRoundStr(float x) {
      if (x >= (float) 0) {
        return Float.toString(x + (float) 0.001);
      } else {
        return Float.toString(x - (float) 0.001);
      } 
    }
    protected String DRoundStr(double x) {
      if (x >= (double) 0) {
        return Double.toString(x + (double) 0.001);
      } else {
        return Double.toString(x - (double) 0.001);
      } 
    }
  
    protected String FF(String FFN) {
      String S = FFN + "00";
      return S.substring(0,S.indexOf(".")+ 3);
    }
    
    protected boolean rename(String newFile, String oldFile)
    {
        String folderName = Sys.getDOMTableValueFor("endofday","folder","");
    	return outFile.rename(folderName + newFile, folderName + oldFile);	
    }
}

