/*
GLSummary.java - Summarizes expense and payment accounts - used by XML2ESS and EOD reports  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

import java.util.*;

//  Example of usage
//  public void setGLSummary() {
//    ESSExpense E;
//    GLSummary P = new GLSummary();
//    GLSummaryInfo.removeAllElements();
//    for (int i = 0; i < ESSExpenses.size(); i++) {
//      E = (ESSExpense) ESSExpenses.elementAt(i);
//      P.add(E.EXPENSE, E.AMOUNT, E.BILLTYPE);
//    }    

public class GLSummary {
  private Vector GLSummaryInfo = new Vector();
  public String NAME = "";   //Normally the account number
  public float SUM = 0;
  public float DEBIT = 0;
  public float CREDIT = 0;
  public float BILLABLE = 0;
  public float PERSONAL = 0;
  public String DEPART = "";
  public String PROJECT = "";
  public String CLIENT = "";  
  public String OTHER = "";
  
  private String referenceType = "PROJECT/CLIENT";
  
  static Double DConvert;
  static Float FConvert;
  
  final String tr = "<tr>\n";
  final String tre = "</tr>\n";
  String btable = "<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"90%\" class=\"offsetColor\" style=\"border: 1px solid\">";
  final String stable = "<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"90%\">";
  
  String backcolor = "class=\"offsetColor\"";
  String oldbackcolor = "";
  String newbackcolor;
 
  private String generalTitle = "Credit Summary Section";
  private String accountTitle = "Account";
 
  public GLSummary (){
  }
  public void setReferenceType(String x)
  {
    referenceType = x;
  }
  public void setGeneralTitle(String x)
  {
    generalTitle = x;
  }
  public void setAccountTitle(String x)
  {
    accountTitle = x;
  }
  
  public void add(String Item, String Value) {
       add(Item, Value, true, false, true, false, "", "", "", "");
  }
  

  public void add(String Item, String Value, String BillType) {
     if (BillType.equals("1")) { 
       add(Item, Value, true, false, true, false, "", "", "", "");
     } else if (BillType.equals("2")) {
       add(Item, Value, true, false, false, false, "", "", "", "");
     } else {
       add(Item, Value, true, false, false, true, "", "", "", "");
     }
  }  
  public void add(String Item, String Value, String BillType, String Depart, String Project) {
     if (BillType.equals("1")) { 
       add(Item, Value, true, false, true, false, Depart, Project, "", "");
     } else if (BillType.equals("2")) {
       add(Item, Value, true, false, false, false, Depart, Project, "", "");
     } else {
       add(Item, Value, true, false, false, true, Depart, Project, "", "");
     }
  }
  public void addDebit(String Item, String Value, String BillType, String Depart, String Project, String Client, String Other) {
     if (BillType.equalsIgnoreCase("YES") || BillType.equalsIgnoreCase("1"))
     {
       add(Item, Value, true, false, true, false, Depart, Project, Client, Other);
     } else {
       add(Item, Value, true, false, false, false, Depart, Project, Client, Other);
     }
  }
  public void addDebit(String Item, String Value, String Depart, String Project, String Client, String Other) {
     add(Item, Value, true, false, false, false, Depart, Project, Client, Other);
  }
  public void addCredit(String Item, String Value, String Depart, String Project, String Client, String Other) {
     add(Item, Value, false, true, false, false, Depart, Project, Client, Other);
  }
  
  public void add(String Item, String Value, boolean Debit, boolean Credit, boolean Billable, 
                  boolean Personal, String Depart, String Project, String Client, String Other)
  {
     boolean found = false;
     GLSummary R;
     for (int i = 0; i < GLSummaryInfo.size(); i++) {
       R = (GLSummary) GLSummaryInfo.elementAt(i);
       if (R.NAME.equals(Item) && R.DEPART.equals(Depart)
                               && R.PROJECT.equals(Project)
                               && R.CLIENT.equals(Client)
                               && R.OTHER.equals(Other)) 
       {
         found = true;
         i = GLSummaryInfo.size();
         R.update(Value, Debit, Credit, Billable, Personal);
       }   
     } 
     if (!found){
       R = new GLSummary();  
       R.NAME = Item;  //Why not pass in Item instead?  - could make private.
       R.update(Value, Debit, Credit, Billable, Personal, Depart, Project, Client, Other);
       GLSummaryInfo.addElement(R);
     }
  }
  
  public GLSummary get(int i)
  {
    return (GLSummary) GLSummaryInfo.get(i);
  }
  
  public int size()
  {
    return GLSummaryInfo.size();
  }
  
  
  private void update(String Value, boolean Debit, boolean Credit, boolean Billable, boolean Personal, String Depart, String Project, String Client, String Other){
    SUM += Double.parseDouble(Value);
    if (Debit) DEBIT += Double.parseDouble(Value);
    if (Credit) CREDIT += Double.parseDouble(Value);
    if (Billable) BILLABLE += Double.parseDouble(Value);
    if (Personal) PERSONAL += Double.parseDouble(Value);
    DEPART = Depart;   //can we make these private and handle correctly?
    PROJECT = Project;
    CLIENT = Client;
    OTHER = Other;
  }
  private void update(String Value, boolean Debit, boolean Credit, boolean Billable, boolean Personal){
    SUM += Double.parseDouble(Value);
    if (Debit) DEBIT += Double.parseDouble(Value);
    if (Credit) CREDIT += Double.parseDouble(Value);
    if (Billable) BILLABLE += Double.parseDouble(Value);
    if (Personal) PERSONAL += Double.parseDouble(Value);
  }
  
  public String printExpListItem(GLSummary R) {
    return l2row(R.NAME, R.SUM, R.BILLABLE, R.PERSONAL); 
  }  
  public String printExpenseSummary() {
    GLSummary R;
    float expTotal = 0;
    float expBillable = 0;
    float expPersonal = 0;
    String retVal = SecHeader("Expense Summary Section");
    retVal += stable;
    retVal += l1row(chd("Expense"),chd("Amount"),chd("Billable"),chd("Personal"));
    for (int i = 0; i < GLSummaryInfo.size(); i++) {
      R = (GLSummary) GLSummaryInfo.elementAt(i);
      retVal += printExpListItem(R);
      expTotal += R.SUM;
      expBillable += R.BILLABLE;
      expPersonal += R.PERSONAL;  
    }
    retVal += l2row("Totals:",expTotal,expBillable,expPersonal);
    retVal += l3row("Non-Pers:",expTotal - expPersonal);
    retVal += "</table>\n";
    return retVal;
  }
  public String printGeneralListItem(GLSummary R) {
    return l3row(R.NAME, R.SUM); 
  }  
  public String printGeneralSummary() {
    GLSummary R;
    float expTotal = 0;
    String retVal = SecHeader(generalTitle);
    retVal += stable;
    retVal += l1row(chd(accountTitle),chd("Amount"),chd(""),chd(""));
    for (int i = 0; i < GLSummaryInfo.size(); i++) {
      R = (GLSummary) GLSummaryInfo.elementAt(i);
      retVal += printGeneralListItem(R);
      expTotal += R.SUM;
    }
    retVal += l3row("Totals:",expTotal);
    retVal += "</table>\n";
    return retVal;
  }
  
  public String printFullItemList() {
    GLSummary R;
    float expDebit = 0;
    float expCredit = 0;
    String retVal = SecHeader(generalTitle);
    retVal += stable;
    retVal += d6row(chd(accountTitle),chd(""),chd(""),chd(""),chd("Debit"),chd("Credit"));
    for (int i = 0; i < GLSummaryInfo.size(); i++) {
      R = (GLSummary) GLSummaryInfo.elementAt(i);
      retVal += printFullItem(R);
      expDebit += R.DEBIT;
      expCredit += R.CREDIT;
    }
    retVal += l6row("","","","Total:",expDebit, expCredit);
    retVal += "</table>\n";
    return retVal;
  }
  
  public String printFullItem(GLSummary R) {
    return l6row(R.NAME, R.DEPART, R.PROJECT, R.CLIENT, R.DEBIT, R.CREDIT); 
  }  
  
  private String l6row(String v1, String v2, String v3, String v4, float v5, float v6) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("150",v2);  //expense
    retVal += htd("150",v3);  //expense
    retVal += htd("150",v4);  //expense
    retVal += htd("100",FF(FRoundStr(v5)));  //amount 
    retVal += htd("100",FF(FRoundStr(v6)));  //amount
    retVal += tre;

    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
  
  private String d6row(String v1, String v2, String v3, String v4, String v5, String v6) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("150",v2);  //expense
    retVal += htd("150",v3);  //expense
    retVal += htd("150",v4);  //expense
    retVal += htd("100",v5);  //amount
    retVal += htd("100",v6);  //amount     
    retVal += tre;
    
    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
  
  public String getAmountString() 
  {
    return FF(FRoundStr(DEBIT));
  }
  
  public String getDebitString() 
  {
    return FF(FRoundStr(DEBIT));
  }
  
  public String getCreditString() 
  {
    return FF(FRoundStr(CREDIT));
  }
  
  public String getName()
  {
    return NAME;
  }
  
  private String l2row(String v1, float v2, float v3, float v4) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("100",FF(FRoundStr(v2)));  //amount 
    retVal += htd("100",FF(FRoundStr(v3)));  //billable
    retVal += htd("150",FF(FRoundStr(v4)));  //personal
    retVal += tre;

    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
  
  private String htd(String w, String v) {
    return "<td width=\"" + w + "\" align=\"left\"" + backcolor + ">" + v + "</td>\n";
  } 
  
  private String FRoundStr(float x) {
    if (x >= (float) 0) {
      return Float.toString(x + (float) 0.001);
    } else {
      return Float.toString(x - (float) 0.001);
    } 
  }
  
  private String chd(String X) {
    //special formatting for column headers
    return "<u>" + X + "</u>";
  }
  private String FF(String FFN) {
    String S = FFN + "00";
    return S.substring(0,S.indexOf(".")+ 3);
  }
  private String SecHeader(String Title) {
    return "<p>" + Title + "</p>\n";
  }  
  private String l3row(String v1, float v2) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("100",FF(FRoundStr(v2)));  //amount 
    retVal += tre;
    
    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
  private String l1row(String v1, String v2, String v3, String v4) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("100",v2);  //amount 
    retVal += htd("100",v3);  //billable
    retVal += htd("150",v4);  //personal
    retVal += tre;
    
    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
//=============================================
  
  public String printDepartListItem(GLSummary R) {
      return d4row(R.DEPART, R.PROJECT, R.NAME, R.SUM);
  }
  private String d1row(String v1, String v2, String v3, String v4) {
    String retVal = tr;
    retVal += htd("50",v1);  
    retVal += htd("100",v2);  
    retVal += htd("100",v3); 
    retVal += htd("150",v4); 
    retVal += tre;
    
    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
  private String d2row(String v1, float v4) {
    String retVal = tr;
    retVal += htd("50",v1);
    retVal += htd("100",""); 
    retVal += htd("100","");
    retVal += htd("150",FF(FRoundStr(v4)));
    retVal += tre;
    
    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
  private String d4row(String v1, String v2, String v3, float v4) {
    String retVal = tr;
    retVal += htd("50",v1);  
    retVal += htd("100",v2);  
    retVal += htd("100",v3);  
    retVal += htd("150",FF(FRoundStr(v4)));  
    retVal += tre;
    
    newbackcolor = backcolor;
    backcolor = oldbackcolor; 
    oldbackcolor = newbackcolor;
    
    return retVal;
  }
  public String printDepartSummary() {
    GLSummary R;
    float expTotal = 0;
    float expBillable = 0;
    float expPersonal = 0;
    String retVal = SecHeader("Expense by Department and " + referenceType);
    retVal += stable;
    retVal += d1row(chd("Depart"),chd(referenceType),chd("Expense"),chd("Amount"));
    for (int i = 0; i < GLSummaryInfo.size(); i++) {
      R = (GLSummary) GLSummaryInfo.elementAt(i);
      retVal += printDepartListItem(R);
      expTotal += R.SUM;
    }
    retVal += d2row("Total:",expTotal);
    retVal += "</table>\n";
    return retVal;
  }
} 