/*
GeneralFeed.java - G/L feed  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.util.*;
import java.sql.*;
import org.jdom.Element;
import org.jdom.Attribute;

public class GeneralFeed extends FeedAPI {
    // This class is used instantiating feeds in the EOD process
    
    private String outFileName = "SimpleGL.dat";
    private String renameFile = null;
    
    public void setOutFile(String x)
    {
        if (x != null) {
        	CustomDate zdate = new CustomDate();
            outFileName = Utilities.replaceStr(x, "$date$", CustomDate.getExportDate(zdate.getDate()) );;
        }
    }
    
    public void setRenameFile(String x)
    {
        if (x != null) {
        	CustomDate zdate = new CustomDate();
            renameFile = Utilities.replaceStr(x, "$date$", CustomDate.getExportDate(zdate.getDate()) );;
        }
    }

    public void init(ess.AdisoftDOM sys, Connection x)
    {
      AuditTrail.println("[000] GeneralFeed.start" );
      Sys = sys;

      Element endofday = Sys.getElement("endofday");
      Element generalFeed = endofday.getChild("generalFeed");
      setDelimitStr(generalFeed.getAttributeValue("delimiter"));     
      setQuotes(generalFeed.getAttributeValue("quotes"));
      setQuoteDate(generalFeed.getAttributeValue("quoteDate"));
      setOutFile(generalFeed.getAttributeValue("file"));
      setDateFormat(generalFeed.getAttributeValue("dateFormat"));
      setRenameFile(generalFeed.getAttributeValue("rename"));
      
      connection = x;
      this.openFile(outFileName);
    }
    
    public void post(PersonnelTable p, Report r)
    {
    	// save the report state
      String stack_quotes = r.getStringQuote();
      boolean stack_quoteDate = r.getQuoteDate();
      String stack_dateFormatStr = r.getDateFormat();
      
   // Need to set the delimit, stringQuote, quoteDate string here
      r.setStringQuote(getQuotes());
      r.setQuoteDate(getQuoteDate());
      r.setDateFormat(getDateFormat());
      
      ArrayList x = r.getDeNormal();
      String D;
      String line;
      for (int i = 0; i < x.size(); i++)
      {
        String[] lineItem = (String[]) x.get(i);
        D = "";
        line = "";
        for (int j = 0; j < lineItem.length; j++)
        {
            line += D + Utilities.removeReturn(lineItem[j], delimitStr);
            D = delimitStr;
        }
        this.println(line);
      }
      
      r.setStringQuote(stack_quotes);  //Need to restore for the next feed process
      r.setQuoteDate(stack_quoteDate);
      r.setDateFormat(stack_dateFormatStr);
      
      AuditTrail.println("[000] GeneralFeed.post: " + r.getVoucherNumber() );
    }

    
    
    public void close()
    {
      AuditTrail.println("[000] GeneralFeed.close" );
      //this.println("End of file");
      this.setClose();
      if (renameFile != null && !renameFile.equals(""))
      {
    	  if (super.rename(renameFile, outFileName))
    	  {
    		  AuditTrail.println("[000] GeneralFeed.close renamed: " + renameFile);
    	  } else {
    		  AuditTrail.println("[500] GeneralFeed.close rename failed: " + outFileName + " ==> " + renameFile);
    	  }
      } else {
    	  AuditTrail.println("[000] GeneralFeed.close: " +  outFileName);
      }
    }
   
}