/*
GeneralImport.java - Common methods for importing an external text file into Prepopulated
Copyright (C) 2006 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

/* USAGE STEPS

// Need to review the usage steps and update.

-- Add steps on connection, SQLTerminator

1. setImportMethod - text from name of a method element in the import menu file 
   (import.xml) that describes this import - this ia a holder for subclasses (REQUIRED)
2. setImportType - Name of element in the import description file 
   (export.xml) that describes this the layouts of the import file (REQUIRED)
3. setRowType - name of the row element (OPTIONAL)
4. setDefaultInputDescFile or setInputDescFile - full path-name of the import 
   description (REQUIRED)
5. setColumnsWanted - The columns from the import description (export.xml) 
   wanted (REQUIRED) (can I figure this out from the description??)
6. setEditName - name of the element in edit.xml that has the logic and physical 
   names (REQUIRED)
7. setOpenFiles - Opens up the input file and readys to bring in entrys (REQUIRED)

*/

package ess;

//import java.io.*;
//import java.util.*; 
//import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Attribute;
import java.sql.*;
import java.util.*;

public class GeneralImport extends ReportAPI {
    
    // This class is used to generate reports and feeds from the EOD process
    
    static AuditTrail Logfile;
    
    // Stuff for reading in the file
    protected String importDescFile = "c:/ess/xmls/export.xml"; //Holds the descriptions of the import (same file as the export.xml)
    //private String importType = "Prepopulated"; //points to major element in importDescFile
    //private String rowType = "row";    //points to the row name under the major element
    protected String inputFileName = "c:/ess/import/import.dat";
    
    protected String importMethod = "standard";
    
    // Stuff for updating the database
    protected String editName = "prepopulated";
    
    ServerSystemTable systemTable = new ServerSystemTable();
    
    protected AdisoftDbase db = new AdisoftDbase();
    protected AdisoftDOM TableDOM = new AdisoftDOM();
    protected AdisoftDOM EditDOM = new AdisoftDOM();
    protected AdisoftDOM ImportControlDOM = new AdisoftDOM();
    protected AdisoftDOM ImportDescDOM = new AdisoftDOM();
    
    protected String[] encrypts = new String[0];  //plugging it for now
    protected String tableName = "";
    protected String sqltableName = "";

    protected org.jdom.Element schemaElement;
    protected org.jdom.Element columns;
    protected org.jdom.Element rowKey;

    String sqlMethod = "simple"; //(Check this out)
    String sqlLockTable; // not used right now
    String sqlUnlockTable; // not used right now

    protected String[] columnsWanted = {};  
    protected String[] attributesWanted = {};
        
    protected String[] fixedColumnNames = {};
    protected String[] fixedColumnValues = {};
    
    protected String[] superColumnNames;  //
    
    protected CustomDate StandardDate = new CustomDate();

    public GeneralImport() 
    {
    }
    
    public void setupFiles(ess.AdisoftDOM sys, Connection x)
    {
        setConnection(x);
        setSystemDOM(sys);
        setupFiles();
    }

    public void setupFiles()  //This is the basic start
    {
        //This is general information for describing the import file
        // importDescFile is the xml file that holds descriptions (e.g. export.xml)
        // impportType is the element with the specific description
        // rowType is there as each importType can have more than one breakdown
        // inputFileName is the actual file to import
        AuditTrail.println("[000] GeneralImport importDescFile: " + importDescFile);
        AuditTrail.println("[000] GeneralImport inputFileName: " + inputFileName);
        AuditTrail.println("[000] GeneralImport rowType: " + rowType);      //Check these two out.  
        //initialization for import
        initElement(importDescFile, importType, rowType);
        if (inputFileFullPath(inputFileName) == 0)
        {
           // DescribDOM is located in the 
           //This file describes the other components, such as the edit routine to use, etc.
           String system_file = super.Sys.getDOMTableValueFor("configuration","xmlimport","c:/ess/xmls/import.xml");
           AuditTrail.println("[000] GeneralImport Import XML: " + system_file);        
           java.io.File SystemFile1 = new java.io.File(system_file);
           ImportControlDOM.setDOM(SystemFile1);
           
           setImportType(getFromControlFile("type"));  
           setJournalName(getFromControlFile("htmljournal"));
           
           AuditTrail.println("[000] GeneralImport importType: " + importType);
           
           //The edit dom contains the logic for mapping to physical table
           system_file = super.Sys.getDOMTableValueFor("configuration","xmledit","c:/ess/xmls/edit.xml");
           AuditTrail.println("[000] GeneralImport Edit XML: " + system_file);        
           java.io.File SystemFile2 = new java.io.File(system_file);
           EditDOM.setDOM(SystemFile2);
           tableName = EditDOM.getDOMTableValueFor(editName,"table");
           sqltableName = EditDOM.getDOMTableValueFor(editName,"sqltable",tableName);
        
           //This opens up the schema file to get information on table to update
           String schemaFileName = super.Sys.getDOMTableValueFor("configuration","xmlschema");
           AuditTrail.println("[000] GeneralImport Schema XML: " + schemaFileName);        
           java.io.File schemaFile = new java.io.File(schemaFileName);
           TableDOM.setDOM(schemaFile);

           AuditTrail.println("[000] GeneralImport Table name: " + tableName);        
           schemaElement = TableDOM.getElement(tableName);
           columns = schemaElement.getChild("columns");
           rowKey = schemaElement.getChild("key");

           sqlMethod = super.Sys.getDOMTableValueFor("sql","insertmethod");
           sqlLockTable = super.Sys.getDOMTableValueFor("sql","locktablesql","");     //not used yet
           sqlUnlockTable = super.Sys.getDOMTableValueFor("sql","unlocktablesql",""); //noteused yet

           db.setSQLDateType(super.Sys.getDOMTableValueFor("sql","datesqlformat"));
           db.setSQLDateStr(super.Sys.getDOMTableValueFor("sql","datesql"));
           db.setSQLNo(super.Sys.getDOMTableValueFor("sql","nosql"));
           db.setSQLYes(super.Sys.getDOMTableValueFor("sql","yessql"));
           db.setAcceptNulls(super.Sys.getDOMTableValueFor("sql","acceptnulls"));
           db.setSQLBlankDate(super.Sys.getDOMTableValueFor("sql","blankdate"));
           db.setSQLReservedWordList(super.Sys.getDOMTableValueFor("sql","reservedwordlist"));
        
           if (openFile(journalName, false) != 0 )  //Used to hold the audit trail for review
           {
              AuditTrail.println("[500] GeneralImport setInputFile failed: cannot open output file: " + journalName); 
           }
           AuditTrail.println("[000] GeneralImport setInputFile finished");
        } else
        { 
            AuditTrail.println("[500] GeneralImport setInputFile failed: cannot open input file: " + inputFileName); 
        }
    }
    
    public void init(ess.AdisoftDOM sys, Connection x)
    {
        // init(sys, x);
        setSystemDOM(sys);
        setConnection(x);
    }
    
    public void setConnection(Connection x) 
    {
        super.connection = x;
        db.setConnection(x);
        systemTable.setConnection(x);
    }
    public void setSQLTerminator(String x)
    {
        db.setSQLTerminator(x);
        systemTable.setSQLTerminator(x);
    }

    public void setSystemDOM(ess.AdisoftDOM sys)
    {
        super.Sys = sys;
    }
       
    public void close()
    {
    }
    
    public Element getTableSchema() {
    	return schemaElement;
    }
    
    public String getTableName()
    {
        return tableName;
    }
    
    public void setColumnsWanted(String[] x)
    {
        columnsWanted = x;  //array that has the column names from export.xml
    }
    
    public String[] getColumnsWanted()
    {
        return columnsWanted;
    }
    
    public void setFixedColumnNames(String[] x)
    {
        fixedColumnNames = x;  //array that has the column names from export.xml
    }
    
    public String[] getFixedColumnNames()
    {
        return fixedColumnNames;
    }
    
    public void setFixedColumnValues(String[] x)
    {
        fixedColumnValues = x;  //array that has the column names from export.xml
    }
    
    public String[] getFixedColumnValues()
    {
        return fixedColumnValues;
    }
    
    public void setAttributesWanted(String[] x)
    {
        attributesWanted = x;  //List each attribute against columnsWanted array
    }
    
    public String[] getAttributesWanted()
    {
        return attributesWanted;
    }
    
// These are used with simple flat files    

    public void setSuperColumnNames() {
        
       AuditTrail.println("[000] setSuperColumnNames - simple");
       int x = fixedColumnNames.length;
       int y = columnsWanted.length; 
       superColumnNames = new String[x + y];
       for (int i = 0; i < x; i++) {
         superColumnNames[i] = fixedColumnNames[i];
       }
       for (int i = 0; i < y; i++) {
         superColumnNames[x + i] = columnsWanted[i];
       }
    }

    public String[] getSuperColumnNames() {
        if (superColumnNames == null) setSuperColumnNames();
        return superColumnNames;
    }

    public String[] getSuperColumnValues(String[] variableColumnValues) {
        
       int x = fixedColumnValues.length;
       int y = variableColumnValues.length; 
       String[] superArray = new String[x + y];
       for (int i = 0; i < x; i++) {
         superArray[i] = fixedColumnValues[i];
       }
       for (int i = 0; i < y; i++) {
         superArray[x + i] = variableColumnValues[i];
       }
       return superArray;
    }

// These are used with vertical files and require a list of elements that have elements 
// named for each column and the value as text.

    public void setSuperColumnNames(java.util.List xx) {
        
      
       Element zz; //Record type element - see import.xml "group"
       java.util.List columns;
       Element columnName;
       AuditTrail.println("[000] setSuperColumnNames - vertical");  
       int x = fixedColumnNames.length;
       int y = 0;
       for (int i = 0; i < xx.size(); i++) {
          zz = (Element) xx.get(i);
          columns = zz.getChildren();
          y = y + columns.size();
       }
       
       superColumnNames = new String[x + y]; //need to figure out the total here.
       
       for (int i = 0; i < x; i++) {
         AuditTrail.println("[000] Super from fixed: " + fixedColumnNames[i]);        
         superColumnNames[i] = fixedColumnNames[i];
       }
       int k = x;
       for (int i = 0; i < xx.size(); i++) {
          zz = (Element) xx.get(i);
          columns = zz.getChildren();
          AuditTrail.println("[000] Super additional items");          
          for (int j = 0; j < columns.size(); j++) {
              columnName = (Element) columns.get(j);
              AuditTrail.println("[000] Super from list: " + columnName.getName());
              superColumnNames[k++] = columnName.getName();
          }
       }
    }

    public String[] getSuperColumnNames(java.util.List xx) {
        if (superColumnNames == null) setSuperColumnNames(xx);
        return superColumnNames;
    }

    public String[] getSuperColumnValues(java.util.List xx) {
        
       Element zz; //Record type element - see import.xml "group"
       java.util.List columns;
       Element columnName;
       
       int x = fixedColumnNames.length;
       String[] superArray = new String[superColumnNames.length];
       
       for (int i = 0; i < x; i++) {
//AuditTrail.println("[---] Values from fixed: " + fixedColumnNames[i]);        
         superArray[i] = fixedColumnValues[i];
       }
       int k = x;
       for (int i = 0; i < xx.size(); i++) {
          zz = (Element) xx.get(i);
          columns = zz.getChildren();
//AuditTrail.println("[---] Value additional items");          
          for (int j = 0; j < columns.size(); j++) {
              columnName = (Element) columns.get(j);
//AuditTrail.println("[---] Value from list: " + columnName.getText());
              superArray[k++] = columnName.getText();
          }
       }
       return superArray;
    }

    public String[] getRecordElements(java.util.List xx, String typeWanted) {
        
//Need to break this into two items - one for the element and one for the array..      
       Element zz; //Record type element - see import.xml "group"
       java.util.List columns;
       Element columnElement;
       Element columnName;
       String recordType;
       String[] retList = null;
       
       for (int i = 0; i < xx.size(); i++) {
          zz = (Element) xx.get(i);
          columns = zz.getChildren();
          recordType = zz.getAttributeValue("recordtype");
          if (recordType.equalsIgnoreCase(typeWanted))
          {
             int cSize = columns.size();
             retList = new String[cSize];
             for (int j = 0; j < cSize; j++) {
                columnName = (Element) columns.get(j);
                retList[j] = columnName.getName();
             }
          }
       }
       return retList;
    }

    public String[] getRecordFormats(String[] wanted, String[] masterNames, String[] masterFormats) {
      
       String[] retList = new String[wanted.length];
       
       for (int i = 0; i < wanted.length; i++) {
          for (int j = 0; j < masterNames.length; j++) {
             if (wanted[i].equalsIgnoreCase(masterNames[j]))
             {
                retList[i] = masterFormats[j];
                j = masterNames.length;
             }
          }
       }
       return retList;
    }



// ------------------------

    public void setFixedColumnsAndValues(AdisoftDOM ICDOM)
    {
        String title_of_import;
        Element r;
        Element aColumn;
        
        Element x = ICDOM.getRootElement();
        java.util.List imports = x.getChildren("method");
        for (int i = 0; i < imports.size(); i++) {
           r = (org.jdom.Element) imports.get(i);
           title_of_import = r.getChild("name").getText();
           if (title_of_import.equalsIgnoreCase(importMethod))
           {
             Element z = r.getChild("defaults"); 
             java.util.List defaults = z.getChildren();
             int newDimension = defaults.size();
             String[] newColumnNames = new String[newDimension]; 
             String[] newColumnValues = new String[newDimension];
             for (int j = 0; j < newDimension; j++) {
                aColumn = (org.jdom.Element) defaults.get(j);
                newColumnNames[j] = aColumn.getName();
                newColumnValues[j] = aColumn.getText();
             }
             setFixedColumnNames(newColumnNames);
             setFixedColumnValues(newColumnValues);
           }
        } 
    }

    public String getFromControlFile(String SpecificItem)
    {
        String retVal = "";
        String mName = "";
        Element r;
        Element x = ImportControlDOM.getRootElement();
        java.util.List imports = x.getChildren("method");
        for (int i = 0; i < imports.size(); i++) {
           r = (org.jdom.Element) imports.get(i);
           mName = r.getChild("name").getText();
           if (mName.equalsIgnoreCase(importMethod))
           {
             Element z = r.getChild(SpecificItem); 
             retVal = z.getText();
             i = imports.size();
           }
        } 
        return retVal;
    }
    
    public java.util.List getListFromControlFile(String SpecificItem)
    {
        java.util.List retVal = null;
        String mName = "";
        Element r;
        Element x = ImportControlDOM.getRootElement();
        java.util.List imports = x.getChildren("method");
        for (int i = 0; i < imports.size(); i++) {
           r = (org.jdom.Element) imports.get(i);
           mName = r.getChild("name").getText();
           if (mName.equalsIgnoreCase(importMethod))
           {
             retVal = r.getChildren(SpecificItem); 
             i = imports.size();
           }
        } 
        return retVal;
    }
    
    public String[] getRow()
    {
        return getRow(columnsWanted, true);
    }
    
    public void setImportMethod(String x)
    {
        importMethod = x;
    }

    public String getImportMethod()
    {
        return importMethod;
    }

    public void setImportDescFile(String x)
    {
        importDescFile = x;
    }

    public void setImportDate(String x, String dateFormat)
    {
        StandardDate.setDate(StandardDate.getDateFromStr(x,dateFormat));
        AuditTrail.println("[000] GeneralImport setImportDate: " + x + " (" + dateFormat + "), " + CustomDate.getExportDate(StandardDate.getDate()));
        
    }

    public void setImportDate(String x)
    {
        StandardDate.setDate(CustomDate.getDateFromStr(x));
        AuditTrail.println("[000] GeneralImport setImportDate: " + x + ", " + CustomDate.getExportDate(StandardDate.getDate()));
        
    }

    public void setDefaultInputDescFile(ess.AdisoftDOM sys) 
    {
        //inputFileName = sys.getDOMTableValueFor("configuration","xmlexport","c:/ess/xmls/export.xml"); 
        setInputFileName(sys.getDOMTableValueFor("configuration","xmlexport","c:/ess/xmls/export.xml")); 
     }
    

    public void setEditName(String x)
    {
        editName = x;
    }
    
    public void setInputFileName(String x)
    {
        inputFileName = x;
    }
    
    public String[] adjust2InternalFormat(String[] x, String[] format)
    {
        final String[] dataTypes = {"packeddollar","numeric","packed","yyyymmdd","mdy","yesno","alpha","simplelanguage","boolean"};
        final int PACKEDDOLLAR = 0;
        final int YYYYMMDD = 3;
        final int MDY = 4;
        final int YESNO = 5;
        final int LANG = 7;
        
        String value;
        for (int i = 0; i < x.length; i++)
        {
           value = x[i]; 
           switch(Utilities.findString( format[i], dataTypes))
           {
              case PACKEDDOLLAR:
                long workVar = Long.valueOf(value).longValue();
                value = makeLong2Currency(workVar);
                x[i] = value.trim();
                break;
                
              case YYYYMMDD:
                java.util.Date dd = CustomDate.getDateFromExportDate(value);
                value = CustomDate.getSimpleDate(dd);
                x[i] = value;
                break;
                
              case MDY:
                  java.util.Date ddd = CustomDate.getDateFromExcelDate(value);
                  value = CustomDate.getSimpleDate(ddd);
                  x[i] = value;
                  break;
                
              case YESNO:
                if (value.equalsIgnoreCase("YES"))
                { 
            	   value = "1";
                } else
                {
            	   value = "0";
                }
                x[i] = value;
                break;
               
              case LANG:
            	  if (value.equalsIgnoreCase("f"))
            	  {
            		  value = "fr";
            	  } else if (value.equalsIgnoreCase("e"))
            	  {
            		  value = "en";
            	  }
            	  x[i] = value;
            	  break;
           }  
            
        }
        return x;
    
    }
    
    public String[] adjust4SuppliedDefaults(String[] x)
    {
        final String[] dataTypes = {"$date$", "$xref$"};
        final int IMPORT_DATE = 0;
        final int IMPORT_XREF = 1;
        String value;
        String replaceThis;
        for (int i = 0; i < x.length; i++)
        {
           value = x[i]; 
           for (int k = 0; k < dataTypes.length; k++) 
           {
              // just list items to replace right here
              // next two are for replacing $date$
        	   switch (k) {
        	   		case IMPORT_DATE:
        	   			value = Utilities.replaceStr(value, dataTypes[k], CustomDate.getSimpleDate(StandardDate.getDate()));
        	   			x[i] = value.trim();
        	   			break;
        	   		case IMPORT_XREF:
        	   			if (value.indexOf(dataTypes[k]) > -1) {
        	   				replaceThis = systemTable.getReferenceString(systemTable.getSystemIncrement("XREF_SEQUENCE"), 8, "");
        	   				value = Utilities.replaceStr(value, dataTypes[k], replaceThis);
        	   				x[i] = value.trim();
        	   			}
        	   			break;
        	   }
           }  
        }
        return x;
    }
    
    public String getSingleAttribute4Column(String columnName)
    {
        String retVal = "";
        for (int i = 0; i < columnsWanted.length; i++)
        {
            if (columnName.equalsIgnoreCase(columnsWanted[i]))
            {
                retVal = attributesWanted[i];
                i = columnsWanted.length;
            }
        }
        return retVal;
    }
    
    public String getSingleAdjValueForColumn(String columnName, String columnValue)
    {
        String columnType = getSingleAttribute4Column(columnName);
        String x[] = new String[1];
        String y[] = new String[1];
        x[0] = columnValue;
        y[0] = columnType;
        x = adjust2InternalFormat(x, y);
        return x[0];
    }
        
}  //General Import

