/*
GeneralPayment.java - EOD A/P feed creation   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.util.*;
import java.sql.*;

import org.jdom.Element;

public class GeneralPayment extends FeedAPI {
    // This class is used instantiating feeds in the EOD process
    private String outFileName = "SimplePayment.dat";
    private String renameFile = null;
    
    public void setOutFile(String x)
    {
        if (x != null) {
        	CustomDate zdate = new CustomDate();
            outFileName = Utilities.replaceStr(x, "$date$", CustomDate.getExportDate(zdate.getDate()) );;
        }
    }
    
    public void setRenameFile(String x)
    {
        if (x != null) {
        	CustomDate zdate = new CustomDate();
            renameFile = Utilities.replaceStr(x, "$date$", CustomDate.getExportDate(zdate.getDate()) );;
        }
    }

    public void init(ess.AdisoftDOM sys, Connection x)
    {
      //CustomDate dt = new CustomDate();  
      AuditTrail.println("[000] GeneralPayment.start" );
      Sys = sys;
      
      Element endofday = Sys.getElement("endofday");
      Element generalPayment = endofday.getChild("generalPayment");
      setDelimitStr(generalPayment.getAttributeValue("delimiter"));     
      setQuotes(generalPayment.getAttributeValue("quotes"));
      setQuoteDate(generalPayment.getAttributeValue("quoteDate"));
      setOutFile(generalPayment.getAttributeValue("file"));
      setDateFormat(generalPayment.getAttributeValue("dateFormat"));
      setRenameFile(generalPayment.getAttributeValue("rename"));
 
      connection = x;
      this.openFile(outFileName);
      //this.println("GL Date:" + dt.logDate());
      //Add logic to bring in the ACCOUNT and CHARGE tables for generating 
      //GL account numbers
    }
    
    public void post(PersonnelTable p, Report r)
    {
      ArrayList x = new ArrayList();    //used to match General payment
        
      String[] xline = new String[36];   //JH 2008-02-21
      Arrays.fill(xline, "");
      xline = addArray2Array(xline,r.Rep.getDeNormal(),0);
      String[] z = p.getACHPaymentInfo();
/*    0 name, 
      1 bankABA 
      2 accountNumber 
      3 isSavingsAccount 
      4 isPreNoted 
      5 preNoteDate 
      6 paymentmethod 
      7 active}; 
*/

      String[] persInfo = new String[8];
      int k = 0;
      persInfo[k++] = getFlat("USER_ACTIVE",z[7],false) ;   //0 
      persInfo[k++] = getFlat("USER_CASH",z[6],true) ;  //1
      persInfo[k++] = getFlat("USER_BANKCODE",z[1],true) ;  //2
      persInfo[k++] = getFlat("USER_BANKACCT",z[2],true) ;   //3
      persInfo[k++] = getFlat("USER_SAVINGS",z[3],false) ;  //4
      persInfo[k++] = getFlat("USER_PRENOTE",z[4],false) ;   //5
      persInfo[k++] = getFlat("USER_PRE_DATE",z[5],true) ;  //6
      persInfo[k++] = getFlat("REIMBURSE_CHECK",r.getReimburse(),false) ;  //7

      xline = addArray2Array(xline,persInfo,28);  //JH 2008-02-21
     
      x.add(xline); 
      
      String D;
      String line;
      for (int i = 0; i < x.size(); i++)
      {
    	  
        String[] lineItem = (String[]) x.get(i);
        D = "";
        line = "";
        for (int j = 0; j < lineItem.length; j++)
        {
            line += D + lineItem[j];
            D = delimitStr;
        }
        this.println(line);
      }
      
      AuditTrail.println("[000] GeneralPayment.post: " + r.getVoucherNumber() );
    }

    private String getFlat(String Name, String Value, boolean treatAsString ) {
      final String Delimiter = "";
      final String Quote = "\""; 
      String retVal;
      if (Value.equals("")) {
        retVal = "null";  
      } else {
        if ( treatAsString){
          retVal = Delimiter + Quote  + Utilities.getFlatString(Value) + Quote;            
        } else {
          retVal = Delimiter + Value;  
        }
      }    
      return retVal;
    } 
    
    private String[] addArray2Array(String[] array1, String[] array2, int startIndex)
    {
      for (int i = 0; i < array2.length; i++)
      {
        array1[startIndex + i] = array2[i];
      }
      return array1;
    }
  

    public void close()
    {
      AuditTrail.println("[000] GeneralPayment.close" );
      //this.println("End of file");
      this.setClose();
    }
}