/*
GuideObject.java - used in guideline checking  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;
import java.util.*;

public class GuideObject {
    
    static AuditTrail Logfile;
    
    String category;
    String method;
    Vector locations = new Vector();
    double amount = 0;
    double weekEndAmount = 0;
    String currency = "";
    double rate = (double) 1.00;
    String company = "";
    
    public GuideObject (String category, String method, String location, double amount, String currency, double rate, String company)
    {
        this.category = getTrim(category);
        method = getTrim(method);
        if (method.equals("WEEKEND") == true)
            this.method = "DAILY";
        else
            this.method = method;
                    
        this.currency = getTrim(currency);
        this.rate = rate;
        this.company = company;
        addLocation(method, getTrim(location), amount);
    }

    private String getTrim(String x) {
      if (x == null) return "";
      return x.trim();
    }
 
    public void addLocation(String method, String location, double amount)
    {
        if (location.length() == 0)
        {
            if (method.equals("WEEKEND") == true)
                this.weekEndAmount = amount;
            else
                this.amount = amount;
        }
        else
            locations.addElement(new Location(method, location, amount));
    }
    
    public void setRate(double x)
    {
        rate = x;
    }
    
    public void setCurrency(String x)
    {
        currency = x;
    }
    
    public String getCurrency()
    {
        return currency;
    }
        
    public void setCompany(String x)
    {
        company = x;
    }
    
    public String getCompany()
    {
        return company;
    }
        
    public String getCategory()
    {
        return(category);
    }
        
    public String getMethod()
    {
        return(method);
    }
        
    public double getAmount()
    {
        return(((double) 1/ rate) * amount);
    }
        
    public double getAmountNoRate()
    {
        return(amount);
    }
        
    public int findLocation(String location)
    {
        location = getTrim(location);
        
        int result = -1;
        
        if ((location == null) ||(location.length() == 0))
            result = -1;
        else
        {
            int i = 0;
            String city;
            
            while ((i < locations.size()) && (result == -1))
            {
                city = ((Location) locations.elementAt(i)).getLocation();
                if (city.equalsIgnoreCase(location)) 
                {
                    result = i;
                    i = locations.size();
                } else
                {
                    i++;
                }
            }
        }
        
        return(result);
    }
    
    public double getAmount(String location)
    {
        double locAmount = 0;
        int i = findLocation(location);
        if (i == -1)
        {
            locAmount = amount;
        } else
        {
            locAmount = ((Location) locations.elementAt(i)).getAmount();
        }   
        locAmount = ((double) 1/ rate) * locAmount;            
            
        return(locAmount);
    }
        
    public double getWeekEndAmount(String location)
    {
        double locAmount = 0;
        
        int i = findLocation(location);
        if (i == -1)
        {
            locAmount = weekEndAmount > 0 ? weekEndAmount : amount;
        } else {
            locAmount = ((Location) locations.elementAt(i)).getWeekEndAmount();
        }
        
        locAmount = ((double) 1/ rate) * locAmount;

        return(locAmount);
    }
        
    public boolean isSame(String category, String method)
    {
        boolean result = false;
            
        if (category.equals(this.category) == true)
        {
            String testMethod = method.equals("WEEKEND") ? "DAILY" : method;
            if (testMethod.equals(this.method)) 
                result = true;
        }
        
        
        return (result);
    }
        
    public String toString()
    {
        return ("  " + category + "   " + method+ 
                "   " + amount+ 
                "  weekEndAmound: " + weekEndAmount+ "\n" 
                + locations );
    }
}   
    
class Location {
    String location;
    double amount = 0;
    double weekEndAmount = 0;
    
    public Location (String method, String location, double amount)
    {
        this.location = location;
        if (method.equals("WEEKEND") == true)
            this.weekEndAmount = amount;
        else        
            this.amount = amount;
    }
    
    public String getLocation()
    {
        return(location);
    }
    
    public double getAmount()
    {
        return(amount);
    }
    public double getWeekEndAmount()
    {
        return( weekEndAmount > 0 ? weekEndAmount : amount);
    }
        
    public String toString()
    {
        return (location + "    " + amount + "   WeekEndAmount:  " + weekEndAmount + "\n");
    }
}   
    
    
