/*
Guideline.java - checks report against guidelines - rturns status and report
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.io.*;
import java.util.*;
import java.sql.*;
import org.jdom.Document;
import org.jdom.Element;
//import org.jdom.Attribute;
//import org.jdom.DocType;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;

public class Guideline {
    public final int LINE = 1;
    public final int DAILY = 2;
    public final int NIGHTS = 3;
    public final int REPORT = 4;
    public final int TOTAL = 5;
    public final int ADVANCE = 6;
    public final int REIMBURSE = 7;
    public final int DAYS = 8;
    public final int WEEKEND = 9;
    public final int DATE = 10;
    public final int MILEAGERATE = 11;
    public final int LODGING = 12;

    
    static AuditTrail Logfile;
    
    private String[] methodNames = {"LINE", "DAILY", "NIGHTS", 
                    "REPORT", "TOTAL", "ADVANCE", "REIMBURSE",
                    "DAYS", "WEEKEND", "DATE", "MILEAGERATE", "LODGING"};
    private String guidelineStatus = "passed"; // PASS, FAILED
    private AdisoftDOM guidelineFile;
    private Document guidelineDOM;
    private Document reportDOM;
    private ExpenseReport reportObj;
    private Vector guidelineObjects = new Vector();
    private Vector reportResults = new Vector();
    String UC_guideline;
    String DbaseURL = "";
    String DbaseUser = "";
    String DbasePassword = "";
    Connection dbConnection;
    String GuideType = "Default";
    private String sqlTerminator = ";";
    private ServerSystemTable ST = new ServerSystemTable();
        
    String currency = "";
    double rate = (double) 1.00;
    String company = "";
    
    private String dateFormatStr = "MM/dd/yyyy";
    private String decimalStr = ".";
    private String separatorStr = ",";
    private ess.Language myLang = new ess.Language();


    public Guideline () {
      Logfile = new AuditTrail();
    }
    public Guideline (String url, String User, String Password) {
      Logfile = new AuditTrail();
      setDB(url, User, Password);
    }
    
    public String getGuideType() {
      return GuideType;
    }    
  
    
    public void setDB(String Url, String User, String Password) {
      DbaseUser = User;
      DbasePassword = Password;
      DbaseURL = Url;
      guidelineFile.setDB(Url, User, Password);
      guidelineFile.setSQLTerminator(sqlTerminator);
      guidelineFile.setDOMFromTable("GUIDE");
      guidelineDOM = guidelineFile.getDOM();
    }
    
    
    public void setDB(String Url) {
       setDB(Url, "", "");
    }
    
    
    public void setUpFiles() {
      guidelineFile = new AdisoftDOM();
      guidelineFile.setConnection(dbConnection);
      guidelineFile.setSQLTerminator(sqlTerminator);
      guidelineFile.setDOMFromTable("GUIDE");
      guidelineDOM = guidelineFile.getDOM();
    } 

    public void setUpFromXML(String guide_file_name) {
      //String guide_file = SysTable.getSystemString("XMLGUIDE","C:\\WORK\\"); 
      guidelineFile = new AdisoftDOM();
      java.io.File guideFile = new java.io.File(guide_file_name);
      guidelineFile.setDOM(guideFile);
      guidelineDOM = guidelineFile.getDOM();
    } 

    public void setUpCurrency(String x)
    {
        setCurrency(x);
        setUpCurrency();
    }    

    public void setUpCurrency()
    {
        ess.Currency Currency = new ess.Currency();
        Currency.setConnection(dbConnection);
        Currency.setSQLTerminator(sqlTerminator);
        Currency.setSQLStrings();
        rate = Currency.getRate(currency);
    }    

    public void setCurrency(String x)
    {
        currency = x;
    }
    
    public String getCurrency()
    {
        return currency;
    }

    public void setCompany(String x)
    {
        company = x;
    }
    
    public String getCompany()
    {
        return company;
    }
    
    public void setLanguage(String x){
      myLang.setLanguage(x);
    }

    public void setLanguage(String x, String y){
      myLang.setLanguage(x,y);
    }
    
    public void setLanguage(ess.Language x) {
    	myLang = x;
    }

    public void setDateFormat(String x) {
      dateFormatStr = x;
    }
      
    public void setDecimal(String x) {
      decimalStr = x;
    }
 
    public void setSeparator(String x) {
      separatorStr = x;
    }

    public void setConnection(Connection dbConn) {
        ST.setConnection(dbConn); 
        dbConnection = dbConn;
    } 
    
    public void setSQLTerminator(String X) {
        ST.setSQLTerminator(X); 
        sqlTerminator = X;
    }
    
    public void setSystemTable (ServerSystemTable sysTable) {
        ST = sysTable;
    }
    
    private String getEmpGuideType()
    {
        PersonnelTable P = new PersonnelTable(); 
        if (DbaseURL.equals("")) {
            P.setConnection(dbConnection);
        } else {
            P.setDB(DbaseURL,DbaseUser,DbasePassword);
        }
        P.setSQLTerminator(sqlTerminator);
        P.setSQLStrings();
        P.setPersNumInfo(reportObj.getPersNum());
        AuditTrail.println("[000] Guideline.setReport - running guidelines with " + P.guideline + " for " + reportObj.getPersNum());
        GuideType = P.guideline;
        return(P.guideline);
    }
    
    public void setReport(Document report) {
        if (DbaseURL.equals("")) {
            reportObj = new XML2Guideline(report, dbConnection, sqlTerminator);
        } else {
            //reportObj = new XML2Guideline(report, DbaseURL,DbaseUser,DbasePassword,sqlTerminator);
        }

        reportObj.setLanguage(myLang);
        reportObj.setDateFormat(dateFormatStr); 
        reportObj.setDecimal(decimalStr);
        reportObj.setSeparator(separatorStr);

        checkGuideline(getEmpGuideType());
    }
    
    public void setDBXMLReport(Document report) {
        
        if (DbaseURL.equals("")) {
            
            reportObj = new XML2Guideline();
            
            reportObj.setConnection(dbConnection, sqlTerminator);

            String[] w = {"PERS_NUM", "NAME", "PHONE", 
                "LOCATION", "DEPART", "CRE_DATE", "ADMIN1", "EMAIL", "SUB_DATE", "CURRENCY"};
            reportObj.setHeaderFieldNames(w);
            String[] x = {"PROJECT","STEPNO","CLIENTNO","CLIENT",
                "LOCATION","COMMENT","BEG_DATE","END_DATE","DAYS","NIGHTS", "PURPOSE"};
            reportObj.setPurposeFieldNames(x);
            String[] y = {"RECEIPT", "AMOUNT", "UNITS", "XRATE", "REC_DATE", "EXPENSE","COMMENT", "CHARGE", "REIMB", "BEG_DATE", "PURPOSE"};
            reportObj.setReceiptFieldNames(y);
            String[] z = {"RECEIPT", "EXPENSE", "CATEGORY", "AMOUNT","EXP_DATE", "PERSPLIT", "COMMENT", "PURPOSE", "REIMB"};
            reportObj.setExpenseFieldNames(z);
            
            reportObj.setReport(report);
            
        } else {
            //reportObj = new XML2Guideline(report, DbaseURL,DbaseUser,DbasePassword,sqlTerminator);
        }
        reportObj.setLanguage(myLang);
        reportObj.setDateFormat(dateFormatStr); 
        reportObj.setDecimal(decimalStr);
        reportObj.setSeparator(separatorStr);
        setUpCurrency(reportObj.getCurrency());
        checkGuideline(getEmpGuideType());
    }
    
    public void setReport(String voucher) {
        if (DbaseURL.equals("")) {
           reportObj = new DB2Guideline(voucher,dbConnection,sqlTerminator,ST);
        } else {
           reportObj = new DB2Guideline(voucher,DbaseURL,DbaseUser,DbasePassword,sqlTerminator,ST);
        }
        reportObj.setLanguage(myLang);
        reportObj.setDateFormat(dateFormatStr); 
        reportObj.setDecimal(decimalStr);
        reportObj.setSeparator(separatorStr);
        setUpCurrency(reportObj.getCurrency());
        checkGuideline(getEmpGuideType());
    }
    
    public void setReport(Document report, String guideline)
    {
        if (DbaseURL.equals("")) {
            reportObj = new XML2Guideline(report,dbConnection,sqlTerminator);
        } else {
            //reportObj = new XML2Guideline(report,DbaseURL,DbaseUser,DbasePassword,sqlTerminator);
        }
        reportObj.setLanguage(myLang);
        reportObj.setDateFormat(dateFormatStr); 
        reportObj.setDecimal(decimalStr);
        reportObj.setSeparator(separatorStr);
        setUpCurrency(reportObj.getCurrency());
        checkGuideline(guideline);
    }
    
    private void checkGuideline(String guideline)
    {
        final int GUIDE = 0;
        final int CATEGORY = GUIDE + 1;
        final int LOCATION = CATEGORY + 1;
        final int AMOUNT = LOCATION + 1;
        final int METHOD = AMOUNT + 1;
        final int COMMENT = METHOD + 1;
        final int COMPANY = COMMENT + 1;
        UC_guideline = guideline.toUpperCase();
        
        Element root;
        buildGuidelineObjects(guideline);
        
/*      System.out.println("guidelineObjects:  " + guidelineObjects);
        Logfile.println("[500] GuidelineObjecs:\n" + guidelineObjects + "\n\n");
*/        
        GuideObject guidelineItem;
        for (int i = 0; i < guidelineObjects.size(); i++)
        {
            guidelineItem = (GuideObject) guidelineObjects.elementAt(i);
            
            switch(getMethodId(guidelineItem.getMethod()))
            {
                case TOTAL:
                    reportObj.checkTOTAL(guidelineItem);
                    break;
                case REPORT:
                    reportObj.checkREPORT(guidelineItem);
                    break;
                case DAILY:
                    reportObj.checkDAILY(guidelineItem);
                    break;
                case LODGING:
                    reportObj.checkLODGING(guidelineItem);
                    break;
                case NIGHTS:
                    reportObj.checkNIGHTS(guidelineItem);
                    break;
                case DAYS:
                    reportObj.checkDAYS(guidelineItem);
                    break;
                case REIMBURSE:
                    reportObj.checkREIMBURSE(guidelineItem);
                    break;
                case WEEKEND:
                //Weekend is being picked up and tested by DAILY.
                //        reportObj.checkDAILY(guidelineItem);
                    break;
                case ADVANCE:
                    reportObj.checkADVANCE(guidelineItem);
                    break;
                case DATE:
                    reportObj.checkDATE(guidelineItem);
                    break;
                case MILEAGERATE:
                    reportObj.checkMILEAGERATE(guidelineItem);
                    break;
                default:
                    AuditTrail.println("[500] Method not found... " + guidelineItem.getMethod());
                    break;
            }
        }
    }
        
    public void buildGuidelineObjects(String guideline)
    {
        final int GUIDE = 0;
        final int CATEGORY = GUIDE + 1;
        final int LOCATION = CATEGORY + 1;
        final int AMOUNT = LOCATION + 1;
        final int METHOD = AMOUNT + 1;
        final int COMMENT = METHOD + 1;
        final int COMPANY = COMMENT + 1;
        UC_guideline = guideline.toUpperCase();
        Element root;
        Element tempElement;
        String[] guideFieldNames = {"GUIDE", "CATEGORY",
                    "LOCATION", "AMOUNT", "METHOD", "COMMENT", "COMPANY"};
//Always check mileagerate, if an entry exists in the systemxml file.        
        addGuideObject("", "MILEAGERATE", "", 0.0);
        root = guidelineDOM.getRootElement();
//Make sure that we have the right guideline                    
        if ((root.getName()).equals("table") && 
                    (root.getAttributeValue("name")).equals("GUIDE"))
        {
            Iterator rowSeqItr = root.getChildren().iterator();
                
            while (rowSeqItr.hasNext())
            {
                Element guideSeq = (Element)rowSeqItr.next();
//get guideline details                    
                String guide = "";
                String category = "";
                String location = "";
                double amount = 0.0;
                String method = "";
                String comment= "";
                String company = "";
                boolean checkThisItem = true;
                    
                Iterator guideItr = guideSeq.getChildren().iterator();
                while ((guideItr.hasNext()) && (checkThisItem == true)) 
                {
                    Element guideEntry = (Element)guideItr.next();
                    
                    int guideId =  Utilities.findString(guideEntry.getName(),
                                        guideFieldNames);
                                    
                    switch (guideId)
                    {
                        case GUIDE:
                            guide = (guideEntry.getText()).toUpperCase();
                            if (guide.equals(UC_guideline) == false)
                                checkThisItem = false;
                            break;
                        case CATEGORY:
                            category = (guideEntry.getText()).toUpperCase();
                            break;
                        case LOCATION:
                            location = (guideEntry.getText()).toUpperCase();
                            break;
                        case AMOUNT:
                            try{
                                amount = Double.parseDouble(guideEntry.getText());
                            }
                            catch (NumberFormatException error)
                            {
                                amount = 0;
                            }
                            break;
                        case METHOD:
                            method = (guideEntry.getText()).toUpperCase();
                            break;
                        case COMMENT:
                            comment = guideEntry.getText();
                            break;
                        case COMPANY:
                            company = guideEntry.getText();
                            break;
                        default:
                            break;
                    }
                }
                if (checkThisItem == true)
                    addGuideObject(category, method, location, amount);
                    
            }
        }
    }
    
    private GuideObject findGuideObject(String category, String method)
    {
        GuideObject guideline = null;
        
        int i = 0;
        while ((i < guidelineObjects.size())&& (guideline == null))
        {
            if (((GuideObject) guidelineObjects.elementAt(i)).isSame(category, method) == true)
                guideline = (GuideObject) guidelineObjects.elementAt(i);
            else
                i++;
        }
        return(guideline);
    }
    
    
    private void addGuideObject(String category, String method, String location, double amount)
    {
        boolean weekend = false;
        GuideObject guideline;
        
            guideline = findGuideObject(category, method);
            
        if (guideline == null)
        {
            guideline = new GuideObject(category, method, location, amount, currency, rate, company);
            guidelineObjects.addElement(guideline);
        }
        else
            guideline.addLocation(method, location, amount);
        
    }

    private int getMethodId(String method)
    {
        return(Utilities.findString(method,methodNames)+1);
    }
   
        
    public String getStatus()
    {
        return reportObj.getStatus();
    }
    
    public boolean getBooleanStatus()
    {
        return (reportObj.getBooleanStatus());
    }
    
    public String toString()
    {
        String result = "";
        result = reportObj.toString();
        return (result);                
    }
    
}
