/*
NACHAFeed.java - Creates an EOD payment feed in the NACHA format   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Attribute;
import java.util.*;
import java.sql.*;

public class NACHAFeed extends FeedAPI {
    // This class is used instantiating feeds in the EOD process
    
    private AdisoftDOM Description = new AdisoftDOM();
    private long batchTotal = 0;   // used to hold a batch debit/credit total
    private long fileTotal = 0;    // used to hold the tapes debit/credit total
    private long batchCount = 0;   // # payments/prenotes in the batch
    private long entryCount = 0;
    private long fileCount = 0;    // # payments/prenotes in the file
    private long recordCount = 0;  // # records in the file
    private long batchNumber = 0;
    private long batchHash = 0;
    private long fileHash = 0;
    private long tracenumber = 0;
    private char packCharacter = ' ';  //space to fill in normal record
    private HTMLFile htmlReport = new HTMLFile();
    protected String transferFile = "NOT_SET";
    private String sqlTerminator = ";";
    
    
    public NACHAFeed() {
    }
    
    public void setTransferFile(String x) {
        transferFile = x;
    }

    public void init(ess.AdisoftDOM sys, Connection x)
    {
      AuditTrail.println("[000] NachaFeed.init started" );
      Sys = sys;
      connection = x;
      sqlTerminator = Sys.getSQLTerminator();
      if (transferFile.equals("NOT_SET")) {
         transferFile = Sys.getDOMTableValueFor("endofday","nacha_file_name","NACHA.FIL");
         CustomDate zdate = new CustomDate();
         transferFile = Utilities.replaceStr(transferFile, "$date$", zdate.getExportDate(zdate.getDate()) ); 
      }
      AuditTrail.println("[000] NachaFeed.init output name: " + transferFile);         
      this.openFile(transferFile);
// put down below later      
      String nachaXMLFileName = Sys.getDOMTableValueFor("configuration","xmlnacha");
      if (!nachaXMLFileName.equals(""))
      {
        AuditTrail.println("[000] NachaFeed.init definition: " + nachaXMLFileName);
        java.io.File nachaXMLFile = new java.io.File(nachaXMLFileName);
        Description.setDOM(nachaXMLFile);

        String layoutStart = Description.getDOMTableValueFor("configuration","offset","1");
        recordLayoutStart = Integer.parseInt(layoutStart);
        
        String calendarXMLFileName = Sys.getDOMTableValueFor("configuration","xmlcalendar");
        AuditTrail.println("[000] NachaFeed.init: " + calendarXMLFileName);
        java.io.File calendarXMLFile = new java.io.File(calendarXMLFileName);
        AdisoftDOM calendarDOM = new AdisoftDOM();
        calendarDOM.setDOM(calendarXMLFile);
        holidays = calendarDOM.getDOMTableArrayFor("bankholidays", "date");
        //opening up the report for html output.
        String htmlOutPutFileName = Sys.getDOMTableValueFor("configuration","htmloutput");
        //Need to do a replace for the $date$
        htmlReport.setHTMLFile(htmlOutPutFileName);
        // end of 'put down below later'

        Element fileheader = Description.getElement("fileheader");
        Attribute recordlength = fileheader.getAttribute("length");

        try {
          recordCount = recordCount + 1;  
          char[] record = new char[recordlength.getIntValue()];
          Arrays.fill(record, packCharacter);
          setField(record,fileheader.getChild("recordtypecontrol"));
          setField(record,fileheader.getChild("prioritycode"));
          setField(record,fileheader.getChild("immediatedestination"));
          setField(record,fileheader.getChild("immediateorigin"));
          setField(record,fileheader.getChild("filecreationdate"),currentDate);
          setField(record,fileheader.getChild("filecreationtime"),currentDate);
          setField(record,fileheader.getChild("fileidmodifier"));
          setField(record,fileheader.getChild("recordsize"));
          setField(record,fileheader.getChild("blockingfactor"));
          setField(record,fileheader.getChild("formatcode"));
          setField(record,fileheader.getChild("immediatedestinationname"));
          setField(record,fileheader.getChild("immediateoriginname"));
          setField(record,fileheader.getChild("referencecode"));
          this.println(String.valueOf(record));

          batchheader(); 
        } catch (org.jdom.DataConversionException e) {
          this.println("Error in file header creation - length attribute!!!");
          AuditTrail.println("[500] NachaFeed.init Error in file header creation - check length attribute");
        }
      } else {
        this.println("Error in file header creation - nacha.xml not found!!!");
        AuditTrail.println("[500] NachaFeed.init Error in file header creation - nacha.xml not found");
      }
    }
    
    
    public void post(PersonnelTable p, Report r)
    {
    /* current layout of the array
    From Personnel Table */
      int locInt = 0;
      final int name = locInt++;
      final int bankABA = locInt++;
      final int accountNumber = locInt++;
      final int isSavingsAccount = locInt++;
      final int isPreNoted = locInt++;
      final int preNoteDate = locInt++;
      final int paymentmethod = locInt++;
      final int active = locInt++;
      final int reimburseAmount = locInt++;
      
      if (p.getReimbursementType().equalsIgnoreCase("ACH") && r.getReimburseAmount() > (double) 0.009)
         {
         AuditTrail.println("[000] NACHAFeed.post: " + r.getVoucherNumber() );
         String[] PaymentInfo = r.getACHPaymentInfo();
         r.setReporterInfo(p);
      
         //Logfile.println("[---] NACHAFeed.post name: " +  PaymentInfo[name]);
         //Logfile.println("[---] NACHAFeed.post bankABA: " +  PaymentInfo[bankABA]);
         //Logfile.println("[---] NACHAFeed.post accountNumber: " +  PaymentInfo[accountNumber]);
         //Logfile.println("[---] NACHAFeed.post isSavingsAccount: " +  PaymentInfo[isSavingsAccount]);
         //Logfile.println("[---] NACHAFeed.post isPreNoted: " +  PaymentInfo[isPreNoted]);
         //Logfile.println("[---] NACHAFeed.post preNoteDate: " +  PaymentInfo[preNoteDate]);
         //Logfile.println("[---] NACHAFeed.post paymentmethod: " +  PaymentInfo[paymentmethod]);
         //Logfile.println("[---] NACHAFeed.post active: " +  PaymentInfo[active]);      
         //Logfile.println("[---] NACHAFeed.post reimburseAmount: " +  PaymentInfo[reimburseAmount]);
      
         Element fileheader = Description.getElement("entrydetail");
         Attribute recordlength = fileheader.getAttribute("length");

         try {
           char[] record = new char[recordlength.getIntValue()];
           Arrays.fill(record, packCharacter);
        
// removed batchcount inc from here on 06/05/2006 - jh
           fileCount = fileCount + 1;
           recordCount = recordCount + 1;
           tracenumber = tracenumber + 1;
           entryCount = entryCount + 1;
        
           setField(record,fileheader.getChild("recordtypecode"));
           if (PaymentInfo[isSavingsAccount].equals("1"))
           {
               setField(record,fileheader.getChild("transactioncode"),"32");
           } else {
               setField(record,fileheader.getChild("transactioncode"),"22");
           }   
           String routingCode = addCheckDigit(PaymentInfo[bankABA]);
           setField(record,fileheader.getChild("receivingdfiidentificationnumber"),routingCode);
           setField(record,fileheader.getChild("rdfiaccountnumber"),PaymentInfo[accountNumber]);
           setField(record,fileheader.getChild("amount"),PaymentInfo[reimburseAmount]);
           setField(record,fileheader.getChild("identificationnumber"),p.getPersNum());
           setField(record,fileheader.getChild("individualname"),PaymentInfo[name]);
           setField(record,fileheader.getChild("discretionarydata"));
           setField(record,fileheader.getChild("addendarecordindicator"));
           setField(record,fileheader.getChild("tracenumber1"));
           setField(record,fileheader.getChild("tracenumber2"), tracenumber);
           this.println(String.valueOf(record));

           batchTotal = addString2LongforCurrency(batchTotal,PaymentInfo[reimburseAmount]); 
           fileTotal = addString2LongforCurrency(fileTotal,PaymentInfo[reimburseAmount]);
           if (routingCode.length() == 9)
           {
             batchHash = addString2Long(batchHash,routingCode.substring(0,8));
             fileHash = addString2Long(fileHash,routingCode.substring(0,8));
           }
        
         } catch (org.jdom.DataConversionException e) {
           this.println("Error in file header creation - length attribute!!!");
           AuditTrail.println("[500] NachaFeed.post Error in post - Data conversion");
         }
      } else {
        AuditTrail.println("[000] NACHAFeed.post item not posted: " + r.getVoucherNumber() );
      }
    }
    
    public void prenote(PersonnelTable p)
    {
    /* current layout of the array
    From Personnel Table */
      int locInt = 0;
      final int name = locInt++;
      final int bankABA = locInt++;
      final int accountNumber = locInt++;
      final int isSavingsAccount = locInt++;
      final int isPreNoted = locInt++;
      final int preNoteDate = locInt++;
      final int paymentmethod = locInt++;
      final int active = locInt++;
            
      if (p.getReimbursementType().equalsIgnoreCase("PRENOTE") )
         {
         AuditTrail.println("[000] NACHAFeed.prenote: " + p.getPersNum() );
         String[] PaymentInfo = p.getACHPaymentInfo() ;
               
         //Logfile.println("[---] NACHAFeed.post name: " +  PaymentInfo[name]);
         //Logfile.println("[---] NACHAFeed.post bankABA: " +  PaymentInfo[bankABA]);
         //Logfile.println("[---] NACHAFeed.post accountNumber: " +  PaymentInfo[accountNumber]);
         //Logfile.println("[---] NACHAFeed.post isSavingsAccount: " +  PaymentInfo[isSavingsAccount]);
         //Logfile.println("[---] NACHAFeed.post isPreNoted: " +  PaymentInfo[isPreNoted]);
         //Logfile.println("[---] NACHAFeed.post preNoteDate: " +  PaymentInfo[preNoteDate]);
         //Logfile.println("[---] NACHAFeed.post paymentmethod: " +  PaymentInfo[paymentmethod]);
         //Logfile.println("[---] NACHAFeed.post active: " +  PaymentInfo[active]);      
         
         Element fileheader = Description.getElement("entrydetail");
         Attribute recordlength = fileheader.getAttribute("length");

         try {
           char[] record = new char[recordlength.getIntValue()];
           Arrays.fill(record, packCharacter);
        
           // batchCount = batchCount + 1;
           fileCount = fileCount + 1;
           recordCount = recordCount + 1;
           tracenumber = tracenumber + 1;
           entryCount = entryCount + 1;
        
           setField(record,fileheader.getChild("recordtypecode"));
           if (PaymentInfo[isSavingsAccount].equals("1"))
           {
               setField(record,fileheader.getChild("transactioncode"),"32");
           } else {
               setField(record,fileheader.getChild("transactioncode"),"22");
           }   
           String routingCode = addCheckDigit(PaymentInfo[bankABA]);
           setField(record,fileheader.getChild("receivingdfiidentificationnumber"),routingCode);
           setField(record,fileheader.getChild("rdfiaccountnumber"),PaymentInfo[accountNumber]);
           setField(record,fileheader.getChild("amount"),"0.00");
           setField(record,fileheader.getChild("identificationnumber"),p.getPersNum());
           setField(record,fileheader.getChild("individualname"),PaymentInfo[name]);
           setField(record,fileheader.getChild("discretionarydata"));
           setField(record,fileheader.getChild("addendarecordindicator"));
           setField(record,fileheader.getChild("tracenumber1"));
           setField(record,fileheader.getChild("tracenumber2"), tracenumber);
           this.println(String.valueOf(record));

           if (routingCode.length() == 9)
           {
             batchHash = addString2Long(batchHash,routingCode.substring(0,8));
             fileHash = addString2Long(fileHash,routingCode.substring(0,8));
           }
        
         } catch (org.jdom.DataConversionException e) {
           this.println("Error in file header creation - length attribute!!!");
           AuditTrail.println("[500] NachaFeed.prenote Error in post - Data conversion");
         }
      } else {
        AuditTrail.println("[000] NACHAFeed.prenote item not posted: " + p.getPersNum() );
      }
    }
    
    public void batchPrenote() 
    {
      PersonnelTable p = new PersonnelTable();
      p.setConnection(connection);
      p.setSQLTerminator(sqlTerminator);
      p.setSQLStrings();
      XML2ESS r = new XML2ESS();
      String personnelNumber = "";
      
      String runPrenote = Sys.getDOMTableValueFor("endofday","prenote"); 
      if (runPrenote.equalsIgnoreCase("YES"))
      {
        AuditTrail.println("[000] NACHAFeed.batchPrenote " );
        AdisoftDbase db = new AdisoftDbase();
        db.setConnection(connection);
        
        String SQLCommand = Sys.getDOMTableValueFor("achprenote","listsql");
        String SQLUpdate = Sys.getDOMTableValueFor("achprenote","batchupdatesql");
        int i = 0;
        
        if (db.setResultSet(SQLCommand)) 
        { 
           try { 
             batchheader();
             do {
               personnelNumber = p.getTrim(db.myResult.getString(1));
               if (p.setPersNumInfo(personnelNumber))
               {
                 r.setReporterInfo(p);
                 AuditTrail.println("[000] NACHAFeed.batchPrenote for " + personnelNumber);
                 post(p,r);
                 i++;
               }
             } while (db.myResult.next());
             batchcontrol();
           
             String newAsOfDate = CustomDate.xBaseDate.format(currentDate.date);
             String SQLDate = Sys.getDOMTableValueFor("sql","datesql");
             String SQLType = Sys.getDOMTableValueFor("sql","dateformat");
             if (SQLType.equals("MM/DD/YYYY")) { 
               SQLDate = db.SQLReplace(SQLDate,"$date$",CustomDate.simpleDate.format(currentDate.date));
             } else { //generate YYYY-MM-DD
               SQLDate = db.SQLReplace(SQLDate,"$date$",newAsOfDate);
             } 
             SQLUpdate = db.SQLReplace(SQLUpdate, "$date$", SQLDate);
             int SQLResult = db.doSQLExecute(SQLUpdate);
             if ( SQLResult > -1) {
                if (SQLResult == i)
                {
                    AuditTrail.println("[000] NACHAFeed.batchPrenote prenotes issued: " + Integer.toString(i));
                } else
                {
                    AuditTrail.println("[500] NACHAFeed.batchPrenote problem - issued " + Integer.toString(i) + ", but updated " + Integer.toString(SQLResult)); 
                }
             } else
             {
                AuditTrail.println("[500] NACHAFeed.batchPrenote - status update error");
             }
           
           } catch (SQLException e)
           {
              AuditTrail.println("[500] NACHAFeed.batchPrenote database access error: " + personnelNumber);
              AuditTrail.println(e.toString());
           }
        }
      }
    }

    public void close()
    {
      AuditTrail.println("[000] NACHAFeed.close" );
      batchcontrol();
      batchPrenote();
      filecontrol();
      this.setClose(); //20060823 JH
    }
    
    //Is this used anywhere???
    public void closeRelease()
    {
      AuditTrail.println("[000] NACHAFeed.closeRelease" );
      filecontrol();
      this.setClose(); //20060823 JH      
    }
    
    // Do I want to abstract the following also??
    public String addCheckDigit(String abaNumber)
    {
        String retVal = "";
        final String[] convert2Str = {"0","1","3","4","5","6","7","8","9","0"};
        final int[] weight = {3,7,1,3,7,1,3,7};
        if (abaNumber.length() == 8)
        {
            int check = 0;
            for (int i = 0; i < 8; i++)
            {
               check += Integer.parseInt(abaNumber.substring(i,i+1)) * weight[i];
            }
            check = 10 - (check % 10);
            retVal = abaNumber + convert2Str[check];
        } else if (abaNumber.length() == 9)
        {
            retVal = abaNumber;
        }
        return retVal;
    }
    
    public void batchheader() 
    {
       Element fileheader = Description.getElement("batchheader");
       Attribute recordlength = fileheader.getAttribute("length");
        
       batchTotal = 0;   
       entryCount = 0; 
       batchNumber = batchNumber + 1;
       recordCount = recordCount + 1;
       batchHash = 0;
       
       CustomDate xdate = new CustomDate();

        try {
          char[] record = new char[recordlength.getIntValue()];
          Arrays.fill(record, packCharacter);
          
          setField(record,fileheader.getChild("recordtypecode"));
          setField(record,fileheader.getChild("serviceclasscode"));
          setField(record,fileheader.getChild("companyname"));
          setField(record,fileheader.getChild("companydiscretionarydata"));
          setField(record,fileheader.getChild("companyidentification"));
          setField(record,fileheader.getChild("standardentryclasscode"));
          setField(record,fileheader.getChild("companyentrydescription"));
          setField(record,fileheader.getChild("companydescriptiondate"),xdate);
          xdate.setDate(CustomDate.addDays(xdate.getDate(),2));  
          xdate = bump4holiday(xdate);
          setField(record,fileheader.getChild("effectiveentrydate"),xdate);
          setField(record,fileheader.getChild("settlementdate"));
          setField(record,fileheader.getChild("originatorstatuscode"));
          setField(record,fileheader.getChild("originatingdfiidentification"));
          setField(record,fileheader.getChild("batchnumber"),batchNumber);
          this.println(String.valueOf(record));

          
        } catch (org.jdom.DataConversionException e) {
          this.println("Error in batch header creation - conversion problem!!!");
          AuditTrail.println("[500] NachaFeed.batchheader Error in file header creation -  conversion problem");
        } catch (Exception e) {
          this.println("Error in batch header creation - general exception!!!");
          AuditTrail.println("[500] NachaFeed.batchheader Error in file header creation -  general exception");
          AuditTrail.println("[500] " + e.toString());
        }
 
    }
    
    public void batchcontrol() 
    {
       Element fileheader = Description.getElement("batchcontrol");
       Attribute recordlength = fileheader.getAttribute("length");
       recordCount = recordCount + 1; 
       batchCount = batchCount + 1;
       
        try {
          char[] record = new char[recordlength.getIntValue()];
          Arrays.fill(record, packCharacter);
          
          setField(record,fileheader.getChild("recordtypecode"));
          setField(record,fileheader.getChild("serviceclasscode"));
          setField(record,fileheader.getChild("entrycount"),entryCount);
          
          String bh = Long.toString(batchHash);
          bh = "00000000" + bh;   //JH 2006-03-09
          int lbh = bh.length();
          setField(record,fileheader.getChild("entryhash"), bh.substring(lbh-10,lbh));
          
          setField(record,fileheader.getChild("totaldebitentrydollaramount"),(long) 0);
          setField(record,fileheader.getChild("totalcreditentrydollaramount"),batchTotal);
          setField(record,fileheader.getChild("companyidentification"));
          setField(record,fileheader.getChild("messageauthenticationcode"));
          setField(record,fileheader.getChild("reserved"));
          setField(record,fileheader.getChild("originatingdfiidentification"));
          setField(record,fileheader.getChild("batchnumber"),batchNumber);
          this.println(String.valueOf(record));
          
        } catch (org.jdom.DataConversionException e) {
          this.println("Error in batch control creation - conversion problem!!!");
          AuditTrail.println("[500] NachaFeed.batchcontrol Error in file control creation -  conversion problem");
        } catch (Exception e) {
          this.println("Error in batch control creation - general exception!!!");
          AuditTrail.println("[500] NachaFeed.batchcontrol Error in file control creation -  general exception");
          AuditTrail.println("[500] " + e.toString());
        }
 
    }
    public void filecontrol() 
    {
       Element fileheader = Description.getElement("filecontrol");
       Attribute recordlength = fileheader.getAttribute("length");
       recordCount = recordCount + 1;
        
        try {
          char[] record = new char[recordlength.getIntValue()];
          Arrays.fill(record, packCharacter);

          setField(record,fileheader.getChild("recordtypecode"));
          setField(record,fileheader.getChild("batchcount"),batchCount);
          long blocking = getBlockCount(recordCount);
          setField(record,fileheader.getChild("blockcount"),blocking);
          setField(record,fileheader.getChild("entrycount"),fileCount);
           
          String bh = Long.toString(fileHash);
          bh = "00000000" + bh;  //JH 2006-03-09 
          int lbh = bh.length();
          setField(record,fileheader.getChild("entryhash"), bh.substring(lbh-10,lbh));
          
          setField(record,fileheader.getChild("totaldebitentrydollaramount"),(long) 0);
          setField(record,fileheader.getChild("totalcreditentrydollaramount"),fileTotal);
          setField(record,fileheader.getChild("reserved"));
          this.println(String.valueOf(record));
          
          for (long i = recordCount; i < blocking * (long) 10; i++)
          {
            record = new char[recordlength.getIntValue()];
            Arrays.fill(record, '9');
            this.println(String.valueOf(record));
          }
          
        } catch (org.jdom.DataConversionException e) {
          this.println("Error in batch control creation - conversion problem!!!");
          AuditTrail.println("[500] NachaFeed.filecontrol Error in file control creation -  conversion problem");
        } catch (Exception e) {
          this.println("Error in batch control creation - general exception!!!");
          AuditTrail.println("[500] NachaFeed.filecontrol Error in file control creation -  general exception");
          AuditTrail.println("[500] " + e.toString());
        }
     }
    
    private long getBlockCount(long rawNumber)
    {
        long blocking = rawNumber/10;
        long modNumber = rawNumber % 10;
        if (modNumber > 0) blocking = blocking + 1;
        return blocking;
    }
}