/*
POP3Batch.java - reads messages from a pop3 account to a folder   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.net.*;
import java.io.*;
//import java.util.*;

public class POP3Batch extends POP3andSMTP implements Runnable {

// This is an extension of the POP3andSMTP object that specializes in doing a POP3 message download to
// text file in batch mode.  It is written for multithreading

   private AuditTrail Logfile = new AuditTrail();
   private String DBUrl = "";
   private ServerSystemTable systemTable;
   private String logon = "";
   private String pwd = "";
   private boolean deleteMessages = true;
   private Register reg;
   
   public POP3Batch(String DBUrl, String DBUser, String DBPassword)
   {
        this.DBUrl = DBUrl;
        systemTable = new ServerSystemTable(DBUrl, DBUser, DBPassword);
        logon = systemTable.getSystemString("pop3_logon","TEST");
        pwd = systemTable.getSystemString("pop3_password","TEST");
        deleteMessages = (systemTable.getSystemString("pop3_delete_messages","Yes")).equalsIgnoreCase("Yes");
        reg = new Register(DBUrl, DBUser, DBPassword);
   }

   public void run () {

// later on will have to make this either static/synchronized or if allowing separate threads for
// various companies another way of controlling for access against a single server

        String ReturnString;
        int NumberOfMessages;
        int i;
        
        outSocketPOP3();        //later on will have to get parameter from the param file stuff
        try {
            ReturnString = readLine();
            System.out.println(ReturnString);
            if( bannerIsOK(ReturnString)) {
                if( pop3Logon(logon, pwd)) {                             
//                    NumberOfMessages = getNumberOfMessages() > 2 ? 2 : getNumberOfMessages();
                    NumberOfMessages = getNumberOfMessages();
                    for (i = 1; i <= NumberOfMessages; i++) {
                        // might want to consider a failure in the getSystemIncrement module
                        System.out.println("Download/ODBC procedure");
                       // replace the second number with something from the SystemFile
                        downLoadMessage(i, systemTable.getSystemIncrement("MESSAGE_SEQUENCE")); 
                        if (deleteMessages == true)
                        {
    System.out.println("deleting message " + i);                            
                                deleteMessage(i);
                        }
                    }
//delete email messages after download                    
                    
                }

            } else {
                AuditTrail.println("Invalid Banner: " + ReturnString);
            }
            quit();

        } catch (IOException e) {
            AuditTrail.println("Error in readLine method");
        }
        close();


    }

    public void downLoadMessage(int MessageNumber, int FileNumber) {
        //Downloads a single message and saves it in a report file

        String MessageRequested;
        String MessageString = "";
        int RetrPortionLength = 0;
        Integer PassThisName = new Integer(FileNumber);
        String EndOfMessage = CommonFunctions.singleDot();

        EMailMessage RPTFile = new EMailMessage(PassThisName.toString(), "xml", DBUrl, "", "");  //Need to update this programs with USER and PASSWORD if this is ever used
        Integer WorkInteger = new Integer(MessageNumber);

        int MessageLength = getMessageLength(MessageNumber);

        if (MessageLength > 0) {
            MessageRequested = WorkInteger.toString();

            try {

                write("RETR " + MessageRequested);
                MessageString = read();
                RPTFile.write(MessageString);

                RetrPortionLength = MessageString.indexOf(10) + 1;    //Adjust index to length
                MessageLength = (MessageLength - MessageString.length()) + RetrPortionLength;

                // Testing for a "." alone on a line instead of the length.  Don't understand why the
                // MessageLength > 0 doesn't work, but it doesn't (Unicode characters?)  -> It seems
                // to generate a "more" negative number for large files and retains a positive number
                // for very small files (if my memory serves my right) -- will need to look into this
                // later as the single dot could truncate a message.  I could also check that the single
                // dot is the last character sequence of the string which would reduce this somewhat, unless
                // an even buffer where to occur that I'd still have the same problem......

                while (MessageString.indexOf(EndOfMessage) == -1) {

                    MessageString = read();
                    RPTFile.write(MessageString);
                    MessageLength = MessageLength - MessageString.length();
                }
                registerFile(RPTFile.getFileName(), FileNumber);

            } catch (IOException e) {
                AuditTrail.println("System error downloading POP3 message");
            }
        }
        try {
            RPTFile.close();
//            reg.setOwner(logon);
            
        } catch (IOException e) {
            AuditTrail.println("Failed to close rpt file");
        }
    }
    
    public void registerFile(String fileName, int fileNumber)
    {
        reg.setOwner(logon);
        reg.setEmailFileName(fileName);
        reg.setStatus("Incoming");
        reg.setXMLReference("00000000");
        reg.setMessage(Utilities.today().toString());
        reg.setReference((new Integer(fileNumber)).toString());
        reg.setSimple(reg.REFERENCE,reg.getReference());
        
    }
        
    public void cleanUp()
    {
        systemTable.close();
        reg.close();
    }

}
