/*
POP3ReceiptImage.java - reads messages from a pop3 account to a folder   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.io.*;
import java.sql.*;
//import java.util.prefs.*;

public class POP3ReceiptImage extends POP3andSMTP implements Runnable {

// This is an extension of the POP3andSMTP object that specializes in doing a POP3 message download to
// text file in batch mode.  It is written for multithreading

   static AuditTrail Logfile = new AuditTrail();
   private ServerSystemTable systemTable;
   private String logon = "";
   private String pwd = "";
   private String pop3address = "";
   private boolean deleteMessages = true;
   private Scan scan;
   private Address address;
   private ApplyScan applyScan;
   private CustomDate Dt = new CustomDate();
   private String base64 = "BASE64";
   private String dbUrl = null;
   private String dbUser = null;
   private String dbPassword = null;
   
   public POP3ReceiptImage(String DBUrl, String DBUser, String DBPassword)
   {
	   // Get a separate connect from the user so that if the user logs off we are not in trouble.
	   	dbUrl = DBUrl;
	   	dbUser = DBUser;
	   	dbPassword = DBPassword;
	   	setMySQLConnection(dbUrl, dbUser, dbPassword);
		logon = systemTable.getSystemString("pop3_logon","TEST");
        pwd = systemTable.getSystemString("pop3_password","TEST");
        pop3address = systemTable.getSystemString("pop3_address","127.0.0.1");
        super.Pop3Port = systemTable.getSystemInt("pop3_port","110");
        deleteMessages = (systemTable.getSystemString("pop3_delete_messages","Yes")).equalsIgnoreCase("Yes");
        
   }
   
   private void setMySQLConnection(String DBUrl, String DBUser, String DBPassword)
   {
	   // Consider adding &waitTimeout=0 to the DBUrl here
	   // MySQL sets that may also help are WAIT_TIMEOUT
	   systemTable = new ServerSystemTable(DBUrl, DBUser, DBPassword);
       scan = new Scan(systemTable.getConnection());
       address = new Address(systemTable.getConnection());
       applyScan = new ApplyScan(systemTable.getConnection(),systemTable.getSQLTerminator());
       address.setSQLTerminator(systemTable.getSQLTerminator());
   }
   
   public POP3ReceiptImage(Connection DBConn)
   {
	   // Get a separate connect from the user so that if the user logs off we are not in trouble.
        systemTable = new ServerSystemTable();
        systemTable.setConnection(DBConn);
        logon = systemTable.getSystemString("pop3_logon","TEST");
        pwd = systemTable.getSystemString("pop3_password","TEST");
        pop3address = systemTable.getSystemString("pop3_address","127.0.0.1");
        super.Pop3Port = systemTable.getSystemInt("pop3_port","110");
        deleteMessages = (systemTable.getSystemString("pop3_delete_messages","Yes")).equalsIgnoreCase("Yes");
        scan = new Scan(systemTable.getConnection());
        address = new Address(systemTable.getConnection());
        address.setSQLTerminator(systemTable.getSQLTerminator());
        applyScan = new ApplyScan(systemTable.getConnection(),systemTable.getSQLTerminator());
   }

   
   public void run () {

// later on will have to make this either static/synchronized or if allowing separate threads for
// various companies another way of controlling for access against a single server

        String ReturnString;
        int NumberOfMessages;
        int i;
        String messageString;
        int incomingReference = 0;
        int receiptReference;
        ess.Email emailMessage = new ess.Email();
        String imageString;
        String encodingType = "";
        String contentType; 
        String persnum;
        String emailAddress;
        String scanComment;
        
        setIP_URL(pop3address);
        outSocketPOP3();        //later on will have to get parameter from the param file stuff
        AuditTrail.println("[331] POP3ReceiptImage.run - checking email"); 
        try {
            ReturnString = readLine();
            AuditTrail.println("[000] POP3ReceiptImage return string: " + ReturnString);
            if( bannerIsOK(ReturnString)) {
                if( pop3Logon(logon, pwd)) {                             
//                    NumberOfMessages = getNumberOfMessages() > 2 ? 2 : getNumberOfMessages();
                    NumberOfMessages = getNumberOfMessages();
                    for (i = 1; i <= NumberOfMessages; i++) {
                        // might want to consider a failure in the getSystemIncrement module
                    	AuditTrail.println("[000] POP3ReceiptImage - Download procedure running");

                    	try {
                    		incomingReference = systemTable.getSystemIncrement("INCOMING_SEQUENCE");
                    	} catch (Exception e)
                    	{
                    		try {
                    			AuditTrail.println("[400] POP3ReceiptImage MySQL error reset attempted: " + e.toString());
                    			setMySQLConnection(dbUrl, dbUser, dbPassword);
                    			incomingReference = systemTable.getSystemIncrement("INCOMING_SEQUENCE");
                    		} catch (Exception ex) {
                    			AuditTrail.println("[500] POP3ReceiptImage MySQL error reset failed: " + ex.toString());
                    			ex.printStackTrace();
                    			close();
                    			System.exit(1);
                    		}
                    	}

                    	messageString = downLoadMessage(i, incomingReference); 
                        emailMessage.set(messageString);
                        emailAddress = emailMessage.getReturnPath();
                        scanComment = "";
                    	persnum = null;
                        if (!emailAddress.equals("")) {
                        	persnum = address.getOwner(emailAddress);
                        } else {
                        	AuditTrail.println("[500] POP3ReceiptImage blank address error - incoming file: " + Integer.toString(incomingReference));
                        }
                        if (persnum != null)
                        {
                        	while(emailMessage.getNext()) {
                        		contentType = emailMessage.getContentType();
                        		AuditTrail.println("[000] POP3ReceiptImage unadjusted header content type: " + contentType);
                        		if (contentType.equals("")) contentType = "TEXT";
                        		if (contentType.toUpperCase().indexOf("IMAGE") > -1)
                        		{
                        			encodingType = emailMessage.getTransferEncoding();
                        			AuditTrail.println("[000] POP3ReceiptImage - transfer encoding: " + encodingType);
                        			if (base64.equalsIgnoreCase(encodingType))
                        			{
                        				imageString = emailMessage.getContent();
                        				receiptReference = systemTable.getSystemIncrement("SCAN_SEQUENCE");
                        				//Need to determine file type
                        				// emailMessage.saveImage(Integer.toString(receiptReference), getFileType(contentType), systemTable, getDecodeImageBytes(imageString), persnum);
                        				ScanFile(receiptReference, persnum, incomingReference, getFileType(contentType), scanComment,emailAddress, getDecodeImageBytes(imageString));
                        				AuditTrail.println("[276] POP3ReceiptImage saved scan: " + receiptReference + " (" + incomingReference + ")");
                            			if (scanComment != null && scanComment.trim().length() > 0)
                            			{
                            				if (applyScan.setExpense(persnum, scanComment))
                            				{
                            					AuditTrail.println("[277] POP3ReceiptImage applied scan type (test): " + applyScan.getCorrectedType());
                            				} else {
                            					AuditTrail.println("[400] POP3ReceiptImage could not apply scan (test): " + scanComment);
                            				}
                            			} else {
                            				AuditTrail.println("[000] POP3ReceiptImage scannable information not detected");
                            			}
                        			}
                        		} else if (contentType.toUpperCase().indexOf("TEXT") > -1)
                        		{
                        			scanComment = emailMessage.getText().trim();
                        			AuditTrail.println("[000] POP3ReceiptImage process transaction detail: " + scanComment);
                        		}
                        	}
                        } else {
                        	//Check for registration message here
                        	if (emailMessage.isRegistration())
                        	{
                        		final int ADDRESS_UPDATE_OK = 0;
                        		final int ADDRESS_EXISTS = 1;
                        		final int ADDRESS_INCOMPLETE = 2;
                        		final int PERSNUM_INVALID = 3;
                        		final int ADDRESS_UPDATE_ERROR = -1;
                        		
                        		String emailPersnum = emailMessage.getRegistrationPersnum();
                        		
                        		switch (address.setNewAddress(emailPersnum, emailAddress))
                        		{
                        		    case ADDRESS_UPDATE_OK :
                        			{ 
                        				AuditTrail.println("[255] POP3ReceiptImage new address: " + emailAddress + " for " + address.getOwner(emailAddress) + ", incoming: " + Integer.toString(incomingReference));
                        				SendAnEmail(emailPersnum, emailAddress, "ADDRESS_UPDATE_OK");
                        				break;
                        			}
                        		    case ADDRESS_EXISTS :
                        		    {
                        		    	AuditTrail.println("[400] POP3ReceiptImage duplicate registration: " + emailAddress + ", incoming: " + Integer.toString(incomingReference));
                        		    	SendAnEmail(emailPersnum, emailAddress, "ADDRESS_EXISTS");
                        				break;
                        		    }
                        		    case ADDRESS_INCOMPLETE :
                        		    {
                        		    	AuditTrail.println("[400] POP3ReceiptImage incomplete registration: " + Integer.toString(incomingReference));
                        		    	//Check if below is a good idea
                                    	//SendAnEmail send = new SendAnEmail(systemTable.getConnection());
                        		    	//send.Send(emailAddress, "ESS", "ADDRESS_INCOMPLETE");
                        		    	break;
                        		    }
                        		    case PERSNUM_INVALID :
                        		    {
                        		    	AuditTrail.println("[400] POP3ReceiptImage invalid personnel number: " + emailPersnum + ", incoming: " + Integer.toString(incomingReference));
                        		    	//Check if below is a good idea
                                    	//SendAnEmail send = new SendAnEmail(systemTable.getConnection());
                        		    	//send.Send(emailAddress, "ESS", "ADDRESS_INCOMPLETE");
                        		    	break;
                        		    }

                        		    case ADDRESS_UPDATE_ERROR :
                        		    {
                        		    	AuditTrail.println("[500] POP3ReceiptImage failure to update registration: " + emailAddress + ", incoming: " + Integer.toString(incomingReference));
                        		    	SendAnEmail(emailPersnum, emailAddress, "ADDRESS_UPDATE_ERROR");
                        				break;
                        		    }
                        		}
                        	} else {
                            	AuditTrail.println("[400] POP3ReceiptImage - invalid return-path for incoming email: " + Integer.toString(incomingReference));
                                //Check if below is a good idea
                            	//SendAnEmail send = new SendAnEmail(systemTable.getConnection());
                                //send.Send(emailAddress, "ESS", "ADDRESS_INCOMPLETE");	
                        	}
                        }
                        if (deleteMessages == true)
                        {
                        	                            
                            deleteMessage(i);
                            AuditTrail.println("[000] POP3ReceiptImage - deleted message " + i);
                        }
                    } //end of the message loop
//delete email messages after download                    
                    
                }

            } else {
                AuditTrail.println("[500] POP3ReceiptImage - Invalid Banner: " + ReturnString);
            }
            quit();

        } catch (IOException e) {
            AuditTrail.println("[500] POP3ReceiptImage - Error in readLine method");
        }
        close();


    }

    public String downLoadMessage(int MessageNumber, int FileNumber) {
        //Downloads a single message and saves it in a report file

        String MessageRequested;
        String MessageString = "";
        int RetrPortionLength = 0;
        Integer PassThisName = new Integer(FileNumber);
        String EndOfMessage = CommonFunctions.singleDot();

        EMailMessage RPTFile = new EMailMessage(PassThisName.toString(), "msg", systemTable);
        Integer WorkInteger = new Integer(MessageNumber);

        int MessageLength = getMessageLength(MessageNumber);

        String returnString = "";
        
        if (MessageLength > 0) {
            MessageRequested = WorkInteger.toString();

            try {
            	AuditTrail.println("[000] POP3ReceiptImage.downLoadMessage writing: RETR " + MessageRequested); 
                
                write("RETR " + MessageRequested);
                MessageString = read();
                AuditTrail.println("[000] POP3ReceiptImage.downLoadMessage 1st read complete");
                RPTFile.write(MessageString);
                returnString = MessageString;

                RetrPortionLength = MessageString.indexOf(10) + 1;    //Adjust index to length
                MessageLength = (MessageLength - MessageString.length()) + RetrPortionLength;

                // Testing for a "." alone on a line instead of the length.  Don't understand why the
                // MessageLength > 0 doesn't work, but it doesn't (Unicode characters?)  -> It seems
                // to generate a "more" negative number for large files and retains a positive number
                // for very small files (if my memory serves me right) -- will need to look into this
                // later as the single dot could truncate a message.  I could also check that the single
                // dot is the last character sequence of the string which would reduce this somewhat, unless
                // an even buffer where to occur that I'd still have the same problem......
                AuditTrail.println("[000] POP3ReceiptImage.downLoadMessage before loop length: " + Integer.toString(MessageLength));
                while (returnString.indexOf(EndOfMessage) == -1) {    //was MessageString - a little less efficient now - look at improving later

                    MessageString = read();
                    RPTFile.write(MessageString);
                    returnString = returnString + MessageString;
                    MessageLength = MessageLength - MessageString.length(); //???? this doesn't make any sense - look into this ????
                }
                AuditTrail.println("[000] POP3ReceiptImage.downLoadMessage after loop length: " + Integer.toString(MessageLength));
            } catch (IOException e) {
                AuditTrail.println("[500] POP3ReceiptImage - System error downloading POP3 message");
            }
        }
        try {
            RPTFile.close();
        } catch (IOException e) {
            AuditTrail.println("[500] POP3ReceiptImage - Failed to close rpt file");
        }
        return returnString;
    }
    
    public void ScanFile(int scanNumber, String owner, int xrefNumber, String fileType, String scanComment, String emailAddress, byte[] scanImage)
    {
    	// "SCAN_REF","PERS_NUM","PVOUCHER","RECEIPT","FILE_TYPE","UP_DATE","UP_TIME","CLERK","STATUS"
        scan.setScan_ref( Integer.toString( scanNumber ) );
        scan.setPers_num(owner);
        scan.setPvoucher("");
        scan.setReceipt("");
        scan.setFile_type(fileType);
        scan.setUp_date(CustomDate.getSQLDate(Dt.date));
        scan.setUp_time(Dt.getLocalTime());
        scan.setClerk(owner);
        scan.setStatus("Incoming");
        scan.setXref(Integer.toString(xrefNumber));
        scan.setComment(scanComment);
        scan.setAddress(emailAddress);
        scan.setSize(scanImage.length);
        scan.setContent(Scan.compress(scanImage));
        
        scan.setSimple(scan.getKeyPosition(),scan.getScan_ref());
        
    }
    
    private String getFileType(String contentType) {
    	contentType = contentType.toLowerCase();
    	if (contentType.indexOf("jpeg") != -1) contentType = "jpeg";  //replace with some link of xml usage for flexibility
    	if (contentType.indexOf("jpg") != -1) contentType = "jpg";
    	if (contentType.indexOf("gif") != -1) contentType = "gif";
    	if (contentType.indexOf("bmp") != -1) contentType = "bmp"; 
    	if (contentType.indexOf("png") != -1) contentType = "png"; 
    	return contentType;
    }

    private byte[] getDecodeImageBytes(String x) {
    	byte[] retVal = null;
    	AuditTrail.println("[000] POP3ReceiptImage.getDecodeImage x: " + x.substring(0, 100));
    	try {
    	     retVal = Base64coder.decodeLines(x); 
    	} catch (IllegalArgumentException e) {
    		AuditTrail.println("[500] POP3ReceiptImage.getDecodeImage IllegalArguement: " + e.toString() + " - see server error logs");
    		e.printStackTrace();
    	}
    	return retVal;
    }

    public void cleanUp()
    {
        systemTable.close();
        scan.close();
    }
    
    private void SendAnEmail(String persnum, String address, String MsgID)
    {
    	SendAnEmail send = new SendAnEmail(systemTable.getConnection());
        send.SendtoUserMsgID(persnum, address, MsgID);	
    }
}