/*
POP3andSMTP.java - interface to SMTP and POP3 servers   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.net.*;
import java.io.*;
import java.util.*;
//import java.lang.*;

public class POP3andSMTP {

    private Socket theSocket;
    private int thePort;
    private Integer WorkingPort;
    private String hostname;
    private DataInputStream inMailStream;

    private PrintStream outMailStream;
    private CommonFunctions Func = new CommonFunctions();
    private AuditTrail Logfile = new AuditTrail();
    private String IP_URL = "127.0.0.1";
    protected int Pop3Port = 110;
    protected int SMTPPort = 25;
    // private String IP_URL = "192.168.0.21";
    private final char newLine[] = {0x0d,0x0a};
    private final String CRLF = new String(newLine);
    private String validReturnCodes = ";+OK;";

    public POP3andSMTP () {
    }

    public void setIP_URL(String Address) {
        IP_URL = Address;
    }
    public void setValidReturnCodes(String x) {
        validReturnCodes = x;
    }

    public boolean outSocketPOP3() {       
        return(outSocket(IP_URL,Pop3Port));
    }

    public boolean outSocketSMTP() {       
        return(outSocket(IP_URL,SMTPPort));
    }

    public boolean outSocket(String IPAddress, int thePort) {
        Integer WorkingPort = new Integer(thePort);    //used for printing purposes

        try {
        	AuditTrail.println("[000] POP3andSMTP Establishing socket " + IPAddress + ", port " + WorkingPort.toString());
            theSocket = new Socket(IPAddress, thePort);
            inMailStream = new DataInputStream(theSocket.getInputStream());
            outMailStream = new PrintStream(theSocket.getOutputStream());
            AuditTrail.println("[000] POP3andSMTP Connected");
            return true;
        } catch (UnknownHostException e) {
            AuditTrail.println("[500] POP3andSMTP outSocket Unknown host exception" );
            System.err.println(e);
            return false;
        } catch (IOException e) {
            AuditTrail.println("[500] POP3andSMTP outSocket IO Exception error");
            System.err.println(e);
            return false;
        }

    }


    public String readLine() throws IOException {
        String readString = inMailStream.readLine();
        return readString;
    }


    public String read() throws IOException {
        byte[] readBytes = new byte[4096];
        int returnedBytes;
        String readString;
        
        returnedBytes = inMailStream.read(readBytes);

        readString = new String(readBytes,0,returnedBytes);

        return readString;
    }


    public String multiRead(String commenceWithThis) throws IOException {
        String readString = commenceWithThis;
        String CheckString = CommonFunctions.singleDot();
        try {
        while (readString.indexOf(CheckString) == -1) {
            readString = readString.concat(read());
        }
        } catch (java.lang.StringIndexOutOfBoundsException e) {
        	AuditTrail.println("[000] POP3andSMTP multiRead no files to process");
        }

        return readString;

    }


    public void write(String outputString) throws IOException {
        //  outMailStream.println(outputString);
        outMailStream.print(outputString + CRLF);
        outMailStream.flush();                          //probably doesn't do anything
    }


    public void close() {

            try {
                theSocket.close();
//                Logfile.println("[000] SMTP..close Close " + theSocket.toString());
                //theSocket.toString() does not return. Need to investigate.
                AuditTrail.println("[000] POP3andSMTP close OK ");
            } catch (IOException e) {
                AuditTrail.println("[000] POP3andSMTP failure closing the email socket");
            }
    }


    public boolean bannerIsOK(String returnString) {
        
        return messageBackIsOK(returnString);

    }

    public boolean messageBackIsOK(String returnString) {
        StringTokenizer st = new StringTokenizer(returnString);
        String errorCode;
        boolean messageStatus = true;
        
        if (st.hasMoreTokens() == true)
        {
            errorCode = st.nextToken();
            AuditTrail.println("[000] POP3andSMTP msg code: '" + errorCode + "'");
            if (validReturnCodes.toUpperCase().indexOf(errorCode.toUpperCase().trim()) == -1)
            {
            	if (errorCode != null)
            	{
            		try {
            			if (Integer.parseInt(errorCode) >= 400) messageStatus = false;
            		} catch (java.lang.NumberFormatException e){
            			AuditTrail.println("[400] POP3andSMTP unknown msg code detected");
            		}
            	}
            }
        }
        return messageStatus;
    }

    public boolean pop3Logon(String UserID, String Password) {
        String ReturnString;
        boolean OKFlag = false;

        try {
            write("user " + UserID);
            AuditTrail.println("[000] POP3andSMTP user entered: " + UserID);
            ReturnString = readLine();
            if( messageBackIsOK(ReturnString)) {
                write("pass " + Password);
                AuditTrail.println("[000] POP3andSMTP password entered");
                ReturnString = read();
                if ( messageBackIsOK (ReturnString)) {
                    OKFlag = true;
                }
            }
        } catch (IOException e) {
            AuditTrail.println("[500] POP3andSMTP pop3Logon System error logging on to POP3 server");
        }
        return OKFlag;
    }

    public int getNumberOfMessages() {
        String ReturnString;
        int OKint = 0;
        String[] ParsedList;
        String[] SecondList;
        String ListString;
        Integer Conversion;
        
        try {
            write("list");
            AuditTrail.println("[000] POP3andSMTP message list requested");
            ReturnString = read();
            AuditTrail.println("[000] POP3andSMTP returnString: " + ReturnString);
            if (messageBackIsOK(ReturnString)) {
                byte[] TheArray = {10};
                String CommenceString = new String(TheArray,0,1);
            	ListString = multiRead(CommenceString);
            	AuditTrail.println("[000] POP3andSMTP ListString: " + ListString);
            	ParsedList = ListString.split("[\n\r]");
                for (int j = 0;j < ParsedList.length;j++){
                	AuditTrail.println("[000] POP3andSMTP msg numbers: '" + ParsedList[j] + "'");
                	SecondList = ParsedList[j].split(" ");
                	if (SecondList.length == 2){
                		Conversion = new Integer(SecondList[0]);
                		OKint = Conversion.intValue();
                	}
                }
            }
        } catch (IOException e) {
            AuditTrail.println("[500] POP3andSMTP getNumber of Messages System error getting number of messages on POP3 server");
        }
        AuditTrail.println("[000] POP3andSMTP message count: " + Integer.toString(OKint)); 
        return OKint;
    }


    public int getMessageLength(int MessageNumber) {
        String ReturnString;
        int OKint = 0;
        String[] ParsedList;
        Integer MessageWanted = new Integer(MessageNumber);  //call with the message number you want info on

        try {
            write("list " + MessageWanted.toString());
            ReturnString = read();                 // only want the first line - some return two lines
            AuditTrail.println("[000] POP3andSMTP MessageLengthString: " + ReturnString);
            if (messageBackIsOK(ReturnString)) {
                ParsedList = CommonFunctions.parsDelimStr(ReturnString.trim()," ");
                Integer Conversion = new Integer(ParsedList[2]);
                OKint = Conversion.intValue();
            }
        } catch (IOException e) {
            AuditTrail.println("[500] SMTP..getMessageLength System error listing message info on POP3 server");
        }
        AuditTrail.println("[000] POP3andSMTP.getMessageLength message length: " + Integer.toString(OKint)); 
        return OKint;
    }


    public boolean requiresGoodReturn(String CheckThisString) {
        // Check if the client requires a response from the SMTP host in order to proceed

        String[] AgainstThisArray = {"HELO","DATA","MAIL FROM:","RCPT TO:"};  //can put this in a system string later too
        int i;

        CheckThisString = CheckThisString.trim();

        if (CheckThisString.equals(".")) {
            return true;
        } else {

            CheckThisString = CheckThisString.toUpperCase();

            for (i = 0; i < AgainstThisArray.length; i++) {

                if (CheckThisString.startsWith(AgainstThisArray[i])) {
                    return true;
                }
            }
        }
        return false;
    }

    public int deleteMessage(int MessageNumber) {
        String ReturnString;
        int OKint = 0;
        Integer MessageWanted = new Integer(MessageNumber);  //call with the message number you want info on

        try {
            write("dele " + MessageWanted.toString());
            ReturnString = read();                 // only want the first line - some return two lines
            AuditTrail.println("[000] POP3andSMTP delete message: " + ReturnString);
        } catch (IOException e) {
            AuditTrail.println("[500] POP3andSMTP getMessageLength System error listing message info on POP3 server");
        }

        return OKint;
    }



    public void quit() {
        String ReturnString;

        try {
            write("quit");
            AuditTrail.println("[000] POP3andSMTP quit issued");
            ReturnString = readLine();

            if (! messageBackIsOK(ReturnString)) {
                AuditTrail.println("[500] POP3andSMTP quit Problem executing quit");
            }
        } catch (IOException e) {
            AuditTrail.println("[500] POP3andSMTP quit System error loggin on to POP3 server IOException: " + e.toString());
        } catch (java.lang.NullPointerException e) {
            AuditTrail.println("[500] POP3andSMTP quit error: " + e.toString());
        }

    }

}   //End of the class