/*
PersonnelTable.java - User information access   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.sql.*;

//
//  This object extends the Dbase class to provide enhancements for reading and writing
//  to the SYSTEM table which contains setup and control parameters
//

// Note: see version of the file in LESS to see XML profile routines.   Might want to 
// add these in the future. JH 6/9/2004

public class PersonnelTable extends AdisoftDbase {

    static Integer IConvert;  //used to manipulate strings, ints, etc
    
    public String persnum = "";
    public String name = "";
    public String phone = "";
    public String location = "";
    public String depart = "";    //add a set method
    public String company = "";
    public String password = "";
    public String guideline = "";
    public String email = "";
    public String mileage = "";
    public String vehicle = "";
    public String servdate = "";
    public String category = "";
    public String active = "";
    public String asofdate = "";
    public String challengecode = "";
    public String manager = "";
    public String level = "";
    public String limit = "";
    public String fname = "";
    public String lname = "";
    public String userField1 = "";
    public String userField2 = "";
    public String userField3 = "";
    public String employeeCode = "";
    public String currency = "";
    public String function = "";
    public String language = "";
    public String country = "";
    public String dateFormat = "";
    public String numberFormat = "";
    
    private boolean departClerk = false;
    private boolean isClerkEstablished = false;
    
    public String bankABA = "";
    public String accountNumber = "";
    public String isSavingsAccount = "";
    public String isPreNoted = "";
    public String preNoteDate = "";
    public String paymentmethod = "";  // was 'cash'
    
    protected DepartmentTable departTable;
    
    // SQL Items below
    private String emailInfoAllSQL = "SELECT * FROM USER WHERE UCASE(EMAIL) = '$email$';";
    private String persInfoSQL = "SELECT * FROM USER WHERE UCASE(EMAIL) = '$email$';";  //to upper case
    private String persInfoAllSQL = "SELECT * FROM USER WHERE PERS_NUM = '$persnum$';" ;
    private String persNumInfoSQL = "SELECT * FROM USER WHERE PERS_NUM = '$persnum$';";
    private String isClerkSQL = "SELECT CLERK FROM DEPART WHERE CLERK = '$persnum$';";
    
    private String blankToPersnum = "No";
    private String persnumEndCount = "";
    //private String sqlTerminator = ";"; commented out JH 2006-03-01 found in AdisoftDBase

    private AdisoftHash AH = new AdisoftHash();

    private String webServer = "http://localhost";
    private String appServer = "http://localhost:8085";
    private String webFolder = "ess";
    private String appFolder = "ess-app";
    
    private String defaultLanguage = "en";
    private String defaultCountry = "US";
    private String defaultDateFormat = "MM/dd/yyyy";
    private String defaultNumberFormat = "decimal";

    
    private AdisoftDOM SecurityDOM = new AdisoftDOM();
    
    private String LDAPProvider = "";
    private String LDAPSearchPrincipal = "";
    private String LDAPCredentials = "";
    private String LDAPFactory = "com.sun.jndi.ldap.LdapCtxFactory";
    private String LDAPSearchString = "CN=Users,DC=expenseservices,DC=ca";
    private String LDAPEmailAttribute = "email";
    private String LDAPPrincipalAttribute = "userPrincipalName";
    private boolean isLDAP = false;

    
    public PersonnelTable() {
      Logfile = new AuditTrail();
    }

    public PersonnelTable(AuditTrail CurrentLog) {
      Logfile = CurrentLog;
    }
    
    //public PersonnelTable(String persNum, String Url, String DBUser, String DBPassword) {
    //  Logfile = new AuditTrail();
    //  setDB(Url, DBUser, DBPassword);
    //  setSQLStrings();
    //  setPersNumInfo(persNum);
    //}
    
    public PersonnelTable(String Url, String DBUser, String DBPassword) {
      Logfile = new AuditTrail();
      setDB(Url, DBUser, DBPassword);
    }
    
    //public PersonnelTable(String persNum, Connection x, String SQLA,String SQLB,String SQLC,String SQLD) {
    //depricated - probably won't work because of the schema link - comment out and retry after the update
    //This optimizes the performance of the Approver class
    //  this.setConnection(x);
    //  setSQLStrings(SQLA,SQLB,SQLC,SQLD);
    //  setPersNumInfo(persNum);
    //}
    
    public void setSQLStrings(String A, String B, String C, String D) {
        emailInfoAllSQL = A;
        persInfoSQL = B;
        persInfoAllSQL = C;
        persNumInfoSQL = D;
    }   
    
    public void setSQLTerminator(String X) {
        sqlTerminator = X;
    }
    
    public String getSQLTerminator() {
        return sqlTerminator;
    }
                
    public void setSQLStrings() {
        //rewrite this for to do the SysTable SQL from this object
        ServerSystemTable SysTable = new ServerSystemTable();
        AdisoftDOM SystemDOM = new AdisoftDOM();
        SysTable.setConnection(this.getConnection());
        SysTable.setSQLTerminator(sqlTerminator);
        
        String system_file = SysTable.getSystemString("XMLSYSTEM","C:\\WORK\\"); 
        java.io.File SystemFile = new java.io.File(system_file);
        SystemDOM.setDOM(SystemFile); 
        
        String security_file = SysTable.getSystemString("XMLSECURITY","C:\\WORK\\"); 
        java.io.File SecurityFile = new java.io.File(security_file);
        SecurityDOM.setDOM(SecurityFile); 

        AuditTrail.println("[000] PersonnelTable.setSQLStrings from " + system_file);
        emailInfoAllSQL = SystemDOM.getDOMTableValueFor("personneltable","emailinfoallsql");
        persInfoSQL = SystemDOM.getDOMTableValueFor("personneltable","persinfosql");
        persInfoAllSQL = SystemDOM.getDOMTableValueFor("personneltable","persinfoallsql");
        persNumInfoSQL = SystemDOM.getDOMTableValueFor("personneltable","persnuminfosql");
        blankToPersnum = SystemDOM.getDOMTableValueFor("personneltable","blanktopersnum");
        persnumEndCount = SystemDOM.getDOMTableValueFor("personneltable","persnumendcount");
        isClerkSQL = SystemDOM.getDOMTableValueFor("personneltable","isclerk");
        webServer = SysTable.getSystemString("webserver","");     
        appServer = SysTable.getSystemString("appserver",""); 
        
        webFolder = SysTable.getSystemString("webfolder","ess");     
        appFolder = SysTable.getSystemString("appfolder","ess-app"); 
        
        defaultLanguage = SysTable.getSystemString("language","en");
        defaultCountry = SysTable.getSystemString("country","US");
        defaultDateFormat = SysTable.getSystemString("dateformat","MM/dd/yyyy");
        defaultNumberFormat = SysTable.getSystemString("numberformat","decimal");
        
        super.setSchemaTable(SysTable.getSystemString("XMLSCHEMA","C:\\WORK\\"));
        super.setColumns("USER");
    }    
    
    
    private void setDepartment(String newDepart, String newCompany) {
       if (departTable == null) {
          departTable = new DepartmentTable();
          departTable.setConnection(dbCon);
          departTable.setSQLTerminator(sqlTerminator);  //JH 2005-09-15
          departTable.setSQLStrings();
       }
       departTable.setDepartInfo(newDepart,newCompany);
    }
    
    public String getCompany() {
       return company;
    }
    
    public ess.DepartmentTable getDepartmentObject() {
       return departTable;
    }
    
    public String getDepartmentName()
    {
    	return(departTable.getDepartmentName());
    }
    public String getDepartmentMgr()
    {
    	return(departTable.getManager());
    }
    public String getDepartmentAdmin()
    {
    	return(departTable.getAdmin());
    }
    public String getDepartmentAudit()
    {
    	return(departTable.getAuditor());
    }

    public String getCompanyName()
    {
    	return(departTable.getCompanyName());
    }
    
    public String getWebServer() {
      return webServer;
    }

    public String getAppServer() {
      return appServer;
    }

    public String getWebFolder() {
      return webFolder;
    }

    public String getAppFolder() {
      return appFolder;
    }

    public String getPaymentMethod()
    {
        return paymentmethod;
    }
    public String getUserField1()
    {
        return userField1;
    }
    public String getUserField2()
    {
        return userField2;
    }
    public String getUserField3()
    {
        return userField3;
    }
    public String getCurrency()
    {
    	if (currency == null || currency.equals("")) currency = departTable.getCurrency();
        return currency;
    }
   
    public ResultSet getResult() {
      return myResult;
    }
    
    public String getChallengeCode() {
      return challengecode;
    }
    
    public void setChallengeCode(String IP, String[] threePartKey) {
        // Need to check if the key has already been provided and don't generate
        // a new code if this is the same key and password combo..
        // I have disabled the random part of the key to allow the same person to be logged
        //   in on a single PC
        
           if (challengecode.equals(""))  //JH changed from a ! condition 2010-10-11
           {
             challengecode = AH.Hash(IP + threePartKey[0] + password);
           }
           // challengecode = AH.Hash(IP + password);  // alternate cc code
    }
    
    public void closeMyState()
    {
        if (myState != null)
        {
            try
            {
                myState.close();
                myState = null;
            }
            catch(Exception e)
            {
                AuditTrail.println("[500] DepartmentTable.closeMyState() close failure");
            }
        }
        
    }
    
    //Look at combining the next two methods and the the complete record memos
    
    public boolean setPersInfo(String SearchEMail) {  //uses email

        boolean exitStatus = true;
        String SQLCommand = "SQL not defined";

        if (SearchEMail == null) return false;

        if (SearchEMail.toUpperCase().equals(email.toUpperCase())) {
            //Logfile.println("[000] PersonnelTable.setPersInfo: Session reuse " + SearchEMail);
            return true;
        } else {
          try {
            myState = dbCon.createStatement();
            SQLCommand = this.SQLReplace(persInfoSQL,"$email$",repStr(getTrim(SearchEMail.toUpperCase())));
            myResult = myState.executeQuery(SQLCommand);
            resetProfile(); //JH 2006-09-28            
            if (myResult.next()) {   //change below later to column locations to generalize
                // AuditTrail.println("[---] PersonnelTable.setPersInfo: setting user: " + SearchEMail);
                departTable = null;  //will reopen department  JH 2008-3-4
                updateInstanceVariables();
            } else {
                AuditTrail.println("[400] PersonnelTable.setPersInfo: Invalid login for " + SearchEMail);
                AuditTrail.println("[400] " + SQLCommand);
                exitStatus = false;
            }
            closeMyState();  
            return (exitStatus);

          } catch (java.lang.NullPointerException ex) {
            AuditTrail.println("[400] PersonnelTable.setPersInfo: null pointer - force relogin: " + SearchEMail);
            return false;
          } catch (java.lang.Exception ex){
            AuditTrail.println("[500] PersonnelTable.setPersInfo: Error finding " + SearchEMail);
            AuditTrail.println("[500] SQL: " + SQLCommand);
            AuditTrail.println("[500] Error: " + ex.toString());
            ex.printStackTrace();
            return false;
          }
        }
    }
    public boolean setPersNumInfo(String SearchPersNum) {  //uses persnum

        boolean exitStatus = true;
        String SQLCommand = "SQL not defined";
        // NEED TO ADD LOGIC FROM THE ABOVE FUNCTION HERE
        try {
            myState = dbCon.createStatement();
            SQLCommand = this.SQLReplace(persNumInfoSQL,"$persnum$",getTrim(SearchPersNum));
            myResult = myState.executeQuery(SQLCommand);
            resetProfile(); //JH 2006-09-28            
            if (myResult.next()) {   //change below later to column locations to generalize
                departTable = null;  //will reopen department  JH 2008-3-4
            	updateInstanceVariables();
                // Logfile.println("[000] PersonnelTable.setPersNumInfo: retrieved info for " + SearchPersNum);
            } else {
                AuditTrail.println("[400] PersonnelTable.setPersNumInfo: Invalid retrieval for " + SearchPersNum);
                AuditTrail.println("[400] " + SQLCommand);
                exitStatus = false;
            }
            if (myState != null) closeMyState();  
            return (exitStatus);

        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] PersonnelTable.setPersNumInfo: Error finding " + SearchPersNum);
            AuditTrail.println("[500] "  + SQLCommand);
            AuditTrail.println("[500] " + ex.toString());
            return false;
        }
    }
    
    private void updateInstanceVariables() throws SQLException {
       persnum = getTrim(myResult.getString(getColumnName("PERS_NUM")));
       fname = getTrim(myResult.getString(getColumnName("FNAME")));
       lname = getTrim(myResult.getString(getColumnName("LNAME")));
       name = rightName(fname)+ " " + rightName(lname);
       phone = getTrim(myResult.getString(getColumnName("PHONE")));
       location = getTrim(myResult.getString(getColumnName("LOCATION")));
       depart = getTrim(myResult.getString(getColumnName("DEPART")));
       company = getTrim(myResult.getString(getColumnName("COMPANY")));
       password = getTrim(myResult.getString(getColumnName("PASSWORD")));  //removed null check jh 2/6/2004
       guideline = getTrim(myResult.getString(getColumnName("GUIDE")));
       email = getTrim(myResult.getString(getColumnName("EMAIL")));
       mileage = getTrim(myResult.getString(getColumnName("MILEAGE")));
       category = getTrim(myResult.getString(getColumnName("SHOWGRP")));
       active = getTrim(myResult.getString(getColumnName("ACTIVE")));
       asofdate = getTrim(myResult.getString(getColumnName("PSWD_DATE")));
       manager = getTrim(myResult.getString(getColumnName("MANAGER")));
       vehicle = getTrim(myResult.getString(getColumnName("VEHICLE")));
       servdate = getTrim(myResult.getString(getColumnName("SERVDATE")));
       bankABA = getTrim(myResult.getString(getColumnName("BANKCODE")));
       accountNumber = getTrim(myResult.getString(getColumnName("BANKACCT")));
       isSavingsAccount = getTrim(myResult.getString(getColumnName("SAVINGS")));
       isPreNoted = getTrim(myResult.getString(getColumnName("PRENOTE")));
       preNoteDate = getTrim(myResult.getString(getColumnName("PRE_DATE")));
       paymentmethod = getTrim(myResult.getString(getColumnName("CASH")));  //CASH
       level = getTrim(myResult.getString(getColumnName("LEVEL")));   
       limit = getTrim(myResult.getString(getColumnName("LIMIT")));
       userField1 = getTrim(myResult.getString(getColumnName("EXTRA1")));
       userField2 = getTrim(myResult.getString(getColumnName("EXTRA2")));
       userField3 = getTrim(myResult.getString(getColumnName("EXTRA3")));
       employeeCode = getTrim(myResult.getString(getColumnName("EMPCODE")));
       currency = getTrim(myResult.getString(getColumnName("CURRENCY")));
       function = getTrim(myResult.getString(getColumnName("FUNCTION")));
       language = getTrim(myResult.getString(getColumnName("LANGUAGE")));
       country = getTrim(myResult.getString(getColumnName("COUNTRY")));
       dateFormat = getTrim(myResult.getString(getColumnName("DATEFORMAT")));
       numberFormat = getTrim(myResult.getString(getColumnName("NUMBERFORMAT")));
       setDepartment(depart, company);
    }
        
    public boolean setPersInfoAll(String SearchKey) {
        boolean exitStatus = true;
        String SQLCommand = "none specified";
        // can replace this with AdisoftDbase usage if I want - JH
        try {

            myState = dbCon.createStatement();
            SQLCommand = this.SQLReplace(persInfoAllSQL,"$persnum$",getTrim(SearchKey));
            myResult = myState.executeQuery(SQLCommand);

            if (myResult.next()) {
                AuditTrail.println("[000] PersonnelTable.setPersInfoAll: Loaded results for " + SearchKey);
            } else {
                AuditTrail.println("[400] PersonnelTable.setPersInfoAll: Invalid key " + SearchKey);
                exitStatus = false;
            }
            closeMyState();  
            return (exitStatus);
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] PersonnelTable.setPersInfoAll: Error finding " + SearchKey);
            return false;
        }
    }
    public boolean setEmailInfoAll(String SearchKey) {
        boolean exitStatus = true;
        String SQLCommand = "none specified";
        // can replace this with AdisoftDbase usage if I want - JH
        try {

            myState = dbCon.createStatement();
            SQLCommand = this.SQLReplace(emailInfoAllSQL,"$email$",repStr(getTrim(SearchKey.toUpperCase())));
            myResult = myState.executeQuery(SQLCommand);

            if (myResult.next()) {
                AuditTrail.println("[000] PersonnelTable.setEmailInfoAll: Loaded results for " + SearchKey);
            } else {
                AuditTrail.println("[400] PersonnelTable.setEmailInfoAll: Invalid key " + SearchKey);
                exitStatus = false;
            }
            closeMyState();  
            return (exitStatus);

        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] PersonnelTable.setEmailInfoAll: Error: " + SQLCommand);
            return false;
        }
    }
    
    public String setRightName(String x) { return rightName(x); }
        
    private String rightName(String x) {
      x = x.toLowerCase();
      char[] z = x.toCharArray();
      for (int i = 0; i < x.length(); i++) 
      {
         if (i == 0) 
         {
            z[0] = toUpperCaseChar(x.substring(0,1));
         } else {
            if (z[i-1] == (char) ' ') 
            {
                z[i] = toUpperCaseChar(x.substring(i,i+1));
            }
            else if (z[i-1] == (char) '\'')
            {
                z[i] = toUpperCaseChar(x.substring(i,i+1));
            }
            else if (z[i-1] == (char) '-')
            {
                z[i] = toUpperCaseChar(x.substring(i,i+1));
            }
         }
      }
      String y = new String(z);
      return y;   
    }
    
    private char toUpperCaseChar(String x)
    {
      x = x.toUpperCase();  
      char[] z = x.toCharArray();
      return z[0];
    }
        
    
    public void resetProfile() {
       persnum = "";
       name = "";
       phone = "";
       location = "";
       depart = "";
       password = "";
       guideline = "";
       email = "";
       mileage = "";
       category = "";
       active = "";
       asofdate = "";
       challengecode = ""; // was an "X" 2006-9-28 JH
       level = "";
       limit = "";
       fname = "";
       lname = "";
       userField1 = "";
       userField2 = "";
       userField3 = "";
       employeeCode = "";
       currency = "";
       function = "";
       language = "";
       country = "";
       dateFormat = "";
       numberFormat = "";
       
       bankABA = "";
       accountNumber = "";
       isSavingsAccount = "";
       isPreNoted = "";
       preNoteDate = "";
       paymentmethod = "";
       
       departTable = null;  //will reopen department  JH 2008-3-4
    }  
    public String getPersNum()
    {
        return(persnum);
    }
    
    public boolean equals(PersonnelTable other)
    {
        return(persnum.equals(other.persnum));
    }
    
    public String getManager()
    {
        return(manager);
    }
    
    public String getDepartment()
    {
        return(depart);
    }
    
    public String getLocation()
    {
        return(location);
    }

    public String getPhone()
    {
        return(phone);
    }
    
    public String getEmailAddress()
    {
        return(email);  
    }
    public String getPrintableEmailAddress() {
      String x;
      if (email == null) {
        x = "";
      } else {
        String oldChar = "'";
        String newChar = "\\'";
        x = repStr(email, oldChar, newChar);
      }
      return x; 
    }
    
    public String getSecurityLevel() {
        String retVal = level;
        if (retVal == null || retVal.equals("") ) retVal = "0";
        return retVal;
    }
    
    public String getMileage() {
        String retVal = mileage;
        if (retVal == null || retVal.equals("") ) retVal = "0.0000";
        return retVal;
    }
    
    public String getCategory() {
        String retVal = category;
        if (retVal == null) retVal = "";
        return retVal;
    }
    
    public String getMenuLevel() {
        String retVal = level;
        if (retVal == null || retVal.equals("") ) retVal = "0";
        return retVal;
    }
   
    public String getFunction() {
        String retVal = function;
        if (retVal == null || retVal.equals("") ) retVal = "Reporter";
        return retVal;
    }
   
    public String getLanguage() {
        String retVal = language;
        if (retVal == null || retVal.equals("") ) retVal = defaultLanguage;      //get from system.xml
        return retVal;
    }
    public String getCountry() {
        String retVal = country;
        if (retVal == null || retVal.equals("") ) retVal = defaultCountry;
        return retVal;
    }

    public String getDateFormat() {
        String retVal = dateFormat;
        if (retVal == null || retVal.equals("") ) retVal = defaultDateFormat;
        return retVal;
    }
    
    public String getNumberFormat() {
        String retVal = numberFormat;
        if (retVal == null || retVal.equals("") ) retVal = defaultNumberFormat;
        return retVal;
    }
    
    public String getDecimal() {
    	String retVal = ".";
    	if (getNumberFormat().equalsIgnoreCase("comma"))
    	{
    		retVal = ",";
    	}
    	return retVal;
    }
    
    public char getDecimalSeparator() {
    	char retVal = '.';
    	if (getNumberFormat().equalsIgnoreCase("comma"))
    	{
    		retVal = ',';
    	}
    	return retVal;
    }
    
    public String getSeparator() {
    	String retVal = ",";
    	if (getNumberFormat().equalsIgnoreCase("comma"))
    	{
    		retVal = ".";
    	}
    	return retVal;
    }

    public char getGroupingSeparator(){
    	char retVal = ',';
    	if (getNumberFormat().equalsIgnoreCase("comma"))
    	{
    		retVal = '.';
    	}
    	return retVal;
   }
    
    public String getServiceDate() {
        String retVal = servdate;
        if (retVal == null) retVal = "";
        return retVal;
    }
   
   
    public static String repStr(String x, String oldChar, String newChar) 
    {
      int startLocate = 0;
      int oldLength = oldChar.length();
      int newLength = newChar.length();
      
      int locate = x.indexOf(oldChar, startLocate);
      while (locate != -1) {
          x = x.substring(0,locate) + newChar + x.substring(locate + oldLength);
          startLocate = locate + newLength;
          locate = x.indexOf(oldChar, startLocate);
      }
      return x;
    }
    
    
    
    
    public boolean validLimit(double reportAmount, double currencyConversionRate)
    {   boolean status = false;
        
        try{
            status = validLimit(reportAmount * currencyConversionRate);
        }
        catch (NumberFormatException error)
        {
            AuditTrail.println("[500] PersonnelTable.validLimit: Invalid amount format. Can't validate limit.");
        }
        
        return(status);
    }
    
    public boolean validLimit(double reportAmount)
    {
        boolean status = false;
        double limitD = 0.00;
        try{
            limitD = Double.parseDouble(limit);
            if ((limitD-reportAmount) >= 0)
                status = true;
        }
        catch (NumberFormatException error)
        {
            AuditTrail.println("[500] PersonnelTable.validLimit: Invalid limit format. Can't validate limit.");
        }
        
        return(status);        
    }
    
    public boolean isClerk(String department)
    {
        //boolean departClerk = null; defined above in instance variables
        if (!isClerkEstablished) {
          if (!isActive()) 
          {
             departClerk = false;
             AuditTrail.println("[400] PersonnelTable.jsp unauthorized department administrator: " + email);
          } else {
             String SQLCommand = "none specified";
           // can replace this with AdisoftDbase usage if I want - JH
             try {
               myState = dbCon.createStatement();
               SQLCommand = this.SQLReplace(isClerkSQL,"$persnum$",repStr(persnum));
               SQLCommand = this.SQLReplace(SQLCommand,"$depart$",repStr(department));
               myResult = myState.executeQuery(SQLCommand);
               if (!myResult.next()) {
                  //AuditTrail.println("[400] PersonnelTable.isClerk not clerk: " + email);
                  departClerk = false;
               } else {
                  departClerk = true;
               }
               isClerkEstablished = true;
               closeMyState();  
             } catch (java.lang.Exception ex){
               ex.printStackTrace();
               AuditTrail.println("[500] PersonnelTable.isClerk: Error: " + SQLCommand);
               departClerk = false;
             }
          }
        }
        return departClerk;
    }
    
    
    public boolean isAdmin()
    {
        boolean secretary = true;
        if ((function.equalsIgnoreCase("ADMIN") || function.equalsIgnoreCase("PROCESSOR")) && isActive()) 
        {
           secretary = true;
        } else {
           secretary = false;
        } 
        return secretary;
    }
    
    public boolean isProcessor()
    {
        boolean secretary = true;
        if (function.equalsIgnoreCase("PROCESSOR") && isActive()) 
        {
           secretary = true;
        } else {
           secretary = false;
        } 
        return secretary;
    }

    
    public boolean isApprover() 
    {
        boolean retVal = false;
        if (isActive()){  //JH 2006-09-26
            retVal = true;
        }
        return retVal;
    }
    
    public boolean isAuditor() 
    {
        boolean retVal = true;
        if (function.equalsIgnoreCase("AUDITOR") && isActive()) 
        {
           retVal = true;
        } else {
           retVal = false;
        } 
        return retVal;
        
    }
    public boolean isGLAdmin() 
    {
        boolean retVal = true;
        if (function.equalsIgnoreCase("G/L ADMIN") && isActive()) 
        {
           retVal = true;
        } else {
           retVal = false;
        } 
        return retVal;
        
    }

    public boolean isSecurityAuditor() 
    {
        boolean securityOfficer = true;
        if (SecurityDOM.getDOMTableValueWhere("audit","email",getTrim(email),"email").equals("") || !isActive()) 
        {
           securityOfficer = false;
           AuditTrail.println("[400] PersonnelTable.jsp unauthorized auditor: " + email);
        } 
        return securityOfficer;
        
    }
    
    private String alternateApprover = "";
    public boolean isAlternateApprover() 
    {
        boolean retVal = true;
        if (alternateApprover.equals(""))
        {
        	if (SecurityDOM.getDOMTableValueWhere("alternateapprover","email",getTrim(email),"email").equals("") || !isActive()) 
        	{
        		alternateApprover = "No";
        	} else
        	{
        		alternateApprover = "Yes";
        	}
        }
        if (alternateApprover.equals("No")) 
        {
           retVal = false;
        } 
        return retVal;
    }
    
    public boolean isCEO() 
    {
        boolean ceo = false;
        if (SecurityDOM.getDOMTableValueWhere("uniquefunctions","function","ceo","email").equalsIgnoreCase(email) && isActive()) //changed from persnum 2007-09-03 
        {
           ceo = true;
        } 
        return ceo;
    }
    
    public boolean isActive() {
        boolean retVal = active.equals("1");
        return retVal;
    }
    public String getAuditMenu() 
    {
        String auditMenu = ""; 
        if (isAuditor())
        {
           auditMenu = SecurityDOM.getDOMTableValueWhere("audit","email",getTrim(email),"menu");
        } else if (isAdmin())
        {
           auditMenu = SecurityDOM.getDOMTableValueWhere("administrator","email",getTrim(email),"menu");
        }
        if (auditMenu.equals(""))
        {
           auditMenu = "backroom" + ".html";
        }
        return auditMenu;
    }
    
    
    public boolean validPassword(String testPassword) 
    {
        boolean retVal = false;
        if (isLDAP) {
        	if (this.isLDAPPassword(testPassword))
        	{
        		password = testPassword;
        		retVal = true;
        	}
        } else {
           if (password.equals("") && blankToPersnum.toUpperCase().equals("YES")) {
            
              if (persnumEndCount.equals("")) {
                 AuditTrail.println("[000] PersonnelTable.validPassword: Checking against the whole persnum");
                 retVal = testPassword.toUpperCase().equals(persnum.toUpperCase());
              } else {
                 int x = Integer.parseInt(persnumEndCount);
                 AuditTrail.println("[000] PersonnelTable.validPassword entered: " + testPassword);
                 if (persnum != null)
                 {
                   AuditTrail.println("[000] PersonnelTable.validPassword password: " + persnum.substring(persnum.length()-x).toUpperCase());
                   retVal = testPassword.toUpperCase().equals(persnum.substring(persnum.length()-x).toUpperCase());
                 } else {
                   AuditTrail.println("[000] PersonnelTable.validPassword password: null value");
                 }
              }  
           } else {
              retVal = testPassword.toUpperCase().equals(password.toUpperCase()); 
           }
        }
        return retVal;
    }
    
    public String getTrim(String x) {
      return Utilities.getTrim(x);
      //if (x == null) return "";
      //return x.trim();
    }

    public String getReimbursementType() {
      return paymentmethod;   
    }

    public String[] getACHPaymentInfo() {
      String[] retVal =  {name, bankABA, accountNumber, isSavingsAccount, isPreNoted, preNoteDate, paymentmethod, active}; 
      return retVal;
    }
    
    public String getName() 
    {
        return name;
    }
    public String getLastName() 
    {
        return lname;
    }
    public String getFirstName() 
    {
        return fname;
    }
    public String getEmployeeCode() 
    {
        return employeeCode;
    }

    public void setLDAP( String _Provider, String _SearchPrincipal, String _Credentials, String _Factory, String _SearchString, String _EmailAttribute, String _PrincipalAttribute)
    {
    	   if (_Provider.equals("")) 
    	   {
    		   isLDAP = false;
    	   } else {
    		   isLDAP = true;   
    		   LDAPProvider = _Provider;
    	       if (!_SearchPrincipal.equals("")) LDAPSearchPrincipal = _SearchPrincipal;
    	       if (!_Credentials.equals("")) LDAPCredentials = _Credentials;
    	       if (!_Factory.equals("")) LDAPFactory = _Factory;
    	       if (!_SearchString.equals("")) LDAPSearchString = _SearchString;
    	       if (!_EmailAttribute.equals("")) LDAPEmailAttribute = _EmailAttribute;
    	       if (!_PrincipalAttribute.equals("")) LDAPPrincipalAttribute = _PrincipalAttribute;
    	   }
    }
    
    public boolean isLDAPPassword(String xPassword) {
    	// javax.naming.Context ctx = null;
    	
    	// Because the email may be different than the LDAP credentials, we first need to find the user with the email id
    	// and then get his ldap id and then use that with the supplied password to see if it is valid.
    	// and that my friends is why there are two logins.
    	
    	javax.naming.directory.DirContext ctx = null;

    	java.util.Hashtable env = new java.util.Hashtable ();
    	boolean isValid = false;
    	try {
    	   //set the name of domain with the user name
    	   env.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, LDAPFactory);
    	   env.put(javax.naming.Context.PROVIDER_URL,LDAPProvider);
     	
    	   AuditTrail.println("[000] PersonnelTable.isLDAPPassword LDAP USER: " + LDAPSearchPrincipal);
    	
    	   env.put(javax.naming.Context.SECURITY_AUTHENTICATION,"simple");
    	
    	   if (!LDAPSearchPrincipal.equals(""))
    	   {
    	     //set user related information
    	     env.put(javax.naming.Context.SECURITY_PRINCIPAL, LDAPSearchPrincipal);
    	     //set user password
    	     env.put(javax.naming.Context.SECURITY_CREDENTIALS, LDAPCredentials);
    	     //validate user
      	   }

    	   ctx = new javax.naming.directory.InitialDirContext(env);

    	   isValid = true;
    	   AuditTrail.println("[000] PersonnelTable.isLDAPPassword LDAP Connection OK");

    	}
    	catch (javax.naming.AuthenticationException ex)
    	{
    		AuditTrail.println("[500] PersonnelTable.isLDAPPassword LDAP Connection AuthenticationException: " + ex);

    	}
    	catch (javax.naming.NamingException ex) 
    	{
    		AuditTrail.println("[500] PersonnelTable.isLDAPPassword LDAP Connection NamingException: " + ex);
    	}
    	catch (java.lang.Exception ex) 
    	{
    		AuditTrail.println("[500] PersonnelTable.isLDAPPassword LDAP Connection Exception: " + ex);
    	}

    	String ldapUser2 = "";
    	
    	if (isValid) {

    	  String[] attrIDs = {LDAPPrincipalAttribute};
    	  javax.naming.directory.SearchControls ctls = new javax.naming.directory.SearchControls();
    	  ctls.setReturningAttributes(attrIDs);
    	  ctls.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);

    	  String filter = LDAPEmailAttribute;   //
    	  if ((filter.indexOf("$email$") > -1) || (filter.indexOf("$persnum$") > -1))
    	  {
    		  filter.replace("$email$", email);
    		  filter.replace("$persnum$", persnum);
    	  } else {
    	       filter = "(" + LDAPEmailAttribute + "=" + email + ")";   //
    	  }
    	  
    	  try { 
    	
    		javax.naming.NamingEnumeration answer = ctx.search(LDAPSearchString, filter,ctls);
    	 
    	    try {
    	      if (answer.hasMore()) 
    	      {  
    	    	 // javax.naming.NameClassPair nc  = (javax.naming.NameClassPair) answer.next();
    	    	 
    	    	 javax.naming.directory.SearchResult nc  = (javax.naming.directory.SearchResult) answer.next();
    	    	 javax.naming.directory.Attribute principalAttrib = (javax.naming.directory.Attribute) nc.getAttributes().get(LDAPPrincipalAttribute);
    	    	 ldapUser2 = (String) principalAttrib.get();
    	    	 
    	         AuditTrail.println("[000] PersonnelTable.isLDAPPassword Return From Server: " + nc);
    	         
    	         AuditTrail.println("[000] PersonnelTable.isLDAPPassword Principal: " + ldapUser2);
    	         
    	         if (ldapUser2.equals(""))    // JH 2011-09-28
    	         {
    	        	 AuditTrail.println("[400] PersonnelTable.isLDAPPassword InValid Principal");
    	        	 isValid = false;   // JH 2011-09-28
    	         }
    	         
    	      } else {
    	    	  AuditTrail.println("[500] PersonnelTable.isLDAPPassword email attribute not found");
    	    	  isValid = false;
    	      }
    	     } catch (javax.naming.PartialResultException ex2)
    	     {

    	    	// AuditTrail.println("[000] PersonnelTable.isLDAPPassword Referral detected and ignored");
    	    	AuditTrail.println("[500] PersonnelTable.isLDAPPassword invalid - Referral detected (Check search string and filter in web.xml)");
    	    	isValid = false;   // JH 2011-09-28

    	     }
    	  }
      	  catch (javax.naming.NamingException ex) 
      	  {
      		AuditTrail.println("[500] PersonnelTable.isLDAPPassword Directory Search NamingException: " + ex);
      		isValid = false;
      	  } 
      	  catch (java.lang.Exception ex)
    	  {
    	     AuditTrail.println("[400] PersonnelTable.isLDAPPassword Directory Search Exception: " + ex); 
    	      isValid = false;
    	  } 

    	}

    	try {
    	   ctx.close();
    	} catch (java.lang.Exception ex)
  	    {
   	       AuditTrail.println("[400] PersonnelTable.isLDAPPassword Close Exception: " + ex); 
   	    } 

    	if (isValid) {


    	   java.util.Hashtable env2 = new java.util.Hashtable ();
           isValid = false;

    	   //String ldapUser2 = "ldap@st-hubert.qc.ca";  //replace with the nc parser
    	
    	    try {
    	      //set the name of domain with the user name
    	      env2.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, LDAPFactory);
    	      env2.put(javax.naming.Context.PROVIDER_URL,LDAPProvider);
    	
    	      AuditTrail.println("[000] PersonnelTable.isLDAPPassword USER: " + ldapUser2);
    	
    	      env2.put(javax.naming.Context.SECURITY_AUTHENTICATION,"simple");
    	      //set user related information
    	      env2.put(javax.naming.Context.SECURITY_PRINCIPAL, ldapUser2);
    	      //set user password
    	      env2.put(javax.naming.Context.SECURITY_CREDENTIALS, xPassword);
    	      //validate user

    	      javax.naming.Context ctx2 = new javax.naming.directory.InitialDirContext(env2);

    	      isValid = true;
    	      AuditTrail.println("[255] PersonnelTable.isLDAPPassword USER validated: " + ldapUser2);

    	    }
    	    catch (javax.naming.AuthenticationException ex)
    	    {
    		  AuditTrail.println("[000] PersonnelTable.isLDAPPassword USER AuthenticationException: " + ex);

         	}
        	catch (javax.naming.NamingException ex) 
    	    {
    		   AuditTrail.println("[500] PersonnelTable.isLDAPPassword USER NamingException is: " + ex);
        	}
    	    catch (java.lang.Exception ex) 
    	    {
    		   AuditTrail.println("[500] PersonnelTable.isLDAPPassword USER Exception: " + ex);
    	    }


    	}
    	return isValid;

    }

    
}

