/*
PrepopulatedMapping.java - assigns expense types based in vendor info  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.net.*;
//import java.io.*;
//import java.util.*;
//import java.sql.*;
//import org.jdom.Document;
import org.jdom.Element;
//import org.jdom.DocType;
import org.jdom.Attribute;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;


public class PrepopulatedMapping extends AdisoftDOM
{
  static AuditTrail AuditLog;  

  private String vendorCard;
  private String sic;
  private String amount;
  private String reference;
  
  private Element mappedInfo;
  private boolean status;
  
  public PrepopulatedMapping () 
  {
  }
  public void setVendorCardSIC(String inputVendorCard, String inputSIC)
  {
        setVendor(inputVendorCard, false);
        setSIC(inputSIC, true);
  }  
  public void setVendorCard(String inputVendorCard)
  {
        setVendor(inputVendorCard, true);
  }  
  public void setSIC(String inputSIC)
  {
        setSIC(inputSIC, true);
  }  
  public void setVendor(String inputVendorCard, boolean recalculate)
  {
        vendorCard = inputVendorCard;
        if (recalculate) status = setPrepopMapping();
  }  
  public void setSIC(String inputSIC, boolean recalculate)
  {
        sic = inputSIC;
        if (recalculate) status = setPrepopMapping();
  }  
  public void setAmount(String inputAmount)
  {
        amount = inputAmount;
  }  
  public void setReference(String inputReference)
  {
        reference = inputReference;
  }  

  public boolean getStatus()
  {
    return status;
  }

  public String getScreen()
  {
    return getMapValue("screen");
  }  
  public String getExpenseType()  
  {
    return getMapValue("expensetype");
  }  
  public String getPaymentMethod()
  {
    return getMapValue("paymentmethod");
  }  
  public String getExpenseAmount()
  {
    String retVal = "";
    if (isMapThere("expenseamount")){
        retVal = amount;
    }
    return retVal;
  }  
  public String getExpenseComment()
  { 
    String retVal = "";
    if (isMapThere("expense_1_comment")){
        retVal = getMapValue("expense_1_comment");
    }
    return retVal;
  }  
  
  private boolean isMapThere(String name)
  {
    boolean retVal = false; 
    if (mappedInfo != null)
    {
       Element x = mappedInfo.getChild(name);
       if (x != null) retVal = true;
    }
    return retVal;
  }
  
  private String getMapValue(String name)
  {
    String retVal = ""; 
    if (mappedInfo != null)
    {
       Element x = mappedInfo.getChild(name);
       if (x != null) retVal = x.getText();
    }
    return retVal;
  }
  
  
//This can be generalized later for a generic mapping routine \|/\|/\|/\|/\|/\|/\|/

  private boolean setPrepopMapping() {
    boolean retVal = true;
    Element mapElement;
    Element x = super.getElement("prepopulatedmap");
    // Need to change later to allow for a sic code range
    mapElement = getElementWhereExact(x, "vendorcard","sic",vendorCard,sic);
    if (mapElement == null)
    {
        mapElement = getElementWhereWildCard(x, "vendorcard","sic",vendorCard,sic);
    } 
    if (mapElement == null)
    {
        mapElement = getElementWhereExact(x, "vendorcard","sic",vendorCard,"*");
    } 
    if (mapElement == null)
    {
        mapElement = getElementWhereExact(x, "vendorcard","sic","*",sic);
    } 
    if (mapElement == null)
    {
        mapElement = getElementWhereWildCard(x, "vendorcard","sic","*",sic);
    } 
    if (mapElement == null)
    {
        mapElement = getElementWhereExact(x, "vendorcard","sic","*","*");
    } 
    if (mapElement == null)
    {
        AuditTrail.println("[500] PrepopulatedMapping.setPrepopMapping - Mapping element has not been found.");
        retVal = false;
    }
    mappedInfo = mapElement;
    return retVal;
  }
  
  public Element getElementWhereExact(Element x, String whereAttribute1, String whereAttribute2, String whereValue1, String whereValue2) {
    /*
    Get element (in this element) where Attribute1 = 3 and Attribute2 = 4 would
    return the second element which contains HotuMotu and ABC.  Will return a null
    if there is nothing found.

      <element_x>
        <rowset whereAttribute1="1" whereAttribute2="2">
          <Agent>KalaKala</Agent>
          <Return>XYZ</Agent>
        </rowset>
        <rowset whereArribute1="3" whereAttribute2="4">
          <Agent>HotuMotu</Agent>
          <Return>ABC</Agent>
        </rowset>
      </element_x>
 
    </root>
    */
	AuditTrail.println("[000] PrepopulatedMapping.getElementWhereExact match: " + whereValue1 + " and " + whereValue2);  
    Element rowset;
    Attribute number1;
    Attribute number2;
    String value1;
    String value2;
    
    Element retVal = null;
     
    java.util.List rowList = x.getChildren();
    int i = 0;
      if (rowList.size() > 0) {
        do {
            
          rowset = (Element) rowList.get(i);  // get a row from the mapping element
          number1 = (Attribute) rowset.getAttribute(whereAttribute1);
          number2 = (Attribute) rowset.getAttribute(whereAttribute2);
          value1 = number1.getValue();
          value2 = number2.getValue();
          if (value1 != null && value2 != null && value1.equals(whereValue1) && value2.equals(whereValue2) ) 
          { 
                retVal = rowset;
                i = rowList.size();
          }    // not equals is not an error condition.
          i = i + 1;
        } while (i < rowList.size());
      }
      return retVal; 
  }
  
  public Element getElementWhereWildCard(Element x, String whereAttribute1, String whereAttribute2, String whereValue1, String whereValue2) {
	    /* See above for a map of the xml file
	    * This does not allow for hierarchical stuff (cannot do 123* versus 12*) but only absolute (1234 versus 12*). 
	    * Hierarchical may be accomplished via order (maybe but not recommended).
	    */

		AuditTrail.println("[000] PrepopulatedMapping.getElementWildCard match: " + whereValue1 + " and " + whereValue2);  
	    Element rowset;
	    Attribute number1;
	    Attribute number2;
	    String value1;
	    String value2;
	    
	    Element retVal = null;
	     
	    java.util.List rowList = x.getChildren();
	    int i = 0;
	      if (rowList.size() > 0) {
	        do {
	            
	          rowset = (Element) rowList.get(i);  // get a row from the mapping element
	          number1 = (Attribute) rowset.getAttribute(whereAttribute1);
	          number2 = (Attribute) rowset.getAttribute(whereAttribute2);
	          value1 = number1.getValue();
	          value2 = number2.getValue();
	          if (value1 != null && value2 != null && value1.equals(whereValue1) && !value2.equals("*") && (whereValue2.indexOf(value2.replace("*", "")) == 0))  
	          { 
	                retVal = rowset;
	                i = rowList.size();
	          }    // not equals is not an error condition.
	          i = i + 1;
	        } while (i < rowList.size());
	      }
	      return retVal; 
	  }
	  
  
}