/*
ProcessESSReport.java - instantiates report in database and starts workflow   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.io.*;
import java.util.*;
import java.sql.*;
//import java.sql.*;
import org.jdom.Document; //See if I can find this
import org.jdom.Element;
//import org.jdom.DocType;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;


public class ProcessESSReport implements ProcessReport
{
//expense report status

    static AuditTrail Logfile;
    String DBUrl = "";
    String DBUser = "";
    String DBPassword = "";
    String xreference = "";
    AdisoftDbase db;
    StateTable st;
    ExpenseReportDOM er;
    XML2ESS ESS;
    String reportRoutingRule = "default";
    
    String processErrorMessage = null;
    PersonnelTable reporter;
    PersonnelTable Submitter;  //needs to be set from outside
    
    boolean msgSentOK = false; 
    
    private Connection dbConn;
    private AdisoftDOM DepartDOM = new AdisoftDOM();
    private AdisoftDOM StatusDOM = new AdisoftDOM();

    ServerSystemTable systemTable;
    private ESSRules rules;
    private String approverEmailAdress;
    
    private String SQL_Find = "SELECT RP_STAT FROM REPORT WHERE PERS_NUM = '$persnum$' AND PVOUCHER = '$pvoucher$' AND RP_STAT <> 'F0' AND RP_STAT <> 'F1' AND RP_STAT <> 'F2';";
//    private String SQL_Find = "SELECT RP_STAT FROM REPORT WHERE RTRIM(PERS_NUM) = '$persnum$' AND PVOUCHER = '$pvoucher$' AND RP_STAT <> 'F0' AND RP_STAT <> 'F1' AND RP_STAT <> 'F2';";
    private String SQL_Char_Update = "UPDATE REPORT SET $column$ = '$value$' WHERE PERS_NUM = '$persnum$' AND PVOUCHER = '$pvoucher$' AND RP_STAT <> 'F0' AND RP_STAT <> 'F1' AND RP_STAT <> 'F2';";
    private String SQL_Update = "UPDATE REPORT SET $column$ = $value$ WHERE PERS_NUM = '$persnum$' AND PVOUCHER = '$pvoucher$' AND RP_STAT <> 'F0' AND RP_STAT <> 'F1' AND RP_STAT <> 'F2';";
    private String SQL_Avoid_Duplicate = "UPDATE REPORT SET RP_STAT = 'F0' WHERE PERS_NUM = '$persnum$' AND PVOUCHER = '$pvoucher$' AND RP_STAT  = '$status$'  AND VOUCHER <> '$voucher$';";
    private String Message_Email = "Entered By";
    private String Message_Reporter = "Report For";
    private String Message_Voucher = "User's Reference";
    private String Message_PVoucher = "Central Voucher";
    private String Message_RouteTo = "Routed to";
    private String Message_Subject = "Expense Report"; 
    private String Message_new_line = "\n";

    private String Message_PVStart = "0";
    private int pVStart = 0;
    private String sqlTerminator = ";";
    private String submitStatus = "ST_SUBMIT_ON";
    
    private String dateFormatStr = "MM/dd/yyyy";
    private String decimalStr = ".";
    private String separatorStr = ",";
    
    private ess.Language myLang = new ess.Language();
    
    private String messageBodyType = "body=8BITMIME";
    private String contentType = "text/plain; charset=utf-8";
    
    public void setDB(String DBUrl, String User, String Password) {
        this.DBUrl = DBUrl;
        DBUser = User;
        DBPassword = Password;
        db = new AdisoftDbase(DBUrl, User, Password );
        dbConn = db.getConnection();
    }
    public void setDB(String DBUrl, String User, String Password, String SQLTerminator) {
        init(DBUrl, User, Password, SQLTerminator);
    }
    
    public void setConnection(Connection dbConnection) {
        dbConn = dbConnection;
        db = new AdisoftDbase();
        db.setConnection(dbConn);
    }
    
    public void setSQLTerminator(String X) {
        sqlTerminator = X;
    }
// ******* next two are related ******

    public void setSubmitter(PersonnelTable x) {
        Submitter = x;
    }
    
    public void setLanguage(String x){
        myLang.setLanguage(x);
    }

    public void setLanguage(String x, String y){
        myLang.setLanguage(x,y);
    }

    public void setDateFormat(String x) {
        dateFormatStr = x;
    }
        
    public void setDecimal(String x) {
        decimalStr = x;
    }
   
    public void setSeparator(String x) {
        separatorStr = x;
    }
    
    public void setMessageBodyType(String x) {
    	messageBodyType = x;
    }
    public void setContentType(String x) {
    	contentType = x;
    }

    private boolean isAuditor() {
        boolean retVal = false;
        if (Submitter != null) retVal = Submitter.isAuditor();
        return retVal;
    }
        

    public void init(String DBUrl, String User, String Password, String SQLTerm)
    {
        this.DBUrl = DBUrl;
        DBUser = User;
        DBPassword = Password;
        db = new AdisoftDbase(DBUrl, User, Password );
        dbConn = db.getConnection();
        sqlTerminator = SQLTerm;
        setUpSystemFiles();
    }
    
    public void init(Connection dbConnection, String SQLTerm)
    {
        dbConn = dbConnection;
        db = new AdisoftDbase();
        db.setConnection(dbConn);
        sqlTerminator = SQLTerm;
        setUpSystemFiles();
    }
    
    public void setUpSystemFiles()
    {
        st = new StateTable(dbConn, sqlTerminator);
        rules = new ESSRules(dbConn, sqlTerminator);
        systemTable = new ServerSystemTable();
        systemTable.setConnection(dbConn);
        systemTable.setSQLTerminator(sqlTerminator);
        reporter = new PersonnelTable();
        reporter.setConnection(dbConn);
        reporter.setSQLTerminator(sqlTerminator);
        reporter.setSQLStrings();

        if (!DepartDOM.getDOMProcessed()) {
          DepartDOM.setConnection(dbConn);
          DepartDOM.setSQLTerminator(sqlTerminator);
          DepartDOM.setDOMFromTable("DEPART"); 
        }
        if (!StatusDOM.getDOMProcessed()) {
            String status_file = systemTable.getSystemString("xmlstatus","C:\\WINEXP\\xmls\\Status.xml"); 
            java.io.File StatusFile = new java.io.File(status_file);
            StatusDOM.setDOM(StatusFile); 
        }
    }
    
    public void setSQLStrings(AdisoftDOM SystemDOM) {
        //rewrite this for to do the SysTable SQL from this object
        AuditTrail.println("[000] ProcessESSReport.setSQLStrings from SystemDOM parameter");
        SQL_Find = SystemDOM.getDOMTableValueFor("reportstatus","findsql",SQL_Find);
        SQL_Char_Update= SystemDOM.getDOMTableValueFor("reportstatus","updatecharsql",SQL_Char_Update);
        SQL_Update= SystemDOM.getDOMTableValueFor("reportstatus","updatesql",SQL_Update);
        SQL_Avoid_Duplicate = SystemDOM.getDOMTableValueFor("reportstatus","avoidduplicatesql",SQL_Avoid_Duplicate);
        Message_PVStart = SystemDOM.getDOMTableValueFor("messages","personal_offset",Message_PVStart);
        pVStart = Integer.parseInt(Message_PVStart);
        submitStatus = SystemDOM.getDOMTableValueFor("reportstatus","submitstatus",submitStatus);
        
        Message_Email = myLang.getString("email_name",SystemDOM.getDOMTableValueFor("messages","email_name",Message_Email));
        Message_Reporter = myLang.getString("email_reporter",SystemDOM.getDOMTableValueFor("messages","email_reporter",Message_Reporter));
        Message_Voucher = myLang.getString("central_reference",SystemDOM.getDOMTableValueFor("messages","central_reference",Message_Voucher));
        Message_PVoucher = myLang.getString("personal_reference",SystemDOM.getDOMTableValueFor("messages","personal_reference",Message_PVoucher));
        Message_RouteTo = myLang.getString("email_routeto",SystemDOM.getDOMTableValueFor("messages","email_routeto",Message_RouteTo));
        Message_Subject = myLang.getString("email_subject",SystemDOM.getDOMTableValueFor("messages","email_subject",Message_Subject));
        Message_new_line = SystemDOM.getDOMTableValueFor("messages","email_new_line",Message_new_line);
        
        messageBodyType = SystemDOM.getDOMTableValueFor("messages","messagebodytype",messageBodyType);
        contentType = SystemDOM.getDOMTableValueFor("messages","contenttype",contentType);
        
    }  
    
    public void setSQLStrings() {
        //rewrite this for to do the SysTable SQL from this object
        ServerSystemTable SysTable = new ServerSystemTable();
        AdisoftDOM SystemDOM = new AdisoftDOM();
        SysTable.setConnection(dbConn);
        SysTable.setSQLTerminator(sqlTerminator);
        String system_file = SysTable.getSystemString("XMLSYSTEM","C:\\WORK\\"); 
        java.io.File SystemFile = new java.io.File(system_file);
        SystemDOM.setDOM(SystemFile); 
        AuditTrail.println("[000] ProcessESSReprot.setSQLStrings from " + system_file);
        SQL_Find = SystemDOM.getDOMTableValueFor("reportstatus","findsql",SQL_Find);
        SQL_Char_Update= SystemDOM.getDOMTableValueFor("reportstatus","updatecharsql",SQL_Char_Update);
        SQL_Update= SystemDOM.getDOMTableValueFor("reportstatus","updatesql",SQL_Update);
        SQL_Avoid_Duplicate = SystemDOM.getDOMTableValueFor("reportstatus","avoidduplicatesql",SQL_Avoid_Duplicate);
        Message_PVStart = SystemDOM.getDOMTableValueFor("messages","personal_offset",Message_PVStart);
        pVStart = Integer.parseInt(Message_PVStart);
        
        Message_Email = myLang.getString("email_name",SystemDOM.getDOMTableValueFor("messages","email_name",Message_Email));
        Message_Reporter = myLang.getString("email_reporter",SystemDOM.getDOMTableValueFor("messages","email_reporter",Message_Reporter));
        Message_Voucher = myLang.getString("central_reference",SystemDOM.getDOMTableValueFor("messages","central_reference",Message_Voucher));
        Message_PVoucher = myLang.getString("personal_reference",SystemDOM.getDOMTableValueFor("messages","personal_reference",Message_PVoucher));
        Message_RouteTo = myLang.getString("email_routeto",SystemDOM.getDOMTableValueFor("messages","email_routeto",Message_RouteTo));
        Message_Subject = myLang.getString("email_subject",SystemDOM.getDOMTableValueFor("messages","email_subject",Message_Subject));
        Message_new_line = SystemDOM.getDOMTableValueFor("messages","email_new_line",Message_new_line);
        
        submitStatus = SystemDOM.getDOMTableValueFor("reportstatus","submitstatus",submitStatus);
        
        messageBodyType = SystemDOM.getDOMTableValueFor("messages","messagebodytype",messageBodyType);
        contentType = SystemDOM.getDOMTableValueFor("messages","contenttype",contentType);

    }  
    
    public void run(Document msgDOM, String xreference)      //jh 11-21-2002
    {
        final int REPLACE_OLD_REPORT = 0;
        final int ADD_REPORT_TO_DATABASE = REPLACE_OLD_REPORT + 1;
        final int AVOID_DUPLICATE_REPORT = ADD_REPORT_TO_DATABASE + 1;
        final int AUDITOR_SUBMISSION = AVOID_DUPLICATE_REPORT + 1;
        final int GUIDELINE = AUDITOR_SUBMISSION + 1;
        final int RANDOM_SELECT = GUIDELINE + 1;
        final int SEND_ALERT = RANDOM_SELECT + 1;
        final int ALL_DONE = SEND_ALERT + 1;
        
        er = new ExpenseReportDOM(msgDOM);
        this.xreference = xreference;
        String persNum = er.getHeaderData("persnum");
        String reportNumber;
        if (xreference.equals("")) 
            reportNumber = er.getHeaderData("reference");   //jh 11-21-2002
        else 
            reportNumber = xreference;
        int task = REPLACE_OLD_REPORT;
        int alertType = 0;
        boolean status = true;
        Vector reportStatus = new Vector();
        String currentStatus;
        String promotedStatus;
        
        
        while (task < ALL_DONE)
        {
                    
            switch (task)
            {
                case REPLACE_OLD_REPORT:
                    if (replaceOldReport(er) == rules.REPORT_ALREADY_PROCESSED)
                    {
                        alertType = rules.REPORT_ALREADY_PROCESSED;
                        reportStatus.addElement(rules.getCondString(rules.REPORT_ALREADY_PROCESSED));
// set it up to send rejection msg                        
                        task = SEND_ALERT - 1;
                        processErrorMessage = "Report with this reference has already been fully processed.";
                    }
                    break;
                    
                case ADD_REPORT_TO_DATABASE:  //automatic
                    reportStatus = addReportToDatabase(msgDOM);
                    if (reportStatus.size() > 0)
                    {
                        alertType = rules.BAD_REPORT_CONTENTS;
                        task = SEND_ALERT - 1;   
                    } else {
                        reportRoutingRule = this.getRoutingRuleName(ESS.getReportDepartment(), reporter.getDepartment());
                    }

                    break;
                    
                case AVOID_DUPLICATE_REPORT:  
                    String SQLString = db.SQLReplace(SQL_Avoid_Duplicate, "$status$", ESS.getReportStatus());
                    SQLString = db.SQLReplace(SQLString,"$voucher$", ESS.getVoucherNumber());
                    updateReportInfo(SQLString, "", "", persNum, reportNumber);
                    break;             
                    
                case AUDITOR_SUBMISSION:  
                    //suggest refactoring the names ESSToNew and NewToESS as they are really just
                    //translation between internal values and table values right now.
                    currentStatus = st.ESSToNew("default", ESS.getReportStatus());
                    final String submitMethod = "AUDITORSUBMIT";
                    promotedStatus = null;
                    if (st.canPerform(currentStatus, submitMethod) == true)
                    {
                        /* Check if this person who submitted the report is
                           set as an auditor
                        */
                        if (this.isAuditor() && !Submitter.getPersNum().equals(ESS.getPersNum()))
                        {
                           promotedStatus = st.NewToESS(reportRoutingRule,st.getPositiveStatus(currentStatus, "AUDITORSUBMIT"));
                        } else {
                           promotedStatus = st.NewToESS(reportRoutingRule,st.getNegativeStatus(currentStatus, "AUDITORSUBMIT"));    
                        }
                        //Need to get rid of the "RP_STAT" hardcode - maybe return the name of the RP_STAT field.
                        ESS.setReportStatus(promotedStatus);
                        updateReportInfo("RP_STAT",promotedStatus,persNum,reportNumber);
                    }
                    break;             
                    
                case GUIDELINE:

                    String ourStatus = st.ESSToNew("default", ESS.getReportStatus());
                    String newStatus;
                    String originalStatus = submitStatus;  //was final, now gotten from the top jh 5-21-04
                    final String guideMethod = "GUIDELINE";
                    String GuidePassName = StatusDOM.getDOMTableValueWhere(reportRoutingRule, "status", originalStatus, "updatesqlsigner");
                    String GuidePassDate = StatusDOM.getDOMTableValueWhere(reportRoutingRule, "status", originalStatus, "updatesqldate");
                    if (st.canPerform(ourStatus, guideMethod) == true)
                    {
                        status = checkGuideline(msgDOM);
                        updateReportInfo("MANAGER","",persNum,reportNumber);  //jh 4-11-2003 route2 field
                            
                        if (status == true)
                        {
                            newStatus = st.NewToESS("default",st.getPositiveStatus(originalStatus, guideMethod));
                            AuditTrail.println("[000] ProcessESSReport.run Guide returns true - " + newStatus); 
                            updateReportInfo("RP_STAT",newStatus,persNum,reportNumber);
                            ESS.setReportStatus(newStatus);
                        //low cohesion with approving reports with the approval module here.
                        //need to review how a passed report is using the "ESS" stamp.
                        //should add another parameter to the Status - default area... (autoapproval??)
                            if (!GuidePassName.equals(""))updateReportInfo(GuidePassName,"ESS",persNum,reportNumber);
                            if (!GuidePassDate.equals("")) {
                                 CustomDate Dt = new CustomDate();
                                 String newAsOfDate = CustomDate.xBaseDate.format(Dt.date);
                                 AdisoftDOM SystemDOM = systemTable.xml.getAdisoftDOM();
                                 String SQLDate = SystemDOM.getDOMTableValueFor("sql","datesql");
                                 String SQLType = SystemDOM.getDOMTableValueFor("sql","dateformat");
                                 if (SQLType.equals("MM/DD/YYYY")) { 
                                    SQLDate = db.SQLReplace(SQLDate,"$date$",CustomDate.simpleDate.format(Dt.date));
                                 } else if (SQLType.equalsIgnoreCase("DD-MMM-YYYY")) {    // -- MM/DD/YYYY
                                    SQLDate = db.SQLReplace(SQLDate,"$date$",CustomDate.oracleDate.format(Dt.date));
                                 } else { //generate YYYY-MM-DD
                                    SQLDate = db.SQLReplace(SQLDate,"$date$",newAsOfDate);
                                 } 
                                 updateReportInfo(SQL_Update,GuidePassDate,SQLDate,persNum,reportNumber); //jh 2003-04-21
                            }
                        }
                        else
                        {
                            newStatus = st.NewToESS("default",st.getNegativeStatus(originalStatus, guideMethod));
                            AuditTrail.println("[000]  ProcessESSReport.run Guide returns false");
                            updateReportInfo("RP_STAT",newStatus,persNum,reportNumber);
                            ESS.setReportStatus(newStatus);
                            reportStatus.addElement(rules.getCondString(rules.FAILED_GUIDELINE));
                            alertType = rules.FAILED_GUIDELINE;
                        }
                    }
                    break;
                    
                case RANDOM_SELECT:   //automatic
                    //suggest refactoring the names ESSToNew and NewToESS as they are really just
                    //translation between internal values and table values right now.
                    currentStatus = st.ESSToNew(reportRoutingRule, ESS.getReportStatus());
                    final String randomMethod = "RANDOMSELECT";
                    if (st.canPerform(currentStatus, randomMethod) == true)
                    {
                        /* Do the random select and set the status appropriately
                        1. get the sample size from the Status file
                        2. instansiate the Random select class
                        3. get the return value
                        4. get the right value from the state table
                        */
                        String sampleString = StatusDOM.getDOMTableValueWhere(reportRoutingRule,"translation",ESS.getReportStatus(),"randomsampleratio");
                        if (!sampleString.equals(""))
                        {
                           double sampleRatio = Double.valueOf(sampleString).doubleValue(); 
                           java.util.Random r = new java.util.Random();
                           double d = r.nextDouble();   
                           double sampleSelect = d * sampleRatio;
                           promotedStatus = null;
                           if (sampleSelect < (double) 1)
                           {
                               promotedStatus = st.NewToESS(reportRoutingRule,st.getPositiveStatus(currentStatus, "RANDOMSELECT"));
                           } else {
                               promotedStatus = st.NewToESS(reportRoutingRule,st.getNegativeStatus(currentStatus, "RANDOMSELECT"));    
                           }
                           //Need to get rid of the "RP_STAT" hardcode - maybe return the name of the RP_STAT field.
                           updateReportInfo("RP_STAT",promotedStatus,persNum,reportNumber);
                           ESS.setReportStatus(promotedStatus);
                        } else {
                           AuditTrail.println("[500] ProcessESSReport.run: RANDOM_SELECT - no ratio for " + currentStatus);   
                        }
                    }
                 
                    break;
                    
                case SEND_ALERT:   //automatic
                    if (alertType == 0)
                        reportStatus.addElement(rules.getCondString(rules.GOOD_REPORT));
                    status = sendAlert(msgDOM, alertType, reportStatus, er);
                    break;

                default:
                    AuditTrail.println("[500] ProcessESSReport.run: INVALID TASK ID");
                    break;
                
            }
            task++;
        }
   }
   
   public boolean isMessageSentOK() 
   {
       return msgSentOK;
   }

   public String getStatus()
   {
        return((ESS==null) ? "" : ESS.getReportStatus());
   }
   
   public Vector getErrorLines()
   {
     return((ESS==null) ? new Vector() : ESS.getStatus());
   }
   
   public String getProcessErrorMessage()
   {
        return processErrorMessage;
   }
   
   public boolean isPersistanceOK()
   {
        return((ESS==null) ? false : ESS.isPersistanceOK());
   }
   
   public String getVoucherNumber()
   {
        return((ESS==null) ? "" : ESS.getVoucherNumber());
   }
   
   public void close()
    {
        if (systemTable != null)
        {
            systemTable.close();
            systemTable = null;
        }
        if (db != null)
        {
            db.close();
            db = null;
        }
        if (rules != null)
        {
            rules.close();
            rules = null;
        }
        if (st != null)
        {
            st.close();
            st = null;
        }
    }

   private int replaceOldReport(ExpenseReportDOM er)
   {
        int status = rules.REPORT_DOES_NOT_EXIST;
        String reportStatus;
        String reportNumber;
        if (xreference.equals("")) {
            reportNumber = er.getHeaderData("reference");   //jh 11-21-2002
        } else {
            reportNumber = xreference;
        }
        String persNum = er.getHeaderData("persnum");
        
        reportStatus = findESSReport(reportNumber, persNum);

        AuditTrail.println("[000] ProcessESSReport.replaceOldReport " + xreference + " reportStatus = " + reportStatus);

        if (reportStatus != "")
        {
            //Consider the subTable determination for future flexibility to replace "default"
            //and also can replace the "ST_REJECTED" with the value from the 
            if (st.canPerform(st.ESSToNew("default",reportStatus), "REPLACEOLD") == true)
                updateReportInfo("RP_STAT", st.NewToESS("default","ST_REJECTED"),
                       persNum, reportNumber );
            else
                status = rules.REPORT_ALREADY_PROCESSED;
        }
        return(status);
   }
    
    private String findESSReport(String reportNumber, String persNum)
    {
        ResultSet rs = null;
        String status = "";
        String SQLString = "SQL note specified";
        
        try
        {
            SQLString = db.SQLReplace(SQL_Find, "$persnum$", persNum);
            SQLString = db.SQLReplace(SQLString,"$pvoucher$", reportNumber);
            AuditTrail.println("[000] ProcessESSReport.findESSReport reportStatus = " + SQLString);
            if (db.setResultSet(SQLString) == true)
            {
                rs = db.getResultSet();
                status = rs.getString(1);
//                System.out.println("rs RP_STAT = " + status);
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
           AuditTrail.println("[500] ProcessESSReport.findESSReport - error");
        }
        return (status);
    }
    
    private void updateReportInfo(String columnId, String columnValue, 
                      String persNum, String reportNumber)
    {
        String statement2Use = SQL_Char_Update;
        updateReportInfo(statement2Use, columnId, columnValue, persNum, reportNumber);
    }
    
    private void updateReportInfo(String statement2Use, String columnId, String columnValue, 
                      String persNum, String reportNumber)
    {
        String SQLStatement = db.SQLReplace(statement2Use, "$persnum$", persNum);
        SQLStatement = db.SQLReplace(SQLStatement,"$pvoucher$", reportNumber);
        SQLStatement = db.SQLReplace(SQLStatement,"$column$", columnId);
        SQLStatement = db.SQLReplace(SQLStatement,"$value$", columnValue);
       
        db.doSQLExecute(SQLStatement);
    }
    
    
    
    
    private Vector addReportToDatabase(Document msgDom)
    {
        Element root;
        Element reportEntry;
        String errorLine;
        
        root = msgDom.getRootElement();
        reportEntry = root.getChild("report");
        ESS = new XML2ESS();
        ESS.setConnection(dbConn);
        ESS.setSQLTerminator(sqlTerminator);
        ESS.setUpSystemFiles();
        
        //ESS.setDateFormat(Submitter.getDateFormat());
        
        // ESS.setLanguage(Submitter.getLanguage());
        ESS.setDateFormat(Submitter.getDateFormat());
        ESS.setDecimal(Submitter.getDecimal());
        ESS.setSeparator(Submitter.getSeparator());

        
        boolean USENEWVOUCHER = true;
        ESS.setESSReport(reportEntry, 0, USENEWVOUCHER);
        ESS.Rep.PVOUCHER = xreference;
        ESS.isValidReport();
        Vector reportStatus = ESS.getStatus();
// log error: Pers_num not found, invalid charge or expense types, etc.        
        if (reportStatus != null && reportStatus.size() > 0)            //check Karen on this logic
        {
            //Consider the subTable determination for future flexibility replace "default"
            ESS.setReportStatus(st.NewToESS("default","ST_REJECTED"));
            AuditTrail.println("[500] ProcessESSReport.addReportToDatabase: Bad Report Contents. ");
            for ( int i = 0; i < reportStatus.size(); i++) {
               errorLine = (String) reportStatus.elementAt(i);
               AuditTrail.println("[500] ProcessESSReport.addReportToDatabase: " + errorLine);
            }
        }
        ESS.setAccountingInfo();
        ESS.setPersistance();
        if (ESS.isPersistanceOK() == false)
        {
            reportStatus.addElement(rules.getCondString(rules.PERSISTANCE_FAILED));
        }
        
        return(reportStatus);
    }
    
    private boolean checkGuideline(Document msgDom)
    {   
        Document newDom = new Document( (Element) msgDom.getRootElement().getChild("report").clone());
        Guideline guideline = new Guideline();
        guideline.setConnection(dbConn);
        guideline.setSQLTerminator(sqlTerminator);
        guideline.setLanguage(myLang);
        guideline.setDateFormat(dateFormatStr);
        guideline.setDecimal(decimalStr);
        guideline.setSeparator(separatorStr);

        guideline.setUpFiles();
        //Logfile.println("[000] Guideline.msgDom:\n" + newDom.toString());
        guideline.setReport(newDom);
        //Logfile.println("[000] Guideline.toString:\n" + guideline.toString());
        return(guideline.getBooleanStatus());
    }
    
    private boolean sendAlert(Document msgDOM, int alertType, Vector statusList, ExpenseReportDOM er)
    {
        Element root = msgDOM.getRootElement();
        String replyTo = root.getChild("reply-to").getText();
        String reference = root.getChild("reference").getText();
        String message;
        String emailMessage = "";
        String notifyMessage;
        String from = systemTable.getSystemString("PAL_EMAIL_ADDRESS","expense@expenseservices.com");
        String smtpServer = systemTable.getSystemString("SMTP_ADDRESS","207.0.0.1");
        String smtpPort = systemTable.getSystemString("SMTP_PORT","25");
        String smtpDomain = systemTable.getSystemString("SMTP_DOMAIN","expenseservices.com");
        String smtpDelimiter = systemTable.getSystemString("SMTP_DELIMITER",".");
        boolean notifyFlag = true;
        
        SMTPSingle smtp = new SMTPSingle();
        smtp.setIP_URL(smtpServer);
        smtp.setPort(smtpPort);
        smtp.setDomain(smtpDomain);
        smtp.setDelimiter(smtpDelimiter);
        smtp.setMessageBodyType(messageBodyType);
        smtp.setContentType(contentType);
        
        msgSentOK = true; //if no message sent it is OK.
          
        message =  myLang.getString("email_msg1") + " "; 
        message += Message_new_line + Message_new_line + "  1. " + Message_Email + ": " + replyTo;
        if (ESS != null) 
        {
            message += Message_new_line + "  2. " + Message_Reporter + ": " +  ESS.getReportName();
            message += Message_new_line + "  3. " + Message_Voucher + ": " +  ESS.getVoucherNumber();
            message += Message_new_line + "  4. " + Message_PVoucher + ": " +  reference.substring(pVStart);
            
            String subjectLine = Utilities.replaceStr(Message_Subject, "$name$", ESS.getReportName());
            subjectLine = Utilities.replaceStr(subjectLine, "$voucher$", ESS.getVoucherNumber());
            smtp.setSubject(subjectLine);
        }
        //if (ESS != null) message += "\n3. " + Message_PVoucher + ": " +  reference + "\n\n";
        Vector notifications = rules.setRoutingObjects("expensereport", statusList);
        
        int i = 0;
        String to = "";
        String sendTo = "";
        String prevTo = "";
        String receiptMsg = "";
        
        //The process below only works because the array is sorted the the ESS.ESS*** string
        //If an ESS.ESSAdmin is added, the routing item #5 won't be filled correctly
        
        while (i < notifications.size())
        {
            to = rules.getRoutingObject(notifications.elementAt(i));
            AuditTrail.println("[000] ProcessESSReport.sendAlert route object: " + to);
            receiptMsg = "";
                            
            if (to.equals(prevTo) == false)
            {
                if ((emailMessage.length() > 0 ) && (sendTo.length() > 0 ))
                {
                   msgSentOK = msgSentOK && smtp.setSMTPMessage (emailMessage, sendTo, from); 
                }
                   // msgSentOK = smtp.setSMTPMessage (emailMessage, sendTo, from); 
                emailMessage = message;
                prevTo = to;
                //Don't like the hard coding of the following check
                if ((to.toUpperCase()).equals("ESS.ESSSUBMITTER"))
                {
                    sendTo = replyTo;
                    if (ESS == null)
                    {
                        receiptMsg = Message_new_line + myLang.getString("email_handling_error") + Message_new_line;
                    }
                     else if (ESS.getPhysicalReceipts() > 0) 
                    {
                        receiptMsg = Message_new_line + myLang.getString("email_receipts") + Message_new_line;  //Later put in call to system.xml
                    } else {
                        receiptMsg = Message_new_line + myLang.getString("email_no_receipts") + Message_new_line;
                    }
                } else if ((to.toUpperCase()).equals("ESS.ESSACCOUNTING"))
                {
                    sendTo = systemTable.getSystemString("PAL_EMAIL_ADDRESS","expense@adisoft-inc.com");
                } else
                {
    //  the variable, to, should contain data such as "adisoft.ESSManager" or "adisoft.ESSApprover"
                    if (to.equalsIgnoreCase("ess.ESSApprover"))
                    {
                        if (reporter.setPersNumInfo(er.getHeaderData("persnum")) && ESS != null)
                        {
                            String depart = reporter.getDepartment();
                            AuditTrail.println("[000] ProcessESSReport.sendAlert depart: " + depart);
                            String guide = DepartDOM.getDOMTableValueWhere("depart",depart, "guide");
                            if (guide.length() == 0)
                                guide = "default";
                             AuditTrail.println("[000] ProcessESSReport.sendAlert guide: " + guide);
                            to = StatusDOM.getDOMTableValueWhere(guide,"translation",ESS.getReportStatus(),"notify");
                            if (to.length() == 0)
                                to =StatusDOM.getDOMTableValueWhere("default","translation",ESS.getReportStatus(),"notify");
                             AuditTrail.println("[000] ProcessESSReport.sendAlert to: " + to);
                        }
                        else
                        {
                            to = "";
                            AuditTrail.println("[500] ProcessESSReport.sendAlert 1: setPersNumInfo error.");
                        }
                    }
                    if (to.length() > 0)
                    {
                      try {
                          //Consider moving this out of here and then interrogate
                          //the report only.
                          RouteTo routeTo = (RouteTo) (Class.forName(to)).newInstance();
                          routeTo.init(dbConn, sqlTerminator);
                          sendTo = routeTo.getEmailAddress(er.getHeaderData("persnum"));
                        
                          //Updating the "Routed to: field. Need to make this optional at some point
                          //Possibly introduce it's own sql update and store in a separate table
                          String reportNumber;
                          if (xreference.equals("")) {
                           reportNumber = er.getHeaderData("reference");   //jh 11-21-2002
                          } else {
                           reportNumber = xreference; //is this still valued jh 030303 ??
                          }
                          String persNum = er.getHeaderData("persnum");

                          this.updateReportInfo("MANAGER",routeTo.getRouteToPersNum(), persNum, reportNumber);
                          ESS.setReportManager(routeTo.getRouteToPersNum());
                          String route2Name = ESS.getRouteToName();
                          if (notifyFlag && !route2Name.equals("")) {
                           message += Message_new_line + "  5. " + Message_RouteTo + ": " + route2Name;
                           notifyFlag = false;
                          }
                      } catch (Exception e) {
                          sendTo = "";
                          e.printStackTrace();
                          AuditTrail.println("[500] ProcessESSReport.sendAlert 2: " + e.getMessage());
                      }
                    }
                }
            }
            // notifyMessage = rules.getMessage(notifications.elementAt(i));
            notifyMessage = myLang.getString(rules.getPropertyMessage(notifications.elementAt(i)));
            AuditTrail.println("[000] ProcessESSReport.notifyMessage: " + notifyMessage);
            emailMessage += Message_new_line + Message_new_line + "  " + notifyMessage + Message_new_line + "  " + receiptMsg;
            i++;
        }
        
        if ((emailMessage.length() > 0 ) && (sendTo.length() > 0 ))
        {
            AuditTrail.println("[000] ProcessESSReport.sendAlert to: " + sendTo + " -from: " + from);
            AuditTrail.println("[000] ProcessESSReport.sendAlert   : " + emailMessage);
            msgSentOK = msgSentOK && smtp.setSMTPMessage (emailMessage, sendTo, from); 
        }    
        
        return (true);
    }

    public String getRoutingRuleName(String ReportDepart, String ReporterDepart) {
      String retVal = "default";

      if (ReportDepart.equals(""))
      {
        ReportDepart = ReporterDepart;
      }

      retVal = DepartDOM.getDOMTableValueWhere("depart",ReportDepart,"guide");
      if (retVal.equals("")) 
      {
        retVal = "default";
        AuditTrail.println("[000] ProcessESSReport.getRoutingRuleName default-used");
      } else {
        AuditTrail.println("[000] ProcessESSReport.getRoutingRuleName " + retVal);
      }
    
      return retVal;
    }
    
}
