/*
RegisterContainer.java - Object representing the user's register file (XMLR)  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.net.*;
import java.io.*;
import java.util.*;
//import java.sql.*;
import org.jdom.Document;
import org.jdom.Content;
import org.jdom.Element;
//import org.jdom.DocType;
//import org.jdom.Attribute;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;


public class RegisterContainer extends AdisoftDOM
{
  static AuditTrail AuditLog;  
  private Element root;
  private String fileName;
  private String key;
  private int itrIndex = -1;
  private int searchIndex = -1;
  ArrayList registers = new ArrayList();

/**
Open xml file and enumerate the register container.
*/

  public RegisterContainer()
  {
  }

  public String getFileName()
  {
	  return fileName;
  }
  
  public void setContainer(String xmlFileName)
  {
    File InputFile = new File(xmlFileName);
    setDOM(InputFile);
    fileName = xmlFileName;
    // key = keyId;
    root = CommonDOM.getRootElement();
    setRegister();
    
  }

  public RegisterContainer(String xmlFileName, String keyId)
  {
    super(xmlFileName);
    fileName = xmlFileName;
    key = keyId;
    root = CommonDOM.getRootElement();
    setRegister();
  }

  public void setKey(String keyId)
  {
    key = keyId;
  }
  
  public String getKey()
  {
    return key;
  }
  
/**
Returns size of the container
*/
  public int size()
  {
    return registers.size();
  }

  public ArrayList getRegisters()
  {
    return(registers);
  }
/**
Reset iterator
*/
public void resetIterator()
{
    registers.clear();
    setRegister();
}

/**
Returns true if there's a next register
*/
public boolean hasNext()
{
    return (itrIndex != -1) && (itrIndex < registers.size());
}

/**
Gets next register in the container.
*/
  public Element next()
  {
    if (itrIndex != -1)
        return(((Register) registers.get(itrIndex++)).getDomElement());
    else
        return null;
    
  }
/** 
Returns next element in the form of AdisoftDOM 
*/
  public AdisoftDOM getNextReport()
  {
    AdisoftDOM dom = null;
    Element el = (Element) next().clone();
    if (el != null)
    {
        dom = new AdisoftDOM();
        dom.setDOM(new Document(el));
    }
    return dom;
  }
  
/**
Gets the element that matches the registerId from the container
*/
  public Element getElement(String keyValue)
  {
    Register reg = findRegister(keyValue);
    
    return (reg != null)? reg.getDomElement() : null;
  }
  
/** 
Returns the element that matches the registerId in the form of AdisoftDOM 
*/
  public AdisoftDOM getReport(String keyValue)
  {
    AdisoftDOM dom = null;
    Register reg = findRegister(keyValue);
    if (reg != null)
    {
        Element el = (Element) reg.getDomElement().clone();
        dom = new AdisoftDOM();
        dom.setDOM(new Document(el));
    }
    return dom;
  }
  
  public String getReportString(String keyValue)
  {
    String retVal = "";
    AdisoftDOM dom = null;
    Register reg = findRegister(keyValue);
    if (reg != null)
    {
        Element el = (Element) reg.getDomElement().clone();
        dom = new AdisoftDOM();
        dom.setDOM(new Document(el));
        retVal = dom.toString(false);
    }
    return retVal;
  }
  
/**

Adds register to the end of the container
*/
  public boolean update(Element inEl)
  {
    boolean updateOK = false;
    Register reg = null;
    
    if (root != null)
    {
        EscapeXML E = new EscapeXML();
        super.setEscapeStrategy( (org.jdom.output.EscapeStrategy) E); 
        if ((reg = findRegister(inEl.getChildText(key))) != null) remove(reg.getDomElement());
        root.addContent(inEl);   //jh
        registers.add(new Register(inEl.getChildText(key), inEl));
        updateOK = true;
        itrIndex = -1;
    }
    
    return updateOK;
  }
  public boolean updateNoEscape(Element inEl)
  {
    boolean updateOK = false;
    Register reg = null;
    
    if (root != null)
    {
        //EscapeXML E = new EscapeXML();
        //super.setEscapeStrategy( (org.jdom.output.EscapeStrategy) E); 
        if ((reg = findRegister(inEl.getChildText(key))) != null) remove(reg.getDomElement());
        root.addContent(inEl);
        registers.add(new Register(inEl.getChildText(key), inEl));
        updateOK = true;
        itrIndex = -1;
    }
    
    return updateOK;
  }
  
/**
Adds register to the end of the container
*/
  public boolean updateReport(Document doc)
  {
    boolean updateOK = false;
    Register reg = null;
    Element inEl;
    
    if (root != null)
    {
        inEl = doc.getRootElement();
        //inEl = doc.detachRootElement();  //JH 2/17/2004
        inEl = (Element) inEl.detach();
        
        if ((reg = findRegister(inEl.getChildText(key))) != null)
            remove(reg.getDomElement());
        
        root.addContent(inEl);
        registers.add(new Register(inEl.getChildText(key), inEl));
        updateOK = true;
        itrIndex = -1;
    }
    
    return updateOK;
  }
  
/**
Removes the register from the container.
*/
  public boolean remove(String keyValue)
  {
    return remove(getElement(keyValue));
  }
  
/** 
Write registers to xml file
*/
  public void persist()
  {
    OutputTextFile outFile = new OutputTextFile();
    outFile.setFile(fileName, toString(false));
  }
  
/*********   private methos  **********/

  private void setRegister()
  {
    Iterator itr = root.getChildren().iterator();
//add register to the arraylist    
    while (itr.hasNext())
    {
        Element regEl = (Element) itr.next();
        registers.add(new Register(regEl.getChildText(key), regEl));
    }
    
    itrIndex = 0;
  }

  private Register findRegister(String keyValue)
  {
    Register reg = null;
    boolean regFound = false;
    
    searchIndex = 0;
    while ((searchIndex < registers.size()) && ( regFound == false))
    {
        reg = (Register) registers.get(searchIndex);
        if (reg != null) 
        {
        regFound = reg.isRegister(keyValue);
        }
        searchIndex++;
    } 
    if (regFound == true)
    {    	
    	AuditTrail.println("[000] RegisterContainer.findRegister found: " + keyValue); 
        searchIndex--;
    } else
    {
    	AuditTrail.println("[000] RegisterContainer.findRegister was not found: " + keyValue + ", total registers in file: " + Integer.toString(registers.size())); 
        reg = null;
        searchIndex = -1;
    }
    itrIndex = -1;
    
    return reg;
  }
/** 
First detaches the element from the associated document and then remove it from the ArrayList, registers.
*/
  private boolean remove(Element el)
  {
    if (el != null)
    {
        el.detach();
        registers.remove(searchIndex); 
        itrIndex = -1;
        return true;
    }
    return false;
  }
  
  private class Register
  {
    String keyValue;
    Element domElement;
    
    public Register(String sKeyValue, Element el)
    {
        keyValue = sKeyValue;
        domElement = el;
    }
    
    public boolean isRegister(String key)
    {
        return keyValue.equals(key);
    }
    
    public Element getDomElement()
    {
        return domElement;
    }
    
    public String toString()
    {
        return keyValue;
    }
  }
  
}