/*
RegisterPurge.java - reads messages from a pop3 account to a folder   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

import java.sql.*;
import org.jdom.output.EscapeStrategy;


public class RegisterPurge implements Runnable {
	
   static AuditTrail Logfile = new AuditTrail();
   private ServerSystemTable systemTable;
   private String ownerSQL;
   private String entrySQL;
   private String deleteSQL;
   private String errorSQL;
   private EscapeStrategy EscStr = new EscapeXML();
    //private String xmlr_folder;
   
   private String SQLTerminator = ";";

   public RegisterPurge(String DBUrl, String DBUser, String DBPassword, String DBSQLTerminator)
   {
        systemTable = new ServerSystemTable(DBUrl, DBUser, DBPassword);
        systemTable.setSQLTerminator(DBSQLTerminator);
        setSQLTerminator(DBSQLTerminator);
        setSQLStatements();
   }
	   
   
   public RegisterPurge(Connection DBConn)
   {
		// Get a separate connect from the user so that if the user logs off we are not in trouble.
        systemTable = new ServerSystemTable();
        systemTable.setConnection(DBConn);
        // need to set setSQLStatements(); manually because of the terminator
   }
   
   public void setSQLStatements()
   {
       ownerSQL = (systemTable.getSystemString("registertable", "purgeOwners", "SELECT DISTINCT(OWNER) FROM REGISTER WHERE STATUS = 'Remove'" + SQLTerminator));
       entrySQL = (systemTable.getSystemString("registertable", "purgeEntry", "SELECT REFERENCE FROM REGISTER WHERE OWNER = '$owner$' AND STATUS = 'Remove'" + SQLTerminator));
       deleteSQL = (systemTable.getSystemString("registertable", "purgeDelete", "UPDATE REGISTER SET STATUS = 'Purged' WHERE REFERENCE = '$reference$'" + SQLTerminator));
       errorSQL = (systemTable.getSystemString("registertable", "purgeError", "UPDATE REGISTER SET STATUS = 'PurgeError' WHERE REFERENCE = '$reference$'" + SQLTerminator));
       // xmlr_folder = (systemTable.getSystemString("xml_folder", "/var/ess/xmlr/"));
   }
   
   public void setSQLTerminator(String x)
   {
	   SQLTerminator = x;
	   systemTable.setSQLTerminator(x);
   }
   
   public void run() 
   {
	   String owner = "";
	   String columnLabel;
	   PersonnelTable Reporter = new PersonnelTable();
	   Reporter.setConnection(systemTable.getConnection());
	   Reporter.setSQLTerminator(SQLTerminator);
	   Reporter.setSQLStrings();
	   
	   try {
		   AdisoftDbase Reg = new AdisoftDbase();
		   Reg.setSQLTerminator(SQLTerminator);
		   Reg.setConnection(systemTable.getConnection());
		   Reg.setColumns("REGISTER");
		   if (Reg.setResultSet(ownerSQL))
		   {
			   do {
				   columnLabel = Reg.getColumnName("OWNER");
				   owner = Reg.myResult.getString(columnLabel);
				   AuditTrail.println("[380] RegisterPurge.run for: " + owner);
				    
				   Reporter.resetProfile();
				   if (Reporter.setPersInfo(owner))
				   {
				       processEntries(Reporter.getEmailAddress());
				   } else {
					   AuditTrail.println("[500] RegisterPurge.run cannot be run for: " + owner + " (Please investigate!)");
				   }
			   }while(Reg.myResult.next());
		   } else 
		   {
			   AuditTrail.println("[400] RegisterPurge.run no items found.");
		   }
		   
	   } catch (java.sql.SQLException e)
	   {
		   AuditTrail.println("[500] RegisterPurge.processEntries error - element not removed: " + owner);
	   }
	   AuditTrail.println("[381] RegisterPurge.run completed");
	   setProcessNotRunning();
	}
   
   
   private void processEntries (String owner)
   {
	   try 
	   {
		   AdisoftDbase Reg = new AdisoftDbase();
		   Reg.setSQLTerminator(SQLTerminator);
		   Reg.setConnection(systemTable.getConnection());
		   Reg.setColumns("REGISTER");

		   AdisoftDbase Update = new AdisoftDbase();
		   Update.setSQLTerminator(SQLTerminator);
		   Update.setConnection(systemTable.getConnection());
		   
		   ReportContainer RegisterDOM = new ReportContainer();
		   RegisterDOM.setSQLTerminator(SQLTerminator); 
		   RegisterDOM.setConnection(systemTable.getConnection());
		   RegisterDOM.setKey("reference");
		   RegisterDOM.setEscapeStrategy(EscStr);
           RegisterDOM.setOwner(owner);

           AuditTrail.println("[000] RegisterPurge.processEntries xmlr file: " + RegisterDOM.getFileName());           
		   
		   String item2Remove;
		   String columnLabel;
		   String resultSQL;
		   if (Reg.setResultSet(entrySQL.replace("$owner$", owner)))
		   {
			   do {
				   columnLabel = Reg.getColumnName("REFERENCE");
				   item2Remove = Reg.myResult.getString(columnLabel);
				   if (RegisterDOM.remove(item2Remove))
				   {
					   AuditTrail.println("[380] RegisterPurge.processEntries element removed for " + owner + ", reference: " + item2Remove);
					   resultSQL = deleteSQL.replace("$reference$", item2Remove);
					   Update.doSQLExecute(resultSQL);
				   } else {
					   AuditTrail.println("[500] RegisterPurge.processEntries error - element not removed for " + owner + ", reference: " + item2Remove);
					   resultSQL = errorSQL.replace("$reference$", item2Remove);
					   Update.doSQLExecute(resultSQL);
				   }
			   }while(Reg.myResult.next());
			   RegisterDOM.persist();
		   } else
		   {
			   AuditTrail.println("[500] RegisterPurge.processEntries error - SQL conflict: " + owner); 
		   }
	   } catch (java.sql.SQLException e)
	   {
		   AuditTrail.println("[500] RegisterPurge.processEntries error: " + e.toString());
		   e.printStackTrace();
	   }

   }
   static Boolean PurgeRunning = new Boolean(false);

   public boolean getAlreadyRunning()
   {
     synchronized(PurgeRunning) {
         boolean retVal = true;
         if (!PurgeRunning.booleanValue())
         {
        	 PurgeRunning = Boolean.valueOf("true");
             retVal = false;
         }
         return retVal;
     }
   }
   
   public void setProcessNotRunning()
   {
	   PurgeRunning = Boolean.valueOf("false");
   }


}
