/*
ESSReport.java - XML to Central Database and Report class, Generic Report Object 
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.io.*;
import java.text.DecimalFormat;
import java.util.*; 
//import java.text.*;
import java.sql.*;
import java.lang.Math;
import org.jdom.Element;
//import org.jdom.Attribute;
//import org.jdom.DocType;

public class Report {
  static Integer IConvert;  //used to manipulate strings, ints, etc.
  static Float FConvert;
  static Math MConvert;  
  static Long LConvert;
  static Double DConvert;
  protected CustomDate date = new CustomDate();
  protected java.util.Date workdate = new java.util.Date();
  protected DecimalFormat moneyFile = new DecimalFormat("0.00");
  protected Locale teanga = new Locale("en");  //jh 2010-01-24
  protected java.text.NumberFormat money = java.text.NumberFormat.getNumberInstance(teanga);
  
  protected ESSReport Rep = new ESSReport();
  protected Vector ESSPurposes = new Vector();
  protected Vector SplitClients = new Vector();
  protected SortVector ESSReceipts = new SortVector(new CompareXMLReceiptsByDate());
  protected ESSReceipt[] ESSRcptsByExpense;  //Used by Audit Module  4/2007
  protected SortVector ESSExpenses = new SortVector(new CompareXMLExpensesByDate());
  protected Vector ESSAttendeeLists = new Vector();
  protected Vector ESSExtendeds = new Vector();
  protected Vector ESSGLInfo = new Vector();
  protected Vector ESSGridInfo = new Vector();
  protected Vector ReceiptScanLinks = new Vector();
  protected Vector ReceiptScanComments = new Vector();
  protected int scanLinks = -1;
  private String DQ = "::";
  private String DE = "::\r\n";
  final String delimit = ",";
  protected int genReference = 1;
  protected String ReportStatus = "B1";
  protected PersonnelTable PT = new PersonnelTable();  //Borrowing for system table access
  protected String MessageReference = "";
  protected String DBUrl = "";
  protected String DBUser = "";
  protected String DBPassword = "";
  protected Connection dbCon = null;  
  protected AdisoftDbase db;
  protected ServerSystemTable ST = new ServerSystemTable(); //can we make these static?? 
  protected StateTable stateTable;
  protected String Pers_ExpenseType;
  protected AdisoftDOM ExpenseTable = new AdisoftDOM(); //can we make these static??
  protected AdisoftDOM PaymentTable = new AdisoftDOM(); // ""
  protected String ReceiptRequired;
  protected String Project_Report_Name = "Project";
  protected String Manager_Report_Name = "Manager";
  protected String Billable_Name = "Billable";
  protected String Show_Company_Name = "No";
  protected String AddMerchant2Comment = "Yes";
  protected String SplitBillOverride = "Yes";
  protected String defaultCompany = "";
  protected String Update_Exp_Approval = "No";
  protected String StepNoDisplay = "";                       //JH 3/17/2004
  protected AdisoftDOM companyTable = new AdisoftDOM(); //can we make these static??
  protected AdisoftDOM StatusDOM = new AdisoftDOM();    //can we make these static??
  protected String bkgrndColor = "#FFFFA4";
  protected String reportReferenceName = "voucher";  //was "Reference #"
  protected boolean denormalizeClient = false;  
  protected String stringQuote = "\"";
  protected boolean quoteDate = true;
  protected String exportNull = "null";
  protected String HomeCurrency = "US Dollar";     //Company Currency
  protected String CentralCurrency = "US Dollar";   //Currency of the processing area
  
// error types  
  protected ESSRules rules;
  protected Vector status = new Vector();
  protected boolean persistanceStatus = false;
  
  protected String encrypt = "No";
  protected int encryptKey = 253;
  protected String encryptBase = "Hex";
  protected String encryptHead = "0x";
  protected String encryptTail = "";
  protected String encryptDelimiter = "";
  
  protected String voucherObject = "ServerSystemTable";  //was "ExpSysTable"
  protected String voucherPrefix = "A";
  protected String voucherLength = "8";
  
  protected String lodgingExpense = "LODGING";
  
  protected String sqlLockTable = "";
  protected String sqlUnlockTable = "";
  protected String sqlTerminator = ";";
  protected String sqlInsertMethod = "SimpleSQL";
  protected String sqlYes = "1";
   
  protected String scanSQL = "SELECT SCAN_REF,FILE_TYPE,COMMENT FROM SCAN WHERE PVOUCHER = '$pvoucher$' AND PERS_NUM = '$persnum$'";
  
  protected String[] personalItems = {"PERSONAL"};
  protected String[] advanceItems = {"ADVANCE"};
  protected String[] returnedItems = {"CASH-RETD"};
  protected String[] mileageItems = {"MILEAGE"};
  
  protected String paymentAccountName = "gen_acct";
  protected String expenseAccountName = "gen_acct";
  
  protected DepartmentTable departTable;
   
  static AuditTrail Logfile;
  
  protected String Miles = "MILES";
  protected String reportInstance = "";  //Used by Audit Report as 8/29/2007
    
  protected String activeCompany;
  protected String expenseSelect; 
  protected String paymentSelect;
  
  protected Currency CurrencyObj = new Currency();
  
  protected String dateFormatStr = "MM/dd/yyyy";
  protected String decimalStr = ".";
  protected String separatorStr = ",";
  
  
  public Report () {
  }
  public Report (String Url, String User, String Password, String SQLTerminator){
    setDB(Url, User, Password, SQLTerminator);
  }
  
  public void reset() {
    
  //ESSReport Rep
     ESSPurposes = new Vector();
     SplitClients = new Vector();
     ESSReceipts = new SortVector(new CompareXMLReceiptsByDate());
     ESSExpenses = new SortVector(new CompareXMLExpensesByDate());
     ESSAttendeeLists = new Vector();
     ESSExtendeds = new Vector();
     ESSGLInfo = new Vector();
     ESSGridInfo = new Vector();
     ReceiptScanLinks = new Vector();
     ReceiptScanComments = new Vector();
     scanLinks = -1;
  }
  
  public void setDB(String Url, String User, String Password, String SQLTerminator) {
    DBUrl = Url;
    DBUser = User;
    DBPassword = Password;
    sqlTerminator = SQLTerminator;
    PT.setDB(Url, User, Password);
    PT.setSQLTerminator(sqlTerminator);    
    ST.setDB(Url, User, Password);
    ST.setSQLTerminator(sqlTerminator);    
    ExpenseTable.setDB(Url, User, Password);
    ExpenseTable.setSQLTerminator(sqlTerminator);    
    PaymentTable.setDB(Url, User, Password);
    PaymentTable.setSQLTerminator(sqlTerminator);    
    companyTable.setDB(Url, User, Password);
    companyTable.setSQLTerminator(sqlTerminator);    
    db = new AdisoftDbase(Url, User, Password);
    db.setSQLTerminator(sqlTerminator);   
    db.setCheckKey(false);  //jh 2005-06-03
    rules = new ESSRules(Url, User, Password, sqlTerminator);
    setDefaults();
    PT.setSQLStrings();
    stateTable = new StateTable(Url, User, Password, sqlTerminator);
  }
    
  public void setConnection(Connection dbConn) {
    dbCon = dbConn;
  }  
  
  public void setDateFormat(String x) {
	  if (x != null) dateFormatStr = x;
  }
  public String getDateFormat() {
	  return dateFormatStr;
  }
  public void setDecimal(String x) {
	  decimalStr = x;
  }
  public void setSeparator(String x) {
	  separatorStr = x;
  }
    
  public void setConnection(Connection dbConn, String dbSQLTerminator) {
    dbCon = dbConn;
    sqlTerminator = dbSQLTerminator;
    setUpSystemFiles();
  }  
  
  public String getReportInstance() {
    return reportInstance;
  }
  
  public void setReportInstance() {
    reportInstance = CustomDate.getInstanceDate();  
  }
  
  public void setUpFiles() {
    setUpSystemFiles();
  }
  
  public void setUpSystemFiles() {
    PT.setConnection(dbCon);
    PT.setSQLTerminator(sqlTerminator);
    ST.setConnection(dbCon);
    ST.setSQLTerminator(sqlTerminator);    
    ExpenseTable.setConnection(dbCon);
    ExpenseTable.setSQLTerminator(sqlTerminator);    
    PaymentTable.setConnection(dbCon);
    PaymentTable.setSQLTerminator(sqlTerminator);    
    companyTable.setConnection(dbCon);
    companyTable.setSQLTerminator(sqlTerminator);    
    db = new AdisoftDbase();
    db.setConnection(dbCon);
    db.setSQLTerminator(sqlTerminator);
    db.setCheckKey(false);  //jh 2005-06-03
    rules = new ESSRules(dbCon,sqlTerminator);
    setDefaults();
    PT.setSQLStrings();
    stateTable = new StateTable(dbCon, sqlTerminator);
    //String schemaFileName = ST.getSystemString("XMLSCHEMA","C:\\WORK\\schema.xml");
    //db.setSchemaTable(schemaFileName);
    if (!CurrencyObj.isAlreadyLoaded()) {
        CurrencyObj.setConnection(dbCon);
        CurrencyObj.setSQLTerminator(sqlTerminator); 
        CurrencyObj.setSQLStrings();
     }
    

  }  
  
  
  
  public void setStringQuote(String X) {
    stringQuote = X;
  }
  public String getStringQuote() {
    return stringQuote;
  }
  public void setQuoteDate(boolean X) {
    quoteDate = X;
  }
  public boolean getQuoteDate() {
    return quoteDate;
  }
  
  public void setExportNull(String X) {
    exportNull = X;
  }
  public String getExportNull() {
    return exportNull;
  }
  
  
  public void setSQLTerminator(String X) {
    sqlTerminator = X;
  }
    
  public void setSQLInsertMethod(String X) {
    sqlInsertMethod = X;
  }    

  public void setDenormalizeClient(boolean X) {
    denormalizeClient = X;
  }
    
  public void close() {
    PT.close();
    ST.close();
    db.close();
    rules.close();
  }
    
  protected void setDefaults() {
    Pers_ExpenseType = ST.getSystemString("PERSONAL_CODE", "PERSONAL");
    ExpenseTable.setDOMFromTable("ACCOUNT");
    PaymentTable.setDOMFromTable("CHARGE");
    companyTable.setDOMFromTable("COMPANY");
    ReceiptRequired = ST.getSystemString("RECEIPT_REQ_AMT_FLD", "LIMIT");
    ReceiptRequired = ReceiptRequired.toLowerCase();
    AuditTrail.println("[000] Report.setDefaults ReceiptRequired name" + ReceiptRequired );
  
    Project_Report_Name = ST.getSystemString("PROJECT_REPORT_NAME", "Project");
    Manager_Report_Name = ST.getSystemString("MANAGER_REPORT_NAME", "Manager");
    Billable_Name = ST.getSystemString("BILLABLE_NAME", "Billable");
    Show_Company_Name = ST.getSystemString("SHOW_COMPANY_NAME", "No");
    AddMerchant2Comment = ST.getSystemString("ADD_MERCHANT_COMMENT", "Yes");
    SplitBillOverride = ST.getSystemString("Split_Bill_Override", "Yes");
    HomeCurrency = ST.getSystemString("currency", "US Dollar");
    CentralCurrency = HomeCurrency;
    Update_Exp_Approval = ST.getSystemString("UPDATE_EXP_APPROVAL", "No");
    StepNoDisplay = ST.getSystemString("STEP_DISPLAY", "");                       //JH 3-17-2004
    bkgrndColor = ST.getSystemString("BACKGROUND_COLOR", "#FFFFA4");

    Miles = ST.getSystemString("DISTANCE", "Miles");
    if (!Miles.equals("") && !Miles.substring(Miles.length() - 1).equalsIgnoreCase("s")) Miles = Miles + "s";  //English!
    Miles = Miles.toUpperCase();

    AdisoftDOM SD = new AdisoftDOM();
    String system_file = ST.getSystemString("XMLSYSTEM","C:\\WORK\\"); 
    java.io.File SystemFile = new java.io.File(system_file);
    SD.setDOM(SystemFile); 
    AuditTrail.println("[000] Report.setSQLStrings from " + system_file);
    db.setSQLDateType(SD.getDOMTableValueFor("sql","dateformat"));
    db.setSQLDateStr(SD.getDOMTableValueFor("sql","datesql"));
    db.setSQLNo(SD.getDOMTableValueFor("sql","nosql"));
    sqlYes = SD.getDOMTableValueFor("sql","yessql");
    db.setSQLYes(sqlYes);
    db.setAcceptNulls(SD.getDOMTableValueFor("sql","acceptnulls"));
    db.setSQLBlankDate(SD.getDOMTableValueFor("sql","blankdate"));
    db.setSQLReservedWordList(SD.getDOMTableValueFor("sql","reservedwordlist"));
    db.setTruncationOff();  //needed because of encryption - we handle truncation here.
    
    sqlLockTable = SD.getDOMTableValueFor("sql","locktablesql","");
    sqlUnlockTable = SD.getDOMTableValueFor("sql","unlocktablesql","");
    sqlInsertMethod = SD.getDOMTableValueFor("sql","insertmethod","SimpleSQL");
    
    scanSQL = SD.getDOMTableValueFor("receiptmanagement","receiptLinkSQL","SELECT SCAN_REF,FILE_TYPE,COMMENT FROM SCAN WHERE PVOUCHER = '$pvoucher$' AND PERS_NUM = '$persnum$'");
    
    encrypt = SD.getDOMTableValueFor("encrypt","apply","No");
    db.setEncryption(encrypt);
    if (encrypt.equalsIgnoreCase("YES")) {
      String encryptKeyStr = SD.getDOMTableValueFor("encrypt","key","15");
      encryptKey = Integer.parseInt(encryptKeyStr);
      encryptBase = SD.getDOMTableValueFor("encrypt","base","Hex");
      encryptHead = SD.getDOMTableValueFor("encrypt","head","0x");
      encryptTail = SD.getDOMTableValueFor("encrypt","tail","");
      encryptDelimiter = SD.getDOMTableValueFor("encrypt","delimit","");
    }
    String status_file = ST.getSystemString("XMLSTATUS","C:\\WORK\\");
    java.io.File StatusFile = new java.io.File(status_file);
    StatusDOM.setDOM(StatusFile); 
    
    voucherObject = SD.getDOMTableValueFor("vouchertable","voucherobject","ServerSystemTable");
    voucherPrefix = SD.getDOMTableValueFor("vouchertable","voucherprefix","");
    voucherLength = SD.getDOMTableValueFor("vouchertable","voucherlength","8");
      
    lodgingExpense = SD.getDOMTableValueFor("process","defaultlodging",SD.getDOMTableValueFor("process","lodging","LODGING"));

    advanceItems = SD.getDOMTableArrayFor("process","advance"); //payment method
    personalItems = SD.getDOMTableArrayFor("process","personal"); //expense type
    returnedItems = SD.getDOMTableArrayFor("process","returned"); //payment method
    mileageItems = SD.getDOMTableArrayFor("process","mileage"); //expense type
    
    defaultCompany = SD.getDOMTableValueFor("process","defaultcompany","ess"); //payment method
    String schemaFileName = ST.getSystemString("XMLSCHEMA","C:\\WORK\\schema.xml");
    db.setSchemaTable(schemaFileName);
    
  }  
  
  protected void setCompany(String xcompany)
  {
	  activeCompany = xcompany;
	  setCompanyInheritance();
  }

  private void setCompanyInheritance() 
  {
     if (activeCompany == null || activeCompany == "") activeCompany = PT.getCompany();

     String CompanySelect = "COMPANY = '" + activeCompany + "'"; //Later can do some tricky things with OR
     db.setResultSet("SELECT EXPENSE, PAYMENT, CURRENCY, UNITS FROM COMPANY WHERE " + CompanySelect);

     try {
       expenseSelect = setComplexSelect(db.myResult.getString(1),activeCompany,CompanySelect); 
       paymentSelect = setComplexSelect(db.myResult.getString(2),activeCompany,CompanySelect);
       HomeCurrency = db.myResult.getString(3);
       Miles = db.myResult.getString(4);
       if (!Miles.equals("") && !Miles.substring(Miles.length() - 1).equalsIgnoreCase("s")) Miles = Miles + "s";  //English!
       Miles = Miles.toUpperCase();
     } catch (Exception e) {
         e.printStackTrace();
         AuditTrail.println("[500] Report.setCompanyInheritance IO error: " + e.getMessage());
         expenseSelect = CompanySelect; 
         paymentSelect = CompanySelect;
     }
  }

  private String setComplexSelect(String targetCompany, String currentCompany, String currentSQL)
  {
    if ((targetCompany != null) && (!targetCompany.equals("")) && (!targetCompany.equals(currentCompany))) 
    {
  	 currentSQL = "(" + currentSQL + " || COMPANY = '" + targetCompany + "')"; 	
    }
    return currentSQL;
  }

  
  
  public void setBackgroundColor(String x) {
    bkgrndColor = x;
  }
   
  public void setReportReferenceName(String x) {
    reportReferenceName = x;
  }
   
  public void setReportStatus(String RPStatus) {
    ReportStatus = RPStatus;
    Rep.RP_STAT = RPStatus;
    Rep.persistance = "update";
  }  
  
  public void setReceiptsReceivedDate(String X) {
    Rep.RECEIVED = X;
  }  
  
  public void setReportManager(String x) {
    Rep.MANAGER = x;
  }  
  
  public String getReportStatus() {
    return (ReportStatus);
  }  
  
  public String getESSReportStatus() {
    return (Rep.RP_STAT);
  }  
  
  public String getESSReportReceiptsReceived() {
    return (Rep.RECEIVED);
  }
  
  public String getVoucherNumber()
  {
    return (Rep.VOUCHER);   
  }
  
  public String getPreviousNumber()
  {
    return (Rep.PVOUCHER);   
  }
  
  public String getReportName()
  {
    return (Rep.NAME);   
  }
  
  public String getReportDepartment()
  {
	  return (Rep.DEPART);
  }
  
  public boolean isHomeCurrency()
  {
	  return HomeCurrency.equalsIgnoreCase(getCurrency());
  }
  
  public String getHomeCurrency()
  {
	  return HomeCurrency;
  }
  
  public String getCurrency()
  {
    String retVal;
    if (Rep.CURRENCY.equals(""))
    {
        retVal = HomeCurrency;
    } else {
        retVal = Rep.CURRENCY;
    }
    return (retVal);
  }
  public String getHistory()
  {
    String retVal = Rep.HISTORY;
    return (retVal);
  }
  
  public void setHistory(String x)
  {
    Rep.HISTORY = x;
  }
  
  public String getReportCompany() {
	  return Rep.COMPANY;
  }
  
  /*
  public String getReportCompany(String depart) {

      if (departTable == null)
      {
         departTable = new DepartmentTable();
         departTable.setConnection(dbCon);
         departTable.setSQLTerminator(sqlTerminator);  //JH 2005-09-15
         departTable.setSQLStrings();
      }
      //if (!ChargedDepart.equalsIgnoreCase(departTable.depart)) 
      departTable.setDepartInfo(depart);
      return departTable.getCompany();
  }
  */
  
  public String getCompany()
  {
    String retVal;
    if (!PT.persnum.equals("") || PT.setPersNumInfo(Rep.PERS_NUM)) 
    {
        retVal = PT.getCompany();
    } else 
    {
        retVal = "";
    }
    return retVal;
  }
  
  public String getCreateDate() {
    return (Rep.CRE_DATE);
  }
  
  public String getCurrentDate() {
    return (Rep.CUR_DATE);
  }
  
  public String getUploadDate() {
    return (Rep.UP_DATE);
  }
  
  public String getPersNum()
  {
    return (Rep.PERS_NUM);   
  }
  public String getName()
  {
    return (PT.getName());   
  }
  
  public String getEmailAddress()
  {
    return (PT.getEmailAddress());   
  }
  
  public String getPrintableEmailAddress()
  {
    return (PT.getPrintableEmailAddress());   
  }
  
  public String getPaymentMethod()
  {
    return (PT.getPaymentMethod());   
  }
  
  
  //===========================================
  
  public void setReportTotalsForced()
  {
     Rep.RC_AMT = Utilities.replaceStr(getReceiptTotal(),",","");
     Rep.RE_AMT = Utilities.replaceStr(getReimburseTotal(),",","");
     AuditTrail.println("[000] Report.setReportTotalsForced RE_AMT: " + Rep.RE_AMT);     
     Rep.AD_AMT = Utilities.replaceStr(getAdvanceTotal(),",","");
     Rep.persistance = "update";
  }
  
  public void setReportTotals()
  {
    String x = getReceiptTotal();
    String y = getReimburseTotal();
    AuditTrail.println("[000] Report.setReportTotals RE_AMT (y): " + y);     
    String z = getAdvanceTotal();
        
    if (!Rep.RC_AMT.equals(x))
    {
        Rep.RC_AMT = x;
        if (Rep.persistance.equals("")) Rep.persistance = "update";
    }
    if (!Rep.RE_AMT.equals(y))
    {
        Rep.RE_AMT = y;
        if (Rep.persistance.equals("")) Rep.persistance = "update";
    }
    if (!Rep.AD_AMT.equals(z))
    {
        Rep.AD_AMT = z;
        if (Rep.persistance.equals("")) Rep.persistance = "update";
    }
  }
  
  
  public String getReimburse() {
      //String retVal = Utilities.replaceStr(getReimburseTotal(),separatorStr,"");
      //String retVal = Utilities.replaceStr(DRoundStr(getReimburseAmount()),separatorStr,"");  //JH 2010-8-19
      //retVal = Utilities.replaceStr(retVal,decimalStr,"."); 
      String x = FF(DRoundStr(getReimburseAmount()));                //Jim 2010-10-26
      AuditTrail.println("[000] Report.getReimburse 1: " + x);
      String retVal;
      try {
          retVal = moneyFile.format(moneyFile.parse(x));
      } catch (java.text.ParseException e) {
          retVal = x;
          AuditTrail.println("[500] Report.getReimburse error: " + retVal);
      }
      AuditTrail.println("[000] Report.getReimburse 2: " + retVal);
      return  retVal;
  }
  
  public String getReimburseTotal() {
    String x = FF(DRoundStr(getReimburseAmount()));                //Jim 2010-10-26     
    String retVal;
    try {
        retVal = money.format(money.parse(x));
    } catch (java.text.ParseException e) {
        retVal = x;
        AuditTrail.println("[500] Report.getReimburseTotal error: " + retVal);
    }
    AuditTrail.println("[000] Report.getReimburseTotal: " + retVal);
    return  retVal;
  }
    
  public String getAdvanceTotal() {
    String x = FF(DRoundStr(getAdvanceAmount()));                //Jim 2010-10-26
    String retVal;
    try {
        retVal = money.format(money.parse(x));
    } catch (java.text.ParseException e) {
        retVal = x;
        AuditTrail.println("[500] Report.getAdvanceTotal error: " + retVal);
    }
    return  retVal;
  }
  public String getReceiptTotal() {
    String x = FF(DRoundStr(getReceiptAmount()));                //Jim 2010-10-26
    String retVal;
    try {
        retVal = money.format(money.parse(x));
    } catch (java.text.ParseException e) {
        retVal = x;
        AuditTrail.println("[500] Report.getReceiptTotal error: " + retVal);
    }
    return  retVal;
  }
    
    
    
    
  // Consider moving to Utilities  
  protected double ifInArrayAmount(String lookFor, String[] lookIn, String stringAmount)
  {
    double returnAmount = (double) 0.00;
    boolean itemFound = false;
    for (int i = 0; i < lookIn.length && !itemFound; i++) {
       if (lookFor.equals(lookIn[i])) {
         itemFound = true;
         returnAmount = Double.parseDouble(stringAmount);
       }
    }
    
    return returnAmount;
  }
  
    // Consider moving to Utilities  
  protected boolean ifInArray(String lookFor, String[] lookIn)
  {
    boolean retVal = false;
    boolean itemFound = false;
    for (int i = 0; i < lookIn.length && !retVal; i++) {
       if (lookFor.equals(lookIn[i])) {
         retVal = true;
       }
    }
    
    return retVal;
  }
  
  public double getReimburseAmount() {  //Need to finish this
    ESSReceipt ESSR;
    ESSExpense ESSE;
    // Can this below be made final??
    String[] paymentArray = PaymentTable.getDOMTableArrayWhere("CHARGE","REIMB",sqlYes);
    double reimburseAmount = (double) 0.00;
    double temporaryAmount = (double) 0.00;
    double paymentAmount = (double) 0.00;
    
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
       
       reimburseAmount += ifInArrayAmount(ESSR.CHARGE, paymentArray, ESSR.AMOUNT);
       reimburseAmount -= ifInArrayAmount(ESSR.CHARGE, advanceItems, ESSR.AMOUNT);
       reimburseAmount += ifInArrayAmount(ESSR.CHARGE, returnedItems, ESSR.AMOUNT);
    } 
    
    for (int i = 0; i < ESSExpenses.size(); i++) {
       ESSE = (ESSExpense) ESSExpenses.elementAt(i);
       ESSR = getESSReceipt(ESSE.RECEIPT);
       
       temporaryAmount = (double) 0.00;   // This avoids double dipping
       paymentAmount = ifInArrayAmount(ESSR.CHARGE, paymentArray, ESSE.AMOUNT);
//Don't need these two items in the current system       
       //temporaryAmount += ifInArrayAmount(ESSR.CHARGE, advanceItems, ESSE.AMOUNT);
       //temporaryAmount += ifInArrayAmount(ESSR.CHARGE, returnedItems, ESSE.AMOUNT);
       if (temporaryAmount == (double) 0.00 && paymentAmount > (double) 0.00 )
       {
          reimburseAmount -= ifInArrayAmount(ESSE.EXPENSE, personalItems, ESSE.AMOUNT);
       }
    }
    return reimburseAmount;
  }  
  
  public double getAdvanceAmount() {  //Need to finish this
    ESSReceipt ESSR;
    double advanceAmount = (double) 0.00;
    
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
       advanceAmount += ifInArrayAmount(ESSR.CHARGE, advanceItems, ESSR.AMOUNT);
    } 
    
    return advanceAmount;
  }  
  
  public double getReceiptAmount() {  //Need to finish this
    ESSReceipt ESSR;
    double receiptAmount = (double) 0.00;
    
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
      
       if (!ifInArray(ESSR.CHARGE, advanceItems) && !ifInArray(ESSR.CHARGE, returnedItems))
       {
          receiptAmount += Double.parseDouble(ESSR.AMOUNT);
       }
    } 
    
    return receiptAmount;
  }  
  
  public void setReporterInfo(PersonnelTable X)
  {
    PT = X;
  }
  
  
  public String[] getACHPaymentInfo() 
  {
    /* current layout of the array = 7 strings
    From Personnel Table
    0    name
    1    bankABA
    2    accountNumber
    3    isSavingsAccount
    4    isPreNoted
    5    preNoteDate
    6    paymentmethod
    7    active
    From the Report
    8    Receipt Amount
    */
    String[] retVal = new String[0];    
    if (!PT.persnum.equals("") || PT.setPersNumInfo(Rep.PERS_NUM))
    {
       String[] personnelInfo = PT.getACHPaymentInfo();
       //String reimbamount = Rep.RE_AMT;
       //if ((reimbamount == null) || (reimbamount.equals(""))) reimbamount = "0.00";
       String reimbamount = getReimburse();
       String[] reportInfo = {reimbamount};
       int retValLength = personnelInfo.length + reportInfo.length;
       retVal = new String[retValLength];
       Arrays.fill(retVal, "");
       retVal = addArray2Array(retVal,personnelInfo,0);
       retVal = addArray2Array(retVal,reportInfo,personnelInfo.length);
    }
    return retVal;
  }
  
  public String[] getFleetInfo() 
  {
    /* current layout of the array = 6 strings
    0    Vehicle #
    1    Starting mileage
    2    Ending mileage
    3    Business miles
    4    In service date
    5    Effective weekending date
    */
    String[] retVal = new String[6];    
    retVal[0] = Rep.VEHICLE;
    retVal[1] = Rep.START;
    retVal[2] = Rep.FINISH;
    retVal[3] = Rep.BUSINESS;
    retVal[4] = Rep.SERVDATE;
    retVal[5] = Rep.WEEKEND;
    return retVal;
  }
  
  public String[] getReportHeader() {
    String[] retVal = Rep.getDeNormal();
    return retVal;
  }
  
  public String getReportEditable() {
     String retVal = "";
     if (Rep.RP_STAT != null && !Rep.RP_STAT.trim().equals("") ) 
     {
        retVal = StatusDOM.getDOMTableValueWhere("default","translation",Rep.RP_STAT,"usereditallowed");
     }
     if (retVal.equals("")) retVal = "Yes";
     return retVal;
  }
  
  public ArrayList getDeNormal () {
    ArrayList retVal = new ArrayList();
    
    //Comeup with something to automatically calculate this...
    String[] line;
    ESSReceipt ESSR;
    ESSExpense ESSE;
    ESSPurpose ESSP;
    boolean receiptOnly = true;
    
    for (int i = 0; i < ESSReceipts.size(); i++) {

         ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
         receiptOnly = true;
            
         for (int j = 0; j < ESSExpenses.size(); j++) {
            ESSE = (ESSExpense) ESSExpenses.elementAt(j);
            if (ESSE.RECEIPT.equals(ESSR.RECEIPT))
            {
                ESSP = getESSPurpBlankOK(ESSE.PURPOSE);
                //Later put in logic to automatically calculate the sizes coming back.
                line = new String[73];    //2011 was 59
                Arrays.fill(line, "null");
                line = addArray2Array(line,Rep.getDeNormal(),0);//jh 2008-2-20 added three - Header
                line = addArray2Array(line,ESSR.getDeNormal(),29);   // 2011 was 28 - receipt
                line = addArray2Array(line,ESSE.getDeNormal(),46);   // 2011 was 37 - expense
                line = addArray2Array(line,ESSP.getDeNormal(),62);   // 2011 was 48 - purpose
                retVal.add(line);
                receiptOnly = false;
            }
        }

        if (receiptOnly)
        {
            line = new String[73];   //2011 was 59
            Arrays.fill(line, exportNull);
            line = addArray2Array(line,Rep.getDeNormal(),0);
            line = addArray2Array(line,ESSR.getDeNormal(),29);
            retVal.add(line);
        }
    } 
    return retVal;
  }

  private String[] addArray2Array(String[] array1, String[] array2, int startIndex)
  {
    for (int i = 0; i < array2.length; i++)
    {
        array1[startIndex + i] = array2[i];
    }
    return array1;
  }
  
  public void setReference(String MsgReference){  //must be used prior to set command to be effective
    MessageReference = MsgReference;
  }
//Replace the existing voucher number with the number from the expsys table
//and increment the number by one.
  public void setNewVoucherNumberFromExpSys(){
    Rep.setNewVoucherNumberFromExpSys();
  }
  
  protected void normalizeWeight () {
    //This section needs to be rewritten as it will generate rounding errors.  See the next 
    //section on splitting clients.
    ESSPurpose ESSP;
    ESSExpense ESSE;
    ESSExpense newExpense;
    float wt = 0;
    float nt;
    float w;
    float amount = 0;
    float remainder = 0;
    int LineLink;
    
    int originalSize = ESSExpenses.size();
    //Need to adjust for the receipt xref to expense items
    AuditTrail.println("[000] Report.normalizeWeight");
    for (int i = 0; i < ESSExpenses.size(); i++) {
        ESSE = (ESSExpense) ESSExpenses.elementAt(i);
        ESSE.xrefNormalize();
    }
    
    //Determine the total weight
    for (int i = 0; i < ESSPurposes.size(); i++) {
      ESSP = (ESSPurpose) ESSPurposes.elementAt(i);
      //FConvert.valueOf(ESSP.weight).isNaN()
      if (ESSP.weight.equals("")) {
        w = 0;
      } else {
        w = Float.valueOf(ESSP.weight).floatValue();
      }
      if (w > 0) {
        AuditTrail.println("[000] Report.normalizeWeight totaling");
                
        wt = wt + w;
      }
    }
    //go through and apply the w/wt to an expense
    if (wt > 0) {
       for (int i = 0; i < ESSPurposes.size(); i++) {
        ESSP = (ESSPurpose) ESSPurposes.elementAt(i);
        if (ESSP.weight.equals("")) {
          w = 0;
        } else {
          //Logfile.println("[000] Report.normalizeWeight establishing w");
          w = Float.valueOf(ESSP.weight).floatValue();
        }
        if (w > 0) {
          originalSize = ESSExpenses.size();
          for (int j = 0; j < originalSize; j++) {
            //Logfile.println("[000] Report.normalizeWeight expense item");
            ESSE = (ESSExpense) ESSExpenses.elementAt(j);
            if (ESSE.PURPOSE.equals("Weight")) {
              //Logfile.println("[000] Report.normalizeWeight found Weight");
              if (w < wt) {
                //Logfile.println("[000] Report.normalizeWeight car");
                try {
                  newExpense = (ESSExpense) ESSE.duplicate();
                  newExpense.PURPOSE = ESSP.PURPOSE;
                  LineLink = genReference++;
                  newExpense.LINEITEM = getESSReferenceString(LineLink);
                  remainder = Float.valueOf(ESSE.AMOUNT).floatValue();
                  amount = (w/wt) * remainder;
                  remainder = remainder - amount;
                  newExpense.AMOUNT = FF(FRoundStr(amount));
                  ESSExpenses.addElement(newExpense);
                  ESSE.AMOUNT = FF(FRoundStr(remainder)); 
                } catch (CloneNotSupportedException e) {
                  ESSE.PURPOSE = ESSP.PURPOSE;
                  //Logfile.println("[500] Report.normalizeWeight cloning error");
                }
              } else {
                //Logfile.println("[000] Report.normalizeWeight cdr");
                ESSE.PURPOSE = ESSP.PURPOSE;
              }
            }
          }
          wt = wt - w; //adjusting the denominator
        }
      }  
    }
  }
  
  protected void normalizeSplitClients () {
    ESSPurpose ESSP;
    ESSPurpose NewPurpose;
    ESSExpense ESSE;
    ESSReceipt ESSR;
    SplitClient Split;
    ESSExpense newExpense;
    double wt = 0;
    double nt;
    double w;
    double amount = 0;
    String amountStr;
    double remainder = 0;
    String remainderStr;
    double origAmt = 0;
    String origAmtStr;
    int LineLink;
    
    //Logfile.println("[000] Report.normalizeSplitClients");
    int originalSize = ESSExpenses.size();
    
    //Need to adjust for the receipt xref to expense items - this should be done
    //in the normalizeWeight function always
    //for (int i = 0; i < ESSExpenses.size(); i++) {
    //    ESSE = (ESSExpense) ESSExpenses.elementAt(i);
    //    ESSE.xrefNormalize();
    //}
    
    //Total weight is always 100(%)
    

    //go through and apply the w/wt to an expense
    //This adjsuts the expense list
    for (int k = 0; k < ESSPurposes.size(); k++) {
        
       ESSP = (ESSPurpose) ESSPurposes.elementAt(k);
       //Logfile.println("[000] Report.normalizeSplitClients Purpose:" +ESSP.PURPOSE );
    
       if (ESSP.multiClient.toUpperCase().equals("YES")) {
         //Logfile.println("[000] Report.normalizeSplitClients Multiclient= Yes");
         wt = (double) 100; //always 100%
         for (int i = 0; i < SplitClients.size(); i++) {
           Split = (SplitClient) SplitClients.elementAt(i);
           Logfile.println("[000] Report.normalizeSplitClients Split: " + Split.PURPOSE);
           if (ESSP.PURPOSE.equals(Split.originalPurpose)) {
             Logfile.println("[000] Report.normalizeSplitClients Split/Purpose match");
             if (Split.weight.equalsIgnoreCase("")) Split.weight = "0";
             w = Double.valueOf(Split.weight).doubleValue();
             originalSize = ESSExpenses.size();  //can I move this up
             for (int j = 0; j < originalSize; j++) {
               ESSE = (ESSExpense) ESSExpenses.elementAt(j);
               if (ESSE.AMOUNT.equals("")) {
                   ESSR = getESSReceipt(ESSE.RECEIPT);
                   ESSE.AMOUNT = ESSR.AMOUNT;
                   if (!ESSR.REC_AMT.equals("") && ESSE.REC_AMT.equals("")) ESSE.REC_AMT = ESSR.REC_AMT;
               }
               if (ESSE.PURPOSE.equals(ESSP.PURPOSE)) {
                 if (wt - w > (double) 0) {
                   try {
                     newExpense = (ESSExpense) ESSE.duplicate();
                     newExpense.PURPOSE = Split.PURPOSE;
                     if (SplitBillOverride.equalsIgnoreCase("Yes")) newExpense.BILLTYPE = Split.billtype;
                     AuditTrail.println("[000] Report.normalizeSplitClient(1) newExpense Billtype: " + newExpense.BILLTYPE);                   
                     LineLink = genReference++;
                     newExpense.LINEITEM = getESSReferenceString(LineLink);
                     //calculating new amount
                     origAmt = Double.valueOf(ESSE.AMOUNT).doubleValue();
                     origAmtStr = FF(DRoundStr(origAmt));
                     AuditTrail.println("[000] Report.normalizeSplitClient(1): start amt = " + origAmtStr);
                     origAmt = Double.valueOf(origAmtStr).doubleValue();
                     amount = (w/wt) * origAmt;
                     amountStr = FF(DRoundStr(amount));
                     amount = Double.valueOf(amountStr).doubleValue();
                     remainder = origAmt - amount;
                     remainderStr = FF(DRoundStr(remainder));
                     newExpense.AMOUNT = amountStr;
                     ESSE.AMOUNT = remainderStr;
                     //calculating receipt amount (mainly for mileage)
                     if (!ESSE.REC_AMT.equals("")) {
                        origAmt = Double.valueOf(ESSE.REC_AMT).doubleValue();
                        origAmtStr = FF(DRoundStr(origAmt));
                        origAmt = Double.valueOf(origAmtStr).doubleValue();
                        amount = (w/wt) * origAmt;
                        amountStr = FF(DRoundStr(amount));
                        amount = Double.valueOf(amountStr).doubleValue();
                        remainder = origAmt - amount;
                        remainderStr = FF(DRoundStr(remainder));
                        newExpense.REC_AMT = amountStr;
                        ESSE.REC_AMT = remainderStr;
                     }
                     //updating the list of expenses
                     ESSExpenses.addElement(newExpense);
                     //updating the list of attendees also
                     setSplitAttendees(ESSE.LINEITEM, newExpense.LINEITEM);
                     
                   } catch (CloneNotSupportedException e) {
                     ESSE.PURPOSE = Split.PURPOSE;
                     if (SplitBillOverride.equalsIgnoreCase("Yes")) ESSE.BILLTYPE = Split.billtype;
                     Logfile.println("[000] Report.normalizeSplitClient(2): " + Split.PURPOSE + "|" + ESSE.AMOUNT + "|" + ESSE.BILLTYPE);
                   }
                 } else {
                   ESSE.PURPOSE = Split.PURPOSE;
                   if (SplitBillOverride.equalsIgnoreCase("Yes")) ESSE.BILLTYPE = Split.billtype;
                   Logfile.println("[000] Report.normalizeSplitClient(3): " + Split.PURPOSE + "|" + ESSE.AMOUNT + "|" + ESSE.BILLTYPE);
                 }
               }
             }
             wt = wt - w; //adjusting the denominator
           } 
         }
       }
     }
     try {
      int Purp_size = ESSPurposes.size();
      for (int k = 0; k < Purp_size; k++) {
        ESSP = (ESSPurpose) ESSPurposes.elementAt(k);
        if (ESSP.multiClient.toUpperCase().equals("YES")) {
          for (int i = 0; i < SplitClients.size(); i++) {
            Split = (SplitClient) SplitClients.elementAt(i);
            if (ESSP.PURPOSE.equals(Split.originalPurpose)) {
             //==Creating a new purpose for this once
              NewPurpose = (ESSPurpose) ESSP.duplicate();
              NewPurpose.PURPOSE = Split.PURPOSE;
              NewPurpose.VOUCHER = Split.VOUCHER;
              if (!Split.CLIENTNO.equals("")) NewPurpose.CLIENTNO = Split.CLIENTNO; //JH 2004-03-23
              if (!Split.CLIENT.equals("")) NewPurpose.CLIENT = Split.CLIENT; //JH 2004-03-23
              if (!Split.PROJECT.equals("")) NewPurpose.PROJECT = Split.PROJECT; //JH 2004-03-23
              if (!Split.STEPNO.equals("")) NewPurpose.STEPNO = Split.STEPNO; //JH 2004-03-23
              if (!Split.LOCATION.equals("")) NewPurpose.LOCATION = Split.LOCATION;
              NewPurpose.billtype = Split.billtype;
              NewPurpose.weight = Split.weight;
              NewPurpose.company = Split.company;
              ESSPurposes.addElement(NewPurpose);
            }
          }
          ESSPurposes.removeElement(ESSP);
          k = k - 1;
          Purp_size = Purp_size - 1;  
        }
      }
     } catch (CloneNotSupportedException e) {
       AuditTrail.println("[500] Report.normalizeSplitClient cloning failure");
     }
   }

  public void setSplitAttendees(String OrigExpenseNo, String NewExpenseNo) {
    //returns the report in ESS format
    ESSExtended x;
    ESSExtended newAttendee;
    int OrigLen = ESSExtendeds.size(); 
    for (int i = 0; i < OrigLen; i++) {
       x = (ESSExtended) ESSExtendeds.elementAt(i);
       if (OrigExpenseNo.equals(x.LINEITEM)) {
         try {
            newAttendee = (ESSExtended) x.duplicate();
            x.LINEITEM = NewExpenseNo;
            ESSExtendeds.addElement(newAttendee);
         } catch (CloneNotSupportedException e) {
            AuditTrail.println("[500] Report.setSplitAttendees cloning failure");
         }
       }
    }
  }

  protected void normalizeESSReport() {
    ESSReceipt ESSR;
    ESSExpense ESSE;
    ESSPurpose ESSP;
    ESSExtended ESSX;
    for (int i = 0; i < ESSPurposes.size(); i++) {
        ESSP = (ESSPurpose) ESSPurposes.elementAt(i);
        ESSP.normalize();
    } 
    for (int i = 0; i < ESSReceipts.size(); i++) {
        ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
        ESSR.normalize();
    } 
    for (int i = 0; i < ESSExpenses.size(); i++) {
        ESSE = (ESSExpense) ESSExpenses.elementAt(i);
        ESSE.normalize();
    }
    for (int i = 0; i < ESSExtendeds.size(); i++) {
        ESSX = (ESSExtended) ESSExtendeds.elementAt(i);
        ESSX.normalize();
    }
    
    // Optimized to stop looking once all purposes are changed.
    // change later to save old purpose number and not lookup again
    int j = ESSPurposes.size();
    for (int i = 0; i < ESSExpenses.size(); i++) 
    {
        ESSE = (ESSExpense) ESSExpenses.elementAt(i);
        if (!ESSE.COMPANY.equals("")) 
        {
           ESSPurpose P = getESSPurpose(ESSE.PURPOSE);
           if (P.company.equals("")) 
           {
              P.company = ESSE.COMPANY;
              j = j - 1;
           }
        }
        if (j == 0) i = ESSExpenses.size();
    }
    for (int i = 0; i < ESSReceipts.size(); i++) {
        ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
        ESSR.normalizeType();  //Goes against each receipt
    } 
    Rep.normalize();
    ESSReceipts.sort();
    ESSExpenses.sort();
  }  



  protected String getValue(Element Parent, String Name) {
    return getValue(Parent, Name, "");
  }  
  protected String getValue(Element Parent, String Name, String Default) {
    //gets a value from a single element
    Element E = Parent.getChild(Name);
    if (E != null) {
      return E.getText();  
    } else {    
      return Default;
    }
  }  
  
  protected String getValueAmount(Element Parent, String Name) {
	    return getValueAmount(Parent, Name, "");
  }
  
  protected String getValueAmount(Element Parent, String Name, String Default) {
	    //gets a value from a single element
	    Element E = Parent.getChild(Name);
	    if (E != null) {
	      String x = E.getText();  
	      String retVal = getValueAmount(x);
	      return retVal;
	    } else {    
	      return Default;
	    }
  }  
  
  protected String getValueAmount(String x)
  {
      String retVal = Utilities.replaceStr(x,separatorStr,"");
      if (!decimalStr.equals(".")) retVal = Utilities.replaceStr(retVal,decimalStr,"."); 
      AuditTrail.println("[000] Report.getValueAmount: " + x + ", with decimal: '" + decimalStr + "', for: " + retVal);
      return retVal;
  }

  protected String getUserAmount(String x) {
	  		String retVal;
	  		if (decimalStr.equals("."))
	  		{
	  			retVal = x;
	  		} else 
	  		{
	  			retVal= Utilities.replaceStr(x,".",decimalStr); 
	  		}
	  		AuditTrail.println("[000] Report.getUserAmount: " + x + ", with decimal: '" + decimalStr + "', for: " + retVal);
            return retVal;
  }  
  
  protected String getValueDate(Element Parent, String Name) {
	    return getValueDate(Parent, Name, "");
  }  
  protected String getValueDate(Element Parent, String Name, String Default) {
	    //gets a value from a single element
	  // note: Default needs to be in the desired format
	    Element E = Parent.getChild(Name);
	    if (E != null) {
	    	String x = E.getText();
            String retVal = date.getStandardDateString(x, dateFormatStr);
            AuditTrail.println("[000] Report.getValueDate: " + x + ", with format: " + dateFormatStr + ", for: " + retVal);
            return retVal;
	    } else {    
	        return Default;
	    }
  }  

  private String getValuePlus(String Value, int Increment){
    if (Value.equals("")) {
      return "";
    } else {
      int NewValue = Integer.parseInt(Value) + Increment;
      return Integer.toString(NewValue);
    }
  }
  
  private String str(String x, int xsize) {
  //limits the size of a field - used for SQL columns  
    if (x.length() > xsize) x = x.substring(0, xsize);
    return x;
  }
  
  private String getESSReferenceString(String Value, String Backup) {
    String retVal = getESSReferenceString(Value);
    if (retVal.equals("") && !Backup.equals("")) {
         retVal = getESSReferenceString(Backup);  
    }  
    return retVal;
  }
  private String getReferenceString(String Value, String Backup, int xLen, String A) {
    String retVal = "";
    if (Value == null || Value.equals("")) {
      try {
        retVal = getReferenceString(Integer.parseInt(Backup),8,A);
      } catch (java.lang.NumberFormatException e) {
        retVal = getReferenceString(Integer.parseInt(numerifyReference (Value)),8,A);
      }
    }else{ 
      try {
        retVal = getReferenceString(Integer.parseInt(Value),8,A);
      } catch (java.lang.NumberFormatException e) {
        retVal = getReferenceString(Integer.parseInt(numerifyReference (Value)),8,A);
      }
    }
    return retVal;
  }
  
  private String numerifyReference (String x) {
    String y = "";
    String z = "0123456789";
    for (int i = 0; i < x.length(); i++)
    {
        if (z.indexOf(x.substring(i, i + 1)) > -1)
        {
            y += x.substring(i, i + 1);
        }
    }
    if (y.equals("")) y = "0";
    return y;
  }
  
  
  protected String getESSReferenceString(String Value) {
    if (Value == null || Value.equals("")) {
      return "";   
    }else{    
      return getESSReferenceString(Integer.parseInt(Value));
    }  
  }
  protected String getESSReferenceString(int Value) { 
    return getReferenceString(Value, 8 );
  }
  private String getReferenceString(int Value, int xLen, String A ) {
    String S = Integer.toString(Value);
    String NewS = "00000000000" + S;
    xLen = xLen - A.length();
    return A + NewS.substring(NewS.length()-xLen);
  }
  private String getReferenceString(int Value, int xLen) {
    return getReferenceString(Value, xLen, "");
  }

  public String getClientHeader() {
    String retVal = Rep.getClientCell();
    //retVal += "]";
    return retVal;
  }

  public String getClientPurpose(String reference) {
    //returns a single purpose as a client array
    String retVal = "[";
    ESSPurpose x = (ESSPurpose) getESSPurpose(reference);
    retVal += x.getClientCell();
    retVal += "]";
    return retVal;
  }

  public String getClientPurposes() {
    //returns the report in ESS format
    ESSPurpose x;
    String DD = "";
    String retVal = "[";
    for (int i = 0; i < ESSPurposes.size(); i++) {
        x = (ESSPurpose) ESSPurposes.elementAt(i);
        retVal += DD + x.getClientCell();
        DD = delimit;
    }
    retVal += "]";
    return retVal;
  }

  public String getClientReceipt(String reference) {
    //returns a single receipt
    String retVal = "[";
    ESSReceipt x = (ESSReceipt) getESSReceipt(reference);
    //x.updateNormalize();  //jh 2006-10-17 
    retVal += x.getClientCell();
    retVal += "]";
    return retVal;
  }

  public String getClientReceipts() {
    //returns the report in ESS format
    ESSReceipt x;
    String DD = "";
    String retVal = "[";
    for (int i = 0; i < ESSReceipts.size(); i++) {
        x = (ESSReceipt) ESSReceipts.elementAt(i);
        //x.updateNormalize();  //jh 2006-10-17        
        retVal += DD + x.getClientCell();
        DD = delimit;
    }
    retVal += "]";
    return retVal;
  }

  public String getClientExpenses(String RcptNo) {
    //returns the report in ESS format
    ESSExpense x;
    String DD = "";
    String retVal = "[";
    for (int i = 0; i < ESSExpenses.size(); i++) {
       x = (ESSExpense) ESSExpenses.elementAt(i);
       if (RcptNo.equals(x.RECEIPT)) {
         retVal += DD + x.getClientCell();
         DD = delimit;
       }
    }
    retVal += "]";
    return retVal;
  }

  public String getDNClientExpenses(String RcptNo) {
    //returns the report in ESS format
    ESSExpense x;
    String DD = "";
    String retVal = "";
    String itemNumber;
    int iN = 1;
    for (int i = 0; i < ESSExpenses.size(); i++) {
       x = (ESSExpense) ESSExpenses.elementAt(i);
       if (RcptNo.equals(x.RECEIPT)) {
         itemNumber = java.lang.Integer.toString(iN++);
         retVal += DD + x.getDNClientCell("expense_" + itemNumber + "_");
         DD = delimit;
       }
    }
    return retVal;
  }

  public String getClientAttendees(String ExpenseNo, String Persplit) {
    //returns the report in ESS format
    ESSExtended x;
    String DD = "";
    String DDX = "";
    String StringX = "";
    String retVal = "";
    for (int i = 0; i < ESSExtendeds.size(); i++) {
       x = (ESSExtended) ESSExtendeds.elementAt(i);
       if (ExpenseNo.equals(x.LINEITEM)) {
         if (DD.equals("")) {
           retVal += "[";
           StringX = getCell("LOCATION",x.LOCATION);  //was 'location'
           retVal += StringX;
           if (!StringX.equals("")) DDX = delimit; 
           StringX = getCell("COMMENT",x.COMMENT,DDX);  //was 'comment'
           retVal += StringX;
           if (!StringX.equals("")) DDX = delimit;           
           retVal += getCell("NUMBER",Persplit,DDX); //was 'number'  
           retVal += delimit;
         }
         retVal += DD + x.getClientAttendeeCell();
         DD = delimit;
       }
    }
    if (!retVal.equals("")) retVal += "]";
    return retVal;
  }
  
  public String getReport() {
    //returns the report in ESS format 
    //Later want to make this return everything!!
    String retVal = Rep.getClientCell();
    for (int i = 0; i < ESSPurposes.size(); i++) {
        ESSPurpose X = (ESSPurpose) ESSPurposes.elementAt(i);
        retVal += "\n\n" + X.getClientCell();
    } 
    for (int i = 0; i < ESSReceipts.size(); i++) {
        ESSReceipt X = (ESSReceipt) ESSReceipts.elementAt(i);
        retVal += "\n\n" + X.getClientCell();
    } 
    return retVal + "\n"; 
  } 
  
  public String getXRate(String receipt)
  {
    String retVal = getESSReceipt(receipt).XRATE;
    return retVal;
  }

  public String getFgnOrMiles(String receipt)
  {
    String retVal = getESSReceipt(receipt).REC_AMT;
    return retVal;
  }

  public String getSource(String receipt)
  {
    String retVal = getESSReceipt(receipt).XSOURCE;
    return retVal;
  }

  public boolean setReceiptCheck(String receipt, String check)
  {
    boolean retVal = false;
    ESSReceipt x = getESSReceipt(receipt);
    if (x.RECEIPT.equals(receipt))
    {
        x.CHECK = check;
        retVal = true;
    } else {
        AuditTrail.println("[500] Report.setReceiptCheck failed: " + Rep.VOUCHER + ", " + receipt + ", " + check);
    }
    return retVal;
  }

  protected ESSReceipt getESSReceipt(String receipt) {
    ESSReceipt ReturnReceipt = new ESSReceipt();
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ReturnReceipt = (ESSReceipt) ESSReceipts.elementAt(i);
       if (ReturnReceipt.RECEIPT.equals(receipt)) i = ESSReceipts.size();
    }
    return ReturnReceipt;
  }
  
  protected ESSReceipt getESSReceipt(int receipt) {
    ESSReceipt ReturnReceipt = new ESSReceipt();
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ReturnReceipt = (ESSReceipt) ESSReceipts.elementAt(i);
       if (ReturnReceipt.ExpenseLink == receipt) i = ESSReceipts.size();
    }
    return ReturnReceipt;
  }
 
  
  protected ESSExpense getESSExpense(String receipt) {
  //Returns the last expense - useful for mileage and single links  
    ESSExpense ReturnExpense = new ESSExpense();
    // for (int i = 0; i < ESSExpenses.size(); i++) {    
    for (int i = ESSExpenses.size() - 1 ; i > -1; i--) {
       ReturnExpense = (ESSExpense) ESSExpenses.elementAt(i);
       if (ReturnExpense.RECEIPT.equals(receipt)) i = -1;
    }
    return ReturnExpense;
  }
  
  protected ESSExpense getESSExpenseByLink(int Link) {
  //Returns a specifice expense for attendees and extended. Defaults last expense.  
    ESSExpense ReturnExpense = new ESSExpense();
    // for (int i = 0; i < ESSExpenses.size(); i++) {    
    for (int i = ESSExpenses.size() - 1 ; i > -1; i--) {
       ReturnExpense = (ESSExpense) ESSExpenses.elementAt(i);
       if (ReturnExpense.LineLink == Link) i = -1;
    }
    return ReturnExpense;
  }
  
  protected ESSExpense getESSExpenseLineItem(String Receipt, String LineItem) {
  //Returns a specifice expense for attendees and extended. Defaults last expense.  
    ESSExpense ReturnExpense = new ESSExpense();
    // for (int i = 0; i < ESSExpenses.size(); i++) {    
    for (int i = ESSExpenses.size() - 1 ; i > -1; i--) {
       ReturnExpense = (ESSExpense) ESSExpenses.elementAt(i);
       if (ReturnExpense.RECEIPT.equals(Receipt) && ReturnExpense.LINEITEM.equals(LineItem)) i = -1;
    }
    return ReturnExpense;
  }
  
  protected ESSPurpose getESSPurpose(String purpose) {
    ESSPurpose ReturnPurpose = new ESSPurpose();
    for (int i = 0; i < ESSPurposes.size(); i++) {
       ReturnPurpose = (ESSPurpose) ESSPurposes.elementAt(i);
       if (ReturnPurpose.PURPOSE.equals(purpose)) i = ESSPurposes.size();
    }
    return ReturnPurpose;
  }  
//development note: Need to review all the getObjects for above and below 
//in the event that the desired object is not found. JH 2006-6-1
  protected ESSPurpose getESSPurpBlankOK(String purpose) {
    ESSPurpose ReturnPurpose = new ESSPurpose();
    ESSPurpose CheckPurpose = new ESSPurpose();
    for (int i = 0; i < ESSPurposes.size(); i++) {
       CheckPurpose = (ESSPurpose) ESSPurposes.elementAt(i);
       if (CheckPurpose.PURPOSE.equals(purpose)) {
          ReturnPurpose = (ESSPurpose) ESSPurposes.elementAt(i);
          i = ESSPurposes.size();
       }
    }
    return ReturnPurpose;
  }  
  
  protected boolean isESSPurpose(String purpose) {
    boolean retVal = false;
    ESSPurpose ReturnPurpose;
    for (int i = 0; i < ESSPurposes.size(); i++) {
       ReturnPurpose = (ESSPurpose) ESSPurposes.elementAt(i);
       if (ReturnPurpose.PURPOSE.equals(purpose))
       {
        retVal = true;
        i = ESSPurposes.size();
       }
    }
    return retVal;
  }  
  
  protected boolean isESSReceipt(String receipt) {
    boolean retVal = false;
    ESSReceipt ReturnReceipt;
    if (receipt != null && !receipt.equals(""))
    {
       for (int i = 0; i < ESSReceipts.size(); i++) {
          ReturnReceipt = (ESSReceipt) ESSReceipts.elementAt(i);
          if (ReturnReceipt.RECEIPT.equals(receipt))
          {
           retVal = true;
           i = ESSReceipts.size();
          }
       }
    }
    return retVal;
  }  
  
  protected boolean isESSExpense(String receipt, String lineitem) {
    boolean retVal = false;
    ESSExpense ReturnExpense;
    if ((receipt != null && !receipt.equals("")) && (lineitem != null && !lineitem.equals("")))
    {
       for (int i = 0; i < ESSExpenses.size(); i++) {
          ReturnExpense = (ESSExpense) ESSExpenses.elementAt(i);
          if (ReturnExpense.RECEIPT.equals(receipt) && ReturnExpense.LINEITEM.equals(lineitem))
          {
            retVal = true;
            i = ESSExpenses.size();
          }
       }
    }
    return retVal;
  }  
  
  protected boolean isLineItem(String lineitem) {
    boolean retVal = false;
    ESSExpense ReturnExpense;
    for (int i = 0; i < ESSExpenses.size(); i++) {
       ReturnExpense = (ESSExpense) ESSExpenses.elementAt(i);
       if (ReturnExpense.LINEITEM.equals(lineitem))
       {
        retVal = true;
        i = ESSExpenses.size();
       }
    }
    return retVal;
  }  
  
  protected boolean isExtraItem(String extraitem) {
    boolean retVal = false;
    ESSExtended ReturnExtended;
    for (int i = 0; i < ESSExtendeds.size(); i++) {
       ReturnExtended = (ESSExtended) ESSExtendeds.elementAt(i);
       if (ReturnExtended.EXTRAITEM.equals(extraitem))
       {
        retVal = true;
        i = ESSExtendeds.size();
       }
    }
    return retVal;
  }  
  
  protected ESSAttendeeList getESSAttendeeList(String lineitem) {
    ESSAttendeeList ReturnList = new ESSAttendeeList();
    for (int i = 0; i < ESSAttendeeLists.size(); i++) {
       ReturnList = (ESSAttendeeList) ESSAttendeeLists.elementAt(i);
       if (ReturnList.LINEITEM.equals(lineitem)) i = ESSAttendeeLists.size();
    }
    return ReturnList;
  }
  
  protected ESSExtended getESSAttendee(String lastLineItem, String extraItem) {
    ESSExtended ReturnAttendee = new ESSExtended();
    for (int i = 0; i < ESSExtendeds.size(); i++) {
       ReturnAttendee = (ESSExtended) ESSExtendeds.elementAt(i);
//SSSSSSSS Need to add in the LINEITEM check      
       if (ReturnAttendee.LINEITEM.equals(lastLineItem) && ReturnAttendee.EXTRAITEM.equals(extraItem)) i = ESSExtendeds.size();
    }
    return ReturnAttendee;
  }
  
  public void setAccountingInfo() {
      CustomDate subdate = new CustomDate();
      Rep.ACT_DATE = CustomDate.getSimpleDate(subdate.date);
      Rep.ACT_TIME = CustomDate.getSimpleTime(subdate.time);
  }
  
  public Vector getStatus()
  {
    return (status);
  }
  
  public boolean isSameReport(String testVoucher)
  {
    boolean retVal = false;
    if (testVoucher != null && Rep.VOUCHER != null && !Rep.VOUCHER.equals(""))
    {
       retVal = testVoucher.equalsIgnoreCase(Rep.VOUCHER);
    }
    return retVal;
  }
  
//Check the report for valid persNum, charge type, expense type, etc.  
  public void isValidReport()
  {
    status = new Vector();
    
    if (Rep.isValidPersNum() == false)
        status.addElement(rules.getCondString(rules.PERS_NUM_NOT_FOUND));
    
//Check charge type    
    int i = 0;
    while ((i < ESSReceipts.size()) && 
                    (((ESSReceipt) ESSReceipts.elementAt(i)).isValidCharge() == true))
        i++;

    if (i < ESSReceipts.size())
        status.addElement(rules.getCondString(rules.INVALID_CHARGE_TYPE));
        
//Check expense type    
    i = 0;
    while ((i < ESSExpenses.size()) && 
                    (((ESSExpense) ESSExpenses.elementAt(i)).isValidExpense() == true))
        i++;

    if (i < ESSExpenses.size())
        status.addElement(rules.getCondString(rules.INVALID_EXPENSE_TYPE));
  }
  
  public boolean isPersistanceOK()
  {
    return(persistanceStatus);
  }
  
  //public void setHeaderPersistance() {
  //  persistanceStatus = Rep.setPersistance();
  //}
  
  public void setPersistance()
  {
    
	String stack_dateformat = db.getInputDateFormat();  
	db.setInputDateFormat("MM/dd/yyyy");
	
    persistanceStatus = Rep.setPersistance();
    
    if (persistanceStatus == true)
    {
        int i = 0;
        while ((i < ESSPurposes.size()) && (persistanceStatus == true))
        {
            persistanceStatus = ((ESSPurpose) ESSPurposes.elementAt(i)).setPersistance();
            i++;
        }
        
        i = 0;
        while ((i < ESSReceipts.size()) && (persistanceStatus == true))
        {
            persistanceStatus = ((ESSReceipt) ESSReceipts.elementAt(i)).setPersistance();
            i++;
        }
        
        i = 0;
        while ((i < ESSExpenses.size()) && (persistanceStatus == true))
        {
            persistanceStatus = ((ESSExpense) ESSExpenses.elementAt(i)).setPersistance();
            i++;
        }
        
        i = 0;
        while ((i < ESSExtendeds.size()) && (persistanceStatus == true))
        {
            persistanceStatus = ((ESSExtended) ESSExtendeds.elementAt(i)).setPersistance();
            i++;
        } 
        
        if (persistanceStatus == false)
            updateReportColumn("RP_STAT", stateTable.NewToESS("default","ST_REJECTED"));
    }
    db.setInputDateFormat(stack_dateformat);
  }
  
  public boolean setConvertReport(String xrate, String multiplier, String xdate, String xsource)
  {
    boolean retVal = false;
    double amount = 0;
    double rec_amt = 0;
    double rate = Double.parseDouble(xrate);
    if (multiplier.equals("1")) rate = 1/rate;
    double rec_rate = 0;
    double exp_amt = 0;
    double exp_rec_amt = 0;
    double exp_rec_rate = 0;
    int i = 0;
    int j;
    int k;
    ESSReceipt receipt;
    ESSExpense expense;
    ESSPurpose purpose;
    ESSExtended extended;
    
    AuditTrail.println("[000] Report.setConvertReport xrate: " + xrate);
    AuditTrail.println("[000] Report.setConvertReport multiplier: " + multiplier);
    AuditTrail.println("[000] Report.setConvertReport xdate: " + xdate);
    AuditTrail.println("[000] Report.setConvertReport xsource: " + xsource);
    xdate = date.getStandardDateString(xdate, dateFormatStr);
    AuditTrail.println("[000] Report.setConvertReport formatted xdate: " + xdate);    
    AuditTrail.println("[000] Report.setConvertReport Rep.CURRENCY: " + Rep.CURRENCY);
    AuditTrail.println("[000] Report.setConvertReport HomeCurrency: " + HomeCurrency);
    
    if (ESSReceipts.size() > 0)
    { 
       setNewVoucherNumberFromExpSys();
       String originalCurrency = Rep.CURRENCY;
       Rep.CURRENCY = HomeCurrency;
        
       while (i < ESSReceipts.size())
       {
           receipt = ((ESSReceipt) ESSReceipts.elementAt(i));
           i++;
            
           AuditTrail.println("[000] Report.setConvertReport Old Voucher: " + receipt.VOUCHER + ", receipt number: " + receipt.RECEIPT);
        
           if (!receipt.RCPTTYPE.equals("4"))   // 4 = mileage
           {
        	   receipt.REC_AMT = receipt.AMOUNT;
               rec_amt = Double.parseDouble(receipt.REC_AMT);
               amount = rec_amt/rate;
               receipt.XDATE = xdate;
               receipt.XRATE = xrate;
               receipt.XSOURCE = xsource;
               receipt.UNITS = originalCurrency;
           } else {
               rec_amt = Double.parseDouble(receipt.AMOUNT);   //In mileage do not convert the receipt amount which is miles - leave other stuff alone
               rec_rate = Double.parseDouble(receipt.XRATE);
               amount = rec_amt/rate;
        	   receipt.XRATE = DRateStr(rec_rate/rate);
           }
           receipt.AMOUNT = FF(DRoundStr(amount));
           receipt.VOUCHER = Rep.VOUCHER;

           j = 0;
           k = -1;  //holds last found expense.
           while (j < ESSExpenses.size())
           {
              expense = ((ESSExpense) ESSExpenses.elementAt(j));
              if (expense.RECEIPT.equals(receipt.RECEIPT))
              {
                 k = j;
                 //expense.REC_AMT = expense.AMOUNT;
                 //exp_rec_amt = Double.parseDouble(expense.REC_AMT);
                 //exp_amt = exp_rec_amt/rate;
                 //expense.AMOUNT = FF(DRoundStr(exp_amt));
                 //amount = amount - exp_amt;
                 //expense.VOUCHER = Rep.VOUCHER;
                 //if (!receipt.RCPTTYPE.equals("4"))
                 //{
                 //   expense.XDATE = xdate;
                 //   expense.XRATE = xrate;
                 //   expense.XSOURCE = xsource;
                 //   expense.UNITS = originalCurrency;
                 //}
                 expense.VOUCHER = Rep.VOUCHER;
                 if (!receipt.RCPTTYPE.equals("4")) // 4 = mileage
                 {
                	 expense.REC_AMT = expense.AMOUNT;
                     exp_rec_amt = Double.parseDouble(expense.REC_AMT);
                     exp_amt = exp_rec_amt/rate;
                     expense.XDATE = xdate;
                     expense.XRATE = xrate;
                     expense.XSOURCE = xsource;
                     expense.UNITS = originalCurrency;
                 } else {
                     exp_rec_amt = Double.parseDouble(expense.AMOUNT);   //In mileage do not convert the receipt amount which is miles - leave other stuff alone
                     exp_rec_rate = Double.parseDouble(expense.XRATE);
                     exp_amt = exp_rec_amt/rate;
              	     expense.XRATE = DRateStr(exp_rec_rate/rate);
                 }
                 expense.AMOUNT = FF(DRoundStr(exp_amt));
                 amount = amount - exp_amt;
                 rec_amt = rec_amt - exp_rec_amt;
              }
              j++;
           }
           if (k > 0 && (rec_amt != (double) 0.00))
           {
              expense = ((ESSExpense) ESSExpenses.elementAt(k));
              exp_rec_amt = Double.parseDouble(expense.REC_AMT);
              exp_rec_amt = exp_rec_amt + rec_amt;
              expense.REC_AMT = FF(DRoundStr(exp_rec_amt));
              AuditTrail.println("[400] Report.setConvertReport(): Receipt rounding - investigate expense: " + expense.LINEITEM); 
           }
           if (k > 0 && (amount != (double) 0.00))
           {
              expense = ((ESSExpense) ESSExpenses.elementAt(k));
              exp_amt = Double.parseDouble(expense.AMOUNT);
              exp_amt = exp_amt + amount;
              expense.AMOUNT = FF(DRoundStr(exp_amt));
              AuditTrail.println("[000] Report.setConvertReport(): Receipt rounding - normal expense: " + expense.LINEITEM); 
           }
       }
       i = 0;
       while (i < ESSPurposes.size())
       {
           purpose = ((ESSPurpose) ESSPurposes.elementAt(i));
           purpose.VOUCHER = Rep.VOUCHER;
           i++;
       }
       i = 0;
       while (i < ESSExtendeds.size())
       {
           extended = ((ESSExtended) ESSExtendeds.elementAt(i));
           extended.VOUCHER = Rep.VOUCHER;
           i++;
       }
       setESSGL();
       setReportTotalsForced();
       retVal = true;
       AuditTrail.println("[000] Report.setConvertReport(): New Voucher: " + Rep.VOUCHER + ", new currency: " + Rep.CURRENCY);
    }
    return retVal;
  }
  
  //Unscrambles a report - call by DB2ESS after the report is loaded
  public void setUnScramble()
  {
    
    if (encrypt.equalsIgnoreCase("YES"))
    {
        AuditTrail.println("[000] Report.setUnScramble encrypt=yes");
        Rep.setUnScramble();
    
        int i = 0;
        while (i < ESSPurposes.size())
        {
            ((ESSPurpose) ESSPurposes.elementAt(i)).setUnScramble();
            i++;
        }
        
        i = 0;
        while (i < ESSReceipts.size())
        {
            ((ESSReceipt) ESSReceipts.elementAt(i)).setUnScramble();
            i++;
        }
        
        i = 0;
        while (i < ESSExpenses.size())
        {
            ((ESSExpense) ESSExpenses.elementAt(i)).setUnScramble();
            i++;
        }
        
        i = 0;
        while (i < ESSExtendeds.size())
        {
            ((ESSExtended) ESSExtendeds.elementAt(i)).setUnScramble();
            i++;
        } 
    } else {
        AuditTrail.println("[000] Report.setUnScramble encrypt=no");
    }    
  }
  
  
  // updates the the data field for the current report
  private void updateReportColumn(String columnId, String columnValue)
  {
    String SQL_Update = "UPDATE REPORT SET $column$ = '$value$' WHERE VOUCHER = '$voucher$'" + sqlTerminator;
    String SQLStatement = db.SQLReplace(SQL_Update,"$column$", columnId);
    SQLStatement = db.SQLReplace(SQLStatement,"$value$", columnValue);
    SQLStatement = db.SQLReplace(SQLStatement,"$voucher$", Rep.getVoucherNumber());
       
    db.doSQLExecute(SQLStatement);
    db.closeMyState();
    
  }
  public String toString() {
    //provided for standardization - prints out the report in ESS format
    return getReport();
  }  
  
  private String getFlat(String Name, String Value,String[] logicalFields,String[] numericFields,String[] dateFields) {
    return getFlat(Name, Value, "", logicalFields, numericFields, dateFields);
  }
  
  private String getFlat(String Name, String Value, String Delimiter,String[] logicalFields,String[] numericFields,String[] dateFields) {
    return getFlat(Name, Value, Delimiter, stringQuote, logicalFields, numericFields, dateFields);  
  } 
  
  private String getFlat(String Name, String Value, String Delimiter, String Quote,String[] logicalFields,String[] numericFields,String[] dateFields ) {
    String retVal;
    if (Value.equals("")) {
      retVal = exportNull;  
    } else {
        if ((Utilities.findString(Name, logicalFields) != -1) || (Utilities.findString(Name, numericFields) != -1))
        {
          retVal = Delimiter + Value;  
        } else if (Utilities.findString(Name, dateFields) != -1 && !quoteDate)
        {
          // Value = CustomDate.getExportDate(CustomDate.getDateFromString(Value));  
          Value = date.getStrFromDate(CustomDate.getDateFromString(Value),dateFormatStr);  
          if (quoteDate) {
        	  retVal = Delimiter + Quote  + Value + Quote;
          } else {
              retVal = Delimiter + Value;
          }
        } else
        {
          retVal = Delimiter + Quote  + Utilities.getFlatString(Value) + Quote;  
        }
    }    
    return retVal;
  } 
   
  private String getCell(String Title, String Value) {
    return getCell(Title, Value, "");
  }
  private String getCell(String Title, String Value, String Delimiter) {
    String retVal;
    retVal = getCell(Title, Value, Delimiter, "\"");  
    return retVal;
  } 
  private String getDateCell(String Title, String Value, String Delimiter) {
	    Value = date.getUserDateStr(Value, dateFormatStr);
	    String retVal = getCell(Title, Value, Delimiter, "\"");  
	    return retVal;
  } 
  private String getAmountCell(String Title, String Value, String Delimiter) {
	    Value = getUserAmount(Value);
	    String retVal = getCell(Title, Value, Delimiter, "\"");  
	    return retVal;
  } 

  private String getCell(String Title, String Value, String Delimiter, String Quote ) {
    String retVal;
    if (Value.equals("")) {
      retVal = "";  
    } else {
      retVal = Delimiter + "[" + Quote + Title + Quote + "," + Quote  + Value + Quote + "]";  
    }    
    return retVal;
  } 
  private String getCellList(String Title, String Value, String Delimiter) {
    //used to print out the standard ESS transmission format
    String retVal;
    if (Value.equals("")) {
      retVal = "";  
    } else {
      retVal = Delimiter + "[\"" + Title + "\","  + Value + "]";  
    }    
    return retVal;
  } 
   
  private String getDNCellList(String Title, String Value, String Delimiter) {
    //used to print out the standard ESS transmission format
    String retVal;
    if (Value.equals("")) {
      retVal = "";  
    } else {
      retVal = Delimiter + Value;  
    }    
    return retVal;
  } 
   
  private String getESSOut(String Title, String Value) {
    //used to print out the standard ESS transmission format
    String retVal;
    if (Value.equals("")) {
      retVal = "";  
    } else {
      retVal = DQ + Title + DQ + Value + DE;  
    }    
    return retVal;
  }  
    
  public ESSReport getReportObject()
  {
    return Rep;
  }
  
  protected class ESSReport implements ReportElementAPI {
   //class to descrbe the report header 
   public String RP_STAT = "";
   public String PERS_NUM = "";
   public String CRE_DATE = "";
   public String CRE_TIME = "";
   public String VOUCHER = "";
   public String PVOUCHER = "";
   public String SUB_DATE = "01/01/01";
   public String SUB_TIME = "01:00:00";
   public String CUR_DATE = "01/01/01";
   public String CUR_TIME = "01:00:00";
   public String ACT_DATE = "";
   public String ACT_TIME = "";
   public String PAYMENT = "No";
   public String NAME = "";
   public String LOCATION = "";
   public String PHONE = "";
   public String DEPART = "";
   public String COMPANY = "";
   public String RE_AMT = "";
   public String RC_AMT = "";
   public String AD_AMT = "";
   public String MONTH = "";
   public String CLERK = "";
   public String PHY_REC = "";
   public String RECEIVED = "";
   public String VEHICLE = "";
   public String START = "";
   public String FINISH = "";
   public String BUSINESS = "";
   public String SERVDATE = "";
   public String WEEKEND = "";
   
   public String COMMENT = "";
   public String ADMIN1 = "";
   public String AD1_DATE = "";
   public String SIGN1 = "";
   public String SG1_DATE = "";
   public String SIGN2 = "";
   public String SG2_DATE = "";
   public String ADMIN2 = "";
   public String AD2_DATE = "";
   public String SIGN3 = "";
   public String AUD_DATE = "";
   public String SIGN4 = "";
   public String UP_DATE = "";
   public String UP_STAT = "";
   public String RC_DATE = "";
   public String MANAGER = "";
   
   public String EXTRA1 = "";
   public String EXTRA3 = "";
   public String FORCED = "";
   public String CURRENCY = "";
   public String HISTORY = "";
   
   public String manager = "";
   private boolean validPersNum = true;
   private boolean exitStatus = true;
   public String persistance = "";
   public String name_sort = "";
   
   
   public boolean set(Element E, boolean useNewVoucher) {
      Rep.PERS_NUM = getValue(E,"persnum");
      RP_STAT = ReportStatus;    //will need to change this for the fullbown version
      CRE_DATE = getValueDate(E,"credate");
      CRE_TIME = getValue(E,"cretime");
      NAME = getValue(E,"name").toUpperCase();
      PHONE = getValue(E,"phone");
      LOCATION = getValue(E,"location");

      DEPART = getValue(E,"depart");
    
      CLERK = getValue(E,"admin1");              //might want to change this
      if (CLERK.equals("")) CLERK = PERS_NUM;
      if (useNewVoucher == false)
        VOUCHER = getESSReferenceString(getValue(E,"reference"),MessageReference);
      else
        setNewVoucherNumberFromExpSys();
      PVOUCHER = getReferenceString(getValue(E,"reference"),MessageReference,8,voucherPrefix);
      SUB_DATE = getValueDate(E,"subdate",CustomDate.getSimpleDate(date.date));
      SUB_TIME = getValue(E,"subtime",CustomDate.getSimpleTime(date.time));
      if (PT.setPersNumInfo(Rep.PERS_NUM)) {
        name_sort = PT.lname + " " + PT.fname;
        //OCCODE = " ";
        manager = PT.manager;
        setCompany(PT.getCompany());  //JH 2-28-2008
      } else {
        AuditTrail.println("[400] Report.ESSReport: Persnum not found: " + Rep.PERS_NUM);
        name_sort = " ";
        //OCCODE = " ";
        manager = "unknown";
        validPersNum = false;
        exitStatus = false;
      }  
      
      //FILENAME = "Web Server";
      //FILENAME = getValue(E,"filename","Web Server");;
      
      RE_AMT = getValueAmount(E,"reamt","0.00");
      RC_AMT = getValueAmount(E,"rcamt","0.00");
      AD_AMT = getValueAmount(E,"adamt","0.00");

      //CUR_DATE = CustomDate.getLocalDate();  //was testing
      //CUR_TIME = CustomDate.getLocalTime();
      CUR_DATE = CustomDate.getSimpleDate(date.date);
      CUR_TIME = CustomDate.getSimpleTime(date.time);
      
      MONTH = getValue(E,"month",CustomDate.getPreviousMonth(CustomDate.getDateFromString(CRE_DATE)));
      WEEKEND = getValueDate(E,"weekend",CustomDate.getPreviousWeekend(CustomDate.getDateFromString(CRE_DATE)));
      PHY_REC = getValueAmount(E,"phyrec");
      CURRENCY = getValue(E,"currency");
      //HISTORY = getValue(E, "history");  // History should not be coming in via XML
      setStandard();
      return true;
   } 
   
   public String getPersistance() {
      return persistance;
   }
   
   public String getIdentifier() {
      return VOUCHER;
   }
   
   protected void setStandard() {
      if (PT.setPersNumInfo(Rep.PERS_NUM)) {
        name_sort = PT.lname + " " + PT.fname;
        //OCCODE = " ";
        manager = PT.manager;
        setCompany(PT.getCompany());
      } else {
        AuditTrail.println("[400] Report.ESSReport: Persnum not found: " + Rep.PERS_NUM);
        name_sort = " ";
        //OCCODE = " ";
        manager = "unknown";
        validPersNum = false;
        exitStatus = false;
      }  
      //FILENAME = "Web Server";
      //if (FILENAME.equals(""))  FILENAME = "Direct Input";
      if (MONTH.equals("")) MONTH = CustomDate.getPreviousMonth(CustomDate.getDateFromString(CRE_DATE));
      if (WEEKEND.equals("")) WEEKEND = CustomDate.getPreviousWeekend(CustomDate.getDateFromString(CRE_DATE));
   }
   
   public boolean isValidPersNum()
   {
        return(validPersNum);
   }
   
   public void setNewVoucherNumberFromExpSys() 
   {
     boolean uniqueNumber = true;
     //if (voucherObject.equalsIgnoreCase("ExpSysTable")) {
     //  ExpSysTable es;
     //   if (dbCon != null)
     //   {
     //       es = new ExpSysTable();
     //       es.setConnection(dbCon);
     //   }
     //   else es = new ExpSysTable(DBUrl);
     //   es.setSQLTerminator(sqlTerminator);
     //   es.setLockTable(sqlLockTable);       //sets up the lock strings
     //   es.setUnlockTable(sqlUnlockTable);
     //   MessageReference = es.getExpSysIncString("VOUCHER", uniqueNumber, voucherPrefix);
     //} else {
        ServerSystemTable es;
        if (dbCon != null)
        {
            es = new ServerSystemTable();
            es.setConnection(dbCon);
        }
        else es = new ServerSystemTable(DBUrl);
        es.setSQLTerminator(sqlTerminator);
        es.setLockTable(sqlLockTable);       //sets up the lock strings
        es.setUnlockTable(sqlUnlockTable);
        MessageReference = es.getExpSysIncString("VOUCHER", uniqueNumber, voucherPrefix);
     //}
     //VOUCHER = getESSReferenceString(MessageReference);
     VOUCHER = MessageReference;
   } 
   
   public String getVoucherNumber()
   {
        return(VOUCHER);
   }
   
   public void set() {
     if (VOUCHER.equals("")) {
        setNewVoucherNumberFromExpSys();
     }
     if (PVOUCHER.equals("")) {     
        PVOUCHER = VOUCHER;
     }
     setStandard();
   }

   public void updateNormalize() {
   };
   
   protected String[] keyList = {"VOUCHER"};
   protected String[] tagList = {"RP_STAT","PERS_NUM","CRE_DATE","CRE_TIME","VOUCHER",
            "PVOUCHER","SUB_DATE","SUB_TIME","CUR_DATE","CUR_TIME",
            "ACT_DATE","ACT_TIME","PAYMENT","EXTENDED",
            "NAME","LOCATION","PHONE","DEPART","COMPANY","RE_AMT","RC_AMT","AD_AMT",
            "SIGN1","SIGN2","SIGN3","SYSVER","CHECK",
            "XCHARGE","AUD_DATE","FORCED","UP_DATE","UP_STAT","VEHICLE",
            "MONTH","SERVDATE","START","FINISH","BUSINESS","CLERK",
            "SG1_DATE","SG2_DATE","EXTRA1","EXTRA2","UNITS","MANAGER","COMMENT",
            "EXTRA3","OLD_STAT","PHY_REC","RECEIVED","EXTRA4",
            "SIGN4","RC_DATE","ADMIN1","AD1_DATE","ADMIN2","AD2_DATE","WEEKEND","CURRENCY","HISTORY"};
   protected String[] updateList = {"PERS_NUM",
            "NAME","LOCATION","PHONE","DEPART","COMPANY","RE_AMT","RC_AMT","AD_AMT",
            "VEHICLE",
            "SERVDATE","START","FINISH","BUSINESS","CLERK",
            "COMMENT",
            "PHY_REC","RECEIVED",
            "WEEKEND","CURRENCY","HISTORY"};
   protected String[] logicalFields = {"PAYMENT","EXTENDED","CHECK","XCHARGE","FORCED"};
   protected String[] numericFields = {"RE_AMT","RC_AMT","AD_AMT","START","FINISH",
             "BUSINESS","EXTRA2","EXTRA3","PHY_REC"};
   protected String[] dateFields = {"CRE_DATE","SUB_DATE","CUR_DATE","ACT_DATE",
             "AUD_DATE","UP_DATE","SERVDATE","SG1_DATE","SG2_DATE",
             "RC_DATE","AD1_DATE","AD2_DATE","RECEIVED","WEEKEND"};
   protected String[] encryptFields = {"NAME"};
        
   public void setPersistanceFlag(String x){
      persistance = x;
   }
        
   public boolean setPersistance()
   {
    boolean status = true;
    
        String[] valueList = {RP_STAT,PERS_NUM,CRE_DATE,CRE_TIME,VOUCHER,
            PVOUCHER,SUB_DATE,SUB_TIME,CUR_DATE,CUR_TIME,
            ACT_DATE,ACT_TIME,PAYMENT,"",
            scramble(str(NAME,36),36),str(LOCATION,30),str(PHONE,24),str(DEPART,16),COMPANY,RE_AMT,RC_AMT,AD_AMT,
            "","","","WEB","",
            "","","","","",VEHICLE,
            MONTH,SERVDATE,START,FINISH,BUSINESS,CLERK,
            "","","","","",PT.manager,"",
            "","",PHY_REC,RECEIVED,"","","",
            "","","","",WEEKEND,CURRENCY,HISTORY};
        
        //status = db.setPersistance(sqlInsertMethod,"REPORT", tagList, logicalFields, numericFields, dateFields, encryptFields, valueList);
        status = db.setPersistance(sqlInsertMethod,"REPORT",tagList,valueList,encryptFields);

        return (status);
   }
   
   public void setUnScramble()
   {
     NAME = unScramble(NAME);
   }
   public String[] getKeyList()
   {
    return keyList;
   }
   public String[] getTagList()
   {
    return tagList;
   }
   public String[] getUpdateList()
   {
    return updateList;
   }
   
   public void setUpdateListWithStatus()
   {
    if (!updateList[0].equals("RP_STAT"))
    {
       String[] x = new String[updateList.length + 1]; 
       x[0] = "RP_STAT";
       for (int i = 0; i < updateList.length; i++) 
       {
          x[i + 1] = updateList[i];
       }
       updateList = x;
    }
   }
   
   public String[] getLogicalFields()
   {
    return logicalFields;
   }
   public String[] getNumericFields()
   {
    return numericFields;
   }
   public String[] getDateFields()
   {
    return dateFields;
   }
   public String[] getEncryptFields()
   {
    return encryptFields;
   }
      
   public String getClientCell() {
     String retVal = "[";
     retVal += getCell("RP_STAT",RP_STAT) ;
     retVal += getCell("PERS_NUM",PERS_NUM,delimit) ;
     retVal += getDateCell("CRE_DATE",CRE_DATE,delimit) ;
     retVal += getCell("CRE_TIME",CRE_TIME,delimit) ;
     retVal += getCell("VOUCHER",VOUCHER,delimit) ;
     retVal += getCell("PVOUCHER",PVOUCHER,delimit) ;
     retVal += getDateCell("SUB_DATE",SUB_DATE,delimit) ;
     retVal += getCell("SUB_TIME",SUB_TIME,delimit) ;
     retVal += getDateCell("CUR_DATE",CUR_DATE,delimit) ;
     retVal += getCell("CUR_TIME",CUR_TIME,delimit) ;
     retVal += getCell("PAYMENT",PAYMENT,delimit) ;
     retVal += getCell("NAME",NAME.toUpperCase(),delimit) ;
     retVal += getCell("LOCATION",LOCATION,delimit) ;
     retVal += getCell("PHONE",PHONE,delimit) ;
     retVal += getCell("DEPART",DEPART,delimit) ;
     retVal += getAmountCell("RE_AMT",RE_AMT,delimit) ;
     retVal += getAmountCell("RC_AMT",RC_AMT,delimit) ;
     retVal += getAmountCell("AD_AMT",AD_AMT,delimit) ;
     retVal += getCell("MONTH",MONTH,delimit) ;
     retVal += getCell("CLERK",CLERK,delimit) ;
     retVal += getCell("PHY_REC",PHY_REC,delimit) ;
     retVal += getCell("RECEIVED",RECEIVED,delimit) ;
     retVal += getCell("VEHICLE",VEHICLE,delimit) ;
     retVal += getAmountCell("START",START,delimit) ;
     retVal += getAmountCell("FINISH",FINISH,delimit) ;
     retVal += getCell("BUSINESS",BUSINESS,delimit) ;
     retVal += getDateCell("SERVDATE",SERVDATE,delimit) ;
     retVal += getDateCell("WEEKEND",WEEKEND,delimit) ;
     retVal += getCell("CURRENCY",CURRENCY,delimit) ;  // 3-19-2007
     retVal += getCell("COMMENT",COMMENT,delimit) ;
     retVal += getCell("ADMIN1",ADMIN1,delimit) ;
     retVal += getDateCell("AD1_DATE",AD1_DATE,delimit) ;
     retVal += getCell("SIGN1",SIGN1,delimit) ;
     retVal += getDateCell("SG1_DATE",SG1_DATE,delimit) ;
     retVal += getCell("SIGN2",SIGN2,delimit) ;
     retVal += getDateCell("SG2_DATE",SG2_DATE,delimit) ;
     retVal += getCell("ADMIN2",ADMIN2,delimit) ;
     retVal += getDateCell("AD2_DATE",AD2_DATE,delimit) ;
     retVal += getCell("SIGN3",SIGN3,delimit) ;
     retVal += getDateCell("AUD_DATE",AUD_DATE,delimit) ;
     retVal += getCell("SIGN4",SIGN4,delimit) ;
     retVal += getDateCell("UP_DATE",UP_DATE,delimit) ;
     retVal += getCell("UP_STAT",UP_STAT,delimit) ;
     retVal += getDateCell("RC_DATE",RC_DATE,delimit) ;
     retVal += getCell("MANAGER",MANAGER,delimit) ;
     retVal += getCell("COMPANY",COMPANY,delimit);
     retVal += getCell("HISTORY",HISTORY,delimit);
     retVal += "]";   //JH 2011-04-14
     //retVal += getCell("editable",getReportEditable(),delimit);
  
     return retVal;
   }
   
   public String[] getDeNormal() {
     String[] retVal = new String[29];
     int j = 0;
     retVal[j++] = getFlat("RP_STAT",RP_STAT,logicalFields,numericFields,dateFields) ;   //0
     retVal[j++] = getFlat("PERS_NUM",PERS_NUM,logicalFields,numericFields,dateFields) ;  //1
     retVal[j++] = getFlat("CRE_DATE",CRE_DATE,logicalFields,numericFields,dateFields) ;  //2
     retVal[j++] = getFlat("VOUCHER",VOUCHER,logicalFields,numericFields,dateFields) ;   //3
     retVal[j++] = getFlat("CUR_DATE",CUR_DATE,logicalFields,numericFields,dateFields) ;  //4
     retVal[j++] = getFlat("PAYMENT",PAYMENT,logicalFields,numericFields,dateFields) ;   //5
     retVal[j++] = getFlat("NAME",NAME.toUpperCase(),logicalFields,numericFields,dateFields) ;  //6
     retVal[j++] = getFlat("LOCATION",LOCATION,logicalFields,numericFields,dateFields) ;  //7
     retVal[j++] = getFlat("CURRENCY",CURRENCY,logicalFields,numericFields,dateFields) ;  //8
     retVal[j++] = getFlat("PVOUCHER",PVOUCHER,logicalFields,numericFields,dateFields) ;    //9
     retVal[j++] = getFlat("HISTORY",HISTORY,logicalFields,numericFields,dateFields) ;    //10
     retVal[j++] = getFlat("COMPANY",COMPANY,logicalFields,numericFields,dateFields) ;    //11
     retVal[j++] = getFlat("DEPART",DEPART,logicalFields,numericFields,dateFields) ;    //12
     retVal[j++] = getFlat("RE_AMT",RE_AMT,logicalFields,numericFields,dateFields) ;    //13
     retVal[j++] = getFlat("RC_AMT",RC_AMT,logicalFields,numericFields,dateFields) ;    //14
     retVal[j++] = getFlat("AD_AMT",AD_AMT,logicalFields,numericFields,dateFields) ;    //15
     retVal[j++] = getFlat("MONTH",MONTH,logicalFields,numericFields,dateFields) ;     //16
     retVal[j++] = getFlat("PHY_REC",PHY_REC,logicalFields,numericFields,dateFields) ;   //17
     retVal[j++] = getFlat("PHONE",PHONE,logicalFields,numericFields,dateFields) ;     //18
     retVal[j++] = getFlat("VEHICLE",VEHICLE,logicalFields,numericFields,dateFields) ;   //19
     retVal[j++] = getFlat("START",START,logicalFields,numericFields,dateFields) ;     //20 
     retVal[j++] = getFlat("FINISH",FINISH,logicalFields,numericFields,dateFields) ;    //21
     retVal[j++] = getFlat("BUSINESS",BUSINESS,logicalFields,numericFields,dateFields) ;  //22
     retVal[j++] = getFlat("SERVDATE",SERVDATE,logicalFields,numericFields,dateFields) ;  //23
     retVal[j++] = getFlat("WEEKEND",WEEKEND,logicalFields,numericFields,dateFields) ;   //24
     retVal[j++] = getFlat("MANAGER",MANAGER,logicalFields,numericFields,dateFields) ;   //25
     retVal[j++] = getFlat("UP_DATE",UP_DATE,logicalFields,numericFields,dateFields) ;   //26
     retVal[j++] = getFlat("UP_STAT",UP_STAT,logicalFields,numericFields,dateFields) ;   //27
     
     retVal[j++] = getFlat("COMMENT",COMMENT,logicalFields,numericFields,dateFields) ;   //28
     
     return retVal;
   }
   
   public void normalize() {
     if (PHY_REC.equals("")){
       PHY_REC = Integer.toString(getPhysicalReceipts());
     }
     RE_AMT = getReimburse();
     AuditTrail.println("[000] Report.normalize RE_AMT: " + RE_AMT);     
     if (COMPANY.equals("")) COMPANY = getCompany(); //2008-01-15
   } 
   
  }
    
  protected class ESSReceipt implements Cloneable,ReportElementAPI {
   public int ExpenseLink; 
   public String xref;
   public String RCPTTYPE = "1";
   public String CHARGE = "";
   public String VOUCHER = "";
   public String RECEIPT = "";
   public String REFER = "";;
   public String REC_AMT = "";
   public String REC_DATE = "01/01/01";
   public String AMOUNT  = "0.00";
   public String REIMB = "No";
   public String FORCED = "No";
   public String CRE_DATE = "01/01/01";
   public String CHECK = "No";
   public String MERCHANT = "";
   public String UNITS = "";
   public String XSOURCE = "";
   public String XRATE = "";
   public String XDATE = "";
   public String COMPANY = "";
   public String SUBTYPE = "";
   public String BEG_DATE = "";
   public String LOCATION = "";
   public String OFFSET = "";
   public String OK_STAT = "";
   public String OK_REF = "";
   public String persistance = "";
   
   public void set(Element E, int ExpenseLink) {
     RCPTTYPE = getValue(E,"rcpttype","1");   //default as a split
     REC_DATE = getValueDate(E, "recdate",getValueDate(E, "rcptdate"));  //we are using rcptdate for ESS for some unknown reason
     AMOUNT = getValueAmount(E, "amount");
     REC_AMT = getValueAmount(E, "recamt");
     if (REC_AMT.equals("")) REC_AMT = AMOUNT;
     CHARGE = getValue(E, "charge");
     RECEIPT = getESSReferenceString(ExpenseLink); //Do we need to check for dup here 2005-12-20??
     VOUCHER = Rep.VOUCHER;
     if (getValue(E,"cre_date").equals("")) {
       CRE_DATE = CustomDate.getSimpleDate(date.date);     
     } else {
       CRE_DATE = getValueDate(E,"cre_date");
     }
     
     xref = getValue(E,"xref");
     if (!xref.equals("Weight")) {
       xref = getValuePlus(getValue(E, "xref"),1);
     }
     
     if (getValue(E, "reimb").equals("1")) {
        REIMB = "Yes";
     } else  if (getValue(E, "reimb").equals("0")){
        REIMB = "No";
     }
     MERCHANT = getValue(E, "merchant");
     UNITS = getValue(E, "units");
     XSOURCE = getValue(E, "xsource");
     XRATE = getValueAmount(E, "xrate");
     XDATE = getValueDate(E, "xdate");
     REFER = getValue(E, "refer");
     COMPANY = getValue(E, "company");
     SUBTYPE = getValue(E, "subtype"); 
     LOCATION = getValue(E, "location"); 
     OFFSET = getValue(E,"offset");
     BEG_DATE = getValueDate(E, "begdate");
     OK_STAT = getValue(E, "okstatus");
     OK_REF = getValue(E, "okref");
     doChargeMapping();
   } 
   
   public void updateNormalize() {
     if (REIMB.equals("1")) {
        REIMB = "Yes";
     } else  if (REIMB.equals("0")){
        REIMB = "No";
     }
   }
   private void doChargeMapping() {
     if (CHARGE.equals("ADVANCE")) {  //Special types for different types of receipts
        RCPTTYPE = "6";
     } else if (CHARGE.equals("CASH RET-D")) {  //need to think of something much better
        RCPTTYPE = "12";                        //for the non-ESS version
     } else if (CHARGE.equals("CASH-RET'D")) {
        RCPTTYPE = "12"; 
     } else if (CHARGE.equals("CASH-RET D")) {
        RCPTTYPE = "12";
     } else if (CHARGE.equals("CASH-RETD")) {
        RCPTTYPE = "12";
     } else if (CHARGE.equals("ADVANCE-RETD")) {
        RCPTTYPE = "12";
     }
   }
   
   public String getPersistance() {
      return persistance;
   }
   public String getIdentifier() {
      return RECEIPT + "-" + CHARGE;
   }

   public ESSReceipt duplicate() throws CloneNotSupportedException {
     return (ESSReceipt) this.clone();
   }

   public boolean isValidCharge()
   {
     return(db.setResultSet("SELECT CHARGE FROM CHARGE WHERE CHARGE = '" +
                                      CHARGE +  "' AND " + paymentSelect) == true);
   }
   
   public void setUnScramble()
   {
   }
   public String[] getKeyList()
   {
    return keyList;
   }
   public String[] getTagList()
   {
    return tagList;
   }
   public String[] getUpdateList()
   {
    return updateList;
   }
   public String[] getLogicalFields()
   {
    return logicalFields;
   }
   public String[] getNumericFields()
   {
    return numericFields;
   }
   public String[] getDateFields()
   {
    return dateFields;
   }
   public String[] getEncryptFields()
   {
    return encryptFields;
   }
   public void set() {
      VOUCHER = Rep.VOUCHER;
      String workingNumber;
      if (RECEIPT == null || RECEIPT.equals("")) { //added null JH 2006-04-25
        do {
             ExpenseLink = genReference++;
             workingNumber = getESSReferenceString(ExpenseLink);
        } while (isESSReceipt(workingNumber));     //JH 2005-12-20
        RECEIPT = workingNumber;
      }
   }
   
   public void setUpdateListWithReceipt()   //Think about modularizing this JH 4-26-2006
   {
    String[] x = new String[updateList.length + 1]; 
    x[0] = "RECEIPT";
    for (int i = 0; i < updateList.length; i++) 
    {
        x[i + 1] = updateList[i];
    }
    updateList = x;
   }
   
   public String[] keyList = {"VOUCHER","RECEIPT"};
   public String[]tagList = {"RCPTTYPE","CHARGE","VOUCHER","RECEIPT","REFER","REC_DATE",
            "REC_AMT","XRATE","UNITS","XSOURCE","XDATE",
            "AMOUNT","ACCOUNT","OFFSET","REIMB","FORCED","OK_DATE","OK_STAT","OK_REF",
            "CRE_DATE","CHECK","DOCUMENT","PERS_NUM","COMPANY","LOC_CODE",
            "UP_DATE","UP_STAT","MERCHANT","SUBTYPE","BEG_DATE","LOCATION"};
   public String[]updateList = {"CHARGE","REFER","REC_DATE",
            "REC_AMT","XRATE","UNITS","XSOURCE","XDATE",
            "AMOUNT","ACCOUNT","OFFSET","REIMB","FORCED","OK_STAT",
            "OK_REF","CHECK","MERCHANT","BEG_DATE","LOCATION"};
   public String[]logicalFields = {"REIMB", "FORCED", "CHECK", "DOCUMENT"};
   public String[]numericFields = {"RCPTTYPE","REC_AMT","XRATE","AMOUNT","EXPSPLIT","SUBTYPE"};
   public String[]dateFields = {"REC_DATE","XDATE","OK_DATE","CRE_DATE","UP_DATE","BEG_DATE"};
   public String[] encryptFields = {};
   
   public void setPersistanceFlag(String x){
      persistance = x;
   }
   
   public boolean setPersistance()
   {
    boolean status = true;
    String[] valueList = {RCPTTYPE,CHARGE,VOUCHER,RECEIPT,str(REFER,23),REC_DATE,
            REC_AMT,XRATE,str(UNITS,20),str(XSOURCE,20),XDATE,
            AMOUNT,"",OFFSET,REIMB,FORCED,"",OK_STAT,OK_REF,
            CRE_DATE,CHECK,"",PT.persnum,str(COMPANY,8),"",
            "","",str(MERCHANT,30),SUBTYPE,BEG_DATE,str(LOCATION,30)};

    status = db.setPersistance(sqlInsertMethod,"RECEIPT",tagList,valueList,encryptFields);
                                  
    return (status);
   }

   public String getClientCell() {
     String retVal = "[\"receipt" + RCPTTYPE + "\",[";
     retVal += getCell("RCPTTYPE",RCPTTYPE);
     retVal += getCell("CHARGE",CHARGE,delimit);
     retVal += getCell("VOUCHER",VOUCHER,delimit);
     retVal += getCell("RECEIPT",RECEIPT,delimit);
     retVal += getCell("REFER",REFER,delimit);
     retVal += getDateCell("REC_DATE",REC_DATE,delimit);
     retVal += getAmountCell("AMOUNT",AMOUNT,delimit);
     retVal += getCell("REIMB",REIMB,delimit);
     retVal += getCell("FORCED",FORCED,delimit);
     retVal += getDateCell("CRE_DATE",CRE_DATE,delimit);
     retVal += getCell("CHECK",CHECK,delimit);
     retVal += getCell("MERCHANT",MERCHANT,delimit);
     retVal += getAmountCell("REC_AMT",REC_AMT,delimit);
     retVal += getCell("UNITS",UNITS,delimit);
     retVal += getCell("XSOURCE",XSOURCE,delimit);
     retVal += getAmountCell("XRATE",XRATE,delimit);
     retVal += getDateCell("XDATE",XDATE,delimit);
     retVal += getCell("SUBTYPE",SUBTYPE,delimit);
     retVal += getDateCell("BEG_DATE",BEG_DATE,delimit);
     retVal += getCell("LOCATION",LOCATION,delimit);
     retVal += getCell("OFFSET",OFFSET,delimit);
     retVal += getCell("OK_STAT",OK_STAT,delimit);
     retVal += getCell("OK_REF",OK_REF,delimit);
     
     if ((RCPTTYPE.equals("4") || RCPTTYPE.equals("5")) && denormalizeClient)
     {
       retVal += getDNCellList("expenselist",getDNClientExpenses(RECEIPT),delimit);
     } else {
       retVal += getCellList("expenselist",getClientExpenses(RECEIPT),delimit);
     }
     retVal += "]]";
     return retVal;
   }
   
   public String[] getDeNormal() {
     String[] retVal = new String[17];
     int j = 0;
     
     String paymentAccountStr =  PaymentTable.getDOMTableValueWhere("charge", CHARGE, paymentAccountName); 
     
     retVal[j++] = getFlat("RCPTTYPE",RCPTTYPE,logicalFields,numericFields,dateFields) ;   //0
     retVal[j++] = getFlat("CHARGE",CHARGE,logicalFields,numericFields,dateFields) ;  //1
     retVal[j++] = getFlat("RECEIPT",RECEIPT,logicalFields,numericFields,dateFields) ;  //2
     retVal[j++] = getFlat("REFER",REFER,logicalFields,numericFields,dateFields) ;  //3
     retVal[j++] = getFlat("REC_DATE",REC_DATE,logicalFields,numericFields,dateFields) ;   //4
     retVal[j++] = getFlat("AMOUNT",AMOUNT,logicalFields,numericFields,dateFields) ;  //5
     retVal[j++] = getFlat("MERCHANT",MERCHANT,logicalFields,numericFields,dateFields) ;  //6
     retVal[j++] = getFlat("ACCOUNT",paymentAccountStr,logicalFields,numericFields,dateFields) ;  //7
     retVal[j++] = getFlat("COSTCENTER","",logicalFields,numericFields,dateFields) ;  //8

     retVal[j++] = getFlat("LOCATION",LOCATION,logicalFields,numericFields,dateFields) ;  //9
     retVal[j++] = getFlat("OFFSET",OFFSET,logicalFields,numericFields,dateFields) ;  //10
     retVal[j++] = getFlat("REC_AMT",REC_AMT,logicalFields,numericFields,dateFields) ;  //11
     retVal[j++] = getFlat("XRATE",XRATE,logicalFields,numericFields,dateFields) ;  //12    
     retVal[j++] = getFlat("UNITS",UNITS,logicalFields,numericFields,dateFields) ;  //13    
     retVal[j++] = getFlat("XSOURCE",XSOURCE,logicalFields,numericFields,dateFields) ;  //14    
     retVal[j++] = getFlat("XDATE",XDATE,logicalFields,numericFields,dateFields) ;  //15
     retVal[j++] = getFlat("SUBTYPE",SUBTYPE,logicalFields,numericFields,dateFields) ;  //16    


     
     return retVal;
   }
   
   public void normalize() {
     if (VOUCHER.equals("")) VOUCHER = Rep.VOUCHER;  //JH 23-24-2004 (move to set?)
     
     ESSExpense E; //denormalize the location for hotels in ESS layout
     ESSPurpose P;
     if (SUBTYPE.equals("1") && LOCATION.equals("")) {
       for (int i = 0; i < ESSExpenses.size(); i++) {
         E = (ESSExpense) ESSExpenses.elementAt(i);
         if (RECEIPT.equals(E.RECEIPT)) {
            for (int j = 0; j < ESSPurposes.size(); j++) {
               P = (ESSPurpose) ESSPurposes.elementAt(j);
               if (E.PURPOSE.equals(P.PURPOSE)) {
                 LOCATION = P.LOCATION;
                 j = ESSPurposes.size();
               }
            }
            i = ESSExpenses.size();
         }
       }
     }
   } 
   public void normalizeType() {
    // if rcptype 3 has more than 1 expense entry it is changed to a 1
    // if rcpttype 4 (mileage) has more than one expense it is broken apart
     ESSReceipt newReceipt;
     ESSExpense E;
     //int newReference;
     int nExpenseCount = 0;
     doChargeMapping();  
     if (RCPTTYPE.equals("3") || RCPTTYPE.equals("4")) {  
       for (int i = 0; i < ESSExpenses.size(); i++) {
         E = (ESSExpense) ESSExpenses.elementAt(i);
         if (RECEIPT.equals(E.RECEIPT)) {
           nExpenseCount++;
           if (RCPTTYPE.equals("4")) {
             if (nExpenseCount > 1) {
               try {
                 newReceipt = (ESSReceipt) this.duplicate();
                 //newReference = genReference++;  //what if this already exists sssssss
                 //E.RECEIPT = getESSReferenceString(newReference);
                 E.RECEIPT = getUnquieReceiptNumber();
                 newReceipt.RECEIPT = E.RECEIPT;
                 newReceipt.AMOUNT = E.AMOUNT;
                 if (!E.REC_AMT.equals("")) newReceipt.REC_AMT = E.REC_AMT;
                 ESSReceipts.addElement(newReceipt);
               } catch (CloneNotSupportedException e) {
                 AuditTrail.println("[500] Report.normalizeReceiptType mileage cloning failure");
               }
             } else {
                AMOUNT = E.AMOUNT;
                if (!E.REC_AMT.equals("")) REC_AMT = E.REC_AMT;
             }
           }    
         }
       }
     }
     if (nExpenseCount > 1  && !RCPTTYPE.equals("4")) {
        RCPTTYPE = "1";
     } 
   } 
   private String getUnquieReceiptNumber() {
      String testReference = "";     
      do {  
         testReference = getESSReferenceString(genReference++);
      } while (isESSReceipt(testReference));
      return testReference;
   }
   
   protected int compareTo(ESSReceipt otherRcpt)
   {
        java.util.Date rcptDate = Utilities.stringToDate(REC_DATE);
        java.util.Date otherRcptDate = Utilities.stringToDate(otherRcpt.REC_DATE);
        return (rcptDate.compareTo(otherRcptDate));
   }
   protected int compare(ESSReceipt otherRcpt)
   {
        String expenseType = Utilities.getTrim(getESSExpense(RECEIPT).EXPENSE);
        String otherExpenseType = Utilities.getTrim(getESSExpense(otherRcpt.RECEIPT).EXPENSE);
        int retVal = expenseType.compareTo(otherExpenseType);
        if (retVal == 0) {
          java.util.Date rcptDate = Utilities.stringToDate(REC_DATE);
          java.util.Date otherRcptDate = Utilities.stringToDate(otherRcpt.REC_DATE);
          retVal = rcptDate.compareTo(otherRcptDate);
        }
        return retVal;
   }
   
  }
  
  protected class ESSExpense implements Cloneable, ReportElementAPI {
   public int ReceiptLink;
   public int LineLink = 0;
   public String EXPENSE = "";
   public String AMOUNT = "";
   public String VOUCHER = "";
   public String RECEIPT = "";
   public String PURPOSE = "";
   public String LINEITEM = "";
   public String BILLTYPE = "";
   public String COMMENT = "";
   public String DEPART = "";
   public String ChargedDepart = "";  //jh 2-3-2004
   public String CRE_DATE = "";
   public String EXP_DATE = "";
   public String REC_AMT = "";
   public String XRATE = "";
   public String XDATE = "";
   public String UNITS = "";
   // public String MILES = "";
   public String XSOURCE = "";
   public String PERSPLIT = "";
   public String COMPANY = "";
   public String APPROVAL = "";
   public String category = "";
   public String project = "";
   public String clientno = "";
   public String merchant = "";
   public String persistance = "";   
   
   public void set(Element E, int ReceiptLink) {
     String xref = "";
     this.ReceiptLink = ReceiptLink;
     AMOUNT = getValueAmount(E,"amount");
     EXPENSE = getValue(E,"expensetype");
     RECEIPT = getESSReferenceString(ReceiptLink);
     xref = getValue(E,"xref");
     if (xref.equals("Weight")) {
       PURPOSE = xref;
     } else {
       PURPOSE = getESSReferenceString(getValuePlus(xref,1));
     }
     VOUCHER = Rep.VOUCHER;
     // LineLink = genReference++;
     // LINEITEM = getESSReferenceString(LineLink);
     String workingNumber;
     if (LINEITEM.equals("")) {
        do {
             LineLink = genReference++;
             workingNumber = getESSReferenceString(LineLink);
        } while (isLineItem(workingNumber));     //JH 2005-12-20
        LINEITEM = workingNumber;
     }
     EXP_DATE = getValueDate(E, "expdate");
     CRE_DATE = getValueDate(E,"credate");
     COMMENT = getValue(E,"comment");
     BILLTYPE = getValue(E,"billtype","2");
     if (EXPENSE.toUpperCase().equals(Pers_ExpenseType)) {
        BILLTYPE = "3";
     } else if (BILLTYPE.toUpperCase().equals("PERS")) {
        BILLTYPE = "3";
     } else if (BILLTYPE.toUpperCase().equals("PERSONAL")) {
        BILLTYPE = "3";          
     } else if (BILLTYPE.toUpperCase().equals("NO")){
        BILLTYPE = "2";
     } else if (BILLTYPE.toUpperCase().equals("YES")) {
        BILLTYPE = "1";
     }
     REC_AMT = getValueAmount(E,"recamt");   //jh 12/22
     UNITS = getValue(E, "units");
     XSOURCE = getValue(E, "xsource");
     XRATE = getValueAmount(E, "xrate");
     XDATE = getValueDate(E, "xdate");
     DEPART = getValue(E, "depart");
     ChargedDepart = DEPART;  //jh 2-3-2004
     PERSPLIT = getValue(E, "persplit");
     COMPANY = getValue(E, "company");
     category = getValue(E, "category");
     merchant = getValue(E, "merchant");
   } 
   
   public ESSExpense duplicate() throws CloneNotSupportedException {
     return (ESSExpense) this.clone();
   }
   
   public boolean isValidExpense()
   {
     return (db.setResultSet("SELECT EXPENSE FROM ACCOUNT WHERE EXPENSE = '" +
                                      EXPENSE + "' AND " + expenseSelect));
//     return (db.setResultSet("SELECT EXPENSE FROM ACCOUNT WHERE RTRIM(EXPENSE) = '" +
//                                      EXPENSE.trim() + "'"));

   }
   public String getPersistance() {
      return persistance;
   }
   public String getIdentifier() {
      return RECEIPT + "-" + LINEITEM + "-" + EXPENSE;
   }
   
   public void setUnScramble()
   {
      COMMENT = unScramble(COMMENT);
   }
   public String[] getKeyList()
   {
    return keyList;
   }
   public String[] getTagList()
   {
    return tagList;
   }
   public String[] getUpdateList()
   {
    return updateList;
   }
   public String[] getLogicalFields()
   {
    return logicalFields;
   }
   public String[] getNumericFields()
   {
    return numericFields;
   }
   public String[] getDateFields()
   {
    return dateFields;
   }
   public String[] getEncryptFields()
   {
    return encryptFields;
   }
   public void set(int Link) {
      VOUCHER = Rep.VOUCHER;
      if (RECEIPT.equals("")) {
        RECEIPT = getESSReferenceString(Link);
      }
      if (LINEITEM.equals("")) {
        LineLink = genReference++;
        LINEITEM = getESSReferenceString(LineLink);
      }
   }
   
   public void updateNormalize() {
   };
   
   protected String[] keyList = {"VOUCHER","RECEIPT","LINEITEM"}; //JH 2006-04-26
   protected String[] tagList = {"EXPENSE", "AMOUNT", "VOUCHER", "RECEIPT", "PURPOSE", 
        "LINEITEM", "DEPART", "ACCOUNT", "UP_DATE", "UP_STAT","BILLTYPE", "BL_DATE", 
        "BL_STAT", "COMMENT", "APPROVAL","CRE_DATE", "EXP_DATE",
        "REC_AMT", "XRATE",  "UNITS", "XSOURCE", "XDATE",
        "COMPANY", "LOC_CODE", "PERSPLIT"};
   protected String[] updateList = {"EXPENSE", "AMOUNT", "PURPOSE", 
        "DEPART", "ACCOUNT", "BILLTYPE", 
        "COMMENT", "EXP_DATE",
        "REC_AMT", "XRATE",  "UNITS", "XSOURCE", "XDATE",
        "PERSPLIT"};
            
   protected String[] logicalFields = {"CHECK","FORCED"};
   protected String[] numericFields = {"AMOUNT","BILLTYPE","REC_AMT","XRATE","PERSPLIT"};
   protected String[] dateFields = {"UP_DATE","BL_DATE","CRE_DATE","EXP_DATE","XDATE"};
   protected String[] encryptFields = {"COMMENT"};
   
   public void setPersistanceFlag(String x){
      persistance = x;
   }
   
   public boolean setPersistance()
   {
    boolean status = true;
    
    // Miles were taken aout on 11/3/2002
    String[] valueList = {EXPENSE,AMOUNT,VOUCHER,RECEIPT,PURPOSE, 
        LINEITEM,str(DEPART,16),"","","",BILLTYPE,"",
        "",scramble(str(COMMENT,60),60),str(APPROVAL,16),CRE_DATE,EXP_DATE,
        REC_AMT,XRATE,str(UNITS,20),str(XSOURCE,20),XDATE,
        str(COMPANY,8),"",PERSPLIT};
            
        //status = db.setPersistance(sqlInsertMethod,"EXPENSE", tagList, logicalFields, numericFields, dateFields, encryptFields, valueList);
        status = db.setPersistance(sqlInsertMethod,"EXPENSE",tagList,valueList,encryptFields);
        
        return (status);
   }
   
   
   public String getClientCell() {
     String retVal = "[\"expense\",[";
     retVal += getCell("EXPENSE",EXPENSE);
     retVal += getAmountCell("AMOUNT",AMOUNT,delimit);
     retVal += getCell("VOUCHER",VOUCHER,delimit);
     retVal += getCell("RECEIPT",RECEIPT,delimit);
     retVal += getCell("PURPOSE",PURPOSE,delimit);
     retVal += getCell("LINEITEM",LINEITEM,delimit);
     retVal += getCell("BILLTYPE",BILLTYPE,delimit);
     retVal += getCell("DEPART",DEPART,delimit);
     retVal += getCell("COMMENT",COMMENT,delimit);
     retVal += getDateCell("CRE_DATE",CRE_DATE,delimit);
     retVal += getDateCell("EXP_DATE",EXP_DATE,delimit);
     retVal += getAmountCell("REC_AMT",REC_AMT,delimit);
     retVal += getAmountCell("XRATE",XRATE,delimit);
     retVal += getDateCell("XDATE",XDATE,delimit);
     retVal += getCell("UNITS",UNITS,delimit);
     // retVal += getCell("MILES",MILES,delimit);
     retVal += getCell("XSOURCE",XSOURCE,delimit);
     retVal += getCell("PERSPLIT",PERSPLIT,delimit);
     retVal += getCell("COMPANY",COMPANY,delimit);
     retVal += getCellList("attendeelist",getClientAttendees(LINEITEM,PERSPLIT),delimit);
     retVal += "]]";
     return retVal;
   }
   public String getDNClientCell(String xLabel) {
     String retVal = "";
     retVal += getCell(xLabel + "EXPENSE",EXPENSE);
     retVal += getCell(xLabel + "AMOUNT",AMOUNT,delimit);
     retVal += getCell(xLabel + "VOUCHER",VOUCHER,delimit);
     retVal += getCell(xLabel + "RECEIPT",RECEIPT,delimit);
     retVal += getCell(xLabel + "PURPOSE",PURPOSE,delimit);
     retVal += getCell(xLabel + "LINEITEM",LINEITEM,delimit);
     retVal += getCell(xLabel + "BILLTYPE",BILLTYPE,delimit);
     retVal += getCell(xLabel + "DEPART",DEPART,delimit);
     retVal += getCell(xLabel + "COMMENT",COMMENT,delimit);
     retVal += getCell(xLabel + "CRE_DATE",CRE_DATE,delimit);
     retVal += getCell(xLabel + "EXP_DATE",EXP_DATE,delimit);
     retVal += getCell(xLabel + "REC_AMT",REC_AMT,delimit);
     retVal += getCell(xLabel + "XRATE",XRATE,delimit);
     retVal += getCell(xLabel + "XDATE",XDATE,delimit);
     retVal += getCell(xLabel + "UNITS",UNITS,delimit);
     // retVal += getCell("MILES",MILES,delimit);
     retVal += getCell(xLabel + "XSOURCE",XSOURCE,delimit);
     retVal += getCell(xLabel + "PERSPLIT",PERSPLIT,delimit);
     retVal += getCell(xLabel + "COMPANY",COMPANY,delimit);
     retVal += getCellList("attendeelist",getClientAttendees(LINEITEM,PERSPLIT),delimit);
     return retVal;
   }
   
   public String[] getDeNormal() {
     String[] retVal = new String[16];  //2011 was 11
     int j = 0;
     
     String ExpenseAccountStr =  ExpenseTable.getDOMTableValueWhere("expense", EXPENSE, expenseAccountName); 
     
     if (departTable == null)
     {
        departTable = new DepartmentTable();
        departTable.setConnection(dbCon);
        departTable.setSQLTerminator(sqlTerminator);  //JH 2005-09-15
        departTable.setSQLStrings();
     }
     //if (!ChargedDepart.equalsIgnoreCase(departTable.depart)) 
     departTable.setDepartInfo(ChargedDepart, COMPANY);
     String costCenter = departTable.gl_term;
     if (costCenter == null || costCenter.equals(""))
     {
        costCenter = ChargedDepart;
     }

     retVal[j++] = getFlat("EXPENSE",EXPENSE,logicalFields,numericFields,dateFields);    //0
     retVal[j++] = getFlat("AMOUNT",AMOUNT,logicalFields,numericFields,dateFields);     //1
     retVal[j++] = getFlat("LINEITEM",LINEITEM,logicalFields,numericFields,dateFields);   //2
     retVal[j++] = getFlat("BILLTYPE",BILLTYPE,logicalFields,numericFields,dateFields);   //3
     retVal[j++] = getFlat("DEPART",ChargedDepart,logicalFields,numericFields,dateFields);     //4
     retVal[j++] = getFlat("EXP_DATE",EXP_DATE,logicalFields,numericFields,dateFields);;  //5
     retVal[j++] = getFlat("PERSPLIT",PERSPLIT,logicalFields,numericFields,dateFields);   //6
     retVal[j++] = getFlat("COMPANY",COMPANY,logicalFields,numericFields,dateFields);    //7
     retVal[j++] = getFlat("ACCOUNT",ExpenseAccountStr,logicalFields,numericFields,dateFields);    //8
     retVal[j++] = getFlat("COSTCENTER",costCenter,logicalFields,numericFields,dateFields) ;  //9    
     retVal[j++] = getFlat("COMMENT",COMMENT,logicalFields,numericFields,dateFields) ;  //10 JH 2006-1-6    
     
     retVal[j++] = getFlat("REC_AMT",REC_AMT,logicalFields,numericFields,dateFields) ;  //11
     retVal[j++] = getFlat("XRATE",XRATE,logicalFields,numericFields,dateFields) ;  //12    
     retVal[j++] = getFlat("UNITS",UNITS,logicalFields,numericFields,dateFields) ;  //13    
     retVal[j++] = getFlat("XSOURCE",XSOURCE,logicalFields,numericFields,dateFields) ;  //14    
     retVal[j++] = getFlat("XDATE",XDATE,logicalFields,numericFields,dateFields) ;  //15
     
     return retVal;
   }
      
   public void normalize() {
    ESSReceipt R = getESSReceipt(RECEIPT);
    if (UNITS.equals("")) UNITS = R.UNITS;
    if (ifInArray(EXPENSE,mileageItems)) R.RCPTTYPE = "4"; 
    if (PURPOSE.equals("")) {
      PURPOSE = getESSReferenceString(0);  
    } else {
      ESSPurpose P = getESSPurpose(PURPOSE);
      if (COMPANY.equals("")) COMPANY = P.company;
      if (project.equals("")) project = P.PROJECT;
      if (clientno.equals("")) clientno = P.CLIENTNO;
      if (Update_Exp_Approval.equalsIgnoreCase("YES")) APPROVAL = COMPANY;
    }
    if (AMOUNT.equals("")) AMOUNT = R.AMOUNT;
    if (REC_AMT.equals("") && !R.SUBTYPE.equals("1")) REC_AMT = R.REC_AMT;
    if (EXP_DATE.equals("")) EXP_DATE = R.REC_DATE;
    if (CRE_DATE.equals("")) CRE_DATE = R.CRE_DATE;
    if (XSOURCE.equals(""))XSOURCE = R.XSOURCE;
    if (XRATE.equals(""))XRATE = R.XRATE;
    if (XDATE.equals(""))XDATE = R.XDATE;
    // if (DEPART.equals("")) DEPART = Rep.DEPART;
    if (ChargedDepart.equals("")) ChargedDepart = Rep.DEPART;
    if (R.MERCHANT.equals("") && !merchant.equals("")) R.MERCHANT = merchant;
    if (EXPENSE.toUpperCase().equals(Pers_ExpenseType)) {  //this is dup from above
        BILLTYPE = "3";
    } else if (BILLTYPE.toUpperCase().equals("PERS")) {
        BILLTYPE = "3";
    } else if (BILLTYPE.toUpperCase().equals("PERSONAL")) {
        BILLTYPE = "3";          
    } else if (BILLTYPE.toUpperCase().equals("NO")){
        BILLTYPE = "2";
    } else if (BILLTYPE.toUpperCase().equals("YES")) {
        BILLTYPE = "1";
    } 
    if (persistance.equals("")) persistance = R.persistance;
   } 
   
   public int companyNormalize() 
   {
    int retVal = 0;
    if (!COMPANY.equals("")) {
        ESSPurpose P = getESSPurpose(PURPOSE);
        if (P.company.equals("")) {
            P.company = COMPANY;
            retVal = 1;
        }
    }
    return retVal;
   }
   
   
   public void xrefNormalize() { //need to do before weight normalizing
    if (PURPOSE.equals("")) {
      ESSReceipt R = getESSReceipt(RECEIPT);  //moved 2009-5-1
      if (R.xref.equals("Weight")) {
        PURPOSE = R.xref; 
      } else {
        PURPOSE = getESSReferenceString(R.xref) ;
      }
    }
   }
    private int compareTo(ESSExpense otherExp)
    {
        //Logfile.println("[000] Report.ESSExpense.compareTo: " + EXP_DATE); 
        java.util.Date expDate = Utilities.stringToDate(EXP_DATE);
        java.util.Date otherExpDate = Utilities.stringToDate(otherExp.EXP_DATE);
        int ReturnCompare = expDate.compareTo(otherExpDate);
        if (ReturnCompare == 0) {
            ReturnCompare = EXPENSE.compareTo(otherExp.EXPENSE);
        }
        return ReturnCompare;
    }
    private int compareDeptProj(ESSExpense otherExp){
        int ReturnCompare = ChargedDepart.compareTo(otherExp.ChargedDepart);
        if (ReturnCompare == 0) {
           ReturnCompare = project.compareTo(otherExp.project);
           if (ReturnCompare == 0) {
              ReturnCompare = EXPENSE.compareTo(otherExp.EXPENSE);
           }
        }
        return ReturnCompare;
    }
  }
    
  protected class ESSPurpose implements Cloneable,ReportElementAPI {
   public int PurposeLink; 
   public String PURPOSE = "";
   public String VOUCHER = "";
   public String CLIENT = "";
   public String LOCATION = "";
   public String COMMENT = "";
   public String PROJECT = "";
   public String STEPNO = "";
   public String BEG_DATE = "";
   public String END_DATE = "";
   public String DAYS = "";
   public String NIGHTS = "";
   public String CLIENTNO = "";
   public String EXTENDED = "No";
   public String TIMESLIP = "No";
   public String billtype = "";
   public String weight = "";
   public String company = "";
   public String multiClient = "";
   public String persistance = "";
   
   public void set(Element E) {
     PURPOSE = getESSReferenceString(getValuePlus(getValue(E,"xref"),1));  //review 2005-12-20??
     VOUCHER = Rep.VOUCHER;
     BEG_DATE = getValueDate(E,"begdate");
     END_DATE = getValueDate(E,"enddate");
     CLIENTNO = getValue(E,"clientno");
     CLIENT = getValue(E,"client");
     LOCATION = getValue(E,"location");
     COMMENT = getValue(E,"comment");
     PROJECT = getValue(E,"project");
     STEPNO = getValue(E,"stepno");
     weight = getValueAmount(E,"weight");
     company = getValue(E,"company");
     multiClient = getValue(E,"multiclient");
   } 
   
   public void set() {
      VOUCHER = Rep.VOUCHER;
      String workingNumber;  //JH 2005-12-20
      if (PURPOSE.equals("")) {
        do {
             workingNumber = getESSReferenceString(genReference++);
        } while (isESSPurpose(workingNumber));     //JH 2005-12-20
        PURPOSE = workingNumber;
      }

   }
   
   public String getPersistance() {
      return persistance;
   }
   public String getIdentifier() {
      return PURPOSE;
   }
   
   public void setUnScramble()
   {
      CLIENT = unScramble(CLIENT);
      LOCATION = unScramble(LOCATION);
      COMMENT = unScramble(COMMENT);
      PROJECT = unScramble(PROJECT);
      CLIENTNO = unScramble(CLIENTNO);
   }
   public String[] getKeyList()
   {
    return keyList;
   }
   public String[] getTagList()
   {
    return tagList;
   }
   public String[] getUpdateList()
   {
    return updateList;
   }
   public String[] getLogicalFields()
   {
    return logicalFields;
   }
   public String[] getNumericFields()
   {
    return numericFields;
   }
   public String[] getDateFields()
   {
    return dateFields;
   }
   public String[] getEncryptFields()
   {
    return encryptFields;
   }
   
   public void updateNormalize() {
   };
   
   protected String[] keyList = {"VOUCHER", "PURPOSE"};
   protected String[] tagList = {"PURPOSE", "VOUCHER", "CLIENT", "LOCATION", "COMMENT",
                "PROJECT", "BEG_DATE", "END_DATE", "DAYS", "NIGHTS",
                "CLIENTNO", "FORCED", "STEPNO", "EXTENDED", "TIMESLIP"};
   protected String[] updateList = {"CLIENT", "LOCATION", "COMMENT",
                "PROJECT", "BEG_DATE", "END_DATE", "DAYS", "NIGHTS",
                "CLIENTNO", "STEPNO", "EXTENDED"};
   protected String[] logicalFields = {"EXTENDED", "TIMESLIP", "FORCED"};
   protected String[] numericFields = {"DAYS", "NIGHTS"};
   protected String[] dateFields = {"BEG_DATE","END_DATE"};
   protected String[] encryptFields = {"CLIENT","LOCATION","COMMENT","PROJECT","CLIENTNO"};
   
   public void setPersistanceFlag(String x){
      persistance = x;
   }
   
   public boolean setPersistance()
   {
        boolean status = true;
        
        if (!weight.equals("")) 
        {
           COMMENT = "(Split " + weight + "%) " + COMMENT;
        }

        String[] valueList = {PURPOSE, VOUCHER, scramble(str(CLIENT,40),40), 
                scramble(str(LOCATION,40),40), scramble(str(COMMENT,250),250),
                scramble(str(PROJECT,24),24), BEG_DATE, END_DATE, DAYS, NIGHTS,
                scramble(str(CLIENTNO,16),16), "", str(STEPNO,12), EXTENDED, TIMESLIP};

        //status = db.setPersistance(sqlInsertMethod,"PURPOSE", tagList, logicalFields, numericFields, dateFields, encryptFields, valueList);
        status = db.setPersistance(sqlInsertMethod,"PURPOSE",tagList,valueList,encryptFields);

        return (status);
   }
   
   public ESSPurpose duplicate() throws CloneNotSupportedException {
     return (ESSPurpose) this.clone();
   }
   public String getClientCell() {
     String retVal = "[\"headGen\",[";
     retVal += getCell("PURPOSE",PURPOSE);
     retVal += getCell("VOUCHER",VOUCHER,delimit);
     retVal += getCell("CLIENT",CLIENT,delimit);
     retVal += getCell("LOCATION",LOCATION,delimit);
     retVal += getCell("COMMENT",COMMENT,delimit);
     retVal += getCell("PROJECT",PROJECT,delimit);
     retVal += getCell("STEPNO",STEPNO,delimit);
     retVal += getDateCell("BEG_DATE",BEG_DATE,delimit);
     retVal += getDateCell("END_DATE",END_DATE,delimit);
     retVal += getCell("DAYS",DAYS,delimit);
     retVal += getCell("NIGHTS",NIGHTS,delimit);
     retVal += getCell("CLIENTNO",CLIENTNO,delimit);
     retVal += getCell("EXTENDED",EXTENDED,delimit);
     retVal += getCell("TIMESLIP",TIMESLIP,delimit);
     retVal += "]]";
     return retVal;
   }
      
   public String[] getDeNormal() {
     String[] retVal = new String[11];
     int j = 0;
     retVal[j++] = getFlat("PURPOSE",PURPOSE,logicalFields,numericFields,dateFields);    //0
     retVal[j++] = getFlat("CLIENT",CLIENT,logicalFields,numericFields,dateFields);     //1
     retVal[j++] = getFlat("LOCATION",LOCATION,logicalFields,numericFields,dateFields);   //2
     retVal[j++] = getFlat("COMMENT",COMMENT,logicalFields,numericFields,dateFields);    //3
     retVal[j++] = getFlat("PROJECT",PROJECT,logicalFields,numericFields,dateFields);    //4
     retVal[j++] = getFlat("STEPNO",STEPNO,logicalFields,numericFields,dateFields);     //5
     retVal[j++] = getFlat("BEG_DATE",BEG_DATE,logicalFields,numericFields,dateFields);   //6
     retVal[j++] = getFlat("END_DATE",END_DATE,logicalFields,numericFields,dateFields);   //7
     retVal[j++] = getFlat("DAYS",DAYS,logicalFields,numericFields,dateFields);       //8
     retVal[j++] = getFlat("NIGHTS",NIGHTS,logicalFields,numericFields,dateFields);     //9
     retVal[j++] = getFlat("CLIENTNO",CLIENTNO,logicalFields,numericFields,dateFields);   //10
     
     return retVal;
   }
      
      
   public void normalize() {
     java.util.Date begDate = Utilities.stringToDate(BEG_DATE);
     java.util.Date endDate = Utilities.stringToDate(END_DATE);
    
     if (DAYS.equals("")) 
     {
        if ((begDate != null) && (endDate != null))
            DAYS = Integer.toString(Utilities.days(endDate, begDate) + 1);
        else
            DAYS = "1";
        
     }
     if (NIGHTS.equals(""))
     {
        NIGHTS = getNights(endDate, begDate);
     //   if ((begDate != null) && (endDate != null))
     //       NIGHTS = Integer.toString(Utilities.days(endDate, begDate));
     //   else
     //       NIGHTS = "0";
     //   
     }
   } 
  }
  
  public String getNights(String begDateStr, String endDateStr) 
  {
     java.util.Date begDate = Utilities.stringToDate(begDateStr);
     java.util.Date endDate = Utilities.stringToDate(endDateStr);
     return getNights(begDate, endDate);
  }
  
  public String getNights(java.util.Date begDate, java.util.Date endDate) 
  {     
      String nights;
      if ((begDate != null) && (endDate != null))
         nights = Integer.toString(Utilities.days(endDate, begDate));
      else
         nights = "0";
      return nights;      
  }
  
  protected void setSplitClient(Element E, int Link) {
    //used to create a Purpose object.
    SplitClient S = new SplitClient();
    S.set(E, Link);
    SplitClients.addElement(S);
  } 
  protected class SplitClient {
   public String PURPOSE = "";
   public String VOUCHER = "";
   public String CLIENT = "";
   public String LOCATION = "";
   public String CLIENTNO = "";
   public String PROJECT = "";  //JH 2004-03-23
   public String STEPNO = "";  //JH 2004-03-23
   public String billtype = "";
   public String weight = "";
   public String company = "";
   public String originalPurpose = "";
   public int PurposeLink;
   public void set(Element E, int Link) {
     PurposeLink = Link;
     originalPurpose = getESSReferenceString(PurposeLink);
     //PURPOSE = originalPurpose + getReferenceString(genReference++,2);
     PURPOSE = getReferenceString(genReference++,8,"S");
     VOUCHER = Rep.VOUCHER;
     CLIENTNO = getValue(E,"clientno");
     CLIENT = getValue(E,"client");
     PROJECT = getValue(E,"project");  //JH 2004-03-23
     STEPNO = getValue(E,"stepno");  //JH 2004-03-23
     LOCATION = getValue(E,"location");
     billtype = getValue(E,"billtype","2"); //JH 12-23-03
     weight = getValue(E,"weight");
     company = getValue(E,"company");
   } 
   
   public void normalize() {
    //Logfile.println("[000] Report.SplitClient.normalize()");
   } 
  }
  
    
  protected class ESSAttendeeList implements Cloneable {
   public int ListLink = 0;
   public String LOCATION = "";
   public String COMMENT = "";
   public String NUMBER = "";
   public String LINEITEM = "";
   public String persistance = "";   
   
   public void set(Element E, int Link) {  
     ListLink = Link;
     LINEITEM = getESSReferenceString(Link);
     LOCATION = getValue(E, "location");
     COMMENT = getValue(E, "comment");
     NUMBER = getValue(E, "number");
   }
   
   public String toString() {
     return "";
   } 
   public void normalize() {
   } 
   public void normalize(int Link) {
    if (LINEITEM.equals("")) {
        ESSExpense x = getESSExpenseByLink(Link);
        LINEITEM = x.LINEITEM;
    }
   } 
   public ESSAttendeeList duplicate() throws CloneNotSupportedException {
     return (ESSAttendeeList) this.clone();
   }
   public String[] getNumericFields()
   {
    return numericFields;
   }
   public String[] getDateFields()
   {
    return dateFields;
   }
   protected String[] numericFields = {};
   protected String[] dateFields = {};

   
  } 
   
  protected class ESSExtended implements Cloneable,ReportElementAPI {
   public int ListLink;
   public int ExtraLink;
   public String VOUCHER = "";
   public String LINEITEM = "";
   public String NAME = "";
   public String REMARKS = "";
   public String CLIENT = "";
   public String LOCATION = "";
   public String COMMENT = "";
   public String EXTRAITEM = "";
   //public String LOGIC1 = "Yes";
   //public String LOGIC2 = "Yes";
   //public String LOGIC3 = "Yes";
   public String persistance = "";   
   
   public void set(Element E, int Link) {
    ListLink = Link;
    LINEITEM = getESSReferenceString(Link);
    NAME = getValue(E, "name");
    //REMARKS = getValue(E, "remarks");
    CLIENT = getValue(E, "client");
    LOCATION = getValue(E, "location");
    COMMENT = getValue(E, "comment");
    ExtraLink = genReference++;
    EXTRAITEM = getESSReferenceString(ExtraLink);
    
   }
   
   public void setUnScramble()
   {
      NAME = unScramble(NAME);
      //REMARKS = unScramble(REMARKS); 
      CLIENT = unScramble(CLIENT);
      LOCATION = unScramble(LOCATION);
      COMMENT = unScramble(COMMENT);
      //PROJECT = unScramble(PROJECT); //this field not being used right now
   }
   
   public ESSExtended duplicate() throws CloneNotSupportedException {
     return (ESSExtended) this.clone();
   }
   
   public String getPersistance() {
      return persistance;
   }
   public String getIdentifier() {
      return EXTRAITEM;
   }
   
   public void updateNormalize() {
   };
   
   protected String[] keyList = {"VOUCHER", "LINEITEM", "EXTRAITEM"};
   protected String[] tagList = {"VOUCHER", "PURPOSE", "LINEITEM", "NAME", "REMARKS", "CLIENT",
            "LOCATION", "COMMENT", "EXTRAITEM"};
   protected String[] logicalFields = {};
   protected String[] numericFields = {};
   protected String[] dateFields = {};
   protected String[] encryptFields = {"NAME","CLIENT","LOCATION","COMMENT"}; //REMARKS removed 1/13
   protected String[] updateList = {"NAME","REMARKS","CLIENT","LOCATION","COMMENT"};
   
   public void setPersistanceFlag(String x){
      persistance = x;
   }
   
   public boolean setPersistance()
   {
        boolean status = true;
 //removed the taglist from here       
        String[] valueList = {VOUCHER, "", LINEITEM, scramble(str(NAME,36),36), 
            str(REMARKS,20), scramble(str(CLIENT,40),40),
            scramble(str(LOCATION,40),40), scramble(str(COMMENT,80),80), EXTRAITEM};
        
        //status = db.setPersistance(sqlInsertMethod,"EXTENDED", tagList, logicalFields, numericFields, dateFields, encryptFields, valueList);
        status = db.setPersistance(sqlInsertMethod,"EXTENDED",tagList,valueList,encryptFields);
        
        return (status);
   }

   public String getClientAttendeeCell() {
     String retVal = "[\"attendee\",[";
     retVal += getCell("NAME",NAME);
     retVal += getCell("REMARKS",REMARKS,delimit);
     retVal += getCell("CLIENT",CLIENT,delimit);
     retVal += getCell("EXTRAITEM",EXTRAITEM,delimit);
     retVal += "]]";
     return retVal;
   }
   
   public String[] getKeyList()
   {
    return keyList;
   }
   public String[] getTagList()
   {
    return tagList;
   }
   public String[] getUpdateList()
   {
    return updateList;
   }
   public String[] getLogicalFields()
   {
    return logicalFields;
   }
   public String[] getNumericFields()
   {
    return numericFields;
   }
   public String[] getDateFields()
   {
    return dateFields;
   }
   public String[] getEncryptFields()
   {
    return encryptFields;
   }
   
   public void normalize(int Link) {
    if (LINEITEM.equals(""))
    {
        ESSExpense x = getESSExpenseByLink(Link);
        LINEITEM = x.LINEITEM;
    }
    normalize();
   }
   
   public void normalize() {
    VOUCHER = Rep.VOUCHER;
    ESSAttendeeList X = getESSAttendeeList(LINEITEM);
    if (LOCATION.equals("")) LOCATION = X.LOCATION;
    if (COMMENT.equals("")) COMMENT = X.COMMENT;
    String workingNumber;
    if (EXTRAITEM.equals("")) {
       do {
          ExtraLink = genReference++;
          workingNumber = getESSReferenceString(ExtraLink);
       } while (isExtraItem(workingNumber));     
       EXTRAITEM = workingNumber;
     }
   } 
  }
  
  protected boolean Check4Delete (ReportElementAPI X) {
    return !X.getPersistance().equalsIgnoreCase("delete");
  }
  protected String unScramble(String X) {
     if (encrypt.equalsIgnoreCase("YES")) {
        X = Utilities.unScramble(X, encryptKey).trim(); 
     }
     return X;
  }
  protected String scramble(String X, int columnLength) {
    if (encrypt.equalsIgnoreCase("YES")) {
      int Y = X.length();
      while ( Y <= columnLength) {
        X = X + " ";
        Y += 1;
      }
      X = encryptHead + Utilities.getBaseString(Utilities.scramble(X, encryptKey),encryptBase,encryptDelimiter) + encryptTail;
    }
    //call encryption utilities here
    return X;
  }
  
  protected String FRoundStr(float x) {
    if (x >= (float) 0) {
      return Float.toString(x + (float) 0.001);
    } else {
      return Float.toString(x - (float) 0.001);
    } 
  }

  protected String DRateStr(double x) {
	 DecimalFormat twoDForm = new DecimalFormat("#.####");
	 Double d = Double.valueOf(twoDForm.format(x)); 
     String S = d.toString() + "00000";
     return S.substring(0,S.indexOf(".") + 5);
  }
  
  protected String DRoundStr(double x) {
    if (x >= (double) 0) {
    	return Double.toString(x + (double) 0.001);
    } else {
    	return Double.toString(x - (double) 0.001);
    } 
  }
  
  protected String FF(String FFN) {
    String S = FFN + "00";
    return S.substring(0,S.indexOf(".")+ 3);
  }
  
  public void setESSGL() {
    ESSExpense E;
    ESSGL P = new ESSGL();
    ESSGLInfo.removeAllElements();
    for (int i = 0; i < ESSExpenses.size(); i++) {
      E = (ESSExpense) ESSExpenses.elementAt(i);
      P.add(E.EXPENSE, E.AMOUNT, E.BILLTYPE);
    }    
    // need to add in the receipt array here eventually if needed.
  }
  
  public void setFullESSGL() {
    ESSExpense E;
    ESSReceipt R;
    ESSGL P = new ESSGL();
    ESSGLInfo.removeAllElements();
    for (int i = 0; i < ESSExpenses.size(); i++) {
      E = (ESSExpense) ESSExpenses.elementAt(i);
      P.add(E.EXPENSE, E.AMOUNT, E.BILLTYPE, E.ChargedDepart, E.project, E.clientno, E.COMPANY);
    }    
    for (int i = 0; i < ESSReceipts.size(); i++) {
      R = (ESSReceipt) ESSReceipts.elementAt(i);
      P.addCr(R.RECEIPT, R.AMOUNT, "", "", "", "");
    }    
  }
  
  
  public class ESSGL {
    public String NAME = "";
    public float SUM = 0;
    public float DEBIT = 0;
    public float CREDIT = 0;
    public float BILLABLE = 0;
    public float PERSONAL = 0;
    public String DEPART = "";
    public String PROJECT = "";
    public String CLIENT = "";
    public String COMPANY = "";
    public String TYPE = "C";  //Normal Type C = credit (expense) D = debit
    public ESSGL (){
    }
    // For depricated calls
    public void add(String Item, String Value, String BillType) {
       if (BillType.equals("1")) { 
         add(Item, Value, true, false, true, false, "", "");
       } else if (BillType.equals("2")) {
         add(Item, Value, true, false, false, false, "", "");
       } else {
         add(Item, Value, true, false, false, true, "", "");
       }
    }
    // For depricated calls
    public void add(String Item, String Value, String BillType, String Depart, String Project) {
       add(Item, Value, BillType, Depart, Project, "", "");
    }
    
    public void add(String Item, String Value, String BillType, String Depart, String Project, String Client, String Company) {
       if (BillType.equals("1")) { 
         add(Item, Value, true, false, true, false, Depart, Project, Client, Company,"D");
       } else if (BillType.equals("2")) {
         add(Item, Value, true, false, false, false, Depart, Project, Client, Company,"D");
       } else {
         add(Item, Value, true, false, false, true, Depart, Project, Client, Company,"D");
       }
    }
    
    public void addCr(String Item, String Value, String Depart, String Project, String Client, String Company) {
       add(Item, Value, false, true, false, false, Depart, Project, Client, Company,"C");
    }
    
    // For depricated calls
    public void add(String Item, String Value, boolean Debit, boolean Credit, boolean Billable, boolean Personal, String Depart, String Project){
         add(Item, Value, Debit, Credit, Billable, Personal, Depart, Project, "", "","D");
    }
    public void add(String Item, String Value, boolean Debit, boolean Credit, boolean Billable, boolean Personal, String Depart, String Project, String Client, String Company, String Type){
       boolean found = false;
       ESSGL R;
       for (int i = 0; i < ESSGLInfo.size(); i++) {
         R = (ESSGL) ESSGLInfo.elementAt(i);
         if (R.NAME.equals(Item) && R.DEPART.equals(Depart) && R.PROJECT.equals(Project)&& R.CLIENT.equals(Client) && R.COMPANY.equals(Company)  && R.TYPE.equals(Type)) {
           found = true;
           i = ESSGLInfo.size();
           R.update(Value, Debit, Credit, Billable, Personal, Depart, Project,Client, Company, Type);
         }   
       } 
       if (!found){
         R = new ESSGL();  
         R.NAME = Item;
         R.update(Value, Debit, Credit, Billable, Personal, Depart, Project, Client, Company, Type);
         ESSGLInfo.addElement(R);
       }
    }
    private void update(String Value, boolean Debit, boolean Credit, boolean Billable, boolean Personal, String Depart, String Project, String Client, String Company, String Type){
      SUM += Double.parseDouble(Value);
      if (Debit) DEBIT += Double.parseDouble(Value);
      if (Credit) CREDIT += Double.parseDouble(Value);
      if (Billable) BILLABLE += Double.parseDouble(Value);
      if (Personal) PERSONAL += Double.parseDouble(Value);
      DEPART = Depart;
      PROJECT = Project;
      CLIENT = Client;
      COMPANY = Company;
      TYPE = Type;
    }
  }  
  static class CompareXMLReceiptsByDate implements Compare{
    public boolean lessThan(Object left, Object right) {
            return (((ESSReceipt) left).compareTo((ESSReceipt) right) < 0);
    }
    public boolean lessThanOrEqual(Object left, Object right) {
            return (((ESSReceipt) left).compareTo((ESSReceipt) right) <= 0);
    }
        
  }
  
  static class CompareXMLExpensesByDate implements Compare{
    public boolean lessThan(Object left, Object right) {
            return (((ESSExpense) left).compareTo((ESSExpense) right) < 0);
    }
    public boolean lessThanOrEqual(Object left, Object right) {
            return (((ESSExpense) left).compareTo((ESSExpense) right) <= 0);
    }
        
  }
  class CompareXMLExpensesDeptProj implements Compare{
    public boolean lessThan(Object left, Object right) {
            return (((ESSExpense) left).compareDeptProj((ESSExpense) right) < 0);
    }
    public boolean lessThanOrEqual(Object left, Object right) {
            return (((ESSExpense) left).compareDeptProj((ESSExpense) right) <= 0);
    }      
  }   
  
  public int getPhysicalReceipts() {  //this is dup of printReceipts - need to fix later
    String throwAway;
    ESSReceipt ESSR;
    RequiredReceipts = 0;
    CashRet = 0;
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
       throwAway = receiptReq(ESSR);
       // throwAway = printReceipt(ESSR, false); - substituted out by above
       // CashReturned(ESSR.RCPTTYPE, ESSR.AMOUNT);
    }
    return RequiredReceipts;
  }
  
  public int getNumberOfExpenses(String purposeNumber) {  
    ESSExpense ESSX;
    int PurposeExpenses = 0;
    for (int i = 0; i < ESSExpenses.size(); i++) {
       ESSX = (ESSExpense) ESSExpenses.elementAt(i);
       if (purposeNumber.equals(ESSX.PURPOSE)) PurposeExpenses += 1;
    }
    return PurposeExpenses;
  }
  
  public String[] getExpenseRefs4Receipt(String receiptNumber) {  
    ArrayList ExpList = new ArrayList();
    ESSExpense ESSX;
    String[] a = new String[0];
    int ReceiptExpenses = 0;
    for (int i = 0; i < ESSExpenses.size(); i++) {
       ESSX = (ESSExpense) ESSExpenses.elementAt(i);
       if (receiptNumber.equals(ESSX.RECEIPT))
       {
           ReceiptExpenses += 1;
           ExpList.ensureCapacity(ReceiptExpenses);
           ExpList.add(ESSX.LINEITEM);
       }
    }
    ExpList.trimToSize();
    return (String[]) ExpList.toArray(a);
  }
  
  public String[] getExtendedRefs4Expense(String lineitem) {  
    ArrayList ExList = new ArrayList();
    ESSExtended ESSX;
    String[] a = new String[0];
    int ExtendedItems = 0;
    for (int i = 0; i < ESSExtendeds.size(); i++) {
       ESSX = (ESSExtended) ESSExtendeds.elementAt(i);
       if (lineitem.equals(ESSX.LINEITEM))
       {
           ExtendedItems += 1;
           ExList.ensureCapacity(ExtendedItems);
           ExList.add(ESSX.EXTRAITEM);
       }
    }
    ExList.trimToSize();
    return (String[]) ExList.toArray(a);
  }
  
  protected String receiptReq(ESSReceipt R) {  
    ESSExpense E;
    boolean MaxLevel = false;
    String ExpenseLimitStr;
    String CheckedAsReceived = "";
    if (R.CHECK.equalsIgnoreCase("Yes")) CheckedAsReceived = "[x]";
    
    for (int i = 0 ; i < ESSExpenses.size(); i++) {
      E = (ESSExpense) ESSExpenses.elementAt(i);
      if (E.RECEIPT.equals(R.RECEIPT)) {
         ExpenseLimitStr =  ExpenseTable.getDOMTableValueWhere("expense", E.EXPENSE, ReceiptRequired); 
         if (ExpenseLimitStr.length() == 0) ExpenseLimitStr = "0.0";
         if (Float.parseFloat(R.AMOUNT) >= Float.parseFloat(ExpenseLimitStr)) MaxLevel = true;
      }
    }    
    if (MaxLevel && !(R.UNITS.toUpperCase().equals(Miles))) {
      RequiredReceipts += 1;  
      return "Required" + CheckedAsReceived;
    } else {
      return " " + CheckedAsReceived;  
    }
  } 
  
  protected int RequiredReceipts = 0;      //Receipts that need to be submitted
  protected double CashRet = 0;             //Cash returned with the report
  
  protected void CashReturned(String Rcpt, String Amount) {
    if (Rcpt.equals("12")) {
        double Amt = Double.parseDouble(Amount);
        CashRet += Amt;
    }
  }
  
  public void setScanLinks()
  {
	  scanLinks = -1;
	  ReceiptScanLinks.clear();
	  ReceiptScanComments.clear();
  }
  
  public int setReceiptScanLinks()
  {
	  int retVal = scanLinks;
	  if (scanLinks == -1)
	  {
		  scanLinks = 0;
		  String SQLCommand = db.SQLReplace(scanSQL, "$pvoucher$", Rep.PVOUCHER);
		  SQLCommand = db.SQLReplace(SQLCommand, "$persnum$", Rep.PERS_NUM);
		  if (db.setResultSet(SQLCommand))
		  {
			  try
			  {
				  do 
				  {
					  ReceiptScanLinks.addElement(db.myResult.getString(1));
					  ReceiptScanComments.addElement(db.myResult.getString(3));
					  scanLinks += 1;
				  } while (db.myResult.next());
			  } catch (java.lang.Exception ex) 
			  {
				  AuditTrail.println("[500] Report.java - setReceiptScanLinks - Error");
				  AuditTrail.println("[500] Report.java - " + ex.toString());
				  ex.printStackTrace();
			  } //try
		  }

	  }
	  return retVal;
  }
  
  public String[] getReceiptScanLinks()
  {
	  String[] retVal = new String[ReceiptScanLinks.size()];
	  for (int i = 0; i < retVal.length; i++)
	  {
		  retVal[i] = (String) ReceiptScanLinks.get(i);
	  }
	  return retVal;
  }
  
  public String[] getReceiptScanComments()
  {
	  String[] retVal = new String[ReceiptScanComments.size()];
	  for (int i = 0; i < retVal.length; i++)
	  {
		  retVal[i] = (String) ReceiptScanComments.get(i);
	  }
	  return retVal;
  }

  
}  // End of class
