
/*
ReportDisplayAudit.java - XML to Central Database and Report class, Generic Report Object 
Copyright (C) 2005 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

import java.util.*; 
import java.sql.*;
import java.lang.Math;
import org.jdom.Document;
import org.jdom.Element;

public class ReportDisplayAudit extends Report {
    
  final String tr = "<tr>\n";
  final String trEtc = "<tr id=extra>\n";
  final String tre = "</tr>\n";
  final String btable = "<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"90%\" class=\"tableBGColor\" style=\"border: 1px solid\">";
  final String stable = "<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"90%\">";
  private ess.Language myLang = new ess.Language();
  
  public ReportDisplayAudit () {
  }
  
  public ReportDisplayAudit (String Url, String User, String Password, String SQLTerminator){
    setDB(Url, User, Password, SQLTerminator);
  }
    
  public void setLanguage(String x){
	  myLang.setLanguage(x);
  }

  public void setLanguage(String x, String y){
	  myLang.setLanguage(x,y);
  }
  
  
  public String printHeader() {
    String retVal = SecHeader(myLang.getString("persInfoSec"));
    retVal += btable;
    retVal += h1row(myLang.getLabel("name"),Rep.NAME, myLang.getLabel(reportReferenceName),Rep.VOUCHER); //Need to look at this for voucher/reference
    retVal += h1row(myLang.getLabel("phone"),Rep.PHONE,myLang.getLabel("persnum"),dispPersNo(Rep.PERS_NUM));
    retVal += h1row(myLang.getLabel("depart"),Rep.DEPART,myLang.getLabel("credate"),date.getUserDateStr(Rep.CRE_DATE, dateFormatStr));
    retVal += h1row(myLang.getLabel(Manager_Report_Name),Rep.LOCATION,myLang.getLabel("pvoucher"),Rep.PVOUCHER);
    if (!Rep.VEHICLE.equals("")) {
      retVal += h1row(myLang.getLabel("vehicle"),Rep.VEHICLE,myLang.getLabel("servdate"),date.getUserDateStr(Rep.SERVDATE, dateFormatStr));
      retVal += h1row(myLang.getLabel("start"),Rep.START,myLang.getLabel("finish"),Rep.FINISH);
      retVal += h1row(myLang.getLabel("business"),Rep.BUSINESS,myLang.getLabel("weekend"),date.getUserDateStr(Rep.WEEKEND, dateFormatStr));
    }    
    retVal += h1row("","",myLang.getLabel("status"),myLang.getDataString(StatusDOM.getDOMTableValueWhere("default","translation",Rep.RP_STAT,"screen"))); 
    if (!getCurrency().equals("")) {
      retVal += h1row(myLang.getLabel("company"),getCompany(),myLang.getLabel("currency"),getCurrency()); 
      if (!getHistory().equals("")) {
          retVal += h1row("","",myLang.getLabel("history"),getHistory()); 
      }
    } else {
      if (!getHistory().equals("")) {
          retVal += h1row(myLang.getLabel("company"),getCompany(),myLang.getLabel("history"),getHistory()); 
      } else {
         retVal += h1row(myLang.getLabel("company"),getCompany(),"",""); 	
      }
    }
    retVal += "</table>\n";
    retVal += "<a href=\"javascript: void newScreen('header','')\">" + myLang.getString("editHeader") + "</a><br>\n";
    return retVal;
  }
  private String dispPersNo(String x) {
  if (x.trim().length() == 11) {
    if (x.substring(3,4).equals("-") && x.substring(6,7).equals("-")) {
      x = "***-**-" + x.substring(7);
    }
  }
  return x;
}  
  private String h1row(String v1, String v2, String v3, String v4) {
    String retVal = tr;
    retVal += headhtd("200",v1);
    retVal += headhtd("200",v2);
    retVal += headhtd("200",v3);
    retVal += headhtd("200",v4);
    retVal += tre;
    return retVal;
  }  
  private String h1rowPlus(String v1, String v2) {
    String retVal = tr;
    retVal += headhtd("100",v1);
    retVal += headhtd("500",v2);
    retVal += tre;
    return retVal;
  }  
  
  String newbackcolor;
  String backcolor= " class=\"offsetColor\""; 
  String hdbackcolor= " class=\"tableBGColor\""; 
  String oldbackcolor = "";

  private void switchColors() {
     newbackcolor = backcolor;
     backcolor = oldbackcolor; 
     oldbackcolor = newbackcolor;
  }
  private void resetColors() {
     backcolor= " class=\"TableData offsetColor\""; 
     oldbackcolor = " class=\"TableData\"";
  }
  private String htd(String w, String v) {
    String retVal = "<td width=\"" + w + "\" align=\"left\"" + backcolor + ">" + v + "</td>\n";
    return retVal;
  } 
  private String htdr(String w, String v) {
    String retVal = "<td width=\"" + w + "\" align=\"right\"" + backcolor + ">" + v + "</td>\n";
    return retVal;
  }
  
  private String headhtd(String w, String v) {
    String retVal = "<td width=\"" + w + "\" align=\"left\"" + hdbackcolor + ">" + v + "</td>\n";
    return retVal;
  } 
  private String headhtdr(String w, String v) {
    String retVal = "<td width=\"" + w + "\" align=\"right\"" + hdbackcolor + ">" + v + "</td>\n";
    return retVal;
  }
  
  private String SecHeader(String Title) {
    return "<h2>" + Title + "</h2>\n";  //was <p>
  }  
  
  public String printCategory(String category) {
    float tripTotal = 0;
    String retVal =  SecHeader(category + " Category");
    ESSPurpose ESSP;
    ESSExpense ESSE;
    ESSExtended ESSX;
    boolean firstAttendee; 
    
    for (int j = 0; j < ESSExpenses.size(); j++) {
      ESSE = (ESSExpense) ESSExpenses.elementAt(j);
      if (ESSE.category.equals(category)) {
        retVal += printExpense(ESSE);
        tripTotal += Float.parseFloat(ESSE.AMOUNT);
        firstAttendee = true; 
        for (int k = 0; k < ESSExtendeds.size(); k++) {
          ESSX = (ESSExtended) ESSExtendeds.elementAt(k);
          if (ESSX.LINEITEM.equals(ESSE.LINEITEM)) {
            if (firstAttendee) {
              retVal += printDiscussion(ESSX.LOCATION, ESSX.COMMENT);
              firstAttendee = false;
            }
            retVal += printAttendee(ESSX);  
          }   
        } 
      }  
    }
    
    retVal += x1row(" ","====== "," "," "," ");
    switchColors();
    retVal += x1row(myLang.getLabel("categoryTotal"),FF(FRoundStr(tripTotal)),"","","");
    retVal += "</table><br>\n";
    
    return retVal;
  }
  
  public String printTrip() {
    float tripTotal = 0;
    String retVal =  myLang.getString("expAndTripSec");
    retVal += "&nbsp;&nbsp;(<a href=\"javascript: void newScreen('purpose','headGen')\">" + myLang.getString("addNewPurp") + "</a>}";
    retVal =  SecHeader(retVal);
    ESSPurpose ESSP;
    ESSExpense ESSE;
    ESSExtended ESSX;
    boolean firstAttendee; 
    for (int i = 0; i < ESSPurposes.size(); i++) {
       ESSP = (ESSPurpose) ESSPurposes.elementAt(i);
       retVal += printPurpose(ESSP) + "<br>\n";
       retVal += stable;
       resetColors();
       if (Show_Company_Name.equalsIgnoreCase("NO")) {
         retVal += x1row(chd(myLang.getString("expdate")),chd(myLang.getString("amount")),chd(myLang.getString("type")),chd(myLang.getString(Billable_Name)),chd(myLang.getString("comment"))) 
         + "<br>\n";
       } else {
         retVal += x1row(chd(myLang.getString("expdate")),chd(myLang.getString("amount")),chd(myLang.getString("type")),chd(myLang.getString(Show_Company_Name)),chd(myLang.getString(Billable_Name)),chd(myLang.getString("comment"))) 
         + "<br>\n";
       }
       tripTotal = 0;
       switchColors();
       for (int j = 0; j < ESSExpenses.size(); j++) {
         ESSE = (ESSExpense) ESSExpenses.elementAt(j);
         if (ESSE.PURPOSE.equals(ESSP.PURPOSE)) {
           retVal += printExpense(ESSE, ESSP);
           tripTotal += Float.parseFloat(ESSE.AMOUNT);
           firstAttendee = true; 
           for (int k = 0; k < ESSExtendeds.size(); k++) {
             ESSX = (ESSExtended) ESSExtendeds.elementAt(k);
             if (ESSX.LINEITEM.equals(ESSE.LINEITEM)) {
               if (firstAttendee) {
                 retVal += printDiscussion(ESSX.LOCATION, ESSX.COMMENT);
                 firstAttendee = false;
               }
               retVal += printAttendee(ESSX);  
             }   
           }
         }  
         switchColors();
       }
       if (Show_Company_Name.equalsIgnoreCase("NO")) {
         retVal += x1row(" ","====== "," "," "," ");
         switchColors();
         retVal += x1row(myLang.getLabel("subtotal"),getUserAmount(FF(FRoundStr(tripTotal))),"","","");
       } else {
         retVal += x1row(" ","====== "," "," "," "," ");
         switchColors();
         retVal += x1row(myLang.getLabel("subtotal"),getUserAmount(FF(FRoundStr(tripTotal))),"","","","");
       }
       retVal += "</table><br>\n";
    }
    return retVal;
  }
  
  private String printDiscussion(String v1, String v2) {
    String retVal = tr;
    retVal += htd("10"," "); 
    retVal += htd("10",itl("location"));  //expense date
    retVal += htd("100",itl(v1));  //amount 
    retVal += htd("100",itl("discussion"));  //expense type
    retVal += htd("150",itl(v2));  //billable
    if (!Show_Company_Name.equalsIgnoreCase("NO")) {
        retVal += htd("10"," "); 
    }
    retVal += tre;
    return retVal;
  }
  
  public String printTripByDay() {
    float tripTotal = 0;
    float dayTotal = 0;
    int numberOfDays = 0;
    String currentDay;
    String retVal =  myLang.getString("expAndTripSec");
    retVal += "&nbsp;&nbsp;(<a href=\"javascript: void newScreen('purpose','headGen')\">" + myLang.getString("addNewPurp") + "</a>)";
    retVal =  SecHeader(retVal);
    
    ESSPurpose ESSP;
    ESSExpense ESSE;
    ESSExtended ESSX;
    boolean firstAttendee; 
    for (int i = 0; i < ESSPurposes.size(); i++) {
       ESSP = (ESSPurpose) ESSPurposes.elementAt(i);
       retVal += printPurpose(ESSP) + "<br>\n";
       retVal += stable;
       resetColors();
       if (Show_Company_Name.equalsIgnoreCase("NO")) {
         retVal += x1row(chd(myLang.getString("expdate")),chd(myLang.getString("amount")),chd(myLang.getString("type")),chd(myLang.getString(Billable_Name)),chd(myLang.getString("comment")))
         + "<br>\n";
       } else {
         retVal += x1row(chd(myLang.getString("expdate")),chd(myLang.getString("amount")),chd(myLang.getString("type")),chd(myLang.getString(Show_Company_Name)),chd(myLang.getString(Billable_Name)),chd(myLang.getString("comment")))
         + "<br>\n";
       }
       tripTotal = 0;
       dayTotal = 0;  //This next 3 items are used to print day totals
       numberOfDays = 0;
       currentDay = "";
       switchColors();
       for (int j = 0; j < ESSExpenses.size(); j++) {        
         ESSE = (ESSExpense) ESSExpenses.elementAt(j);
         if (ESSE.PURPOSE.equals(ESSP.PURPOSE)) {
           if (currentDay.equals("")) currentDay = ESSE.EXP_DATE;
           if (!currentDay.equals(ESSE.EXP_DATE)) {
             if (Show_Company_Name.equalsIgnoreCase("NO")) {
               retVal += x1row("Day Total",getUserAmount(FF(FRoundStr(dayTotal))),"***","","");
             }else{
               retVal += x1row("Day Total",getUserAmount(FF(FRoundStr(dayTotal))),"***","","","");
             }
             switchColors();
             numberOfDays++;
             dayTotal = 0;
             currentDay = ESSE.EXP_DATE;
           }
           retVal += printExpense(ESSE, ESSP);
           tripTotal += Float.parseFloat(ESSE.AMOUNT);
           dayTotal += Float.parseFloat(ESSE.AMOUNT);
           firstAttendee = true; 
           for (int k = 0; k < ESSExtendeds.size(); k++) {
             ESSX = (ESSExtended) ESSExtendeds.elementAt(k);
             if (ESSX.LINEITEM.equals(ESSE.LINEITEM)) {
               if (firstAttendee) {
                 retVal += printDiscussion(ESSX.LOCATION, ESSX.COMMENT);
                 firstAttendee = false;
               }
               retVal += printAttendee(ESSX);  
             }   
           } 
         }  
         switchColors();
       } //Expense loop
       if (numberOfDays > 0) {
          if (Show_Company_Name.equalsIgnoreCase("NO")) {
            retVal += x1row(myLang.getString("dayTotal"),getUserAmount(FF(FRoundStr(dayTotal))),"***","","");
          } else {
            retVal += x1row(myLang.getString("dayTotal"),getUserAmount(FF(FRoundStr(dayTotal))),"***","","","");
          }
          switchColors();
       }
       if (Show_Company_Name.equalsIgnoreCase("NO")) {
         retVal += x1row(" ","====== "," "," "," ");
         retVal += x1row(myLang.getLabel("subtotal"),getUserAmount(FF(FRoundStr(tripTotal))),"","","");
       } else {
         retVal += x1row(" ","====== "," "," "," "," ");
         retVal += x1row(myLang.getLabel("subtotal"),getUserAmount(FF(FRoundStr(tripTotal))),"","","","");
       }
       retVal += "</table><br>\n";
    }
    return retVal;
  }
  
  
  private String printPurpose(ESSPurpose ESSP) {
    String retVal = btable;
    retVal += h1row(myLang.getLabel("client"),ESSP.CLIENT,myLang.getLabel("begdate"),date.getUserDateStr(ESSP.BEG_DATE,dateFormatStr)); 
    retVal += h1row(myLang.getLabel("location"),ESSP.LOCATION,myLang.getLabel("enddate"),date.getUserDateStr(ESSP.END_DATE, dateFormatStr));
    retVal += h1row(myLang.getLabel(Project_Report_Name),ESSP.PROJECT,myLang.getLabel("clientno"),ESSP.CLIENTNO);
    if (!ESSP.weight.equals("") || !ESSP.company.equals("")) {
      if (ESSP.weight.equals("")) {
        retVal += h1row("",ESSP.weight,myLang.getLabel("company"),getCompanyName(ESSP.company));
      } else if (ESSP.company.equals("")){  
        retVal += h1row(myLang.getLabel("split"),ESSP.weight,"","");  //Parameterize this
      } else {  
        retVal += h1row(myLang.getLabel("split"),ESSP.weight,myLang.getLabel("company"),getCompanyName(ESSP.company));
      }
    }
    retVal += h1rowPlus(myLang.getLabel("purpose"),formatNoRets(ESSP.COMMENT)); 
    if (!StepNoDisplay.equals("")) {                                //JH 3-17-2004
       retVal += h1rowPlus(StepNoDisplay,ESSP.STEPNO); 
    }   
    retVal += "</table>\n";
    retVal += "<a href=\"javascript: void editPurpose('" + ESSP.PURPOSE + "')\">" + myLang.getString("editPurp") + "</a>";
    if (getNumberOfExpenses(ESSP.PURPOSE) == 0)
    {   
        retVal += "&nbsp;&nbsp;&nbsp;<a href=\"javascript: void deletePurpose('" + ESSP.PURPOSE + "')\">" + myLang.getString("delPurp") + "</a>";
    }
    retVal += "<br>\n";
        
    return retVal;
  }
  
  private String getCompanyName(String company) 
  {
    String retVal = companyTable.getDOMTableValueWhere("company", company, "descrip");
    if (retVal == null || retVal.equals("")) retVal = company;
    return retVal; 
  }
  
  
  private String formatNoRets(String x) {
    return replaceStr(x,"\\n"," ");
  }
  
  private String replaceStr(String x, String oldVal, String newVal) {
    int y = x.indexOf(oldVal);
    while (y > -1) {
        x = x.substring(0,y-1) + newVal + x.substring(y + oldVal.length());
        y = x.indexOf(oldVal);
    }    
    return x;
  }
  private String chs(String X, String tabl, int col) {
    //special formatting for column headers
    return "<a href=\"javascript: void sortTable(" + Integer.toString(col) + ",\'" + tabl + "\')\">" + X + "</a>";
  }
    
  private String chd(String X) {
    //special formatting for column headers
    return "<u>" + X + "</u>";
  }
  private String itl(String X) {
    return "<i>" + X + "</i>";
  }
  
  private String x1row(String v1, String v2, String v3, String v4, String v5) {
    String retVal = tr;
    retVal += htd("100",v1);  //expense date
    retVal += htd("100",v2);  //amount 
    retVal += htd("120",v3);  //expense type
    retVal += htd("50",v4);  //billable
    retVal += htd("230",v5);    //comment
    retVal += tre;
    return retVal;
  }
  
  private String x1row(String v1, String v2, String v3, String v4, String v5, String v6) {
    String retVal = tr;
    retVal += htd("100",v1);  //expense date
    retVal += htd("100",v2);  //amount 
    retVal += htd("120",v3);  //expense type
    retVal += htd("50",v4);   //company
    retVal += htd("50",v5);   //billable
    retVal += htd("230",v6);  //comment
    retVal += tre;
    return retVal;
  }
  
  private String printExpense(ESSExpense ESSE, ESSPurpose ESSP) {
    String x;
    ESSReceipt ESSR = getESSReceipt(ESSE.RECEIPT);
    String rcptEditString = "<a href=\"javascript: void editReceipt('" + ESSR.RECEIPT +  "')\"><br>" + myLang.getString("edit") + "</a>&nbsp;&nbsp;"
         + "<a href=\"javascript: void deleteReceipt('" + ESSR.RECEIPT +  "')\">" + myLang.getString("del") + "</a>" ;
    String sComment = ESSE.COMMENT; 
    if (AddMerchant2Comment.equalsIgnoreCase("YES"))
    {
        if (ESSR.MERCHANT != null && !ESSR.MERCHANT.equals(""))
        {
           sComment = sComment + " (" + ESSR.MERCHANT + ")";
        }
    }
    if (ESSR.REFER != null && !ESSR.REFER.equals(""))
    {
       sComment = sComment + " Ref #: " + ESSR.REFER;
    }
    sComment = sComment + rcptEditString;
    String nights;
    if (Show_Company_Name.equalsIgnoreCase("NO")) {
      x = x1row(date.getUserDateStr(ESSE.EXP_DATE,dateFormatStr), getUserAmount(ESSE.AMOUNT), myLang.getDataString(ESSE.EXPENSE), billConvert(ESSE.BILLTYPE),sComment);
      if (ESSE.EXPENSE.equals(lodgingExpense)) {
        if (ESSR.BEG_DATE.equals("")) {
           nights = ESSP.NIGHTS;
        } else {
           nights = getNights(ESSR.BEG_DATE, ESSR.REC_DATE);
        }
        x += x1row("", itl(myLang.getLabel("dailyrate")) , getUserAmount(itl(divide4Currency(ESSE.AMOUNT,nights))) , "", "");
      }
    } else {    
      x = x1row(date.getUserDateStr(ESSE.EXP_DATE,dateFormatStr), getUserAmount(ESSE.AMOUNT), myLang.getDataString(ESSE.EXPENSE), ESSE.COMPANY, billConvert(ESSE.BILLTYPE),sComment);
      if (ESSE.EXPENSE.equals(lodgingExpense)) {
        if (ESSR.BEG_DATE.equals("")) {
           nights = ESSP.NIGHTS;
        } else {
           nights = getNights(ESSR.BEG_DATE, ESSR.REC_DATE);
        }
        if (Double.valueOf(nights).doubleValue() == 0) nights = "1";
        x += x1row("", itl(myLang.getLabel("dailyrate")), getUserAmount(itl(divide4Currency(ESSE.AMOUNT,nights))), "", "", "");
      }
    } 
    if (!ESSR.OFFSET.equals("")) {
        x += x1row("", itl(myLang.getLabel("from")), myLang.getDataString(ESSR.LOCATION) , itl(myLang.getLabel("to")), myLang.getDataString(ESSR.OFFSET), "");
    } else {
       if (!ESSR.LOCATION.equals(ESSP.LOCATION)) {
          x += x1row("", itl(myLang.getLabel("location")), ESSR.LOCATION , "", "", "");
       }
    }
    return x;
  } 
  
  private String divide4Currency(String dividend, String divisor) {
        String retVal = "undefined";
        double numerator = Double.valueOf(dividend).doubleValue();
        double denominator = Double.valueOf(divisor).doubleValue();
        if (denominator != 0) 
        {
           double answer = numerator/denominator;
           retVal = FF(DRoundStr(answer));
        }
        return retVal;
  }
    
  private String printExpense(ESSExpense ESSE) {
    String x;
    if (Show_Company_Name.equalsIgnoreCase("NO")) {
      x = x1row(date.getUserDateStr(ESSE.EXP_DATE,dateFormatStr), getUserAmount(ESSE.AMOUNT), myLang.getDataString(ESSE.EXPENSE), billConvert(ESSE.BILLTYPE),ESSE.COMMENT);
    } else {    
      x = x1row(date.getUserDateStr(ESSE.EXP_DATE,dateFormatStr), getUserAmount(ESSE.AMOUNT), myLang.getDataString(ESSE.EXPENSE), ESSE.COMPANY, billConvert(ESSE.BILLTYPE),ESSE.COMMENT);
    } 
    return x;
  } 
  
  private String billConvert(String BillType) {
    if (BillType.equals("1")) {
        return myLang.getString("yes");
    } else {
        return " ";
    }
  }    
  
  private String x2row(String v1, String v2, String v3) {
    String retVal = tr;
    retVal += htd("10"," "); 
    retVal += htd("10",itl(myLang.getLabel("attendee")));  //expense date
    retVal += htd("100",itl(v1));  //amount 
    retVal += htd("100",itl(v2));  //expense type
    retVal += htd("150",itl(v3));  //billable
    if (!Show_Company_Name.equalsIgnoreCase("NO")) {
        retVal += htd("10"," "); 
    }
    retVal += tre;
    return retVal;
  }
  
  private String printAttendee(ESSExtended ESSX) {
    String retVal;
    if (Check4Delete( (ReportElementAPI) ESSX ) ) {
        retVal = x2row(ESSX.NAME, ESSX.REMARKS, ESSX.CLIENT);
    } else {
        retVal = "";
    }
    return retVal;
  }
  
  
  private String r1row(String v1, String v2, String v3, String v4, String v5, String v6) {
    String retVal = tr;
    retVal += htd("80",v1);  //reference
    retVal += htd("80",v2);  //receipt date
    retVal += htd("80",v3);  //amount
    retVal += htd("100",v4);  //charge type
    retVal += htd("60",v5);    //Reimburseable
    retVal += htd("100",v6);    //Receipt required
    retVal += tre;
    return retVal;
  }
  
  private String r1row(String v1, String v2, String v3, String v4, String v5, String v6, String v7) {
    String retVal = tr;
    retVal += htd("30",v7);
    retVal += htd("80",v1);  //reference
    retVal += htd("80",v2);  //receipt date
    retVal += htd("80",v3);  //amount
    retVal += htd("100",v4);  //charge type
    retVal += htd("60",v5);    //Reimburseable
    retVal += htd("100",v6);    //Receipt required
    retVal += tre;
    return retVal;
  }
  
  private String r1rowEtc(String v1, String v2, String v3, String v4, String v5, String v6) {
    String retVal = trEtc;
    retVal += htd("30","");
    retVal += htd("80",v1);  //reference
    retVal += htd("80",date.getUserDateStr(v2, dateFormatStr));  //receipt date
    retVal += htd("80",v3);  //amount
    retVal += htd("100",v4);  //charge type
    retVal += htd("60",v5);    //Reimburseable
    retVal += htd("100",v6);    //Receipt required
    retVal += tre;
    return retVal;
  }
  
  private String printReceipt(ESSReceipt R, boolean checkBox) {
    String retVal = ""; 
    String v7;  //Checkbox logic
    ESSExpense ESSX;
    String ExpenseType;  //Replaces R.RECEIPT below
    
    String required = receiptReq(R);
    ExpenseType = Utilities.getTrim(getESSExpense(R.RECEIPT).EXPENSE);
          
    // Don't use "if (checkBox)" anymore

    if (R.CHECK.equalsIgnoreCase("YES"))
    {
       v7 = "<input type='checkbox' name='select_this_receipt' reference='" + R.RECEIPT + "' paper='" + required + "' checked changed='No' onClick='JavaScript: void hasChanged(this)'>" 
          + "<a href=\"javascript: void editReceipt('" + R.RECEIPT +  "')\"><br>" + myLang.getString("edit") + "</a>&nbsp;&nbsp;&nbsp;"
          + "&nbsp;&nbsp;<a href=\"javascript: void deleteReceipt('" + R.RECEIPT +  "')\">" + myLang.getString("del") + "</a>" ;
    } else {
       v7 = "<input type='checkbox' name='select_this_receipt' reference='" + R.RECEIPT + "' paper='" + required + "' changed='No' onClick='JavaScript: void hasChanged(this)'>"
          + "<a href=\"javascript: void editReceipt('" + R.RECEIPT +  "')\"><br>" + myLang.getString("edit") + "</a>&nbsp;&nbsp;&nbsp;"
          + "&nbsp;&nbsp;<a href=\"javascript: void deleteReceipt('" + R.RECEIPT +  "')\">" + myLang.getString("del") + "</a>" ;
    }
    retVal = r1row(date.getUserDateStr(R.REC_DATE,dateFormatStr),getUserAmount(R.AMOUNT),myLang.getDataString(ExpenseType),myLang.getDataString(R.CHARGE),myLang.getDataString(R.REIMB),required,v7);
    
    if (!R.MERCHANT.equals("")) {
        
       retVal += r1rowEtc("","","","",itl(myLang.getLabel("merchant")),itl(R.MERCHANT));   //took out the last ""
    }
    return retVal;
  }  
  
  public String printReceipts() {
    return printReceipts(false);
  }
  
  public String printReceipts(boolean checkBox) {
    //has the getPhysicalReceipt code embedded in it.
    String cmt;
    //float tripTotal = 0;
    String retVal =  myLang.getString("receiptsSec");
    retVal += "&nbsp;&nbsp;( " + myLang.getString("addANew") + " <a href=\"javascript: void newScreen('receipt','receipt1')\">" + myLang.getString("receipt") + "</a>";
    retVal += "&nbsp;<a href=\"javascript: void newScreen('receipt','receipt4')\">" + myLang.getString("mileage") + "</a>";
    retVal += "&nbsp;<a href=\"javascript: void newScreen('receipt','receipt6')\">" + myLang.getString("advance") + "</a>";
    retVal += "&nbsp;<a href=\"javascript: void newScreen('receipt','receipt12')\">" + myLang.getString("return") + "</a>)";
    retVal =  SecHeader(retVal);

    resetColors();
    
    ESSReceipt ESSR;
    //ESSExpense ESSX;
    //String ExpenseType;
    RequiredReceipts = 0;
    CashRet = 0;
    retVal += "<table id=\"RcptList\" name=\"RcptList\" cols=7 border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"90%\">";
    
    if (checkBox) 
    {
      // First title was "Reference"  
      retVal += r1row(chs(myLang.getString("date"),"RcptList",1),chs(myLang.getString("amount"),"RcptList",2),chs(myLang.getString("expense"),"RcptList",3),chd(myLang.getString("charge")),chd(myLang.getString("reimb")),chd(myLang.getString("required")),chd(myLang.getString("ok")));
    } else {
      retVal += r1row(chs(myLang.getString("date"),"RcptList",0),chs(myLang.getString("amount"),"RcptList",1),chs(myLang.getString("expense"),"RcptList",2),chd(myLang.getString("charge")),chd(myLang.getString("reimb")),chd(myLang.getString("required")));
    }
    switchColors();
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
       if (!ESSR.RCPTTYPE.equals("5") && !ESSR.RCPTTYPE.equals("6") && !ESSR.RCPTTYPE.equals("12")) {
          retVal += printReceipt(ESSR, checkBox);
          if (!(ESSR.AMOUNT.equals(ESSR.REC_AMT)) && !(ESSR.REC_AMT.equals("0.00"))) {  //printout mileage, foreign ex if applicable jh 11/5/2002
             cmt = getCmt(ESSR.RECEIPT ,ESSR.UNITS,ESSR.XSOURCE,date.getUserDateStr(ESSR.XDATE,dateFormatStr));
            if (checkBox) 
            {
               retVal +=  r1rowEtc(" ",itl(getXLabel(ESSR.UNITS)),itl(getUserAmount(ESSR.REC_AMT)),itl(ESSR.UNITS),itl(getUserAmount(ESSR.XRATE)),itl(cmt));
            } else {
               retVal +=  r1row(" ",itl(getXLabel(ESSR.UNITS)),itl(getUserAmount(ESSR.REC_AMT)),itl(ESSR.UNITS),itl(getUserAmount(ESSR.XRATE)),itl(cmt));
            }
          }
       }
       CashReturned(ESSR.RCPTTYPE, ESSR.AMOUNT);
       switchColors();
    }
    retVal += "</table>\n";
    retVal += "<br>" + myLang.getLabel("requiredRec") + Integer.toString(RequiredReceipts);
    return retVal;
  }
  
  public String printReceiptsByExpense(boolean checkBox) {
    //Can later carry in the compare class
    String cmt;
    float tripTotal = 0;
    String retVal =  myLang.getString("receiptsSec");
    // retVal += "&nbsp;&nbsp;(Add&nbsp;a&nbsp;new&nbsp;<a href=\"javascript: void newScreen('receipt','receipt1')\">" + myLang.getString("receipt") + "</a>";
    retVal += "&nbsp;&nbsp;( " + myLang.getString("addANew") + " <a href=\"javascript: void newScreen('receipt','receipt1')\">" + myLang.getString("receipt") + "</a>";
    retVal += "&nbsp;<a href=\"javascript: void newScreen('receipt','receipt4')\">" + myLang.getString("mileage") + "</a>";
    retVal += "&nbsp;<a href=\"javascript: void newScreen('receipt','receipt6')\">" + myLang.getString("advance") + "</a>";
    retVal += "&nbsp;<a href=\"javascript: void newScreen('receipt','receipt12')\">" + myLang.getString("return") + "</a>)";
    retVal =  SecHeader(retVal);

    resetColors();
    
    ESSReceipt ESSR;
    ESSExpense ESSX;
    String ExpenseType;
    ESSReceipt[] RcptArray = new ESSReceipt[ ESSReceipts.size() ];
    RcptArray = (ESSReceipt[]) ESSReceipts.toArray((ESSReceipt[]) RcptArray);
    CompareReceiptsByExpense compareObject = new CompareReceiptsByExpense();
    Arrays.sort(RcptArray, compareObject);
    RequiredReceipts = 0;
    CashRet = 0;
    retVal += "<table id=\"RcptList\" name=\"RcptList\" cols=7 border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"90%\">";
    
    if (checkBox) 
    {
      // First title was "Reference"  
      retVal += r1row(chs(myLang.getString("date"),"RcptList",1),chs(myLang.getString("amount"),"RcptList",2),chs(myLang.getString("expense"),"RcptList",3),chd(myLang.getString("charge")),chd(myLang.getString("reimb")),chd(myLang.getString("required")),chd(myLang.getString("ok")));
    } else {
      retVal += r1row(chs(myLang.getString("date"),"RcptList",0),chs(myLang.getString("amount"),"RcptList",1),chs(myLang.getString("expense"),"RcptList",2),chd(myLang.getString("charge")),chd(myLang.getString("reimb")),chd(myLang.getString("required")));
    }
    switchColors();
    for (int i = 0; i < RcptArray.length; i++) {
       ESSR = (ESSReceipt) RcptArray[i];
       if (!ESSR.RCPTTYPE.equals("5") && !ESSR.RCPTTYPE.equals("6") && !ESSR.RCPTTYPE.equals("12")) {
          retVal += printReceipt(ESSR, checkBox);
          if (!(ESSR.AMOUNT.equals(ESSR.REC_AMT)) && !(ESSR.REC_AMT.equals("0.00"))) {  //printout mileage, foreign ex if applicable jh 11/5/2002
             cmt = getCmt(ESSR.RECEIPT ,ESSR.UNITS,ESSR.XSOURCE,ESSR.XDATE);
            if (checkBox) 
            {
               retVal +=  r1rowEtc(" ",itl(getXLabel(ESSR.UNITS)),itl(getUserAmount(ESSR.REC_AMT)),itl(ESSR.UNITS),itl(getUserAmount(ESSR.XRATE)),itl(cmt));
            } else {
               retVal +=  r1row(" ",itl(getXLabel(ESSR.UNITS)),itl(getUserAmount(ESSR.REC_AMT)),itl(ESSR.UNITS),itl(getUserAmount(ESSR.XRATE)),itl(cmt));
            }
          }
       }
       CashReturned(ESSR.RCPTTYPE, ESSR.AMOUNT);
       switchColors();
    }
    retVal += "</table>\n";
    retVal += "<br>" + myLang.getLabel("requiredRec") + " " + Integer.toString(RequiredReceipts);
    return retVal;
  }
  class CompareReceiptsByExpense implements Comparator {
    public int compare(Object left, Object right) {
            return ((ESSReceipt) left).compare((ESSReceipt) right);
    }
  }
  
  public String printFleetItems() {
    //has the getPhysicalReceipt code embedded in it.
    String cmt;
    String retVal =  SecHeader(myLang.getString("fleetExpSec"));
    ESSReceipt ESSR;
    ESSExpense ESSE;
    int ReceiptFound = 0;
    retVal += stable;
    resetColors();
    retVal += printFleetRow(chd(myLang.getString("date")),chd(myLang.getString("amount")),chd(myLang.getString("quantity")),chd(myLang.getString("expense")),chd(myLang.getString("comment")));
    switchColors();
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
       if (ESSR.RCPTTYPE.equals("5")) { 
         ESSE = getESSExpense(ESSR.RECEIPT);
         retVal += printFleetRcpt(ESSR, ESSE);
         ReceiptFound += 1;
         switchColors();
       }
    }
    retVal += "</table>\n";
    if (ReceiptFound == 0) retVal = "";
    return retVal;
  }
  
  private String printFleetRcpt(ESSReceipt R, ESSExpense E) {
    String rcptEditString = "<a href=\"javascript: void editReceipt('" + R.RECEIPT +  "')\"><br>" + myLang.getString("edit") + "</a>&nbsp;&nbsp;"
         + "<a href=\"javascript: void deleteReceipt('" + E.RECEIPT +  "')\">" + myLang.getString("del") + "</a>" ;
    String sComment = E.COMMENT + rcptEditString;
    
    return printFleetRow(date.getUserDateStr(R.REC_DATE,dateFormatStr),getUserAmount(R.AMOUNT),R.REC_AMT,myLang.getDataString(E.EXPENSE),sComment);
  }  
    
  private String printFleetRow(String v1, String v2, String v3, String v4, String v5) {
    String retVal = tr;
    retVal += htd("60",v1);  //date
    retVal += htd("60",v2);  //amount
    retVal += htd("60",v3);  //quantity
    retVal += htd("100",v4);  //expense type
    retVal += htd("200",v5);    //comment
    retVal += tre;
    return retVal;
  }
  
  public String printAdvanceItems() {
    //has the getPhysicalReceipt code embedded in it.
    String cmt;
    String retVal =  myLang.getString("AdvSec");
    retVal += "&nbsp;&nbsp;(" + myLang.getString("addANew") + " <a href=\"javascript: void newScreen('receipt','receipt6')\">";
    retVal += myLang.getString("advance") + "</a>";
    retVal += "&nbsp;<a href=\"javascript: void newScreen('receipt','receipt8')\">"; 
    retVal += myLang.getString("return") + "</a>)";
    retVal =  SecHeader(retVal);
    
    ESSReceipt ESSR;
    ESSExpense ESSE;
    int ReceiptFound = 0;
    retVal += stable;
    resetColors();
    retVal += printAdvanceRow(chd(myLang.getString("date")),chd(myLang.getString("amount")),chd(myLang.getString("type")),chd(myLang.getString("refer")));
    switchColors();
    for (int i = 0; i < ESSReceipts.size(); i++) {
       ESSR = (ESSReceipt) ESSReceipts.elementAt(i);
       if (ESSR.RCPTTYPE.equals("6") || ESSR.RCPTTYPE.equals("12") ) { 
         retVal += printAdvanceRcpt(ESSR);
         ReceiptFound += 1;
         switchColors();
       }
    }
    retVal += "</table>\n";
    if (ReceiptFound == 0) retVal = "";
    return retVal;
  }
  
  private String printAdvanceRcpt(ESSReceipt R) {
    String rcptEditString = "<a href=\"javascript: void editReceipt('" + R.RECEIPT +  "')\"><br>" + myLang.getString("edit") + "</a>&nbsp;&nbsp;"
         + "<a href=\"javascript: void deleteReceipt('" + R.RECEIPT +  "')\">" + myLang.getString("del") + "</a>";
    String sComment = R.REFER + rcptEditString;
    return printAdvanceRow(date.getUserDateStr(R.REC_DATE,dateFormatStr),getUserAmount(R.AMOUNT),myLang.getDataString(R.CHARGE),sComment);
  }  
    
  private String printAdvanceRow(String v1, String v2, String v3, String v4) {
    String retVal = tr;
    retVal += htd("60",v1);  //date
    retVal += htd("60",v2);  //amount
    retVal += htd("100",v3);  //advance/return
    retVal += htd("60",v4);    //reference
    retVal += tre;
    return retVal;
  }
  
  private String getXLabel(String units) {
    if(units.toUpperCase().equals(Miles)) {
        return myLang.getLabel("mileage");
    } else {
        return myLang.getLabel("exchange");
    }
  }
  
  private String getCmt(String Receipt, String Units, String Xsource, String Xdate) {
    ESSExpense E;
    E = getESSExpense(Receipt);
    if (Units.toUpperCase().equals(Miles)) {
      return Xsource + " " + myLang.getString("to") + " " + E.COMMENT;
    } else {
      return Xsource + " " + date.getUserDateStr(Xdate,dateFormatStr);
    }
  }
  
  public String printTitle() {
    String retVal =  printTitle(myLang.getString("expenseReport")); 
    return retVal;
  }  
  public String printTitle(String xTitle) {
    String retVal =  "<p align=\"center\"><big>Expense Submittal System&#153;<br>" + xTitle + " - " + Rep.SUB_DATE + "</p>\n"; 

    return retVal;
  }
  
  public String printEApproval() {
    return printMoneyDue(); 
  }  
  
  
  final String lign = "___________________";
  
  public String printApproval() {
   String retVal = SecHeader(myLang.getString("approvalSec")) + printMoneyDue() + SignatureBox();
   return retVal;
  }

  public String printCrossChargeWarning() {
   String retVal = "";
   if (PT.setPersNumInfo(Rep.PERS_NUM)) 
   {
     if (!Rep.DEPART.equals(PT.getDepartment()))
     {
    	 retVal += myLang.getString("WARNING_XCHARGE_REPORT") + "<br><br>";	 
    	 retVal = retVal.replace("$persDepart$", PT.getDepartment());
    	 retVal = retVal.replace("$reportDepart$", Rep.DEPART);
    	 
      // retVal += "Warning - User has cross charged this report.";
      // retVal += "<br>Reporter's department is : " + PT.getDepartment();
      // retVal += "<br>Report is charged to: " + Rep.DEPART;
      // retVal += "<br><br>";
     } else {
      ESSExpense CheckExpense = new ESSExpense();
      for (int i = 0; i < ESSExpenses.size(); i++) {
        CheckExpense = (ESSExpense) ESSExpenses.elementAt(i);
        if (!CheckExpense.ChargedDepart.equals(PT.getDepartment()))
        {
            retVal += myLang.getString("WARNING_XCHARGE_EXPENSES") + "<br><br>";
//            retVal += "Warning - Report contains cross charged expenses.<br><br>";
            i = ESSExpenses.size();
        }
      }
  
     }
   } else {
	      retVal += "<br>" + myLang.getString("WARNING_XCHARGE_VERIFICATION") + "<br><br>";
//     retVal += "<br>Warning - cannot verify the reporter's department.<br><br>";
   }
    
   return retVal;
  }
  
  public String printMoneyDue() {
   String retVal = "";
   //work on this later ************************************************** there a rounding/trunc problem

   
   double PlugAmount = Double.valueOf("99.99").doubleValue();
   double Advances = Double.valueOf(Rep.AD_AMT).doubleValue();
   double Reimbursement = Double.valueOf(Rep.RE_AMT).doubleValue();
   double Receipts = Double.valueOf(Rep.RC_AMT).doubleValue();
   //int IPay = Math.round((Reimbursement - Advances)*100);
   double Payment = Reimbursement;   // Advances CashRet is not currently being used here
   if (Payment >= (double) 0) {
      Payment = Payment + (double) 0.001;  //add a mille for rounding because of truncation
   } else {
      Payment = Payment - (double) 0.001;
   }
   retVal += "<table>\n";
   if (!CentralCurrency.equalsIgnoreCase(getCurrency()))
   {
	   retVal += "<tr class=\"TableData\"><td><u>" + myLang.getString("currencyConverter") + "</u></td><td><u>" + getCurrency() + "</u></td><td><u>" + CentralCurrency + "</u></td></tr>\n";
   }
   retVal += moneyDueLine(myLang.getString("totalReport"),Receipts + Advances + CashRet, "offsetColor");
   retVal += moneyDueLine(myLang.getString("totalExpenses"),Receipts,"");
   
   retVal += moneyDueLine(myLang.getString("reimbursables"),(Reimbursement + Advances) - CashRet, "offsetColor");    //jh 2009-07-08
   
   retVal += moneyDueLine(myLang.getString("prepaidPersonal"),Receipts - ((Reimbursement + Advances) - CashRet),"");        //jh 2009-07-08
   
   
   retVal += moneyDueLine(myLang.getString("advances"),Advances, "offsetColor");
   retVal += moneyDueLine(myLang.getString("returnedFunds"),CashRet,"");
   retVal += moneyDueLine(myLang.getString("amountDue"),Payment, "offsetColor");
   retVal += "</table>";
   return retVal;
  }
  private String moneyDueLine(String title, double amt, String extraAttr)
  {
	  String retVal = "";
	  String centralAmt = "";
	  String classY = "class=\"TableData";
	  if (!extraAttr.equals(""))
	  {
		  classY += " ";
		  classY += extraAttr;
	  }
	  classY += "\"";
	  
	  if (!CentralCurrency.equalsIgnoreCase(getCurrency()))
	  {
		  if (!this.getCurrency().equalsIgnoreCase(CurrencyObj.getCurrency())) CurrencyObj.setCurrency(this.getCurrency());  
		  centralAmt = FF(DRoundStr(CurrencyObj.getConvertedAmount(amt)));;
	  }
	  retVal += "<tr " + classY +"><td>" + title + ":</td><td align=\"right\">" + getUserAmount(FF(DRoundStr(amt))) + "</td><td align=\"right\">" + getUserAmount(centralAmt) + "</td></tr>\n";
	  return retVal; 
  }
  
  
  public String SignatureBox() {
   String retVal = "<small>"; 
   if (CashRet > (float) 0) {
   retVal += "A check for $" + DRoundStr(CashRet) + " is included with this report.<br>";
   }
   retVal += "Attach required receipts to this copy and forward to A/P.</small><br>";
   retVal += btable;
   retVal += h1row("Employee Signature:",lign,"Approver Signature:",lign);
   retVal += h1row("Employee Name:",lign,"Approver Name:",lign);
   retVal += h1row("Date:",lign,"Date:",lign);
   retVal += "\n</table>";
   return retVal;
  } 
  private String l1row(String v1, String v2, String v3, String v4) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("100",v2);  //amount 
    retVal += htd("100",v3);  //billable
    retVal += htd("150",v4);  //personal
    retVal += tre;
    return retVal;
  }
  private String l2row(String v1, float v2, float v3, float v4) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("100",getUserAmount(FF(FRoundStr(v2))));  //amount 
    retVal += htd("100",getUserAmount(FF(FRoundStr(v3))));  //billable
    retVal += htd("150",getUserAmount(FF(FRoundStr(v4))));  //personal
    retVal += tre;
    return retVal;
  }
  private String l3row(String v1, float v2) {
    String retVal = tr;
    retVal += htd("150",v1);  //expense
    retVal += htd("100",getUserAmount(FF(FRoundStr(v2))));  //amount 
    retVal += htd("100","");  //billable
    retVal += htd("150","");  //personal
    retVal += tre;
    return retVal;
  }
  //private String l4row(String v1, String v2, String v3, float v4) {
  //  String retVal = tr;
  //  retVal += htd("150",v1);  
  //  retVal += htd("100",v2);  
  //  retVal += htd("100",v3);  
  //  retVal += htd("150",getUserAmount(FF(FRoundStr(v4))));  
  //  retVal += tre;
  //  return retVal;
  //}
  public String printExpListItem(ESSGL R) {
    return l2row(R.NAME, R.SUM, R.BILLABLE, R.PERSONAL); 
  }  
  public String printExpenseSummary() {
    ESSGL R;
    float expTotal = 0;
    float expBillable = 0;
    float expPersonal = 0;
    String retVal = SecHeader(myLang.getString("expSumSec"));
    retVal += stable;
    resetColors();
    retVal += l1row(chd(myLang.getString("expense")),chd(myLang.getString("amount")),chd(myLang.getString(Billable_Name)),chd(myLang.getString("personal")));
    switchColors();
    for (int i = 0; i < ESSGLInfo.size(); i++) {
      R = (ESSGL) ESSGLInfo.elementAt(i);
      retVal += printExpListItem(R);
      switchColors();
      expTotal += R.SUM;
      expBillable += R.BILLABLE;
      expPersonal += R.PERSONAL;  
    }
    retVal += l2row(myLang.getLabel("totals"),expTotal,expBillable,expPersonal);
    switchColors();
    retVal += l3row(myLang.getLabel("nonpers"),expTotal - expPersonal);
    retVal += "</table>\n";
    return retVal;
  }
  public String printDepartListItem(ESSGL R) {
      return d4row(R.DEPART, R.PROJECT, R.NAME, R.SUM);
  }
  private String d1row(String v1, String v2, String v3, String v4) {
    String retVal = tr;
    retVal += htd("50",v1);  
    retVal += htd("100",v2);  
    retVal += htd("100",v3); 
    retVal += htd("150",v4); 
    retVal += tre;
    return retVal;
  }
  private String d2row(String v1, float v4) {
    String retVal = tr;
    retVal += htd("50",v1);
    retVal += htd("100",""); 
    retVal += htd("100","");
    retVal += htd("150",getUserAmount(FF(FRoundStr(v4))));
    retVal += tre;
    return retVal;
  }
  private String d4row(String v1, String v2, String v3, float v4) {
    String retVal = tr;
    retVal += htd("50",v1);  
    retVal += htd("100",v2);  
    retVal += htd("100",v3);  
    retVal += htd("150",getUserAmount(FF(FRoundStr(v4))));  
    retVal += tre;
    return retVal;
  }
  public String printDepartSummary() {
    String RefType = ST.getSystemString("REP_REFERENCE", "Project");
    ESSGL R;
    float expTotal = 0;
    //float expBillable = 0;
    //float expPersonal = 0;
    String retVal = SecHeader(myLang.getString("expByDepart" + RefType));
    ESSExpenses.reLoad(new CompareXMLExpensesDeptProj());
    ESSExpenses.sort();
    setESSGLwDept(RefType);
    retVal += stable;
    resetColors();
    retVal += d1row(chd(myLang.getString("depart")),chd(myLang.getString(RefType)),chd(myLang.getString("expense")),chd(myLang.getString("amount")));
    switchColors();
    for (int i = 0; i < ESSGLInfo.size(); i++) {
      R = (ESSGL) ESSGLInfo.elementAt(i);
      retVal += printDepartListItem(R);
      switchColors();
      expTotal += R.SUM;
    }
    retVal += d2row(myLang.getLabel("total"),expTotal);
    retVal += "</table>\n";
    return retVal;
  }
  
  public void setESSGLwDept(String ReferenceType) {
    ESSExpense E;
    ESSGL P = new ESSGL();
    ESSGLInfo.removeAllElements();
    for (int i = 0; i < ESSExpenses.size(); i++) {
      E = (ESSExpense) ESSExpenses.elementAt(i);
      if (ReferenceType.toUpperCase().equals("CLIENTNO")) {
        P.add(E.EXPENSE, E.AMOUNT, E.BILLTYPE, E.ChargedDepart, E.clientno);
      } else if (ReferenceType.toUpperCase().equals("ENTITY/CLIENT")) {
        P.add(E.EXPENSE, E.AMOUNT, E.BILLTYPE, E.ChargedDepart, E.COMPANY + "/" + E.clientno);
      } else if (ReferenceType.toUpperCase().equals("PROJECT/CLIENT")) {
        P.add(E.EXPENSE, E.AMOUNT, E.BILLTYPE, E.ChargedDepart, E.project + "/" + E.clientno);
      } else {
        P.add(E.EXPENSE, E.AMOUNT, E.BILLTYPE, E.ChargedDepart, E.project);
      }
    }    
    // need to add in the receipt array here eventually if needed.
  }
  
  
  public void createGrid() {
     
  }    
  public void setESSGrid() {
    ESSExpense E;
    ESSGrid P = new ESSGrid();
    ESSGridInfo.removeAllElements();
    for (int i = 0; i < ESSExpenses.size(); i++) {
      E = (ESSExpense) ESSExpenses.elementAt(i);
      P.add(E.EXPENSE, CustomDate.getDateFromStr(E.EXP_DATE), Double.parseDouble(E.AMOUNT));
    }    
    // need to add in the receipt array here eventually if needed.
  }
  public class ESSGrid {
    public String expenseType;
    public java.util.Date expenseSunday;
    public double[] expenseAmts = new double[7];
    public ESSGrid (){
    }
    public void add(String expenseType, java.util.Date expenseDate, double expenseAmount){
       boolean found = false;
       ESSGrid R;
       expenseSunday = CustomDate.getPrevSunday(expenseDate);
       for (int i = 0; i < ESSGridInfo.size(); i++) {
         R = (ESSGrid) ESSGridInfo.elementAt(i);
         if (R.expenseType.equals(expenseType) && R.expenseSunday.equals(expenseSunday)) {
           found = true;
           i = ESSGridInfo.size();
           R.update(expenseType, expenseDate, expenseAmount);
         }   
       } 
       if (!found){
         R = new ESSGrid();  
         R.expenseType = expenseType;
         R.expenseSunday = expenseSunday;
         for (int j = 0; j < 7; j++) R.expenseAmts[j] = 0;
         R.update(expenseType, expenseDate, expenseAmount);
         ESSGridInfo.addElement(R);
       }
    }
    private void update(String expenseType, java.util.Date expenseDate, double expenseAmount){
      int i = CustomDate.getDayOfWeek(expenseDate) - 1;
      expenseAmts[i] += expenseAmount;
    }
  } 
  public String printExpenseGrid() {
    double[] expenseAmts = new double[7];
    for (int j = 0; j < 7; j++) expenseAmts[j] = 0;
    ESSGrid R;
    java.util.Date D = new java.util.Date();
    String retVal = SecHeader(myLang.getString("weeklyExpMatrix"));
    ESSExpenses.reLoad(new CompareXMLExpensesByDate());
    ESSExpenses.sort();
    setESSGrid();
    retVal += stable;
    for (int i = 0; i < ESSGridInfo.size(); i++) {
      R = (ESSGrid) ESSGridInfo.elementAt(i);
      if (i == 0 || !R.expenseSunday.equals(D)) {
        if (i != 0) {
           retVal += printGridLine(myLang.getLabel("totals"),expenseAmts); 
           retVal += "<tr><td width=\"100\">&nbsp;</td></tr>";
        }
        resetColors();
        retVal += printGridTitle(chd(" "),R.expenseSunday);
        switchColors();
        D = R.expenseSunday;
        for (int j = 0; j < 7; j++) expenseAmts[j] = 0;    
      }
      retVal += printGridLine(myLang.getDataString(R.expenseType),R.expenseAmts);
      switchColors();
      for (int j = 0; j < 7; j++) expenseAmts[j] += R.expenseAmts[j]; 
    }
    retVal += printGridLine(myLang.getLabel("totals"),expenseAmts);
    retVal += "</table>\n";
    return retVal;
  }
  private String printGridLine(String v1, double[] v2) {
    String retVal = tr;
    double total = (double) 0.00;
    retVal += htd("100",v1);
    for (int j = 0; j < 7; j++)
    {
       retVal += htdr("70",getUserAmount(FF(DRoundStr(v2[j]))));  
       total += v2[j]; 
    }
    retVal += htdr("70",getUserAmount(FF(DRoundStr(total))));
    
    retVal += tre;
    return retVal;
  }
  
  private String printGridTitle(String v1, java.util.Date v2) {
    String retVal = tr;
    retVal += htd("100",v1); 
    for (int i = 0; i < 7; i++) {
      retVal += htdr("70",date.getUserMonthDay(v2,dateFormatStr));
      v2 = CustomDate.addDays(v2, 1);
    }
    retVal += htdr("70",myLang.getString("total"));
    retVal += tre;
    return retVal;
  }
  
  public String printAuditInfo() {
    String uploadStatus;
    if (Rep.UP_STAT.equals("")) {
        uploadStatus = "";
    } else {
        uploadStatus = StatusDOM.getDOMTableValueWhere("default","translation",Rep.UP_STAT,"auditmessage");
    }
    if (uploadStatus.equals("")) uploadStatus = Rep.UP_STAT;
    String retVal = SecHeader(myLang.getString("auditInfoSec"));
    retVal += btable;
    retVal += h1row(myLang.getLabel("repComment"),Rep.COMMENT,myLang.getLabel("currency"),Rep.CURRENCY);
    retVal += h1row(myLang.getLabel("pvoucher"),Rep.PVOUCHER,myLang.getLabel("inputBy"),getNameFmPers(Rep.CLERK));
    retVal += h1row(myLang.getLabel("curdate"),date.getUserDateStr(Rep.CUR_DATE,dateFormatStr),Rep.CUR_TIME,"");
    retVal += h1row(myLang.getLabel("subdate"),date.getUserDateStr(Rep.SUB_DATE,dateFormatStr),Rep.SUB_TIME,"");
    retVal += h1row(myLang.getLabel("actdate"),date.getUserDateStr(Rep.ACT_DATE,dateFormatStr),Rep.ACT_TIME,"");
    retVal += h1row(myLang.getLabel("admin1"),getNameFmPers(Rep.ADMIN1),myLang.getLabel("ad1date"),date.getUserDateStr(Rep.AD1_DATE,dateFormatStr));
    retVal += h1row(myLang.getLabel("sign1"),getNameFmPers(Rep.SIGN1),myLang.getLabel("sg1date"),date.getUserDateStr(Rep.SG1_DATE,dateFormatStr));
    retVal += h1row(myLang.getLabel("sign2"),getNameFmPers(Rep.SIGN2),myLang.getLabel("sg2date"),date.getUserDateStr(Rep.SG2_DATE,dateFormatStr));
    retVal += h1row(myLang.getLabel("admin2"),getNameFmPers(Rep.ADMIN2),myLang.getLabel("ad2date"),date.getUserDateStr(Rep.AD2_DATE,dateFormatStr));
    retVal += h1row(myLang.getLabel("sign3"),getNameFmPers(Rep.SIGN3),myLang.getLabel("auddate"),date.getUserDateStr(Rep.AUD_DATE,dateFormatStr));
    retVal += h1row(myLang.getLabel("upstat"),uploadStatus,myLang.getLabel("received"),date.getUserDateStr(Rep.RECEIVED,dateFormatStr));
    retVal += h1row(myLang.getLabel("sign4"),getNameFmPers(Rep.SIGN4),myLang.getLabel("update"),date.getUserDateStr(Rep.UP_DATE,dateFormatStr));
    retVal += h1row(myLang.getLabel("extra1"),Rep.EXTRA1,myLang.getLabel("extra3"),Rep.EXTRA3);
    retVal += h1row(myLang.getLabel("manager"),getRouteToName(),myLang.getLabel("forced"),Rep.FORCED);
    
    // retVal += h1row("Approved:"," "," "," ");
    retVal += "</table>\n";
    return retVal;
  }
  
  public String getRouteToName() {
    return getNameFmPers(Rep.MANAGER);
  }
  private String getNameFmPers(String uid) {
    String retVal = "";
    if (!uid.equals("") && !uid.equalsIgnoreCase("GATEWAY")){
      if (PT.setPersNumInfo(uid)) {
        retVal = PT.name;
      } else {
        retVal = dispPersNo(uid);
      }
    } else if (uid.equalsIgnoreCase("GATEWAY")){
        retVal = uid;
    }
    return retVal;
  }

}
