/*
SMTPBatch.java - Send 'out' messages in batch via SMTP  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.net.*;
import java.io.*;
//import java.util.*;

public class SMTPBatch extends POP3andSMTP implements Runnable {

// This is an extension of the POP3andSMTP object that specializes in doing an SMTP message upload to
// text file in batch mode.  It is written for multithreading


// This static object also exists in the parent object but I used it here just to keep
// the code more readable and modular - don't know if this makes sense, the jury's out
    private CommonFunctions F = new CommonFunctions();
    private AuditTrail Logfile = new AuditTrail();


    public void run () {

        ServerSystemTable SystemTable = new ServerSystemTable();
        String OutMailBox = SystemTable.getSystemString("OUTMAILBOX","C:/WORK");
        File OutMailDir = new File(OutMailBox);
        String[] OutFilesArray;
        int NumberOfFiles;
        int i;
        String ReturnString;
        int FirstReturn;

        SystemTable.close();

        FileTypeFilter OutTypeOfFile = new FileTypeFilter(".out");  //later get this from Audit
        OutFilesArray = OutMailDir.list(OutTypeOfFile);
        NumberOfFiles = OutFilesArray.length;

        if(NumberOfFiles > 0) {

            outSocketSMTP();        //later on will have to get parameter from the param file stuff
            try {
                ReturnString = read();  //need to make somekind of time out parameter
                FirstReturn = ReturnString.indexOf(13);
                if (FirstReturn == -1) {
                    AuditTrail.println(ReturnString);
                } else {
                    AuditTrail.println(ReturnString.substring(0,FirstReturn));
                }
            } catch (IOException e) {
                AuditTrail.println("Did not receive SMTP Banner");
            }

            OutMailBox = CommonFunctions.addSlash(OutMailBox);

            try {
                write("helo");
                ReturnString = read();
                if (messageBackIsOK(ReturnString)) {
                    for (i = 0; i < NumberOfFiles; i++ ) {
                        uploadMessageFile(OutMailBox + OutFilesArray[i]);
                    }
                } else {
                    AuditTrail.println("SMTP server is not responding properly");

                }
                write("quit");
                ReturnString = read();

            } catch (IOException e) {
                AuditTrail.println("Cannot access SMTP server");
            }
            close();
        } else {
            AuditTrail.println("No messages in the outbox");
        }
    }


    public void uploadMessageFile(String UploadFile) {
        InputTextFile Message = new InputTextFile();
        String InputLine;
        String ReturnString;
        boolean LoopThruFile = true;    // Flag that signifies
        boolean DotFlag = false;        // Signifies that a message has ended and won't continue until next

// Might want to rewrite this to require a return on all sends except those within
// the DATA . function instead of it's current positive checking for
// specific commands.

        try {
            Message.createFileHandle(UploadFile);
            while (LoopThruFile && (InputLine = Message.readString()) != null) {

                if (DotFlag) {
                    if (requiresGoodReturn(InputLine)) {
                        DotFlag = false;
                    } else {
                        break;
                    }
                }


                write(InputLine);

                if (requiresGoodReturn(InputLine)) {

                    ReturnString = readLine();

                    if (!messageBackIsOK(ReturnString)) {
                        LoopThruFile = false;
                        break;
                    }

                    if (InputLine.equals(".")) {
                        AuditTrail.println(ReturnString);
                        DotFlag = true;
                    }
                }

            }
        }catch (IOException e) {
            AuditTrail.println("Failed to open outmail message file: " + UploadFile);
        }
    }
}

