/*
Scan.java - interface to the Scan table
Copyright (C) 2012 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

import java.sql.*;
import javax.sql.rowset.serial.*;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;
import java.util.zip.GZIPInputStream;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import java.util.List;
import java.awt.image.BufferedImage;
import com.sun.image.codec.jpeg.*;
import javax.imageio.*;

public class Scan extends AdisoftDbase {

    private ServerSystemTable systemTable;

    public final int SCAN_REF = 0;
    public final int PERS_NUM = SCAN_REF + 1;
    public final int PVOUCHER = PERS_NUM + 1;
    public final int RECEIPT = PVOUCHER + 1;
    public final int FILE_TYPE = RECEIPT + 1;
    public final int UP_DATE = FILE_TYPE + 1;
    public final int UP_TIME = UP_DATE + 1;
    public final int CLERK = UP_TIME + 1;
    public final int STATUS = CLERK + 1;
    public final int XREF = STATUS + 1;
    public final int COMMENT = XREF + 1;
    public final int ADDRESS = COMMENT + 1;
    public final int FILE_SIZE = ADDRESS + 1;
    public final int FILE_CONTENT = FILE_SIZE + 1;
    
//    private ServerSystemTable SystemTable;
//    private AuditTrail AuditLog = new AuditTrail();
//    private String smtp_address;
    //private String[] tags = {"SCAN_REF","PERS_NUM","PVOUCHER","RECEIPT","FILE_TYPE","UP_DATE","UP_TIME","CLERK","STATUS","XREF","COMMENT","ADDRESS","FILE_SIZE","FILE_CONTENT"};
    //private String[] values = {"","","","","","","","","","","","","",""};
    private String[] tags = {"SCAN_REF","PERS_NUM","PVOUCHER","RECEIPT","FILE_TYPE","UP_DATE","UP_TIME","CLERK","STATUS","XREF","COMMENT","ADDRESS"};
    private String[] values = {"","","","","","","","","","","",""};
    private int file_size = 0;
    private byte[] file_content;
    private List <PDDocument> pdfs = new java.util.ArrayList<PDDocument>();


    public Scan()
    {
//caller needs to call setDB().        
//        SystemTable = new ServerSystemTable();
//        smtp_address = SystemTable.getSystemString("SMTP_ADDRESS","192.168.0.21");
    }
    
    public Scan(String DBUrl, String DBUser, String DBPassword)
    {
        setDB(DBUrl, DBUser, DBPassword);
    }
    
    public Scan(Connection DBConnection)
    {
        setConnection(DBConnection);
    }

    public void setSystemTable(ServerSystemTable x)
    {
    	systemTable = x;
    }
    
    public int getKeyPosition()
    {
    	return SCAN_REF;
    }
    
    public void setScan_ref(String x)
    {
        values[SCAN_REF] = x;
    }
    public void setPers_num(String x)
    {
        values[PERS_NUM] = x;
    }
    public void setPvoucher(String x)
    {
        values[PVOUCHER] = x;
    }
    public void setReceipt(String x)
    {
        values[RECEIPT] = x;
    }
    public void setFile_type(String x)
    {
        values[FILE_TYPE] = x;
    }
    public void setUp_date(String x)
    {
        values[UP_DATE] = x;
    }
    public void setUp_time(String x)
    {
        values[UP_TIME] = x;
    }
    public void setClerk(String x)
    {
        values[CLERK] = x;
    }
    public void setStatus(String x)
    {
        values[STATUS] = x;
    }
    public void setXref(String x)
    {
        values[XREF] = x;
    }
    public void setComment(String x)
    {
        values[COMMENT] = Utilities.getSQLString(x);
    }
    public void setAddress(String x)
    {
        values[ADDRESS] = x;
    }
    public void setSize(int x)
    {
        file_size = x;
    }
    public void setContent(byte[] x)
    {
        file_content = x;
    }

    public String getScan_ref()
    {
        return(values[SCAN_REF]);
    }
    public String getPers_num()
    {
    	return(values[PERS_NUM]);
    }
    public String getPvoucher()
    {
    	return(values[PVOUCHER]);
    }
    public String getReceipt()
    {
    	return(values[RECEIPT]);
    }
    public String getFile_type()
    {
    	return(values[FILE_TYPE]);
    }
    public String getUp_date()
    {
    	return(values[UP_DATE]);
    }
    public String getUp_time()
    {
    	return(values[UP_TIME]);
    }
    public String getClerk()
    {
    	return(values[CLERK]);
    }
    public String getStatus()
    {
        return(values[STATUS]);
    }
    public String getXref()
    {
        return(values[XREF]);
    }
    public String getComment()
    {
        return(values[COMMENT]);
    }
    
    public int getSize()
    {
        return(file_size);
    }
    
    public byte[] getContent()
    {
        return(file_content);
    }
   
    public String[] getTags()
    {
        return(tags);
    }
    
    public String[] getValues() 
    {
        return(values);
    }
    public void updateColumn(String reference, String tag, String newValue)
    {
        String[] tags = {tag};
        String[] values = {newValue};
        setSimple(SCAN_REF,reference, tags, values);
    }
    
    //public void setSimple(String[] elementTags, String[] elementValues) {
    //The key here is the first member of the ElementTags array    
    //   setSimple("SCAN", elementTags[0], elementValues[0], elementTags, elementValues);
   // }
  
    public void setSimple(int keyTag, String KeyValue, String[] elementTags, String[] elementValues)
    {
        setSimple("SCAN", tags[keyTag], KeyValue, elementTags, elementValues);
    }
    
    public void setSimple(int keyTag, String KeyValue)
    {
        setSimple("SCAN", tags[keyTag], KeyValue, tags, values);
		String INSERT_PICTURE = "UPDATE SCAN SET FILE_SIZE = ?, FILE_CONTENT = ? where SCAN_REF = '" + KeyValue + "'";
		// put zipping here
		try {
			java.sql.Blob picture_as_blob = new SerialBlob(getContent());   
			PreparedStatement ps = this.getConnection().prepareStatement(INSERT_PICTURE);
			ps.setInt(1,(int) picture_as_blob.length());
			ps.setBlob(2, picture_as_blob);   
			ps.executeUpdate();  
	    } catch (SerialException e) {
	    	AuditTrail.println("[500] Scan.setSimple - SerialException - see logs");
			e.printStackTrace();
	    } catch (SQLException e) {
	    	AuditTrail.println("[500] Scan.setSimple - SQLException - see logs");
			e.printStackTrace();
	    }

    }
    
    public boolean isScansAvailable(String persNumber) {
    	  String sql2FindScans = systemTable.getSystemString("receiptmanagement", "scanstoapply", "SELECT * FROM SCAN WHERE PERS_NUM = '$persnum$' AND STATUS = 'Incoming';");
    	  sql2FindScans = Utilities.replaceStr(sql2FindScans, "$persnum$", persNumber);
	      return this.setResultSet(sql2FindScans);  //check for multiple records
	    }
	public String[] getScan()
	{
		 String[] retVal = null;
		 try {
			retVal = new String[3];
			retVal[0] = myResult.getString("SCAN_REF");
			retVal[1] = myResult.getString("FILE_TYPE");
			retVal[2] = myResult.getString("COMMENT");
	    } catch (SQLException e) {
	    	retVal = null;
			AuditTrail.println("[500] Scan.getScan - error in getting scan information - see logs");
			e.printStackTrace();
	    }
	    return retVal;
	}
	public String[] getNextScan()
	{
	    String[] retVal = null;
	    try {
		  if (myResult.next())
		  {
			  retVal = getScan();
		  }
	    } catch (SQLException e) {
	    	retVal = null;
	    	AuditTrail.println("[500] Scan.getNextScan - error in getting scan information - see logs");
	    	e.printStackTrace();
	    }
	    return retVal;
	}

	public byte[] getScanImage()
	{
		 byte[] retVal = null;
		 try {
			 Blob file_blob = myResult.getBlob("FILE_CONTENT");
			 retVal = file_blob.getBytes((long) 1, (int) file_blob.length());
	    } catch (SQLException e) {
	    	retVal = null;
			AuditTrail.println("[500] Scan.getScanImage - error in getting scan information - see logs");
			e.printStackTrace();
	    }
	    return retVal;
	}
	
	public static byte[] compress(byte[] item2Compress)
	{
	    if (item2Compress == null || item2Compress.length == 0) {
	        return item2Compress;
	    }
	    byte[] compressedBytes = null;
	    try {
	    	ByteArrayOutputStream out = new ByteArrayOutputStream();
	    	GZIPOutputStream gzip = new GZIPOutputStream(out);
	    	gzip.write(item2Compress);
	    	gzip.close();
	    	compressedBytes = out.toByteArray();
	    	out.close();
	    } catch (IOException e) {
			AuditTrail.println("[500] Scan.compress - error - see logs");
			e.printStackTrace();
	    }
	    return compressedBytes;
	 }
	
	public static byte[] decompress(byte[] item2decompress)
	{
	    if (item2decompress == null || item2decompress.length == 0) {
	        return item2decompress;
	    }
	    byte[] decompressedBytes = null;
	    try {
	    	ByteArrayInputStream incoming = new ByteArrayInputStream(item2decompress);
	    	GZIPInputStream gunzip = new GZIPInputStream(incoming);
	    	ByteArrayOutputStream out = new ByteArrayOutputStream();
	        int len;
	        byte[] buffer = new byte[1024];
	        while ((len = gunzip.read(buffer)) > 0) {
	        	out.write(buffer, 0, len);
	        }
	    	decompressedBytes = out.toByteArray();
	    	out.close();
	    	gunzip.close();
	    	incoming.close();
	    } catch (IOException e) {
			AuditTrail.println("[500] Scan.compress - error - see logs");
			e.printStackTrace();
	    }
	    return decompressedBytes;
	 }
	
	public boolean loadPDFDocumentObject(byte[] item2load)
	{
		boolean retVal = false;
		ByteArrayInputStream incoming = new ByteArrayInputStream(item2load);
		
        try {
        	PDDocument pdf = PDDocument.load( incoming, true );
        	retVal = pdfs.add(pdf);
        } catch (IOException e)
        {
			AuditTrail.println("[500] Scan.loadPDFDocumentObject - error - see logs");
			e.printStackTrace();
			retVal = false;
	    }
		return retVal;
	}
	
	public void resetPDFsList()   
	{
		if (pdfs != null) pdfs.clear();
		
	}
	
	public int getNumberOfPages(int pdfNumber)
	{
		return (pdfs.get(pdfNumber).getNumberOfPages()); 
	}
	
	public synchronized byte[] getPDFPageImage(int pdfNumber, int pageNumber)
	{
		byte[] returnImage = null;
		List <PDPage> pages = pdfs.get(pdfNumber).getDocumentCatalog().getAllPages();
		PDPage thisPage = pages.get(pageNumber);
		try {
			//BufferedImage image = page.convertToImage(imageType, resolution); 
			BufferedImage image = thisPage.convertToImage();
			ByteArrayOutputStream os = new ByteArrayOutputStream();
			JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(os);
			encoder.encode(image);
			returnImage =  os.toByteArray();
		} catch (IOException e) {
			AuditTrail.println("[500] Scan.getPDFPageImage - error - see logs");
			e.printStackTrace();
		}
        return returnImage;
	}
		
	public static String getImageFormats() { 
        StringBuffer retval = new StringBuffer(); 
        String[] formats = ImageIO.getReaderFormatNames(); 
        for (int i = 0; i < formats.length; i++) { 
            retval.append(formats[i]); 
            if (i + 1 < formats.length) { 
                retval.append(","); 
            } 
        } 
        return retval.toString(); 
    }
    
}



    
