/*
ServerSystemTable.java - interface to system table  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.net.URL;
import java.sql.*;

//
//  This object extends the Dbase class to provide enhancements for reading and writing
//  to the SYSTEM table which contains setup and control parameters
//
//  Also see the IncrementDbase obj as it is used here.

public class ServerSystemTable extends AdisoftDbase {

    //Statement myState;                          // every instance gets a pair of these
    private String DBUrl = "jdbc:odbc:adisoftdbase";
    SystemXML xml;
    //Need to make the values in the DBFfields more dynamic with a select statement
    String[] DBFfields = {"XMLSYSTEM", "MESSAGE_SEQUENCE","COMPANY_SEQUENCE","PERSNUM_SEQUENCE","SCAN_SEQUENCE","VOUCHER","XREF_SEQUENCE"};
    static IncrementDbase IncDB = new IncrementDbase();

    
    public ServerSystemTable() {
    }

    public ServerSystemTable(AuditTrail CurrentLog, String DBUrl) {
        setDB(DBUrl);
    }

    public ServerSystemTable(String DBUrl) {
        setDB(DBUrl);
    }

    public ServerSystemTable(String DBUrl, String DBUser, String DBPassword) {
        setDB(DBUrl, DBUser, DBPassword);
    }
    
    public void setLockTable (String x) {
        IncDB.setLockTable(x);
    }
    
    public void setDB (String DBUrl, String DBUser, String DBPassword)
    {
        super.setDB(DBUrl, DBUser, DBPassword);
    }
    
    public void setSQLTerminator(String X)
    {
        IncDB.setSQLTerminator(X);  //might want to look at not always reseting this
        super.setSQLTerminator(X);  //each time that the increment takes place. JH 11/7/2003
    }
    
    private boolean isInDBF(String fieldID)
    {
        return(Utilities.findString(fieldID, DBFfields) == -1? false : true);
    }
    
    public String getSystemString(String ElementTag, String ElementDefault) {

        String result;
        
        if (isInDBF(ElementTag) == true)
        {
            result = getSystemStringPrivate(ElementTag.toLowerCase(), ElementDefault);
            closeMyState();
        }
        else
        {
            if (xml == null) xml = new SystemXML(this,sqlTerminator);
            result = xml.getConfig(ElementTag.toLowerCase());
            if ((result == null) || (result.length() == 0))
                result = ElementDefault;
        }
            
        return (result);
    }

    public String getSystemString(String ElementGroup, String ElementTag, String ElementDefault) {

        String result;
        
        if (isInDBF(ElementTag) == true)
        {
            result = getSystemStringPrivate(ElementTag.toLowerCase(), ElementDefault);
            closeMyState();
        }
        else
        {
            if (xml == null) xml = new SystemXML(this,sqlTerminator);
            result = xml.getConfig(ElementGroup.toLowerCase(), ElementTag.toLowerCase());
            if ((result == null) || (result.length() == 0))
                result = ElementDefault;
        }
            
        return (result);
    }


    private String getSystemStringPrivate(String ElementTag, String ElementDefault) {
        String SQLCommand = "not defined";
        try {

            myState = dbCon.createStatement();
            SQLCommand = "SELECT ELEMENT FROM SYSTEM WHERE TAG = '" + ElementTag.toUpperCase() + "'" + sqlTerminator;
            myResult = myState.executeQuery(SQLCommand);

            if (myResult.next()) {
                return myResult.getString("ELEMENT").trim();
            } else {
            	AuditTrail.println("[400] ServerSystemTable.getSystemString failed for : " + ElementTag.trim());
                AuditTrail.println("[400] ServerSystemTable.getSystemString SQL : " + SQLCommand);
                return ElementDefault;
            }


        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] ServerSystemTable.getSystemString Default for " + ElementTag.trim() + " used: " + ElementDefault);
            AuditTrail.println("[500] " + SQLCommand);
            AuditTrail.println("[500] " + ex.toString());
            return ElementDefault;
        }
    }

    
    public void setSystemString(String ElementTag, String ElementValue) {
        putSystemString(ElementTag, ElementValue);
    }    
    
    public void putSystemString(String ElementTag, String ElementValue) {
        int returnResult;
        Integer returnInteger;
        String SQLStatement = "SQL statement not yet constructed";
        try {
            if (getSystemString(ElementTag,"NONE_NONE").equals("NONE_NONE")) {
                SQLStatement = "INSERT INTO SYSTEM (TAG, ELEMENT) VALUES ('" + ElementTag.toUpperCase() + "', '"+ ElementValue + "')"  + sqlTerminator;
            } else {
                SQLStatement = "UPDATE SYSTEM SET ELEMENT = '" + ElementValue + "' WHERE TAG = '" + ElementTag.toUpperCase() + "'" + sqlTerminator;
            }
            returnResult = doSQLExecute(SQLStatement);
            returnInteger = new Integer(returnResult);

            if (returnResult == 1) {
                AuditTrail.println("[000] ServerSystemTable.putSystemString succeeded: " + ElementTag.trim() + ", " + ElementValue);
            } else {
                AuditTrail.println("[500] ServerSystemTable.putSystemString failed: " + ElementTag.trim() + ", " + ElementValue + ", Error: " + returnInteger.toString());
            }
        } catch (java.lang.Exception ex){
            ex.printStackTrace();    //want to get this to the log file
            AuditTrail.println("[500] ServerSystemTable.putSystemString failed for " + ElementTag.trim() + ", value: " + ElementValue);
            AuditTrail.println("[500] " + ex.toString());
            AuditTrail.println("[500] ServerSystemTable.putSystemString SQL: " + SQLStatement);

        }
        finally
        {
            closeMyState();
        }
    }

    public void setSystemInt(String ElementTag, int ElementValue){
        putSystemInt(ElementTag, ElementValue);
    }    

    public void putSystemInt(String ElementTag, int ElementValue){
        Integer IntResult = new Integer(ElementValue);
        putSystemString(ElementTag, IntResult.toString());
    }


    public int getSystemInt(String ElementTag) {
// This method gets a system string and converts it to an int.
        String StringResult = getSystemString(ElementTag, "-1");
        Integer IntResult = new Integer(StringResult);
        return IntResult.intValue();
    }

    public int getSystemInt(String ElementTag, String defaultString) {
    	// This method gets a system string and converts it to an int.
    	        String StringResult = getSystemString(ElementTag, defaultString);
    	        Integer IntResult = new Integer(StringResult);
    	        return IntResult.intValue();
    	    }

    public int getSystemIncrement(String ElementTag) {
// This method gets a system string and converts it to an int and then increments
// by calling the appropriate class in a static ID object.
        String getSQL = "SELECT ELEMENT FROM SYSTEM WHERE TAG = '" + ElementTag.toUpperCase() + "'" + sqlTerminator;
        String putSQL = "UPDATE SYSTEM SET ELEMENT = '$columnvalue$' WHERE TAG = '" + ElementTag.toUpperCase() + "'" + sqlTerminator;
        int IntResult = IncDB.IncrementInDbase(dbCon, "ELEMENT", "SYSTEM",getSQL, putSQL);
        return IntResult;
    }

    public String getSystemIncString(String ElementTag) {
        int IntValue = getSystemIncrement(ElementTag);
        String StringResult = new Integer(IntValue).toString(); 
        return StringResult;
    } 
    public void closeMyState()
    {
      if (myState != null)
      {
        try{
            myState.close();
            myState = null;
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] AdisoftDbase.closeMyState close failure");
        }
      }
    }
    public void close()
    {
        try {
            closeMyState();
            dbCon.close();
        } catch (java.lang.Exception ex){
            ex.printStackTrace();
            AuditTrail.println("[500] ServerSystem.close DB close failure");
        }
    }
    
    public String getExpSysIncString(String elementTag, boolean unique, String sequencePrefix) {
        int nextSequence = 0;
        String searchString;
        int searchNum;
        boolean numberUsed = false;
        String result = null; 
        int test = -1;
        nextSequence = getSystemIncrement(elementTag);
        searchNum = nextSequence;
        
        do
        {
            searchString = getReferenceString(searchNum, 8, sequencePrefix);
            if (unique && findDBFReport(searchString) == true)
            {
                numberUsed = true;
                searchNum += 1000;
            }
            else 
                numberUsed = false;
        } while (numberUsed == true);    
        
        if (searchNum != nextSequence)
            putSystemInt(elementTag, ++searchNum);
        return (searchString);
    }    
    public String getReferenceString(int Value, int xLen, String sequencePrefix) {
      String S = Integer.toString(Value);
      String NewS = "0000000000" + S;
      if (!sequencePrefix.equals("") && sequencePrefix != null) xLen = xLen - sequencePrefix.length();
      return sequencePrefix + NewS.substring(NewS.length()-xLen);
    }
    
    private boolean findDBFReport(String voucher)
    {
        Statement myState;                          
        ResultSet myResult;
        boolean result = false;
        try
        {            
            myState = dbCon.createStatement();
            myResult = myState.executeQuery("SELECT VOUCHER FROM REPORT WHERE VOUCHER = '" + voucher.trim() + "' ORDER BY VOUCHER" + sqlTerminator);
            result = myResult.next();
         }
        catch (Exception e)
        {
            e.printStackTrace();    //want to get this to the log file
            AuditTrail.println("[500] ExpSysTable.findDBFReport failed for voucher " + voucher);
        }
        
            return (result);
    }
    
    
    
    
}