/*
StateTable.java - interface to the STATE table
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.io.*;
//import java.util.*;
import java.sql.*;
import org.jdom.Document; //See if I can find this
import org.jdom.Element;
//import org.jdom.DocType;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;


public class StateTable
{
   static AuditTrail Logfile;
   String DBUrl = "";
   String DBUser = "";
   String DBPassword = "";
   Connection dbConn;
   AdisoftDbase db;
    
   private ServerSystemTable systemTable;
    
    
    final int STATUS = 1;
    final int ACTION = 2;
    final int TRUE = 3;
    final int FALSE = 4;
    
    public StateTable() {
    }
    
    public StateTable(String DBUrl, String DBUser, String DBPassword, String DBSQLTerminator)
    {
        this.DBUrl = DBUrl;
        this.DBUser = DBUser;
        this.DBPassword = DBPassword;
        db = new AdisoftDbase(DBUrl, DBUser, DBPassword);
        db.setSQLTerminator(DBSQLTerminator);
        systemTable = new ServerSystemTable(DBUrl, DBUser, DBPassword);
        systemTable.setSQLTerminator(DBSQLTerminator);
    }
    public StateTable(Connection dbConnection, String DBSQLTerminator)
    {
        dbConn = dbConnection;
        db = new AdisoftDbase();
        db.setConnection(dbConn);
        db.setSQLTerminator(DBSQLTerminator);
        systemTable = new ServerSystemTable();
        systemTable.setConnection(dbConn);
        systemTable.setSQLTerminator(DBSQLTerminator);
    }
    
    public void setStateTable(Connection dbConnection, String DBSQLTerminator)
    {
        dbConn = dbConnection;
        db = new AdisoftDbase();
        db.setConnection(dbConn);
        db.setSQLTerminator(DBSQLTerminator);
        systemTable = new ServerSystemTable();
        systemTable.setConnection(dbConn);
        systemTable.setSQLTerminator(DBSQLTerminator);
    }
    
    public void close()
    {
        systemTable.close();
        db.close();
    }
    public boolean canPerform(String status, String action)
    {
        String result = getColumnData("SELECT ACTION FROM STATE WHERE STATUS = '" + status + "' AND " +
                        "ACTION = '" + action + "'", "ACTION");
//        String result = getColumnData("SELECT ACTION FROM STATE WHERE RTRIM(STATUS) = '" + status + "' AND " +
//                        "RTRIM(ACTION) = '" + action + "'", "ACTION");
        return (result.length() > 0);
          
    }
    
    public String getPositiveStatus(String status, String action)
    {
        return(getColumnData("SELECT POSITIVE FROM STATE WHERE STATUS = '" + status + "' AND " +
                        "ACTION = '" + action + "'", "POSITIVE"));
//        return(getColumnData("SELECT POSITIVE FROM STATE WHERE RTRIM(STATUS) = '" + status + "' AND " +
//                        "RTRIM(ACTION) = '" + action + "'", "POSITIVE"));
    }
    
    public String getNegativeStatus(String status, String action)
    {
        return(getColumnData("SELECT NEGATIVE FROM STATE WHERE STATUS = '" + status + "' AND " +
                        "ACTION = '" + action + "'", "NEGATIVE"));
//        return(getColumnData("SELECT NEGATIVE FROM STATE WHERE RTRIM(STATUS) = '" + status + "' AND " +
//                        "RTRIM(ACTION) = '" + action + "'", "NEGATIVE"));
    }
    
    private String getColumnData(String sqlString, String columnId)
    {
        AuditTrail.println("[000] State.getColumnData:" + columnId + " with "+ sqlString);
        String result = "";
        try
        {
            if (db.setResultSet(sqlString) == true)
            {
                ResultSet rs = db.getResultSet();
                result += rs.getString(columnId);
                AuditTrail.println("[000] State.getColumnData result:" + result);
            }
        }
        catch(Exception e)
        {
            AuditTrail.println("[500] State.getColumnData:" + e.getMessage());
            e.printStackTrace();
        }
        return(result);
    }
    
    public String NewToESS(String sourceSubTable, String sourceStatus)
    {
        return(translateStatus(sourceSubTable, sourceStatus, "status", "translation"));
    }
    
    public String ESSToNew(String sourceSubTable, String sourceStatus)
    {
        return(translateStatus(sourceSubTable, sourceStatus, "translation", "status"));
    }
    
    public String translateStatus(String sourceSubTable, String sourceStatus, String fromType, String toType)
    {
        String result = "";
        String statFile = systemTable.getSystemString("XMLSTATUS","c:/visualcafe/projects/adisoft/status.xml");
        AdisoftDOM statusTable = new AdisoftDOM(statFile);
        Document statusDOM = statusTable.getDOM();
        Element root = statusDOM.getRootElement();
        result = statusTable.getDOMTableValueWhere(sourceSubTable, fromType, sourceStatus, toType);
        return(result);
    }
    
    public String toString()
    {
        String result = "";
        try
        {
            if (db.setResultSet("SELECT ALL * FROM STATE") == true)
            {
                ResultSet rs = db.getResultSet();
                do
                {
                    result += rs.getString(1)+ ", ";
                    result += rs.getString(2)+ ", ";
                    result += rs.getString(3)+ ", ";
                    result += rs.getString(4)+ "\n";
                    
                }while (rs.next() == true);
            }
            
         
        }
        catch(Exception e)
        {
            AuditTrail.println("[500] State.toString:" + e.getMessage());
            e.printStackTrace();
        }
        
        
        return(result);
    }
    
    
}
