/*
SystemXML.java - Used by guidelines to access System information (system table and system.xml)
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;
//import java.io.*;
import java.util.*;
import java.sql.*;
import org.jdom.Document; //See if I can find this
import org.jdom.Element;

public class SystemXML
{
    static AuditTrail Logfile;
    String DBUrl = "";
    String DBUser = "";
    String DBPassword = "";
    Connection dbConn;
    private ServerSystemTable systemTable;
    private SortVector mileageRates = new SortVector(new compareMileageRates());
//    String systemXML;
    static AdisoftDOM systemXMLTable;
    static Document systemXMLDOM;
    static Element root;
    MileageRate rateObj = null;
    private String sqlTerminator = ";";
    static AdisoftDOM mileageXMLTable = new AdisoftDOM();
  
    public SystemXML(String DBUrl, String DBUser, String DBPassword, String DBSQLTerminator)
    {
        this.DBUrl = DBUrl;
        this.DBUser = DBUser;
        this.DBPassword = DBPassword;
        this.sqlTerminator = DBSQLTerminator;
        systemTable = new ServerSystemTable(DBUrl, DBUser, DBPassword);
        systemTable.setSQLTerminator(this.sqlTerminator);
        this.dbConn = systemTable.getConnection();
        this.sqlTerminator = systemTable.getSQLTerminator();
        setXMLTable();
    }
    public SystemXML(Connection dbConnection, String DBSQLTerminator)
    {
        dbConn = dbConnection;
        this.sqlTerminator = DBSQLTerminator;
        systemTable = new ServerSystemTable();
        systemTable.setConnection(dbConn);
        systemTable.setSQLTerminator(this.sqlTerminator);
        this.dbConn = dbConnection;
        this.sqlTerminator = DBSQLTerminator;
        
        setXMLTable();
/*        
        String systemXML = systemTable.getSystemString("XMLSYSTEM","c:/winexp/xmls/system.xml");
        systemXMLTable = new AdisoftDOM(systemXML);
        systemXMLDOM = systemXMLTable.getDOM();
        root = systemXMLDOM.getRootElement();
*/        
    }
    
    public SystemXML(ServerSystemTable sysTable, String DBSQLTerminator)
    {
        systemTable = sysTable;
        this.sqlTerminator = DBSQLTerminator;
        this.dbConn = sysTable.getConnection();
        systemTable.setSQLTerminator(this.sqlTerminator);
        setXMLTable();
/*        
        String systemXML = systemTable.getSystemString("XMLSYSTEM","c:/winexp/xmls/system.xml");
        systemXMLTable = new AdisoftDOM(systemXML);
        systemXMLDOM = systemXMLTable.getDOM();
        root = systemXMLDOM.getRootElement();
*/        
    }    
    
    private void setXMLTable()
    {
        String systemXML = systemTable.getSystemString("XMLSYSTEM","c:/winexp/xmls/system.xml");
        if (systemXMLTable == null)
        {
            systemXMLTable = new AdisoftDOM(systemXML);
            systemXMLDOM = systemXMLTable.getDOM();
            root = systemXMLDOM.getRootElement();
        }
        mileageXMLTable.setConnection(this.dbConn);
        mileageXMLTable.setSQLTerminator(this.sqlTerminator);
    }
    
    public AdisoftDOM getAdisoftDOM() {
        return systemXMLTable;
    }
    
    public void close() {
        systemTable.close();
        
    }

  public String getConfig(String fieldId)
  {
        return(systemXMLTable.getDOMTableValueFor("configuration", fieldId.toLowerCase()));
  }
    
  public String getConfig(String sectionID, String fieldId)
  {
        return(systemXMLTable.getDOMTableValueFor(sectionID.toLowerCase(), fieldId.toLowerCase()));
  }

  public Vector getMileageRates(String forThisCompany)
  {
        // AdisoftDOM X = new AdisoftDOM();
        String company;
        String effective;
        String rate;
        mileageXMLTable.setDOMFromTable("MILEAGE");
    
        Element systemEl = mileageXMLTable.getRootElement();
        mileageRates.removeAllElements();
 
 
        Iterator rateItr = systemEl.getChildren().iterator();
                       
        while (rateItr.hasNext())
        {
           Element rateEl = (Element) rateItr.next();
           company = rateEl.getChildText("company"); 
           
           if (forThisCompany.equalsIgnoreCase(company))
           {
              effective = ess.CustomDate.setInternalFromSQL(rateEl.getChildText("effective"));
              rate = rateEl.getChildText("rate");
              mileageRates.addElement(new MileageRate(Float.parseFloat(rate),effective));
           }
        }
            
        mileageRates.sort();
    return((Vector) mileageRates);    
  }
  
  
  
  
  public float getEffectiveRate(java.util.Date effectiveDate)
  {
    int i = 0;
    float rateFound = -1.0f;
    
    while ((i < mileageRates.size()) && (rateFound == -1.0f))
    {
        rateObj = (MileageRate) mileageRates.get(i);
        if (rateObj.compareEffectiveDates(effectiveDate) >= 0)
            rateFound = rateObj.getRate();
            
        i++;
    }
    
    return rateFound;
  }
  
  public java.util.Date getEffectiveDate()
  {
    return(rateObj != null ? rateObj.getEffectiveDate() : null);
  }
  
  class MileageRate
  {
    public float value = 0.0f;
    public java.util.Date effectiveDate;
        
    public MileageRate(float value, String effective)
    {
        this.value = value;
        this.effectiveDate = Utilities.stringToDate(effective);
    }
/**
compareEffectiveDates - return >0 if it is after the effective date; 0 if on the effective date;
< 0 if before the effective date.
*/
    public int compareEffectiveDates(java.util.Date effectiveDate)
    {
        return(Utilities.compareDates(effectiveDate, this.effectiveDate));
    }
   
    public java.util.Date getEffectiveDate()
    {
        return(effectiveDate);
    }
    
    public float getRate()
    {
        return value;
    }
    
    public int compareTo(MileageRate other)
    {
//  put rate entries in the order of most recent dates.
        return(Utilities.compareDates(other.effectiveDate, this.effectiveDate));
    }
    
    public String toString()
    {
        return(value + ", " + effectiveDate + "\n");
    }
    
  }
  
    class compareMileageRates implements Compare{
        public boolean lessThan(Object left, Object right) {
                return (((MileageRate) left).compareTo((MileageRate) right) < 0);
        }
        public boolean lessThanOrEqual(Object left, Object right) {
                return (((MileageRate) left).compareTo((MileageRate) right) <= 0);
        }
    }
        
  
}
