/*
Utilities.java - various common utility methods  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;
import java.io.*;
import java.util.*;
import java.text.*;

public class Utilities {
/** scramble - scrambles the string using an xor operation
*/
public static String scramble(String inString, int inKey)
{
    char[] outChars = new char[inString.length()];
    String outString = "";
    inString.getChars(0, inString.length(),outChars,0);
    char cypherFeedback = (char) inKey;
    for (int i = 0; i < outChars.length; i++)
    {
        cypherFeedback ^= outChars[i];
        outString +=  cypherFeedback;
    }
    return outString;
}

public static String unScramble(String inString, int inKey)
{
    char[] outChars = new char[inString.length()];
    String outString = "";
    if (inString.length() > 0) 
    {
      inString.getChars(0, inString.length(),outChars,0);
      char cypherFeedback = (char) inKey;
      cypherFeedback ^= outChars[0];
      outString += cypherFeedback;
      for (int i = 1; i < outChars.length; i++)
      {
        cypherFeedback = (char) (outChars[i] ^ outChars[i-1]);
        outString +=  cypherFeedback;
      }
    }
    return outString;
}

public static String getBaseString(String inString, String type, String delimiter)
{
    String result = inString;
    
    if (type.equalsIgnoreCase("OCTAL") == true)
        result = toOctal(inString, delimiter);
    else if (type.equalsIgnoreCase("HEX") == true)
        result = toHex(inString, delimiter);
    else if (type.equalsIgnoreCase("DECIMAL") == true)
        result = toDecimal(inString, delimiter);
        
    return result;
}

public static String toOctal(String inString, String delimiter)
{
    String result = "";
    String xDelimiter = "";
    char[] outChars = new char[inString.length()];
    
    inString.getChars(0, inString.length(),outChars,0);
    for (int i = 0; i < outChars.length; i++)
    {
        String octString = Integer.toOctalString((int)outChars[i]);
        switch (octString.length())
        {
            case 0:
                result += xDelimiter + "000";
                break;
            case 1:
                result += xDelimiter + "00" + octString;
                break;
            case 2:
                result += xDelimiter + "0" + octString;
                break;
            default:
                result += xDelimiter + octString;
                break;
        }
        xDelimiter = delimiter;
    }
    return(result);
}

public static String toHex(String inString, String delimiter)
{
    String result = "";
    String xDelimiter = "";
    char[] outChars = new char[inString.length()];
    
    inString.getChars(0, inString.length(),outChars,0);
    for (int i = 0; i < outChars.length; i++)
    {
        String hexString = Integer.toHexString((int)outChars[i]);
        switch (hexString.length())
        {
            case 0:
                result += xDelimiter + "00";
                break;
            case 1:
                result += xDelimiter + "0" + hexString;
                break;
            default:
                result += xDelimiter + hexString;
                break;
        }
        xDelimiter = delimiter;
    }
    return(result);
}
public static String toDecimal(String inString, String delimiter)
{
    String result = "";
    String xDelimiter = "";
    char[] outChars = new char[inString.length()];
    
    inString.getChars(0, inString.length(),outChars,0);
    for (int i = 0; i < outChars.length; i++)
    {
        String decString = Integer.toString((int)outChars[i]);
        switch (decString.length())
        {
            case 0:
                result += xDelimiter + "0";
                break;
            case 1:
                result += xDelimiter + decString;
                break;
            default:
                result += xDelimiter + decString;
                break;
        }
        xDelimiter = delimiter;
    }
    return(result);
}


/****   format methods *****/    
    public static String format(String pattern, double value)
    {
        DecimalFormat formatter = new DecimalFormat(pattern);
        return (formatter.format(value));
    }
    
/****   Date/time functions ****/    
    public static Date stringToDate(String dateString)
    {
        return (new Date(dateString));
        
    }
    
    public static Date today()
    {
        Calendar day = new GregorianCalendar();
        return (day.getTime());
    }
    
//calculate the time differences between current time and the time
//given in the parameter. Time format: hh:mm
    public static Date setTime(String time)
    {
        StringTokenizer st = new StringTokenizer(time, ":");
        int i = 0;
        int hour = 0;
        int minute = 0;
        
        while (st.hasMoreTokens())
        {
            switch (i)
            {
                case 0:
                    hour = Integer.parseInt(st.nextToken());
                    break;
                case 1:
                    minute = Integer.parseInt(st.nextToken());
                    break;
            }
            i++;
        }
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, hour);
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.SECOND, 0);
        return (calendar.getTime());
    }
    
    
    public static int days(Date secondDate, Date firstDate)
    {
        int totalDays = 0;
        boolean flipDates = false;
        GregorianCalendar day2 = new GregorianCalendar();
        day2.setTime(secondDate);
        GregorianCalendar day1 = new GregorianCalendar();
        day1.setTime(firstDate);
        
        int y1= day1.get(Calendar.YEAR);
        int y2 = day2.get(Calendar.YEAR);
//same year
        if (y2 == y1)
        {
            totalDays = day2.get(Calendar.DAY_OF_YEAR) - day1.get(Calendar.DAY_OF_YEAR);
        }
        else
        {
//two different years: swap if secondDate is before the firstDate
            if (y2 < y1)
            {
                GregorianCalendar temp = day2;
                day2 = day1;
                day1 = temp;
                y1= day1.get(Calendar.YEAR);
                y2 = day2.get(Calendar.YEAR);
                flipDates = true;
            }
            
            GregorianCalendar yearEnd = new GregorianCalendar();
            while ((y2-y1) > 0)
            {
                yearEnd.setTime(Utilities.stringToDate("12/31/" + y1));
                totalDays += yearEnd.get(Calendar.DAY_OF_YEAR) - day1.get(Calendar.DAY_OF_YEAR);
                totalDays++;  //add one day for crossing the year.
                y1++;
                day1.setTime(Utilities.stringToDate("01/01/"+y1));
            }
            totalDays += day2.get(Calendar.DAY_OF_YEAR) - day1.get(Calendar.DAY_OF_YEAR);
            if (flipDates == true)
                totalDays = totalDays * -1;
        }
        return(totalDays);
    }   
    public static int compareDates(Date firstDate, Date secondDate)
    {
        return (days(firstDate, secondDate));
    }
    
    public static boolean isWeekend(Date date)
    {
        
        Calendar day = new GregorianCalendar();
        day.setTime(date);

        int dayOfWeek = day.get(Calendar.DAY_OF_WEEK);
        
        return(((dayOfWeek == 1) || (dayOfWeek == 7)) ? true : false);
    }
    
/**** Keyboard functions *****/ 
    public static void pressAnyKey(String message)
    {
        System.out.println(message);
        pressAnyKey();
        
    }
    public static void pressAnyKey()
    {
        try
        {
            InputStreamReader isr = new InputStreamReader(System.in);
            BufferedReader input = new BufferedReader(isr);

            System.out.print("Press Enter Key to continue.");
            String strIn = input.readLine();
        }
        catch(IOException exception)
        {
            System.out.println("I got an i/o error!!!   " + exception);
        }
    }
    
/****** String functions *****/ 

    public static String getTrim(String x) {
      if (x == null) return "";
      return x.trim();
    }

    public static String rightTrim(String string)
    {
        int strLen = string.length() - 1;
            
        while ((strLen >= 0) && (string.charAt(strLen) == ' '))
            strLen--;
        
        return(string.substring(0,strLen+1));
    }
    
    public static String removeDecimals(String num)
    {
        StringTokenizer tk = new StringTokenizer(num, ".");
        return(tk.nextToken());
               
    }
    
    public static int findString(String searchString, String[] table)
    {
        int index = 0;
        boolean found = false;
        
        if ((searchString != null) && (table != null))
        {
            while ((index < table.length) && (found == false))
            {
                if (searchString.equalsIgnoreCase(table[index]) == true)
                    found = true;
                else
                    index++;
            }
        }   
        
        if (found == false)
            index = -1;
            
        return(index);
        
    }
    
    public static int findString(String searchString, Vector table)
    {
        int index = 0;
        boolean found = false;
        
        if ((searchString != null) && (table != null))
        {
            while ((index < table.size()) && (found == false))
            {
                if (searchString.equalsIgnoreCase((String) table.elementAt(index)) == true)
                    found = true;
                else
                    index++;
            }
        }   
        
        if (found == false)
            index = -1;
            
        return(index);
        
    }
    
    public static String replaceStr(String x, String oldChar, String newChar) 
    {
      int startLocate = 0;
      int oldLength = oldChar.length();
      int newLength = newChar.length();
      
      int locate = x.indexOf(oldChar, startLocate);
      while (locate != -1) {
          x = x.substring(0,locate) + newChar + x.substring(locate + oldLength);
          startLocate = locate + newLength;
          locate = x.indexOf(oldChar, startLocate);
      }
      return x;
    }
    
    //Originally in DB2Client - ensures a JS compatible array
    public static String getClientString(String x) {
      x = replaceStr(x, "\"","'");
      x = replaceStr(x, "\\","\\\\"); // was "&#92"
      x = replaceStr(x, "\n","\\n");  // was "&#10;"
      x = replaceStr(x, "\r","");

      return x; 
    }
    
    
    //Used to ensure string for XML String
    public static String getXMLString(String x) {
      // x = repStr(x, "\'", "\'");
      x = replaceStr(x, ";", ":");
      x = replaceStr(x, "&", "&amp;");
      x = replaceStr(x, "<", "");
      x = replaceStr(x, "", "\'");
      x = replaceStr(x, ">", "&gt;");
      x = replaceStr(x, "\"", "\'");
      x = replaceStr(x, "\n", "&#10;");
      x = replaceStr(x, "\r", "");
      x = replaceStr(x, "\\", "&#92;");
      x = replaceStr(x, "[", "&#91;");
      x = replaceStr(x, "]", "&#93;");
      // Do we need to add the greater than 127 here?
      return x; 
    }

    //Used to ensure string for XML String
    public static String getIncomingXMLString(String x) {
      x = replaceStr(x, "&", "&amp;");
      return x; 
    }
    
    //Used to ensure string for JS Arrays
    public static String getJSArrayString(String x) {
      x = replaceStr(x, "'", "\\\'");
      x = replaceStr(x, "", "\\\'");
      x = replaceStr(x, "[", "&#91;");
      x = replaceStr(x, "]", "&#93;");
      return x; 
    }
    
   //Used to ensure string for SQL - Need to add & replacement
   public static String getSQLString(String x) {
      x = replaceStr(x, "\'", "\'\'");
/*
Need to figure out which characters need to be escaped.
*/
      return x; 
   }
   
   
   //Used to ensure string for SQL 
   public static String getFlatString(String x) {
      x = replaceStr(x, "\"", "\'");
      x = replaceStr(x, ",", ";");
      x = replaceStr(x, "\n", ":");
      x = replaceStr(x, "\r", ":");
      return x; 
   }
   
   //Used to ensure string for export file 
    public static String removeReturn(String x) {
       x = Utilities.replaceStr(x, ",", ";");
       x = Utilities.replaceStr(x, "\n", ":");
       x = Utilities.replaceStr(x, "\r", ":");
       return x; 
    }

    //Used to ensure string for export file 
    public static String removeReturn(String x, String delimiter) {
    	if (delimiter == "," )
    	{
    		x = Utilities.replaceStr(x, ",", ";");
    	} else {
    		x = Utilities.replaceStr(x, delimiter, " ");
    	}
    	x = Utilities.replaceStr(x, "\n", " ");
    	x = Utilities.replaceStr(x, "\r", " ");
       return x; 
    }

   
  public static String getNReplace(String x, String oldVal, String newVal) {
    // x = rough sql statement e.g., "select $name$ from USERTABLE;" 
    // oldVal = token e.g., '$name$'
    // newVal = value e.g., 'employee_name'
    if (newVal != null) {
      int y = x.indexOf(oldVal);
      while (y > -1) {
         x = x.substring(0,y) + newVal + x.substring(y + oldVal.length());
         y = x.indexOf(oldVal);
      }   
    }
    return x;
  }
  
 public static String getEscapeStrategy(String x) {
    
   char z = 0;
   String y = "";
   for (int j = 0; j < x.length(); j++) {
     z = x.charAt(j);
     if (z > 127) {
        if (z < 256) {
           y = "&#" + Short.toString((short) z) + ";";
        } else {
           y = "?";
        }
        x = x.substring(0,j) + y + x.substring(j+1);
        j = j + y.length() - 1;
     }
   }
   return x;
 }   

 public int searchStringArray(String[] S, String X)
 {
    int retVal = -1;
    for (int i = 0; i < S.length; i++)
    {
        if (S[i].equals(X))
        {
            retVal = i;
            i = S.length;
        }
    }
    return retVal;
 }
 
 // Number functions
    public static long addString2LongforCurrency(long total, String value)
    {
        double workVar = Double.valueOf(value).doubleValue();
        
        workVar = dRound(workVar) * (double) 100;
        Double x = new Double(workVar);
        long y = x.longValue() + total;
        return y;
    }
    
    public static long addString2Long(long total, String value)
    {
        double workVar = Double.valueOf(value).doubleValue();
        Double x = new Double(workVar);
        long y = x.longValue() + total;
        return y;
    }
    
    public static String makeLong2Currency(long value)
    {
        String retVal = Long.toString(value);
        if (retVal.length() == 2) retVal = "0" + retVal;  //JH 2006-10-30
        if (retVal.length() == 1) retVal = "00" + retVal; //JH 2006-10-30
        if (retVal.length() == 0) retVal = "000" + retVal; //JH 2006-10-30        
        int decimalPos = retVal.length() - 2;
        retVal = retVal.substring(0,decimalPos) + "." + retVal.substring(decimalPos, decimalPos + 2);
        return retVal;
    }
    
  private static double dRound(double x) {
    if (x >= (double) 0) {
      return x + (double) 0.001;
    } else {
      return x - (double) 0.001;
    } 
  }
     
  public static ArrayList make1SListFrom2(String[] tagList, String[]valueList)
  {
    ArrayList values = new ArrayList(); 
    String ParamName;
    String ParamValue;
    String[] Params;
    for (int i = 0; i < tagList.length; i++){
       Params = new String[2];
       ParamName = tagList[i];
       ParamValue = valueList[i]; 
       Params[0] = ParamName;
       if (ParamValue == null) ParamValue = "";
       Params[1] = ParamValue;     
       values.add(Params);
    }
    return values;
  }
  
  public static boolean equalStrings(String x)
  {
    // format is =("x","y")
    // change =(" to /
    // change the ") to /
    // change the "," to /
    x = x.trim();
    AuditTrail.println("[000] Utilities.equalStrings: " + x);    
    x = replaceStr(x,"\",\"","/");
    x = replaceStr(x,"=(\"","/");
    x = replaceStr(x,"\")","/");
    return equalStrings(x, "/");
  }
     
  public static boolean equalStrings(String x, String delim)
  {
    boolean retVal = false;
    // x format is /x/y/ if delim is /
    x = x.trim();    
    StringTokenizer st = new StringTokenizer(x, delim, false);
    int i = 0;
    String[] items = new String[2];   
    while (st.hasMoreTokens() && i < 3)
    {
        items[i++] = st.nextToken();
    }
    switch (i)
    {
       case 0:
         AuditTrail.println("[500] Utilities.equalStrings 0 item parse error: " + x);
         break;
       case 1:
         AuditTrail.println("[500] Utilities.equalStrings 1 item parse error: " + x);
         break;
       case 2:
         retVal = items[0].equalsIgnoreCase(items[1]);
         break;
       default:
         AuditTrail.println("[500] Utilities.equalStrings too many items parse error: " + x);
         break;
    }
    return retVal;
  }
  public static boolean equalStringsIgnoreCase(String pattern, String x)
  {
    // matches against a simple pattern of either 'Abc' or 'Ab*' against the String x.
    // if x has an asterick, the wild card is disabled
    
    boolean retVal = false;
    int asterick = pattern.indexOf("*");
    if (asterick > -1 && x.indexOf("*") == -1)
    {
        pattern = pattern.substring(0, asterick);
        if (x.length() >= asterick)
        {
            x = x.substring(0,asterick);
        }
    }
    retVal = x.equalsIgnoreCase(pattern);
    return retVal;
  }
  
  public static boolean isEmptyStr(String x)
  {
	  return ((x == null) || (x.length() == 0) );
  }
}
