
/*
ReportDisplayESS.java - XML to Central Database and Report class, Generic Report Object 
Copyright (C) 2005 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.io.*;
//import java.util.*; 
//import java.text.*;
//import java.sql.*;
//import org.jdom.Document;
//import org.jdom.Element;
//import org.jdom.Attribute;
//import org.jdom.DocType;

import ess.DepartmentTable;
import ess.ReportElementAPI;
import ess.Utilities;
//import ess.Report.ESSExpense;
//import ess.Report.ESSExtended;
//import ess.Report.ESSReceipt;

//import ess.Report.CompareXMLExpensesByDate;
//import ess.Report.CompareXMLExpensesDeptProj;
//import ess.Report.ESSGL;
//import ess.ReportDisplayAudit.ESSGrid;

public class V2DB2ESS extends ess.DB2ESS {

  private ess.Language reportLang = new ess.Language();

  public String printHeader() {
	    String retVal = SecHeader(myLang.getString("persInfoSec"));
	    retVal += btable;
	    retVal += h1row(myLang.getLabel("name"),Rep.NAME, myLang.getLabel(reportReferenceName),Rep.VOUCHER); //Need to look at this for voucher/reference
	    retVal += h1row(myLang.getLabel("persnum"),dispPersNo(Rep.PERS_NUM),myLang.getLabel("status"),myLang.getDataString(StatusDOM.getDOMTableValueWhere("default","translation",Rep.RP_STAT,"screen")));
	    // retVal += h1row(myLang.getLabel("depart"),departTable.descrip +  " (" + Rep.DEPART + ")",myLang.getLabel("credate"),date.getUserDateStr(Rep.CRE_DATE, dateFormatStr));
	    retVal += h1row(myLang.getLabel("depart"), Rep.DEPART +  " (" + getDepartDescrip() + ")",myLang.getLabel("credate"),date.getUserDateStr(Rep.CRE_DATE, dateFormatStr));
	    retVal += h1row("","",myLang.getLabel("pvoucher"),Rep.PVOUCHER);
	    if (!Rep.VEHICLE.equals("")) {
	      retVal += h1row(myLang.getLabel("vehicle"),Rep.VEHICLE,myLang.getLabel("servdate"),date.getUserDateStr(Rep.SERVDATE, dateFormatStr));
	      retVal += h1row(myLang.getLabel("start"),Rep.START,myLang.getLabel("finish"),Rep.FINISH);
	      retVal += h1row(myLang.getLabel("business"),Rep.BUSINESS,myLang.getLabel("weekend"),date.getUserDateStr(Rep.WEEKEND, dateFormatStr));
	    }    
	    if (!getCurrency().equals("")) {
	      retVal += h1row(myLang.getLabel("company"),getCompany(),myLang.getLabel("currency"),getCurrency()); 
	      if (!getHistory().equals("")) {
	          retVal += h1row("","",myLang.getLabel("history"),getHistory()); 
	      }
	    } else {
	      if (!getHistory().equals("")) {
	          retVal += h1row(myLang.getLabel("company"),getCompany(),myLang.getLabel("history"),getHistory()); 
	      } else {
	         retVal += h1row(myLang.getLabel("company"),getCompany(),"",""); 	
	      }
	    }
	    retVal += "</table>\n";
	    return retVal;
  }


  private String getDepartDescrip() {
	  if (super.departTable == null)
	  {
		  super.departTable = new DepartmentTable();
		  super.departTable.setConnection(super.dbCon);
		  super.departTable.setSQLTerminator(super.sqlTerminator);  //JH 2005-09-15
		  super.departTable.setSQLStrings();
	  }
	  super.departTable.setDepartInfo(Rep.DEPART, Rep.COMPANY);
	  return departTable.descrip;
  }
  public String printAuditInfo() {
	    String uploadStatus;
	    if (Rep.UP_STAT.equals("")) {
	        uploadStatus = "";
	    } else {
	        uploadStatus = StatusDOM.getDOMTableValueWhere("default","translation",Rep.UP_STAT,"auditmessage");
	    }
	    if (uploadStatus.equals("")) uploadStatus = Rep.UP_STAT;
	    String retVal = SecHeader(myLang.getString("auditInfoSec"));
	    retVal += btable;
	    resetColors();
	    retVal += h1row(myLang.getLabel("pvoucher"),Rep.PVOUCHER,myLang.getLabel("inputBy"),getNameFmPers(Rep.CLERK));
	    retVal += h1row(myLang.getLabel("subdate"),date.getUserDateStr(Rep.SUB_DATE,dateFormatStr),"","");
	    retVal += h1row(myLang.getLabel("supervisor"),getNameFmPers(PT.getManager()),myLang.getLabel("curdate"),date.getUserDateStr(Rep.CUR_DATE,dateFormatStr));
	    retVal += h1row(myLang.getLabel("sign1"),getNameFmPers(Rep.SIGN1),myLang.getLabel("sg1date"),date.getUserDateStr(Rep.SG1_DATE,dateFormatStr));
	    // retVal += h1row(myLang.getLabel("sign3"),getNameFmPers(Rep.SIGN3),myLang.getLabel("auddate"),date.getUserDateStr(Rep.AUD_DATE,dateFormatStr));
	    
	    // retVal += h1row("Approved:"," "," "," ");
	    retVal += "</table>\n";
	    return retVal;
  }

  public String printExpenseSummary() {
	    ESSGL R;
	    float expTotal = 0;
	    //float expBillable = 0;
	    float expPersonal = 0;
	    String retVal = SecHeader(myLang.getString("expSumSec"));
	    retVal += btable;
	    resetColors();
	    retVal += l1row(chd(myLang.getString("expense")),chd(myLang.getString("amount")),chd(myLang.getString("personal")),"");
	    switchColors();
	    for (int i = 0; i < ESSGLInfo.size(); i++) {
	      R = (ESSGL) ESSGLInfo.elementAt(i);
	      retVal += printItem(R);
	      expTotal += R.SUM;
	      expPersonal += R.PERSONAL;  
	      switchColors();
	    }
	    retVal += l2row(myLang.getLabel("totals"),expTotal,expPersonal);
	    switchColors();
	    // retVal += l3row(myLang.getLabel("nonpers"),expTotal - expPersonal);
	    retVal += "</table>\n";
	    return retVal;
  }
  
  private String printItem(ESSGL R) {
	    return l2row(myLang.getDataString(R.NAME), R.SUM, R.PERSONAL); 
  }  

  public String printExpense(ESSExpense ESSE, ESSReceipt ESSR) {
	    String x;
	    String cmt;
	    String sComment = ESSE.COMMENT;

	    //sComment = Utilities.replaceStr(sComment, myLang.getDataString("JS_ATTENDEE"), " ");
	     
	    sComment = sComment.replaceFirst(reportLang.getDataString("JS_ATTENDEE").trim(), "").trim();

	    //ess.AuditTrail.println("[999] sComment: '" + sComment + "'");
	    //ess.AuditTrail.println("[999] JS_ATTENDEE: '" + myLang.getDataString("JS_ATTENDEE") + "'");
	    
	    if (AddMerchant2Comment.equalsIgnoreCase("YES"))
	    {
	        if (ESSR.MERCHANT != null && !ESSR.MERCHANT.equals(""))
	        {
	           sComment = sComment + " (" + ESSR.MERCHANT + ")";
	        }
	    }
	    if (ESSR.REFER != null && !ESSR.REFER.equals(""))
	    {
	    	if (Utilities.isEmptyStr(sComment)) {
	    		sComment = myLang.getLabel("expReference") + " " + ESSR.REFER;
	    	} else {
	            sComment = sComment + " " + myLang.getLabel("expReference") + " " + ESSR.REFER;
	    	}
	    }
	    
        x = x1rowStH(date.getUserDateStr(ESSE.EXP_DATE,dateFormatStr), ESSE.AMOUNT, myLang.getDataString(ESSE.EXPENSE), myLang.getDataString(ESSR.LOCATION),sComment);
	    
	    if (!ESSR.OFFSET.equals("")) {
	    	x += x1rowStH("", "", itl(myLang.getLabel("to")), myLang.getDataString(ESSR.OFFSET),"");
	    }

	    if (!(ESSR.AMOUNT.equals(ESSR.REC_AMT)) && !(ESSR.REC_AMT.equals("0.00"))) {  //printout mileage, foreign ex if applicable jh 11/5/2002
            cmt = getCmt(ESSR.RECEIPT ,ESSR.UNITS,ESSR.XSOURCE,ESSR.XDATE);
            x +=  x1rowStH(" ",itl(myLang.getDataLabel(ESSR.UNITS)),itl(ESSR.REC_AMT),itl(myLang.getLabel("rate")) + " " + itl(ESSR.XRATE),itl(cmt));
        }

	    return x;
  }
  public String printExpenses() {
	    float tripTotal = 0;
	    String retVal =  SecHeader(myLang.getString("expenseList"));
	    retVal += btable;
	    ESSExpense ESSE;
	    ESSExtended ESSX;
	    ESSReceipt ESSR;
	    
	    reportLang.setLanguage(PT.getLanguage());
	    
	    boolean firstAttendee; 
	       resetColors();
	       retVal = expenseListColumnTitles(retVal);
	       tripTotal = 0;
	       switchColors();

	    for (int j = 0; j < ESSExpenses.size(); j++) {
	      ESSE = (ESSExpense) ESSExpenses.elementAt(j);
	      ESSR = (ESSReceipt) super.getESSReceipt(ESSE.RECEIPT);
          retVal += printExpense(ESSE, ESSR);
	      tripTotal += Float.parseFloat(ESSE.AMOUNT);
	      firstAttendee = true; 
	      for (int k = 0; k < ESSExtendeds.size(); k++) {
	         ESSX = (ESSExtended) ESSExtendeds.elementAt(k);
	         if (ESSX.LINEITEM.equals(ESSE.LINEITEM)) {
	           if (firstAttendee) {
	             retVal += printDiscussion(ESSX.LOCATION, ESSX.COMMENT);
	             firstAttendee = false;
	           }
	           retVal += printAttendee(ESSX);  
	         }   
	      } 
	      switchColors();
	    }
	    retVal += x1rowStH(" ","====== "," "," "," ");
	    switchColors();
	    retVal += x1rowStH(myLang.getLabel("total"),FF(FRoundStr(tripTotal)),"","","");
	    retVal += "</table><br>\n";
	    
	    return retVal;
  }
  private String printDiscussion(String v1, String v2) {
	    String retVal = tr;
	    retVal += htd("100"," ");
	    if (v1 == null || Utilities.isEmptyStr(v1)) {
	        retVal += htd("100","");  //place of meeting
	    } else {
	    	retVal += htd("100",itl(myLang.getLabel("location")));
	    }
	    retVal += htd("120",itl(v1));  
	    retVal += htd("120",itl(myLang.getLabel("discussion")));  //discussion
	    retVal += htd("300",itl(v2));  
         
	    retVal += tre;
	    return retVal;
  }
  protected String x1rowStH(String v1, String v2, String v3, String v4, String v5) {
	    String retVal = tr;
	    retVal += htd("100",v1);  //expense date
	    retVal += htd("100",v2);  //amount 
	    retVal += htd("120",v3);  //expense type
	    retVal += htd("120",v4);  //billable
	    retVal += htd("300",v5);    //comment
	    retVal += tre;
	    return retVal;
  }
  private String printAttendee(ESSExtended ESSX) {
	    String retVal;
	    if (Check4Delete( (ReportElementAPI) ESSX ) ) {
	        retVal = x2rowStH(ESSX.NAME, ESSX.CLIENT);
	    } else {
	        retVal = "";
	    }
	    return retVal;
  }
  private String x2rowStH(String v1, String v3) {
        return x1rowStH(" ",itl(myLang.getLabel("attendee")),itl(v1),itl(myLang.getLabel("company")),itl(v3));	  
  }

  protected String expenseListColumnTitles(String retVal) {
          retVal += x1rowStH(chd(myLang.getString("expdate")),chd(myLang.getString("amount")),chd(myLang.getString("type")),chd(myLang.getString("location")),chd(myLang.getString("comment"))) 
          + "<br>\n";
       return retVal;
  }
  
}