/*
WorkFlow.java - Batch email workflow controller  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.net.*;
import java.io.*;
//import java.util.Timer.*;
//import java.util.TimerTask.*;
import java.util.*;
//import java.awt.*;  //for testing the Frame class

//import org.jdom.DocType;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;

public class WorkFlow implements Runnable 
{
    static AuditTrail Logfile;
    //private static ProcessESSReport procReport = null;
    //private static ProcessESSApproval procApproval = null;
    //private static ProcessReport procUnrecognized = null;
    private static AdisoftDbase db;
    private static Register reg;
    
/* variables for the WorkFlow class */    
    private ServerSystemTable systemTable;
    final int GETMAIL = 0;
    final int PROCESSMAIL = 1;
    final int SENDTESTMAIL = 2;
    
    Timer getMailTimer = new Timer();
    Timer cancelMailTimer = new Timer();
    
    private int task2perform;
    static Boolean ProcessRunning = new Boolean(false);
    
    public WorkFlow(int task, String DataBaseUrl, String DataBaseUser, String DataBasePassword, String DBSQLTerminator)
    {
    	setWorkFlow(task, DataBaseUrl, DataBaseUser, DataBasePassword, DBSQLTerminator, "");
    }
    
    public WorkFlow(int task, String DataBaseUrl, String DataBaseUser, String DataBasePassword, String DBSQLTerminator, String DBDriverClassName)
    {
    	setWorkFlow(task, DataBaseUrl, DataBaseUser, DataBasePassword, DBSQLTerminator, DBDriverClassName);
    }

    public void setWorkFlow(int task, String DBUrl, String DBUser, String DBPassword, String sqlTerminator, String DBDriver)
    {
        db = new AdisoftDbase(DBUrl, DBUser, DBPassword,DBDriver);
        db.setSQLTerminator(sqlTerminator);
        reg = new Register();
        reg.setConnection(db.getConnection());
        reg.setSQLTerminator(sqlTerminator);
        systemTable = new ServerSystemTable();
        systemTable.setConnection(db.getConnection());
        systemTable.setSQLTerminator(sqlTerminator);
        task2perform = task;
    }
    
    public boolean getAlreadyRunning()
    {
      synchronized(ProcessRunning) {
          boolean retVal = true;
          if (!ProcessRunning.booleanValue())
          {
              ProcessRunning = Boolean.valueOf("true");
              retVal = false;
          }
          return retVal;
      }
    }
    
    public void setProcessNotRunning()
    {
      ProcessRunning = Boolean.valueOf("false");
    }


    public void run() {

        switch (task2perform )
        {
            case GETMAIL:
                scheduleGetMail(systemTable);
                break;
//remove this after test.                
            case PROCESSMAIL:
                GetMail gm = new GetMail(systemTable);
                gm.run();
                break;
            case SENDTESTMAIL:
                sendTestEmail(systemTable);
                break;
            default: 
            	AuditTrail.println("[500] WorkFlow.run: Task not defined ");
                break;
        }
    }
        private void sendTestEmail(ServerSystemTable systemTable)
        {
                EMailXMLFile sendEmail = new EMailXMLFile();
                sendEmail.setSystemTable(systemTable);
                sendEmail.setOwner("jim@expenseservices.com");
                sendEmail.setXMLMessage("<TEST>This is a test message</TEST>");
                System.out.println("sendEmail status = " + sendEmail.getStatus());
        }

/**************************************************
Mail retrival time is defined in GATEWAY_INTERVAL.
Default is set to retrieving the mail every 15 minutes.

***************************************************/
    private void scheduleGetMail(ServerSystemTable systemTable)
    {
//set timer to get mail            
        int minutes = 0;
        minutes = systemTable.getSystemInt("gateway_interval");
//default to 15 minutes        
        minutes = minutes > 0 ? minutes : 15;
        AuditTrail.println("[000] WorkFlow.scheduleGetMail interval: " + Integer.toString(minutes));
        getMailTimer.schedule(new GetMail(systemTable), 1, minutes * 60*1000);
//Set timer to cancel get mail task        
        String stopTime = systemTable.getSystemString("gateway_stop_time", "21:45");
        if (!stopTime.equalsIgnoreCase("No"))
        {
           AuditTrail.println("[000] WorkFlow.scheduleGetMail stop time: " + stopTime); 
           cancelMailTimer.schedule(new CancelMail(),Utilities.setTime(stopTime));
        } else {
        	AuditTrail.println("[000] WorkFlow.scheduleGetMail no stop time specified");
        }
    }
/*******************************************************************
cancelMail terminates the getMail timer so that it would not 
download anymore emails until the program is reloaded again.
*******************************************************************/
    class CancelMail extends TimerTask
    {
        public void run()
        {
        	AuditTrail.println("[000] WorkFlow.CancelMail timer cancelled");
            getMailTimer.cancel();
            cancelMailTimer.cancel();
        }
        
    }
    
    class GetMail extends TimerTask
    {
        ServerSystemTable systemTable;
        POP3ReceiptImage conduit;
        
        public GetMail(ServerSystemTable systemTable)
        {
        	this.systemTable = systemTable;
        	conduit = new POP3ReceiptImage(systemTable.getConnection());
        }
        
        public void run()
        {
        	AuditTrail.println("[000] WorkFlow.run: GetMail task has been called.");
        		conduit.run();

        }
        
        
     }
    
        
}
