/*
WorkFlow.java - Batch email workflow controller  
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.net.*;
//import java.io.*;
//import java.util.Timer.*;
//import java.util.TimerTask.*;
import java.util.*;
//import java.awt.*;  //for testing the Frame class
import java.sql.*;
import org.jdom.Document; //See if I can find this
import org.jdom.Element;
//import org.jdom.DocType;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;

public class WorkFlow_Original 
{
    static AuditTrail Logfile;
    private static ProcessESSReport procReport = null;
    private static ProcessESSApproval procApproval = null;
    private static ProcessReport procUnrecognized = null;
    private static AdisoftDbase db;
    private static Register reg;
    
/* variables for the WorkFlos class */    
    private ServerSystemTable systemTable;
    final int GETMAIL = 0;
    final int PROCESSMAIL = 1;
    final int SENDTESTMAIL = 2;
    
    Timer getMailTimer = new Timer();
    Timer cancelMailTimer = new Timer();
    
    private String sqlTerminator = ";";
    
    public WorkFlow_Original(int task, String DBUrl, String DBUser, String DBPassword, String DBSQLTerminator)
    {
        sqlTerminator = DBSQLTerminator;
        db = new AdisoftDbase(DBUrl);
        db.setSQLTerminator(sqlTerminator);
        reg = new Register(DBUrl, DBUser, DBPassword);
        reg.setSQLTerminator(sqlTerminator);
        systemTable = new ServerSystemTable(DBUrl);
        systemTable.setSQLTerminator(sqlTerminator);

        switch (task)
        {
            case GETMAIL:
                scheduleGetMail(DBUrl, DBUser, DBPassword);
                break;
//remove this after test.                
            case PROCESSMAIL:
                GetMail gm = new GetMail(DBUrl, DBUser, DBPassword);
                gm.processIncomingMessages(DBUrl, DBUser, DBPassword);
                break;
            case SENDTESTMAIL:
                sendTestEmail(DBUrl, DBUser, DBPassword);
                break;
            default: 
                System.out.println("Task not defined.");
                break;
        }
    }
        private void sendTestEmail(String DBUrl,  String DBUser, String DBPassword)
        {
//            for (int i = 0; i < 2; i++)
//            {
            
                EMailXMLFile sendEmail = new EMailXMLFile(DBUrl, DBUser, DBPassword);
                sendEmail.setOwner("kpchen@pacbell.net");
                sendEmail.setXMLMessage("<TEST>This is a test message</TEST>");
                System.out.println("sendEmail status = " + sendEmail.getStatus());
//            }
        }

/**************************************************
Mail retrival time is defined in GATEWAY_INTERVAL.
Default is set to retrieving the mail every 15 minutes.

***************************************************/
    private void scheduleGetMail(String DBUrl, String DBUser, String DBPassword)
    {
//set timer to get mail            
        int minutes = 0;
        minutes = systemTable.getSystemInt("GATEWAY_INTERVAL");
//default to 15 minutes        
        minutes = minutes > 0 ? minutes : 15;
        getMailTimer.schedule(new GetMail(DBUrl, DBUser, DBPassword), 1, minutes * 60*1000);
        
//Set timer to cancel get mail task        
        String stopTime = systemTable.getSystemString("GATEWAY_STOP_TIME", "21:45");
        cancelMailTimer.schedule(new CancelMail(),Utilities.setTime(stopTime));
    }
/*******************************************************************
cancelMail terminates the getMail timer so that it would not 
download anymore emails until the program is reloaded again.
*******************************************************************/
    class CancelMail extends TimerTask
    {
        public void run()
        {
            System.out.println("Cancelling Get Mail Task..."); 
            getMailTimer.cancel();
            cancelMailTimer.cancel();
        }
        
    }
    
    class GetMail extends TimerTask
    {
        final int REPORT = 0;
        final int APPROVAL = REPORT + 1;
        String[] fileTypes = {"report", "approval"};
        String DBUrl = "";
        String DBUser = "";
        String DBPassword = "";
//        ServerSystemTable systemTable;
        
        public GetMail(String DBUrl, String DBUser, String DBPassword)
        {
            this.DBUrl = DBUrl;
            this.DBUser = DBUser;
            this.DBPassword = DBPassword;
            
            systemTable = new ServerSystemTable(DBUrl, DBUser, DBPassword);
        }
        
        public void run()
        {
            processIncomingMessages(DBUrl, DBUser, DBPassword);
//debug send another batch for the next time.            
//            sendTestEmail(DBUrl);
        }
        
        
        public void processIncomingMessages(String DBUrl, String DBUser, String DBPassword)
        {
            int exitStatus = 0;
            
            if (db.setResultSet("SELECT XMLFILE, REFERENCE FROM REGISTER WHERE STATUS = 'Incoming'"))
            {
                try{
                    ResultSet rs = db.getResultSet();
                    do{
                        String file = rs.getString("XMLFILE");
//debug code                        file = "c:/visualcafe/projects/adisoft/testcases/1.xml";
//Convert xml file to dom format                        
                        AdisoftDOM incoming = new AdisoftDOM(file);
                        Document incomingDOM = incoming.getDOM();
                        Element root = incomingDOM.getRootElement();
//get reference # from register.dbf                        
                        String reference = rs.getString("REFERENCE");
//Update register - stamp sender in the owner field 
                        try
                        {
                            reg.updateColumn(reference, "OWNER",(root.getChild("Reply-To")).getText());
                        }
                        catch(Exception e)
                        {
                            AuditTrail.println("[500] WorkFlow- Can't update register file: No Reply-To in the message: " + file);
                        }
                        if ((exitStatus = routeFile(DBUrl, DBUser, DBPassword,
                                           incomingDOM,file,getFileType(root))) == 0)
                            reg.updateColumn(reference,"STATUS", "Processed");
                    }while ((rs.next() == true) && (exitStatus == 0));
                    
                    closeRoutingProcesses();
                }
                catch (java.lang.Exception ex){
                    AuditTrail.println("[000] Workflow.processIncomingMessages: No message to process");
                }
            } else {
            AuditTrail.println("[000] Workflow.processIncomingMessages: No message to process");  
            }
//            return(exitStatus);          
        }
        
        private String getFileType(Element root)
        {            
            String fileType = "";
            
            String rootName = (root.getName()).toUpperCase();
            if ((rootName.toUpperCase()).equals("REGISTER"))
            {
                try{
                    
                    fileType = root.getAttributeValue("type");
                }
                catch (Exception e)
                {
                    AuditTrail.println("[500] WorkFlow.processFile: Missing file type");
                }
            }
            else
            {
                AuditTrail.println("[500] WorkFlow.processFile: Unrecognizable root element - " + root.getName());
            }
            return(fileType);
        }
        
        private int routeFile(String DBUrl, String DBUser, String DBPassword, Document fileDOM, String fileName, String fileType)
        {
            ProcessReport nextBatch = null;
            String executeObject = null;
            int exitStatus = -1;
            
    //Need to batch fils so that we don't run out of resource openning too many files        
            try {
                switch (Utilities.findString(fileType, fileTypes))
                {
                case REPORT:
                    if (procReport == null)
                    {
                        procReport = (ProcessESSReport) getExecuteObject(fileType);
                        nextBatch = procReport;
                        if (nextBatch != null)
                            nextBatch.init(DBUrl, DBUser, DBPassword, sqlTerminator);
                    }
                    else
                        nextBatch = procReport;
                    break;
                case APPROVAL:
                    if (procApproval == null)
                    {
                        procApproval = (ProcessESSApproval) getExecuteObject(fileType);
                        nextBatch = procApproval;
                        if (nextBatch != null)
                            nextBatch.init(DBUrl, DBUser, DBPassword, sqlTerminator);
                    }
                    else
                        nextBatch = procApproval;
                    break;
                default:
                    if (procUnrecognized == null)
                    {
                        procUnrecognized = getExecuteObject("unrecognized");
                        nextBatch = procUnrecognized;
                        if (nextBatch != null)
                            nextBatch.init(DBUrl, DBUser, DBPassword, sqlTerminator);
                    }
                    else
                        nextBatch = procUnrecognized;
                    break;
                }
                
                if (nextBatch != null)
                {
                    nextBatch.run(fileDOM, fileName);
                    exitStatus = 0;
                }
                else
                    AuditTrail.println("[500] WorkFlow.routeFile: Can't find processing object (nextBatch = null)");
            } catch (Exception e) {
                e.printStackTrace();
                AuditTrail.println("[500] WorkFlow.routeFile:" + e.getMessage());
//                Utilities.pressAnyKey();
            }
            return(exitStatus);
        }
        
        private void closeRoutingProcesses()
        {
            if (procReport != null)
            {
                procReport.close();
                procReport = null;
            }
            if (procApproval != null)
            {
                procApproval.close();
                procApproval = null;
            }            
        }
        
    }
    
        private ProcessReport getExecuteObject(String fileType)
        {
            String result = "";
            String processFile = systemTable.getSystemString("XMLPROCESSTABLE","c:/visualcafe/projects/adisoft/processTable.xml");
            AdisoftDOM processTable = new AdisoftDOM(processFile);
            Document processDOM = processTable.getDOM();
            Element root = processDOM.getRootElement();
            String executeObject = "";
            ProcessReport processReport = null;

            if ((root.getName()).equals("table") && 
                        (root.getAttributeValue("name")).equals("fileprocess"))
            {
                Iterator rowSeqItr = root.getChildren().iterator();
                    
                while ((rowSeqItr.hasNext())&& (result.length() == 0))
                {
                    Element executeSeq = (Element)rowSeqItr.next();
    //get guideline details                    
                    
                    if (fileType.equals(executeSeq.getChildText("type")))
                        executeObject = executeSeq.getChildText("executeObject");
                        
                }
            }
            if (executeObject.length() > 0)
            {
                try
                {
                    processReport = (ProcessReport) (Class.forName(executeObject)).newInstance();
                }
                catch(Exception e)
                {
                    AuditTrail.println("[500] WorkFlow.getExceuteObject: Can' instantiate executeObject: " + executeObject);
                }
            }
                
            return(processReport);
        }
        
}
