
/*
XML2ESS.java - XML to Central Database and Report class, Generic Report Object 
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/

package ess;

//import java.io.*;
import java.util.*; 
//import java.text.*;
import java.sql.*;
import java.lang.Math;
import org.jdom.Document;
import org.jdom.Element;
//import org.jdom.Attribute;
//import org.jdom.DocType;

public class XML2ESS extends ReportDisplayESS {
    
  public XML2ESS () {
  }
  public XML2ESS (String Url, String User, String Password, String SQLTerminator){
    setDB(Url, User, Password, SQLTerminator);
  }
  
  public void set(AdisoftDOM ReportDOM) {
  //compatible to the Adisoft DOM extension  
    set(ReportDOM.getDOM());
  }  
  public void set(Document ReportDOM) {
    Element report = ReportDOM.getRootElement();
    set(report, 0);
  }
  
  public void set(Element report, int Link){
    setReport(report, Link);
    normalizeWeight();
    normalizeSplitClients();
    normalizeESSReport();
    setESSGL();
  }
    
  public void setReport(Element report, int Link) {
    setReport(report, Link, false);
  }  
                                                                                                              
  public void setReport(Element report, int Link, boolean useNewVoucher) {
  //calls the routines to load up the objects representing the various sections of an ESS report  
    java.util.List sections = report.getChildren();
    for (int i = 0; i < sections.size(); i++) {   //these are the sections
      Element section = (Element) sections.get(i);
      java.util.List sectionList = section.getChildren();
      if (sectionList.size() > 0) {
        String secType = section.getName();
        if (secType.equals("header")) {
          if (!setESSReport(section, useNewVoucher)){
            AuditTrail.println("[400] XML2ESS.setReport: Report process aborted");
            i = sections.size();
          }    
        } else if (secType.equals("purpose")) { 
          setESSPurpose(section); 
        } else if (secType.equals("split")) { 
          setSplitClient(section, Link); 
        } else if (secType.equals("receipt")) { 
          setESSReceipt(section);  
        } else if (secType.equals("expense")) { 
          setESSExpense(section, Link); 
        } else if (secType.equals("attendeelist")) { 
          setESSAttendeeList(section, Link);
        } else if (secType.equals("attendee")) { 
          setESSAttendee(section, Link);    
        } 
      }
    }
  }  
  
  public void setESSReport(Element report, int Link,  boolean useNewVoucher){
    setReport(report, Link, useNewVoucher);
    normalizeWeight();
    normalizeSplitClients();
    normalizeESSReport();
    setESSGL();
  }
  
  private boolean setESSReport(Element E, boolean useNewVoucher) {
    //used to create a report object.
    
    if (Rep.set(E, useNewVoucher)) {
      return true;
    } else {    
      AuditTrail.println("[500] XML2ESS.setESSReport: Could not find " + Rep.PERS_NUM);
      return false;
    }
  } 
  
  private void setESSReceipt(Element E) {
    //used to create a receipt object.
    String s1 = getValue(E, "odototal");
    if (s1.equals("")) {
    } else {   
      Rep.VEHICLE = getValue(E, "vehicle");
      Rep.START = getValue(E, "start");
      Rep.FINISH = getValue(E, "finish");
      Rep.BUSINESS = getValue(E, "business");
      Rep.SERVDATE = getValue(E, "servdate");
      Rep.WEEKEND = getValue(E, "weekend");
    } 
    String s2 = getValue(E, "amount"); //if now amounts - no expenses
    if (s2.equals("0.00")) s2 = "";
    String s3 = getValue(E, "recamt");
    if (s3.equals("0.00")) s3 = "";
    if (s2.equals("") && s3.equals("")) {
    } else {
      ESSReceipt P = new ESSReceipt();
      P.ExpenseLink = genReference++;
      P.set(E, P.ExpenseLink);
      ESSReceipts.addElement(P);
      setReport(E, P.ExpenseLink); 
    }
  } 
  
  private void setESSExpense(Element E, int ReceiptLink) {
    //used to create an expense object.
    ESSExpense P = new ESSExpense();
    P.set(E, ReceiptLink);
    ESSExpenses.addElement(P);
    setReport(E, P.LineLink);
  } 
  
  private void setESSPurpose(Element E) {
    //used to create a Purpose object.
    ESSPurpose P = new ESSPurpose();
    P.set(E);
    P.PurposeLink = Integer.parseInt(P.PURPOSE.substring(P.PURPOSE.length()-7));
    ESSPurposes.addElement(P);
    setReport(E, P.PurposeLink);
  } 
  
  protected void setESSAttendeeList (Element E, int ListLink) {
    //used to create a Purpose object.
    ESSAttendeeList P = new ESSAttendeeList();
    P.set(E, ListLink);
    ESSAttendeeLists.addElement(P);
    setReport(E, ListLink);
  }  
  
  private void setESSAttendee (Element E, int Link) {
    //used to create a Purpose object.
    ESSExtended P = new ESSExtended();
    P.set(E, Link);
    P.REMARKS = getValue(E, "title");  //Done as a "correction"
    P.CLIENT = getValue(E, "company"); //Done as a "correction"
    ESSExtendeds.addElement(P);
  }    
    
  
}
    
