/*
XML2Guideline.java - runs guidelines from the XML format of the ESS (WFE) report   
Copyright (C) 2004 R. James Holton

This program is free software; you can redistribute it and/or modify it 
under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2 of the License, or (at your option) 
any later version.  This program is distributed in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, Inc., 
675 Mass Ave, Cambridge, MA 02139, USA. 
*/


package ess;

//import java.io.*;
import java.util.*;
//import java.text.*;
import java.sql.*;
import org.jdom.Document;
import org.jdom.Element;
//import org.jdom.Attribute;
//import org.jdom.DocType;
//import org.jdom.input.SAXBuilder;
//import org.jdom.output.XMLOutputter;

public class XML2Guideline extends ExpenseReport {
   
    private String[] headerFieldNames = {"persnum", "name", "phone", 
                "location", "depart", "credate", "admin1", "email", "subDate", "currency"};
      
    private String[] purposeFieldNames = {"project",
                "stepno","clientno","client",
                "location","comment","begdate",
                "enddate","days","nights", "xref"};
                
    private String[] receiptFieldNames = {"Amount", "units", "xrate", "Rcptdate", "expense",
                    "Comment", "Charge", "Reimb", "begdate", "xref"};
                    
    private String[] expenseFieldNames = {"ExpenseType", "Category", "Amount",
                    "expdate", "perSplit", "Comment", "Xref", "Reimb"};
   
    public XML2Guideline(){
    }
   
    public XML2Guideline (Document reportDOM, Connection dbConn,String SQLTerminator) {
      this.dbConnection = dbConn;
      super.sqlTerminator = SQLTerminator; 
      super.PT.setConnection(dbConn);  //JH 1/19/04
      super.PT.setSQLTerminator(this.sqlTerminator);
      super.PT.setSQLStrings();
      setReport(reportDOM);
    }
    
    public void setConnection(Connection dbConn,String SQLTerminator) {
      this.dbConnection = dbConn;
      super.sqlTerminator = SQLTerminator; 
      super.PT.setConnection(dbConn);  //JH 1/19/04
      super.PT.setSQLTerminator(this.sqlTerminator);
      super.PT.setSQLStrings();
    }
    
    public void setHeaderFieldNames(String[] x)
    {
        headerFieldNames = x;
    }
    public void setPurposeFieldNames(String[] x)
    {
        purposeFieldNames = x;        
    }
    public void setReceiptFieldNames(String[] x)
    {
        receiptFieldNames = x;
    }
    public void setExpenseFieldNames(String[] x)
    {
        expenseFieldNames = x;
   }
    
   
    public void setReport (Document reportDOM)
    {
        Element root;
        root = reportDOM.getRootElement();
        Iterator itr = root.getChildren().iterator();
//        int i = 0;
        while (itr.hasNext()) {
            Element firstLevel = (Element)itr.next();
            if ((firstLevel.getName()).equals("receipt"))
            {
                receipts.addElement(buildReceiptObject(firstLevel));
            }
            else if ((firstLevel.getName()).equals("purpose"))
            {
                purposes.addElement(buildPurposeObject(firstLevel));
                
            }
            else // must be the header record
            {
                header = buildHeaderObject(firstLevel);
            }
        }
        
        rptExpenses.sort();
//System.out.print("ExpenseReport =" + rptExpenses);          
    }
    
    private Header buildHeaderObject(Element firstLevel)
    {
        
      final int PERS_NUM = 0;
      final int NAME = PERS_NUM + 1;
      final int PHONE = NAME + 1;
      final int LOCATION = PHONE + 1;
      final int DEPART = LOCATION + 1;
      final int CRE_DATE = DEPART + 1;
      final int ADMIN1 = CRE_DATE + 1;
      final int EMAIL = ADMIN1 + 1;
      final int SUB_DATE = EMAIL + 1;
      final int CURRENCY = SUB_DATE + 1;
      String persNum = "";
      String name = "";
      String phone = "";
      String location = "";
      String depart = "";
      java.util.Date creDate = null;
      String admin1 = "";
      String email = "";
      String currency = "";
      java.util.Date subDate = null;

      //String[] headerFieldNames = {"persnum", "name", "phone", 
      //          "location", "depart", "credate", "admin1", "email", "subDate"};
                
       
        Iterator itr = firstLevel.getChildren().iterator();
        int i = 0;
        while (itr.hasNext()) {
            i++;
            Element headerEntry = (Element)itr.next();
            int headerId = Utilities.findString(headerEntry.getName(), headerFieldNames);
            
            switch (headerId)
            {
                case PERS_NUM:
                    persNum = headerEntry.getText();
                    break;
                case NAME:
                     name = headerEntry.getText();
                    break;
                case PHONE:
                     phone = headerEntry.getText();
                    break;
                case LOCATION:
                     location = (headerEntry.getText()).toUpperCase();
                    break;
                case DEPART:
                     depart = headerEntry.getText();
                    break;
                case CRE_DATE:
                     //creDate = new java.util.Date(headerEntry.getText());
                	creDate = getDateFromString(headerEntry.getText());
                    break;
                case ADMIN1:
                     admin1 = headerEntry.getText();
                    break;
                case EMAIL:
                     email = headerEntry.getText();
                    break;
                case SUB_DATE:
                     String xdate = headerEntry.getText();
                     if (xdate != null && !xdate.equals("")) {
                        //subDate = new java.util.Date(xdate);
                        subDate = getDateFromString(xdate);
                     }
                    break;
                case CURRENCY:
                     currency = headerEntry.getText();
                    break;
                default:
                    break;
            }
        }
        
        return(new Header(persNum, name, phone, location, depart,
                creDate, admin1, email, subDate, currency));
    }
            
    private Purpose buildPurposeObject(Element firstLevel)
    {
        
      final int PROJECT = 0;
      final int STEPNO = PROJECT + 1;
      final int CLIENTNO = STEPNO + 1;
      final int CLIENT = CLIENTNO + 1;
      final int LOCATION = CLIENT + 1;
      final int COMMENT = LOCATION + 1;
      final int BEGDATE = COMMENT + 1;
      final int ENDDATE = BEGDATE + 1;
      final int DAYS = ENDDATE + 1;
      final int NIGHTS = DAYS + 1;
      final int XREF = NIGHTS + 1;
      
      String project = "";
      String stepno = "";
      String clientNo = "";
      String client = "";
      String location = "";
      String comment = "";
      java.util.Date begDate = null;
      java.util.Date endDate = null;
      int days = 0;
      int nights = 0;
      String xref = "";
      
      //String[] purposeFieldNames = {"project",
      //          "stepno","clientno","client",
      //          "location","comment","begdate",
      //          "enddate","days","nights", "xref"};
        
        
        Iterator itr = firstLevel.getChildren().iterator();
        int i = 0;
        while (itr.hasNext()) {
            i++;
            Element purposeEntry = (Element)itr.next();
            int purposeId = Utilities.findString(purposeEntry.getName(),
                                purposeFieldNames);
            switch (purposeId)
            {
                case PROJECT:
                    project = purposeEntry.getText();
                    break;
                case STEPNO:
                    stepno = purposeEntry.getText();
                    break;
                case CLIENTNO:
                    clientNo = purposeEntry.getText();
                    break;
                case CLIENT:
                    client = purposeEntry.getText();
                    break;
                case LOCATION:
                    location = (purposeEntry.getText()).toUpperCase();
                    break;
                case COMMENT:
                    comment = purposeEntry.getText();
                    break;
                case BEGDATE:
                    //begDate = Utilities.stringToDate(purposeEntry.getText());
                	begDate = getDateFromString(purposeEntry.getText());
                    break;
                case ENDDATE:
                    //endDate = Utilities.stringToDate(purposeEntry.getText());
                	endDate = getDateFromString(purposeEntry.getText());
                    break;
                case DAYS:
                    days = Integer.parseInt(purposeEntry.getText());
                    break;
                case NIGHTS:
                    nights = Integer.parseInt(purposeEntry.getText());
                    break;
                case XREF:
                    xref = purposeEntry.getText();
                    break;
                default:
                    break;
            }
        }
        
        return(new Purpose(project, stepno, clientNo, client, 
                location, comment, begDate, endDate,
                days, nights, xref));
    }

    private Receipt buildReceiptObject(Element firstLevel)
    {
      final int AMOUNT = 0;
      final int UNITS = AMOUNT + 1;
      final int XRATE = UNITS + 1;
      final int RCPTDATE = XRATE + 1;
      final int EXPENSE = RCPTDATE + 1;
      final int COMMENT = EXPENSE + 1;
      final int CHARGE = COMMENT + 1;
      final int REIMB = CHARGE + 1;
      final int BEGDATE = REIMB + 1;
      final int XREF = BEGDATE + 1;
      
      double amount = 0;
      String units = "";
      float xrate = 0.0f;
      java.util.Date rcptDate = null;
      String sequence = "";
      Vector rcptExpenses = new Vector();        
      String comment = "";
      String charge = "";
      String reimbStatus = "";
      java.util.Date begDate = null;
      String xref = "";
      Expense expense;
      String workVar;
      
      //String[] receiptFieldNames = {"Amount", "units", "xrate", "Rcptdate", "expense",
      //              "Comment", "Charge", "Reimb", "xref"};
                
 
        Iterator itr = firstLevel.getChildren().iterator();
        int i = 0;
        while (itr.hasNext()) {
            i++;
            Element receiptEntry = (Element)itr.next();
            int receiptId =  Utilities.findString(receiptEntry.getName(),
                                receiptFieldNames);
                                
            switch (receiptId)
            {
                case AMOUNT:
                    try{
                        amount = Double.parseDouble(receiptEntry.getText());
                    }
                    catch (NumberFormatException error)
                    {
                        amount = 0;
                    }
                    break;
                case UNITS:
                    units = receiptEntry.getText();
                    break;
                case XRATE:
                    try{
                        xrate = Float.parseFloat(receiptEntry.getText());
                    }
                    catch (NumberFormatException error)
                    {
                        xrate = 0.0f;
                    }
                    break;
                case RCPTDATE:
                    //rcptDate = Utilities.stringToDate(receiptEntry.getText());
                	rcptDate = getDateFromString(receiptEntry.getText());
                    break;
                case EXPENSE:
                    sequence = receiptEntry.getAttributeValue("sequence");
                    expense = buildExpenseObject(sequence, rcptDate, amount, xref,
                                receiptEntry);
                    rcptExpenses.addElement(expense);
                    rptExpenses.addElement(expense);
                    break;
                case COMMENT:
                    comment = receiptEntry.getText();
                    break;
                case CHARGE:
                    charge = receiptEntry.getText();
                    break;
                case REIMB:
                    reimbStatus = receiptEntry.getText();
                    break;
                case BEGDATE:
                    workVar = receiptEntry.getText();
                    
                    //begDate = Utilities.stringToDate(workVar);
                    begDate = getDateFromString(workVar);
                    break;

                case XREF:
                    xref = receiptEntry.getText();
                    break;
                default:
                    break;
            }
        }
                    
        return(new Receipt(xref, amount, units, xrate, rcptDate,rcptExpenses, comment, charge,
                           reimbStatus, begDate, xref));
    }
    
   
    private Expense buildExpenseObject(String sequence, java.util.Date rcptDate, double rcptAmount,
                 String rcptXref, Element firstLevel)
    {
      final int EXPENSE_TYPE = 0;
      final int CATEGORY = EXPENSE_TYPE + 1;
      final int AMOUNT = CATEGORY + 1;
      final int EXPDATE = AMOUNT + 1;
      final int PERSPLIT = EXPDATE + 1;
      final int COMMENT = PERSPLIT + 1;
      final int XREF = COMMENT + 1;
      final int REIMB = XREF + 1;
      String expenseType =  "";
      String expenseCategory = "";
      double amount = 0.0;
      java.util.Date expenseDate = rcptDate;
      int perSplit = 0;
      String comment = "";
      String xref = "";
      String reimburseStatus = null;
      
      //String[] expenseFieldNames = {"ExpenseType", "Category", "Amount",
      //              "expdate", "perSplit", "Comment", "Xref", "Reimb"};
      
        amount = rcptAmount;  //defaults for receipt amount
        Iterator itr = firstLevel.getChildren().iterator();
        int i = 0;
        while (itr.hasNext()) {
            i++;
            Element expenseEntry = (Element)itr.next();
            int expenseId =  Utilities.findString(expenseEntry.getName(),
                                expenseFieldNames);

            switch (expenseId)
            {
                case EXPENSE_TYPE:
                    expenseType = expenseEntry.getText();
                    break;
                case CATEGORY:
                    expenseCategory = expenseEntry.getText();
                    break;
                case AMOUNT:
                    try{
                        amount = Double.parseDouble(expenseEntry.getText());
                    }
                    catch (NumberFormatException error)
                    {
                        amount = rcptAmount;
                    }
                    break;
                case EXPDATE:
                    //expenseDate = Utilities.stringToDate(expenseEntry.getText());
                    expenseDate = getDateFromString(expenseEntry.getText());
                    break;
                case PERSPLIT:
                    perSplit = Integer.parseInt(expenseEntry.getText());
                    break;
                case COMMENT:
                    comment = expenseEntry.getText();
                    break;
                case XREF:
                    xref = expenseEntry.getText();
                     break;
                case REIMB:
                    reimburseStatus = expenseEntry.getText();
                     break;
                default:
                    break;
            }
            
        }

        // if (amount == 0)
        //       amount = rcptAmount;
               
        if (expenseDate == null)
            expenseDate = rcptDate;
        if (xref == "")
            xref = rcptXref;
            
        return(new Expense(rcptXref, sequence, expenseDate, expenseType, expenseCategory, amount,
                           perSplit, comment, xref, reimburseStatus));
    }
    
   
}
